@@ -7,9 +7,11 @@ def header(file_):
     content = "# Nabu configuration parameters\nThis file lists all the current configuration parameters available in the [configuration file](nabu_config_file.md)."
     print(content, file=file_)
 
+
 def generate(file_):
     def write(content):
         print(content, file=file_)
+
     header(file_)
     for section, values in nabu_config.items():
         if section == "about":
@@ -22,17 +24,19 @@ def generate(file_):
             if "---" in help_content:
                 help_content = help_content.replace("--", "")
             write(help_content + "\n")
-            write(
-                "```ini\n%s = %s\n```"
-                % (key, val["default"])
-            )
+            write("```ini\n%s = %s\n```" % (key, val["default"]))
 
 
+def format_lists(fname):
+    with open(fname, "r") as f:
+        content = f.read()
 
-if __name__ == "__main__":
-    fname = os.path.join(
-        os.path.dirname(os.path.realpath(__file__)),
-        "nabu_config_items.md"
-    )
     with open(fname, "w") as f:
-        generate(f)
+        content = content.replace(":\n - ", ":\n\n  - ").replace(":\n-", ":\n\n  - ").replace("\n-", "\n  - ")
+        f.write(content)
+
+
+fname = os.path.join("docs", "nabu_config_items.md")
+with open(fname, "w") as f:
+    generate(f)
+format_lists(fname)
