@@ -8,6 +8,7 @@ from nabu.pipeline.config_validators import str2bool
 
 from tomoscan.framereducer.method import ReduceMethod
 
+reduce_methods = tuple(member.value for member in ReduceMethod)
 
 BootstrapConfig = {
     "bootstrap": {
@@ -48,35 +49,6 @@ BootstrapConfig = {
     },
 }
 
-# Default configuration for "zsplit" command
-ZSplitConfig = {
-    "input_file": {
-        "help": "Input HDF5-Nexus file",
-        "mandatory": True,
-    },
-    "output_directory": {
-        "help": "Output directory to write split files.",
-        "mandatory": True,
-    },
-    "loglevel": {
-        "help": "Logging level. Can be 'debug', 'info', 'warning', 'error'. Default is 'info'.",
-        "default": "info",
-    },
-    "entry": {
-        "help": "HDF5 entry to take in the input file. By default, the first entry is taken.",
-        "default": "",
-    },
-    "n_stages": {
-        "help": "Number of expected stages (i.e different 'Z' values). By default it is inferred from the dataset.",
-        "default": -1,
-        "type": int,
-    },
-    "use_virtual_dataset": {
-        "help": "Whether to use virtual datasets for output file. Not using a virtual dataset duplicates data and thus results in big files ! However virtual datasets currently have performance issues. Default is False",
-        "default": 0,
-        "type": int,
-    },
-}
 
 # Default configuration for "histogram" command
 HistogramConfig = {
@@ -147,7 +119,7 @@ ReconstructConfig = {
     },
     "phase_margin": {
         "help": "Specify an explicit phase margin to use when performing phase retrieval.",
-        "default": -1,
+        "default": 0,
         "type": int,
     },
     "force_use_grouped_pipeline": {
@@ -162,7 +134,7 @@ MultiCorConfig = ReconstructConfig.copy()
 MultiCorConfig.update(
     {
         "cor": {
-            "help": "Positions of the center of rotation. It must be a list of comma-separated scalars, or in the form start:stop:step, where start, stop and step can all be floating-point values.",
+            "help": "Absolute positions of the center of rotation. It must be a list of comma-separated scalars, or in the form start:stop:step, where start, stop and step can all be floating-point values.",
             "default": "",
             "mandatory": True,
         },
@@ -175,38 +147,6 @@ MultiCorConfig.update(
 )
 
 
-GenerateInfoConfig = {
-    "hist_file": {
-        "help": "HDF5 file containing the histogram, either the reconstruction file or a dedicated histogram file.",
-        "default": "",
-    },
-    "hist_entry": {
-        "help": "Histogram HDF5 entry. Defaults to the first available entry.",
-        "default": "",
-    },
-    "output": {
-        "help": "Output file name",
-        "default": "",
-        "mandatory": True,
-    },
-    "bliss_file": {
-        "help": "HDF5 master file produced by BLISS",
-        "default": "",
-    },
-    "bliss_entry": {
-        "help": "Entry in the HDF5 master file produced by BLISS. By default, take the first entry.",
-        "default": "",
-    },
-    "info_file": {
-        "help": "Path to the .info file, in the case of a EDF dataset",
-        "default": "",
-    },
-    "edf_proj": {
-        "help": "Path to a projection, in the case of a EDF dataset",
-        "default": "",
-    },
-}
-
 RotateRadiosConfig = {
     "dataset": {
         "help": "Path to the dataset. Only HDF5 format is supported for now.",
@@ -322,6 +262,64 @@ CompareVolumesConfig = {
     },
 }
 
+EstimateMotionConfig = {
+    "dataset": {
+        "help": "Path to the dataset.",
+        "default": "",
+        "mandatory": True,
+    },
+    "flatfield": {
+        "help": "Whether to perform flatfield normalization. Default is True.",
+        "default": "1",
+        "type": int,
+    },
+    "rot_center": {
+        "help": "Center of rotation. If not provided, will be estimated.",
+        "default": None,
+    },
+    "subsampling": {
+        "help": "For 360-degrees scan, angular subsampling for matching opposite projections. Default is 10.",
+        "default": 10,
+        "type": int,
+    },
+    "deg_xy": {
+        "help": "Polynomial degree in x-y for sample movement polynomial model",
+        "default": 2,
+        "type": int,
+    },
+    "deg_z": {
+        "help": "Polynomial degree in z (vertical) for sample movement polynomial model",
+        "default": 2,
+        "type": int,
+    },
+    "win_size": {
+        "help": "Size of the look-up window for half-tomography",
+        "default": 300,
+        "type": int,
+    },
+    "verbose": {
+        "help": "Whether to plot the movement estimation fit",
+        "default": 1,
+    },
+    "output_file": {
+        "help": "Path of the output file containing the sample translations projected in the detector reference frame. This file can be directly used in 'translation_movements_file' of nabu configuration",
+        "default": "correct_motion.txt",
+    },
+    "only": {
+        "help": "Whether to only generate motion file for horizontal or vertical movement: --only horizontal  or --only vertical",
+        "default": "",
+    },
+    "ccd_filter_size": {
+        "help": "Size of conditional median filter to apply on radios. Default is zero (disabled)",
+        "default": 0,
+        "type": int,
+    },
+    "ccd_filter_threshold": {
+        "help": "Threshold for median filter, 'ccd_filter_size' is not zero. Default is 0.04",
+        "default": 0.04,
+        "type": float,
+    },
+}
 
 # Default configuration for "stitching" command
 StitchingConfig = {
@@ -344,7 +342,7 @@ StitchingConfig = {
 # Default configuration for "stitching-bootstrap" command
 BootstrapStitchingConfig = {
     "stitching-type": {
-        "help": f"User can provide stitching type to filter some parameters. Must be in {StitchingType.values()}.",
+        "help": f"User can provide stitching type to filter some parameters. Must be in {[sst for sst in StitchingType]}.",
         "default": None,
     },
     "level": {
@@ -462,11 +460,6 @@ DiagToRotConfig = {
     "--diag_file": dict(
         required=True, help="The reconstruction file obtained by nabu-helical using the diag_zpro_run option", type=str
     ),
-    "--entry_name": dict(
-        required=False,
-        help="entry_name. Defauls is entry0000",
-        default="entry0000",
-    ),
     "--near": dict(
         required=False,
         help="This is a relative offset respect to the center of the radios. The cor will be searched around the provided value. If not given the optinal parameter original_scan must be the original nexus file; and the estimated core will be taken there. The netry_name parameter also must be provided in this case",
@@ -564,12 +557,12 @@ ReduceDarkFlatConfig = {
         "required": False,
     },
     "dark-method": {
-        "help": f"Define the method to be used for computing darks. Valid methods are {ReduceMethod.values()}",
+        "help": f"Define the method to be used for computing darks. Valid methods are {reduce_methods}",
         "default": ReduceMethod.MEAN,
         "required": False,
     },
     "flat-method": {
-        "help": f"Define the method to be used for computing flats. Valid methods are {ReduceMethod.values()}",
+        "help": f"Define the method to be used for computing flats. Valid methods are {reduce_methods}",
         "default": ReduceMethod.MEDIAN,
         "required": False,
     },
@@ -613,6 +606,45 @@ ReduceDarkFlatConfig = {
     },
 }
 
+PCAFlatsConfig = {
+    "datasets": {"help": "datasets to be stitched together", "default": tuple(), "nargs": "+", "mandatory": True},
+    "nsigma": {
+        "help": "Paramter to select PCA components. Default is 3. Higher nsigma, less components.",
+        "default": 3.0,
+        "type": float,
+        "required": False,
+    },
+    "flat-method": {
+        "help": f"Define the method to be used for computing flats. Valid methods are {reduce_methods}",
+        "default": ReduceMethod.MEDIAN,
+        "required": False,
+    },
+    "dark-method": {
+        "help": f"Define the method to be used for computing darks. Valid methods are {reduce_methods}",
+        "default": ReduceMethod.MEAN,
+        "required": False,
+    },
+    "overwrite": {
+        "dest": "overwrite",
+        "action": "store_true",
+        "default": False,
+        "help": "Overwrite dark/flats if exists",
+    },
+    "debug": {
+        "dest": "debug",
+        "action": "store_true",
+        "default": False,
+        "help": "Set logging system in debug mode",
+        "required": False,
+    },
+    "output-filename": {
+        "aliases": ("orfl",),
+        "default": None,
+        "help": "Where to save PCA flats. If not provided will be dumped in the current folder as{scan_prefix}_PCAFlats.hdf5",
+        "required": False,
+    },
+}
+
 ShowReconstructionTimingsConfig = {
     "logfile": {
         "help": "Path to the log file.",
@@ -629,3 +661,25 @@ ShowReconstructionTimingsConfig = {
         "type": str,
     },
 }
+
+FlipVolumeVerticallyConfig = {
+    "input": {
+        "help": "Path to the volume to flip upside-down, either a directory or a file.",
+        "default": "",
+        "mandatory": True,
+    },
+    "vol_type": {
+        "help": "Which type of volume is to be flipped. Possible values are: hdf5, tiff, tiff3d, edf, jp2, raw",
+        "mandatory": True,
+        "default": "",
+    },
+    "file_prefix": {
+        "help": "File prefix, if relevant",
+        "default": "",
+    },
+    "mem_fraction": {
+        "help": "For HDF5/tiff3D/Raw volumes, amount of available RAM to use. Should be a number between 0 and 1 (eg. 0.5). Default is 0.25",
+        "default": 0.25,
+        "type": float,
+    },
+}
