@@ -31,7 +31,7 @@ from ..pipeline.estimators import oversample
   we must bring to the pixel size
 
   An example of collection is this :
-    
+
    |_____ diagnostics
    |               |  
                   |__ 0
@@ -72,10 +72,10 @@ def transform_images(diag, ovs):
 
 def detailed_merit(diag, shift):
     # res will become the merit summed over all the pairs theta, theta+180
-    res = 0.0
+    # res = 0.0
 
     # need to account for the weight also. So this will become the used weight for the pairs theta, theta+180
-    res_w = 0.0
+    # res_w = 0.0
 
     ## The following two variables are very important information to be collected.
     ## On the the z translation over a 360 turn
@@ -149,7 +149,7 @@ def build_total_merit_list(diag, oversample_factor, args):
     # calculats the merit at all the tested extra adjustment shifts.
 
     transform_images(diag, [oversample_factor, 1])
-    h_ima = diag.radios[0].shape[0]
+    # h_ima = diag.radios[0].shape[0]
     # search_radius_v = min(oversample_factor * args.search_radius_v, h_ima - 1)
     search_radius_v = oversample_factor * args.search_radius_v
 
@@ -233,7 +233,6 @@ def main(user_args=None):
     # For each key there is a sequence of radio, the corresponding sequence of weights map, the corresponding z translation, and angles
 
     zpix_mm = None
-    observed_oneturn_total_shift_zpix = None
 
     argument_list = [
         (DictToObj(h5todict(args.diag_file, os.path.join(diag_url, my_key))), oversample_factor, args)
@@ -244,7 +243,7 @@ def main(user_args=None):
     with Pool(processes=ncpus) as pool:
         all_res_plus_infos = pool.starmap(build_total_merit_list, argument_list)
 
-    observed_oneturn_total_shift_zpix, zpix_mm = None, None
+    _, zpix_mm = None, None
 
     # needs to flatten the result of pool.map
     for_all_pairs_detailed_merit_lists = []
