@@ -31,7 +31,7 @@ from .cli_configs import DiagToRotConfig
   The collection is not done here. Here we exploit the result of a previous collection to deduce, looking at the correlations, the cor
 
   An example of collection is this :
-    
+
    |_____ diagnostics
    |               |  
                   |__ 0
@@ -68,12 +68,13 @@ def transform_images(diag, args):
     But beforehand it is beneficial to remove low spatial frequencies.
     And we do oversampling on the fly.
 
-    Parameters:
+    Parameters
+    ----------
        diag: object
          used member of diag are radios and weights
        args: object
          its member are the application parameters. Here we use only:
-             low_pass, high_pass, ovs ( oversampling factor for the horisontal dimension )
+            low_pass, high_pass, ovs ( oversampling factor for the horizontal dimension )
 
 
 
@@ -196,7 +197,6 @@ def find_best_interpolating_line(args):
     z_b = np.nanmax(all_z_transl)
 
     best_error = np.nan
-    best_off_pair = None
 
     for index_ovlp_a in index_overlap_list_a:
         for index_ovlp_b in index_overlap_list_b:
@@ -204,7 +204,7 @@ def find_best_interpolating_line(args):
             indexes = (np.arange(all_energies.shape[0]))[~np.isnan(index_ovlps)].astype("i")
 
             index_ovlps = index_ovlps[~np.isnan(index_ovlps)]
-            index_ovlps = np.round_(index_ovlps).astype("i")
+            index_ovlps = np.round(index_ovlps).astype("i")
 
             diff_enes = all_res[(indexes, index_ovlps)]
             orig_enes = all_energies[(indexes, index_ovlps)]
@@ -313,13 +313,12 @@ def main(user_args=None):
             and this is not enough to do correlation + interpolation between sections
             """
             raise RuntimeError(message)
-    else:
-        if n_pairings_with_data < 1:
-            message = f""" The diagnostics collection has probably been run over a too thin section of the scan
+    elif n_pairings_with_data < 1:
+        message = f""" The diagnostics collection has probably been run over a too thin section of the scan
             or you scan does not allow to form pairs of theta, theta+360. I only found {n_pairings_with_data}
             pairings
             """
-            raise RuntimeError(message)
+        raise RuntimeError(message)
 
     # all_merits, all_energies, all_z_transls = zip( result_list )
 
@@ -345,15 +344,15 @@ def main(user_args=None):
 
 def do_height_by_height(args, overlap_list, all_diff, all_energies, all_z_transl):
     # now we find the best cor for each chunk, or nan if no overlap is found
-    z_a = np.min(all_z_transl)
-    z_b = np.max(all_z_transl)
+    # z_a = np.min(all_z_transl)
+    # z_b = np.max(all_z_transl)
 
     grouped_diff = {}
     grouped_energy = {}
 
     for diff, energy, z in zip(all_diff, all_energies, all_z_transl):
         found = z
-        for key in grouped_diff.keys():
+        for key in grouped_diff:
             if abs(key - z) < 2.0:  # these are in pixel units
                 found = key
                 break
