@@ -5,8 +5,8 @@ from silx.io.dictdump import h5todict
 from ..utils import check_supported
 from ..io.utils import get_first_hdf5_entry, get_h5_value
 from ..io.writer import NXProcessWriter
-from ..misc.histogram import PartialHistogram, VolumeHistogram, hist_as_2Darray
-from ..misc.histogram_cuda import CudaVolumeHistogram
+from ..processing.histogram import PartialHistogram, VolumeHistogram, hist_as_2Darray
+from ..processing.histogram_cuda import CudaVolumeHistogram
 from ..resources.logger import Logger, LoggerOrPrint
 from .utils import parse_params_values
 from .cli_configs import HistogramConfig
@@ -190,6 +190,7 @@ def histogram_cli():
     hist, configs = histogramer.get_histogram(return_config=True)
     config = histogramer.merge_histograms_configurations(configs)
     histogramer.write_histogram(hist, config=config)
+    return 0
 
 
 if __name__ == "__main__":
