@@ -4,7 +4,6 @@ from scipy.special import erf  # pylint: disable=all
 import sys
 import os
 from scipy.ndimage import gaussian_filter
-from nxtomo.nxobject.nxdetector import ImageKey
 from nabu.resources.nxflatfield import update_dataset_info_flats_darks
 from nabu.resources.dataset_analyzer import HDF5DatasetAnalyzer
 from ..io.reader import load_images_from_dataurl_dict
@@ -21,7 +20,7 @@ def main(argv=None):
     provide a default construction of such maps. The double-flat is set to one and
     the weight is build on the basis of the flat fields   from the dataset with an apodisation on the borders
     which allows to eliminate discontinuities in the contributions from the borders, above and below
-    for the z-translations, and on the left or roght border for half-tomo.
+    for the z-translations, and on the left or right border for half-tomo.
 
     The usage is ::
 
@@ -57,7 +56,7 @@ def main(argv=None):
     beam_profile = 0
     my_flats = load_images_from_dataurl_dict(dataset_info.flats)
 
-    for key, flat in my_flats.items():
+    for flat in my_flats.values():
         beam_profile += flat
     beam_profile = beam_profile / len(list(dataset_info.flats.keys()))
 
