@@ -1,5 +1,5 @@
-from tomoscan.io import HDF5File
 from .. import version
+from ..io.reader import list_hdf5_entries
 from ..utils import list_match_queries
 from ..pipeline.config import parse_nabu_config_file
 from ..pipeline.config_validators import convert_to_int
@@ -92,12 +92,6 @@ def get_reconstructor(args, overwrite_options=None):
     return reconstructor
 
 
-def list_hdf5_entries(fname):
-    with HDF5File(fname, "r") as f:
-        entries = list(f.keys())
-    return entries
-
-
 def main():
     args = parse_params_values(
         ReconstructConfig,
