@@ -67,12 +67,17 @@ def main():
         futures = {}
         # 2.1 launch jobs
         slurm_job_progress_bars: dict = {}
+
+        # set job name
+        final_output_object_identifier = stitching_config.get_output_object().get_identifier().to_str()
+        stitching_config.slurm_config.job_name = f"stitching-{final_output_object_identifier}"
+
         for i_job, (job, sub_config) in enumerate(
             split_stitching_configuration_to_slurm_job(stitching_config, yield_configuration=True)
         ):
             _logger.info(f"submit job nb {i_job}: handles {sub_config.slices}")
-            output_volume = sub_config.get_output_object().get_identifier().to_str()
-            futures[output_volume] = submit(job, timeout=999999)
+            output_object = sub_config.get_output_object().get_identifier().to_str()
+            futures[output_object] = submit(job, timeout=999999)
             # note on total=100: we only consider percentage in this case (providing advancement from slurm jobs)
             slurm_job_progress_bars[job] = tqdm(
                 total=100,
