@@ -10,7 +10,12 @@ def parse_params_values(Params, parser_description=None, program_version=None, u
             optional = not (vals.pop("mandatory", False))
             if optional:
                 param_name = "--" + param_name
-        parser.add_argument(param_name, **vals)
+            aliases = vals.pop("aliases", tuple())
+            if optional:
+                aliases = tuple(["--" + alias for alias in aliases])
+        else:
+            aliases = ()
+        parser.add_argument(param_name, *aliases, **vals)
     if program_version is not None:
         parser.add_argument("--version", "-V", action="version", version=program_version)
 
