@@ -87,20 +87,20 @@ class AlignmentBase:
         if not len(shape_stack) == 3:
             raise ValueError(
                 "A stack of 2-dimensional images is required. Shape of stack: %s"
-                % (" ".join(("%d" % x for x in shape_stack)))
+                % (" ".join("%d" % x for x in shape_stack))
             )
         if not len(shape_pos) == 1:
             raise ValueError(
                 "Positions need to be a 1-dimensional array. Shape of the positions variable: %s"
-                % (" ".join(("%d" % x for x in shape_pos)))
+                % (" ".join("%d" % x for x in shape_pos))
             )
         if not shape_stack[0] == shape_pos[0]:
             raise ValueError(
                 "The same number of images and positions is required."
                 + " Shape of stack: %s, shape of positions variable: %s"
                 % (
-                    " ".join(("%d" % x for x in shape_stack)),
-                    " ".join(("%d" % x for x in shape_pos)),
+                    " ".join("%d" % x for x in shape_stack),
+                    " ".join("%d" % x for x in shape_pos),
                 )
             )
 
@@ -110,23 +110,23 @@ class AlignmentBase:
         shape_2 = np.squeeze(img_2).shape
         if not len(shape_1) == 2:
             raise ValueError(
-                "Images need to be 2-dimensional. Shape of image #1: %s" % (" ".join(("%d" % x for x in shape_1)))
+                "Images need to be 2-dimensional. Shape of image #1: %s" % (" ".join("%d" % x for x in shape_1))
             )
         if not len(shape_2) == 2:
             raise ValueError(
-                "Images need to be 2-dimensional. Shape of image #2: %s" % (" ".join(("%d" % x for x in shape_2)))
+                "Images need to be 2-dimensional. Shape of image #2: %s" % (" ".join("%d" % x for x in shape_2))
             )
         if not np.all(shape_1 == shape_2):
             raise ValueError(
                 "Images need to be of the same shape. Shape of image #1: %s, image #2: %s"
                 % (
-                    " ".join(("%d" % x for x in shape_1)),
-                    " ".join(("%d" % x for x in shape_2)),
+                    " ".join("%d" % x for x in shape_1),
+                    " ".join("%d" % x for x in shape_2),
                 )
             )
 
     @staticmethod
-    def refine_max_position_2d(f_vals: np.ndarray, fy=None, fx=None):
+    def refine_max_position_2d(f_vals: np.ndarray, fy=None, fx=None, logger=None):
         """Computes the sub-pixel max position of the given function sampling.
 
         Parameters
@@ -150,10 +150,11 @@ class AlignmentBase:
             Estimated (vertical, horizontal) function max, according to the
             coordinates in fy and fx.
         """
+        logger = LoggerOrPrint(logger)
         if not (len(f_vals.shape) == 2):
             raise ValueError(
                 "The fitted values should form a 2-dimensional array. Array of shape: [%s] was given."
-                % (" ".join(("%d" % s for s in f_vals.shape)))
+                % (" ".join("%d" % s for s in f_vals.shape))
             )
         if fy is None:
             fy_half_size = (f_vals.shape[0] - 1) / 2
@@ -161,7 +162,7 @@ class AlignmentBase:
         elif not (len(fy.shape) == 1 and np.all(fy.size == f_vals.shape[0])):
             raise ValueError(
                 "Vertical coordinates should have the same length as values matrix. Sizes of fy: %d, f_vals: [%s]"
-                % (fy.size, " ".join(("%d" % s for s in f_vals.shape)))
+                % (fy.size, " ".join("%d" % s for s in f_vals.shape))
             )
         if fx is None:
             fx_half_size = (f_vals.shape[1] - 1) / 2
@@ -169,7 +170,7 @@ class AlignmentBase:
         elif not (len(fx.shape) == 1 and np.all(fx.size == f_vals.shape[1])):
             raise ValueError(
                 "Horizontal coordinates should have the same length as values matrix. Sizes of fx: %d, f_vals: [%s]"
-                % (fx.size, " ".join(("%d" % s for s in f_vals.shape)))
+                % (fx.size, " ".join("%d" % s for s in f_vals.shape))
             )
 
         fy, fx = np.meshgrid(fy, fx, indexing="ij")
@@ -190,14 +191,7 @@ class AlignmentBase:
         vertex_max_yx = [np.max(fy), np.max(fx)]
         if np.any(vertex_yx < vertex_min_yx) or np.any(vertex_yx > vertex_max_yx):
             raise ValueError(
-                "Fitted (y: {}, x: {}) positions are outside the input margins y: [{}, {}], and x: [{}, {}]".format(
-                    vertex_yx[0],
-                    vertex_yx[1],
-                    vertex_min_yx[0],
-                    vertex_max_yx[0],
-                    vertex_min_yx[1],
-                    vertex_max_yx[1],
-                )
+                f"Fitted (y: {vertex_yx[0]}, x: {vertex_yx[1]}) positions are outside the input margins y: [{vertex_min_yx[0]}, {vertex_max_yx[0]}], and x: [{vertex_min_yx[1]}, {vertex_max_yx[1]}]"
             )
         return vertex_yx
 
@@ -225,10 +219,10 @@ class AlignmentBase:
         float
             Estimated function max, according to the coordinates in fx.
         """
-        if not len(f_vals.shape) in (1, 2):
+        if len(f_vals.shape) not in (1, 2):
             raise ValueError(
                 "The fitted values should be either one or a collection of 1-dimensional arrays. Array of shape: [%s] was given."
-                % (" ".join(("%d" % s for s in f_vals.shape)))
+                % (" ".join("%d" % s for s in f_vals.shape))
             )
         num_vals = f_vals.shape[0]
 
@@ -264,16 +258,9 @@ class AlignmentBase:
         upper_bound_ok = vertex_x < vertex_max_x
         if not np.all(lower_bound_ok * upper_bound_ok):
             if len(f_vals.shape) == 1:
-                message = "Fitted position {} is outide the input margins [{}, {}]".format(
-                    vertex_x, vertex_min_x, vertex_max_x
-                )
+                message = f"Fitted position {vertex_x} is outide the input margins [{vertex_min_x}, {vertex_max_x}]"
             else:
-                message = "Fitted positions outside the input margins [{}, {}]: {} below and {} above".format(
-                    vertex_min_x,
-                    vertex_max_x,
-                    np.sum(1 - lower_bound_ok),
-                    np.sum(1 - upper_bound_ok),
-                )
+                message = f"Fitted positions outside the input margins [{vertex_min_x}, {vertex_max_x}]: {np.sum(1 - lower_bound_ok)} below and {np.sum(1 - upper_bound_ok)} above"
             raise ValueError(message)
         if return_vertex_val:
             vertex_val = coeffs[0, :] + vertex_x * coeffs[1, :] / 2
@@ -354,7 +341,7 @@ class AlignmentBase:
         if not (len(img_shape) == 2):
             raise ValueError(
                 "The input image should be either a 1 or 2-dimensional array. Array of shape: [%s] was given."
-                % (" ".join(("%d" % s for s in cc.shape)))
+                % (" ".join("%d" % s for s in cc.shape))
             )
         other_axis = (axis + 1) % 2
         # get pixel having the maximum value of the correlation array
@@ -551,7 +538,7 @@ class AlignmentBase:
     def close_last_plot_windows(self, n=1):
         """
         Close the last "n" plot windows.
-        Applicable only if the class was instanciated with verbose=True.
+        Applicable only if the class was instantiated with verbose=True.
 
         Parameters
         -----------
