@@ -80,7 +80,7 @@ class CenterOfRotation(AlignmentBase):
         return_validity: a boolean, defaults to false
             if set to True adds a second return value which may have three string values.
             These values are "unknown", "sound", "questionable".
-            It will be "uknown" if the  validation method is not implemented
+            It will be "unknown" if the  validation method is not implemented
             and it will be "sound" or "questionable" if it is implemented.
 
 
@@ -132,7 +132,7 @@ class CenterOfRotation(AlignmentBase):
         img_2 = self._prepare_image(img_2, roi_yxhw=roi_yxhw, median_filt_shape=median_filt_shape)
 
         cc = self._compute_correlation_fft(img_1, img_2, padding_mode, high_pass=high_pass, low_pass=low_pass)
-        img_shape = img_2.shape
+        img_shape = cc.shape  # Because cc.shape can differ from img_2.shape (e.g. in case of odd nb of cols)
         cc_vs = np.fft.fftfreq(img_shape[-2], 1 / img_shape[-2])
         cc_hs = np.fft.fftfreq(img_shape[-1], 1 / img_shape[-1])
 
@@ -177,7 +177,7 @@ class CenterOfRotationSlidingWindow(CenterOfRotation):
         return_relative_to_middle=None,
     ):
         """Semi-automatically find the Center of Rotation (CoR), given two images
-        or sinograms. Suitable for half-aquisition scan.
+        or sinograms. Suitable for half-acquisition scan.
 
         This method finds the half-shift between two opposite images,  by
         minimizing difference over a moving window.
@@ -269,10 +269,9 @@ class CenterOfRotationSlidingWindow(CenterOfRotation):
         win_ind_max = np.argmax(diffs_mean)
 
         diffs_std = diffs_std.min() - diffs_std
-        if not win_ind_max == np.argmax(diffs_std):
+        if win_ind_max != np.argmax(diffs_std):
             self.logger.warning(
-                "Minimum mean difference and minimum std-dev of differences do not coincide. "
-                + "This means that the validity of the found solution might be questionable."
+                "Minimum mean difference and minimum std-dev of differences do not coincide. This means that the validity of the found solution might be questionable."
             )
             validity_check_result = cor_result_validity["questionable"]
         else:
@@ -331,7 +330,7 @@ class CenterOfRotationGrowingWindow(CenterOfRotation):
         return_relative_to_middle=None,
     ):
         """Automatically find the Center of Rotation (CoR), given two images or
-        sinograms. Suitable for half-aquisition scan.
+        sinograms. Suitable for half-acquisition scan.
 
         This method finds the half-shift between two opposite images,  by
         minimizing difference over a moving window.
@@ -435,10 +434,9 @@ class CenterOfRotationGrowingWindow(CenterOfRotation):
         win_ind_max = np.argmax(diffs_mean)
 
         diffs_std = diffs_std.min() - diffs_std
-        if not win_ind_max == np.argmax(diffs_std):
+        if win_ind_max != np.argmax(diffs_std):
             self.logger.warning(
-                "Minimum mean difference and minimum std-dev of differences do not coincide. "
-                + "This means that the validity of the found solution might be questionable."
+                "Minimum mean difference and minimum std-dev of differences do not coincide. This means that the validity of the found solution might be questionable."
             )
             validity_check_result = cor_result_validity["questionable"]
         else:
@@ -480,7 +478,7 @@ class CenterOfRotationAdaptiveSearch(CenterOfRotation):
     which can possibly contain the good center of rotation.
     The whole image is spanned during several applications of the apodisation. At each application
     the apodisation function, which is a gaussian, is moved to a new guess position.
-    The lenght of the step, by which the gaussian is moved, and its sigma are
+    The length of the step, by which the gaussian is moved, and its sigma are
     obtained by multiplying the shortest distance from the left or right border with
     a self.step_fraction and  self.sigma_fraction factors which ensure global overlapping.
     for each step a region around the CoR  of each image is selected, and the regions of the two images
@@ -499,6 +497,7 @@ class CenterOfRotationAdaptiveSearch(CenterOfRotation):
         self,
         img_1: np.ndarray,
         img_2: np.ndarray,
+        side=None,
         roi_yxhw=None,
         median_filt_shape=None,
         padding_mode=None,
@@ -534,6 +533,10 @@ class CenterOfRotationAdaptiveSearch(CenterOfRotation):
             )
             return_relative_to_middle = True  # the kwarg above will be False by default in a future release
         # ---
+
+        if side is not None:
+            self.logger.error("CenterOfRotationOctaveAccurate: the 'side' keyword argument will have no effect")
+
         validity_check_result = cor_result_validity["unknown"]
 
         self._check_img_pair_sizes(img_1, img_2)
@@ -578,8 +581,7 @@ class CenterOfRotationAdaptiveSearch(CenterOfRotation):
 
         if lim_2 <= lim_1:
             message = (
-                "Image shape or cropped selection too small for global search."
-                + " After removal of the margins the search limits collide."
+                "Image shape or cropped selection too small for global search. After removal of the margins the search limits collide."
                 + " The cropped size is %d\n" % (dim_radio)
             )
             raise ValueError(message)
@@ -616,11 +618,11 @@ class CenterOfRotationAdaptiveSearch(CenterOfRotation):
                 if "positions are outside the input margins" in str(err):
                     x_cor = min(x_cor + x_cor * self.step_fraction, x_cor + (dim_radio - x_cor) * self.step_fraction)
                     continue
-            except:
-                message = "Unexpected error from base class CenterOfRotation.find_shift in CenterOfRotationAdaptiveSearch.find_shift  : {err}".format(
-                    err=err
+            except Exception as err:
+                self.logger.error(
+                    "Unexpected error from base class CenterOfRotation.find_shift in CenterOfRotationAdaptiveSearch.find_shift: %s"
+                    % (str(err))
                 )
-                self.logger.error(message)
                 raise
 
             p_1 = cor_position * 2
@@ -643,8 +645,8 @@ class CenterOfRotationAdaptiveSearch(CenterOfRotation):
                 * self.sigma_fraction
             )
 
-            M1 = int(round(cor_position + img_1.shape[1] // 2)) - int(round(tmp_sigma))
-            M2 = int(round(cor_position + img_1.shape[1] // 2)) + int(round(tmp_sigma))
+            M1 = round(cor_position + img_1.shape[1] // 2) - round(tmp_sigma)
+            M2 = round(cor_position + img_1.shape[1] // 2) + round(tmp_sigma)
 
             piece_1 = img_filtered_1[:, M1:M2]
             piece_2 = img_filtered_2[:, img_1.shape[1] - M2 : img_1.shape[1] - M1]
@@ -656,16 +658,13 @@ class CenterOfRotationAdaptiveSearch(CenterOfRotation):
                 diff_energy = np.array((piece_1 - piece_2) * (piece_1 - piece_2), "d").sum()
                 cost = diff_energy / energy
 
-                if not np.isnan(cost):
-                    if tmp_sigma * 2 > abs(x_cor_rel - cor_position):
-                        found_centers.append([cost, abs(x_cor_rel - cor_position), cor_position, energy])
+                if not np.isnan(cost) and tmp_sigma * 2 > abs(x_cor_rel - cor_position):
+                    found_centers.append([cost, abs(x_cor_rel - cor_position), cor_position, energy])
 
             x_cor = min(x_cor + x_cor * self.step_fraction, x_cor + (dim_radio - x_cor) * self.step_fraction)
 
         if len(found_centers) == 0:
-            message = "Unable to find any valid CoR candidate in {my_class}.find_shift ".format(
-                my_class=self.__class__.__name__
-            )
+            message = f"Unable to find any valid CoR candidate in {self.__class__.__name__}.find_shift "
             raise ValueError(message)
 
         # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
@@ -674,6 +673,7 @@ class CenterOfRotationAdaptiveSearch(CenterOfRotation):
 
         filtered_found_centers = []
         for i in range(len(found_centers)):
+            # ruff: noqa: SIM102
             if i > 0:
                 if abs(found_centers[i][2] - found_centers[i - 1][2]) < 0.5:
                     filtered_found_centers.append(found_centers[i])
@@ -683,7 +683,7 @@ class CenterOfRotationAdaptiveSearch(CenterOfRotation):
                     filtered_found_centers.append(found_centers[i])
                     continue
 
-        if len(filtered_found_centers):
+        if len(filtered_found_centers) > 0:
             found_centers = filtered_found_centers
 
         min_choice = min(found_centers)
@@ -711,8 +711,7 @@ class CenterOfRotationAdaptiveSearch(CenterOfRotation):
             validity_check_result = cor_result_validity["sound"]
         else:
             self.logger.warning(
-                "Minimum mean difference and minimum std-dev of differences do not coincide. "
-                + "This means that the validity of the found solution might be questionable."
+                "Minimum mean difference and minimum std-dev of differences do not coincide. This means that the validity of the found solution might be questionable."
             )
             validity_check_result = cor_result_validity["questionable"]
 
@@ -802,7 +801,7 @@ class CenterOfRotationOctaveAccurate(CenterOfRotation):
                 return res
         return res
 
-    def _interpolate(self, input, shift, mode="mean", interpolation_method="linear"):
+    def _interpolate(self, input_, shift, mode="mean", interpolation_method="linear"):
         """Applies to the input a translation by a vector `shift`. Based on
         `scipy.ndimage.affine_transform` function.
         JL: This Octave function was initially used in the refine clause of the local_correlation (Octave find_shift).
@@ -811,7 +810,7 @@ class CenterOfRotationOctaveAccurate(CenterOfRotation):
 
         Parameters
         ----------
-        input : array
+        input_ : array
             Array to which the translation is applied.
         shift : tuple, list or array of length 2.
         mode : str
@@ -862,19 +861,19 @@ class CenterOfRotationOctaveAccurate(CenterOfRotation):
 
         if mode == "mean":
             mode = "constant"
-            cval = input.mean()
-            return affine_transform(input, matrix, mode=mode, order=order, cval=cval)
+            cval = input_.mean()
+            return affine_transform(input_, matrix, mode=mode, order=order, cval=cval)
         elif mode not in admissible_modes:
             raise ValueError(f"Pad method is {mode} and should be in {admissible_modes}.")
 
-        return affine_transform(input, matrix, mode=mode, order=order)
+        return affine_transform(input_, matrix, mode=mode, order=order)
 
     def _local_correlation(
         self,
         z1,
         z2,
-        maxsize=[5, 5],
-        cor_estimate=[0, 0],
+        maxsize=5,
+        cor_estimate=0,
         refine=None,
         pmcc=False,
         normalize=True,
@@ -896,8 +895,8 @@ class CenterOfRotationOctaveAccurate(CenterOfRotation):
         z1,z2 : 2D arrays.
             The two (sub)images to be compared.
 
-        maxsize : 2-list. Default [5,5]
-            Size of the search window.
+        maxsize : 5
+            Size of the (horizontal) search window.
 
         cor_estimate:
             Initial guess of the center of rotation.
@@ -913,7 +912,7 @@ class CenterOfRotationOctaveAccurate(CenterOfRotation):
 
         Returns
         -------
-        c = [row,column] (or [NaN,NaN] if unsuccessful.)
+        c = cor (or [NaN,NaN] if unsuccessful.)
 
         2007-01-05 P. Cloetens cloetens@esrf.eu
         * Initial revision
@@ -921,12 +920,8 @@ class CenterOfRotationOctaveAccurate(CenterOfRotation):
         * Python conversion.
         """
 
-        if type(maxsize) in (float, int):
-            maxsize = [int(maxsize), int(maxsize)]
-        elif type(maxsize) in (tuple, list):
-            maxsize = [int(maxsize[0]), int(maxsize[1])]
-        elif maxsize in ([], None, ""):
-            maxsize = [5, 5]
+        if maxsize in ([], None, ""):
+            maxsize = 5
 
         if refine is None:
             refine = np.allclose(maxsize, 0.0)
@@ -935,43 +930,24 @@ class CenterOfRotationOctaveAccurate(CenterOfRotation):
             z1 /= np.mean(z1)
             z2 /= np.mean(z2)
 
-        #####################################
-        # JL : seems useless since func is always called with a first approximate.
-        ## determination of approximative shift (manually or Fourier correlation)
-        # if isinstance(cor_estimate,str):
-        #    if cor_estimate in ('fft','auto','fourier'):
-        #        padding_mode = None
-        #        cor_estimate = self._compute_correlation_fft(
-        #            z1,
-        #            z2,
-        #            padding_mode,
-        #            high_pass=self.high_pass,
-        #            low_pass=self.low_pass
-        #        )
-        #    elif cor_estimate in ('manual','man','m'):
-        #        cor_estimate = None
-        #        # No ImageJ plugin here :
-        #        # rapp = ij_align(z1,z2)
-
-        ####################################
         # check if refinement with realspace correlation is required
         # otherwise keep result as it is
         if np.allclose(maxsize, 0):
             shiftfound = 1
             if refine:
-                c = np.round(np.array(cor_estimate, dtype=int))
+                c = np.round(cor_estimate)
             else:
-                c = np.array(cor_estimate, dtype=int)
+                c = cor_estimate
         else:
             shiftfound = 0
-            cor_estimate = np.round(np.array(cor_estimate, dtype=int))
+            cor_estimate = np.round(cor_estimate)
 
         rapp_hist = []
-        if np.sum(np.abs(cor_estimate) + 1 >= z1.shape):
+        if np.sum(np.abs(cor_estimate) + 1 >= z1.shape[1]):
             self.logger.debug(f"Approximate shift of [{cor_estimate[0]},{cor_estimate[1]}] is too large, setting [0 0]")
-            cor_estimate = np.array([0, 0])
-        maxsize = np.minimum(maxsize, np.floor((np.array(z1.shape) - 1) / 2)).astype(int)
-        maxsize = np.minimum(maxsize, np.array(z1.shape) - np.abs(cor_estimate) - 1).astype(int)
+            cor_estimate = 0
+        maxsize = np.minimum(maxsize, np.floor((z1.shape[1] - 1) / 2)).astype(int)
+        maxsize = int(np.minimum(maxsize, z1.shape[1] - np.abs(cor_estimate) - 1))
 
         while not shiftfound:
             # Set z1 region
@@ -982,52 +958,54 @@ class CenterOfRotationOctaveAccurate(CenterOfRotation):
             # Of course, care must be taken with borders, hence the various max,min calls.
 
             # Extract the reference block
-            shape_ar = np.array(z1.shape)
-            cor_ar = np.array(cor_estimate)
-            maxsize_ar = np.array(maxsize)
+            # shape_ar = np.array(z1.shape)
+            shape_x = z1.shape[1]
+            # cor_ar = np.array(cor_estimate)
+            cor = cor_estimate
+            # maxsize_ar = np.array(maxsize)
 
-            z1beg = np.maximum(cor_ar + maxsize_ar, np.zeros(2, dtype=int))
-            z1end = shape_ar + np.minimum(cor_ar - maxsize_ar, np.zeros(2, dtype=int))
+            # z1beg = np.maximum(cor_ar + maxsize_ar, np.zeros(2, dtype=int))
+            z1beg = np.maximum(cor + maxsize, 0)
+            # z1end = shape_ar + np.minimum(cor_ar - maxsize_ar, np.zeros(2, dtype=int))
+            z1end = z1.shape[1] + np.minimum(cor - maxsize, 0)
 
-            z1p = z1[z1beg[0] : z1end[0], z1beg[1] : z1end[1]].flatten()
+            z1p = z1[:, z1beg:z1end].flatten()
 
             # Build local correlations array.
-            window_shape = (2 * int(maxsize[0]) + 1, 2 * int(maxsize[1]) + 1)
+            window_shape = 2 * int(maxsize) + 1
             cc = np.zeros(window_shape)
 
             # Prepare second block indices
-            z2beg = (cor_ar + maxsize_ar > 0) * cc.shape + (cor_ar + maxsize_ar <= 0) * (shape_ar - z1end + z1beg) - 1
+            # z2beg = (cor_ar + maxsize_ar > 0) * cc.shape + (cor_ar + maxsize_ar <= 0) * (shape_ar - z1end + z1beg) - 1
+            z2beg = (cor + maxsize > 0) * window_shape + (cor + maxsize <= 0) * (shape_x - z1end + z1beg) - 1
             z2end = z2beg + z1end - z1beg
 
             if pmcc:
                 std_z1p = z1p.std()
-            if normalize == 2:
+            if normalize:
                 z1p /= z1p.mean()
 
-            for k in range(cc.shape[0]):
-                for l in range(cc.shape[1]):
-                    if pmcc:
-                        z2p = z2[z2beg[0] - k : z2end[0] - k, z2beg[1] - l : z2end[1] - l].flatten()
-                        std_z2p = z2p.std()
-                        cc[k, l] = -np.cov(z1p, z2p, rowvar=True)[1, 0] / (std_z1p * std_z2p)
+            for l in range(len(cc)):  # noqa: E741
+                if pmcc:
+                    z2p = z2[:, z2beg - l : z2end - l].flatten()
+                    std_z2p = z2p.std()
+                    cc[l] = -np.cov(z1p, z2p, rowvar=True)[1, 0] / (std_z1p * std_z2p)
+                else:
+                    if normalize:
+                        z2p = z2[:, z2beg - l : z2end - l].flatten()
+                        z2p /= z2p.mean()
+                        z2p -= z1p
                     else:
-                        if normalize == 2:
-                            z2p = z2[z2beg[0] - k : z2end[0] - k, z2beg[1] - l : z2end[1] - l].flatten()
-                            z2p /= z2p.mean()
-                            z2p -= z1p
-                        else:
-                            z2p = z2[z2beg[0] - k : z2end[0] - k, z2beg[1] - l : z2end[1] - l].flatten()
-                            z2p -= z1p
-                        cc[k, l] = ((z2p - z2p.mean()) ** 2).sum()
-                        # cc(k,l) = std(z1p./z2(z2beg(1)-k:z2end(1)-k,z2beg(2)-l:z2end(2)-l)(:));
+                        z2p = z2[:, z2beg - l : z2end - l].flatten()
+                        z2p -= z1p
+                    cc[l] = ((z2p - z2p.mean()) ** 2).sum()
 
-            c = np.unravel_index(np.argmin(cc, axis=None), shape=cc.shape)
+            c = np.argmin(cc)
+            (f_vals, _) = self.extract_peak_regions_1d(-cc, peak_radius=1)
+            win_pos_max, _ = self.refine_max_position_1d(f_vals, return_vertex_val=True)
+
+            c = float(np.argmin(cc)) + win_pos_max
 
-            if not np.sum((c == 0) + (c == np.array(cc.shape) - 1)):
-                # check that we are not at the edge of the region that was sampled
-                x = np.array([-1, 0, 1])
-                tmp = self.refine_max_position_2d(cc[c[0] - 1 : c[0] + 2, c[1] - 1 : c[1] + 2], x, x)
-                c += tmp
             shiftfound = True
 
             c += z1beg - z2beg
@@ -1040,63 +1018,32 @@ class CenterOfRotationOctaveAccurate(CenterOfRotation):
                     self.logger.debug("Stuck in loop?")
                     refine = True
                     shiftfound = True
-                    c = np.array([np.nan, np.nan])
+                    c = np.nan
                 else:
                     rapp_hist.append(cor_estimate)
                     self.logger.debug(f"Changing shift estimate: {cor_estimate}")
-                    maxsize = np.minimum(maxsize, np.array(z1.shape) - np.abs(cor_estimate) - 1).astype(int)
-                    if (maxsize == 0).sum():
+                    maxsize = np.minimum(maxsize, z1.shape[1] - np.abs(cor_estimate) - 1).astype(int)
+                    if maxsize == 0:
                         self.logger.debug("Edge of image reached")
                         refine = False
                         shiftfound = True
-                        c = np.array([np.nan, np.nan])
+                        c = np.nan
             elif len(rapp_hist) > 0:
                 self.logger.debug("\n")
 
-        ####################################
-        # refine result; useful when shifts are not integer values
-        # JL: I don't understand why this refine step should be useful.
-        # In Octave, from fastomo.m, refine is always set to False.
-        # So this could be ignored.
-        # I keep it for future use if it proves useful.
-        # if refine:
-        #    if debug:
-        #        print('Refining solution ...')
-        #    z2n = self.interpolate(z2,c)
-        #    indices = np.ceil(np.abs(c)).astype(int)
-        #    z1p = np.roll(z1,((c>0) * (-1) * indices),[0,1])
-        #    z1p = z1p[1:-indices[0]-1,1:-indices[1]-1].flatten()
-        #    z2n = np.roll(z2n,((c>0) * (-1) * indices),[0,1])
-        #    z2n = z2n[:-indices[0],:-indices[1]]
-        #    ccrefine = np.zeros([3,3])
-        #    [n2,m2] = z2n.shape
-        #    for k in range(3):
-        #        for l in range(3):
-        #            z2p = z1p - z2n[2-k:n2-k,2-l:m2-l].flatten()
-        #            ccrefine[k,l] = ((z2p - z2p.mean())**2).sum()
-        #    x = np.array([-1,0,1])
-        #    crefine = self.refine_max_position_2d(ccrefine, x, x)
-        #    #crefine = min2par(ccrefine)
-
-        #    # Check if the refinement is effectively confined to subpixel
-        #    if (np.abs(crefine) >= 1).sum():
-        #        self.logger.info("Problems refining result\n")
-        #    else:
-        #        c += crefine
-
         return c
 
     def find_shift(
         self,
         img_1,
         img_2,
-        side="center",
+        side=None,
         roi_yxhw=None,
         median_filt_shape=None,
         padding_mode=None,
         low_pass=0.01,
         high_pass=None,
-        maxsize=[5, 5],
+        maxsize=5,
         refine=None,
         pmcc=False,
         normalize=True,
@@ -1112,6 +1059,10 @@ class CenterOfRotationOctaveAccurate(CenterOfRotation):
             )
             return_relative_to_middle = True  # the kwarg above will be False by default in a future release
         # ---
+
+        if side is not None:
+            self.logger.error("CenterOfRotationOctaveAccurate: the 'side' keyword argument will have no effect")
+
         self._check_img_pair_sizes(img_1, img_2)
 
         img_shape = img_2.shape
@@ -1124,21 +1075,15 @@ class CenterOfRotationOctaveAccurate(CenterOfRotation):
             img_1,
             img_2,
             padding_mode,
+            axes=(-1,),
             high_pass=high_pass,
             low_pass=low_pass,
         )
 
         # We use fftshift to deal more easily with negative shifts.
         # This has a cost of subtracting half the image shape afterward.
-        shift = np.unravel_index(np.argmax(np.fft.fftshift(cc)), img_shape)
-        shift -= np.array(img_shape) // 2
-
-        # The real "accurate" starts here (i.e. the octave findshift() func).
-        if np.abs(shift[0]) > 10 * limz:
-            # This is suspiscious. We don't trust results of correlate.
-            self.logger.warning("Pre-correlation yields {shift[0]} pixels vertical motion")
-            self.logger.warning("We do not consider it.")
-            shift = (0, 0)
+        shift = int(np.argmax(np.fft.fftshift(cc).sum(axis=0)))
+        shift -= img_shape[1] // 2
 
         # Limit the size of region for comparison to cutsize in both directions.
         # Hard-coded?
@@ -1146,9 +1091,7 @@ class CenterOfRotationOctaveAccurate(CenterOfRotation):
         oldshift = np.round(shift).astype(int)
         if (img_shape[0] > cutsize) or (img_shape[1] > cutsize):
             im0 = self._cut(img_1, min(img_shape[0], cutsize), min(img_shape[1], cutsize))
-            im1 = self._cut(
-                np.roll(img_2, oldshift, axis=(0, 1)), min(img_shape[0], cutsize), min(img_shape[1], cutsize)
-            )
+            im1 = self._cut(np.roll(img_2, oldshift, axis=(1,)), min(img_shape[0], cutsize), min(img_shape[1], cutsize))
             shift = oldshift + self._local_correlation(
                 im0,
                 im1,
@@ -1167,18 +1110,14 @@ class CenterOfRotationOctaveAccurate(CenterOfRotation):
                 pmcc=pmcc,
                 normalize=normalize,
             )
-        if ((shift - oldshift) ** 2).sum() > 4:
-            self.logger.warning(f"Pre-correlation ({oldshift}) and accurate correlation ({shift}) are not consistent.")
-            self.logger.warning("Please check!!!")
+        if (shift - oldshift) ** 2 > 4:
+            self.logger.warning(
+                f"Pre-correlation ({oldshift}) and accurate correlation ({shift}) are not consistent. Please check !"
+            )
 
-        offset = shift[1] / 2
+        offset = shift / 2
 
-        if np.abs(shift[0]) > limz:
-            self.logger.debug("Verify alignment or sample motion.")
-            self.logger.debug(f"Verical motion: {shift[0]} pixels.")
-            self.logger.debug(f"Offset?: {offset} pixels.")
-        else:
-            self.logger.debug(f"Offset?: {offset} pixels.")
+        self.logger.debug(f"Offset?: {offset} pixels.")
 
         if not (return_relative_to_middle):
             offset += (img_shape[1] - 1) / 2
