@@ -21,12 +21,10 @@ class SinoCor:
     """
 
     def __init__(self, img_1, img_2, logger=None):
-        """ """
         self.logger = LoggerOrPrint(logger)
         self.sx = img_1.shape[1]
 
         # algorithm cannot accept odd number of projs. This is handled in the SinoCORFinder class.
-        nproj2 = img_1.shape[0]
 
         # extract upper and lower part of sinogram, flipping H the upper part
         self.data1 = img_1
@@ -141,9 +139,6 @@ class SinoCor:
         xc1 = self.rcor_abs - int(xwin / 2)
         xc2 = self.sx - self.rcor_abs - int(xwin / 2) - 1
 
-        im1 = self.data1[:, xc1 : xc1 + xwin]
-        im2 = self.data2[:, xc2 : xc2 + xwin]
-
         pixs = p_sign * (np.arange(neighborhood) - ng2)
         diff0 = 1000000000.0
 
@@ -213,7 +208,7 @@ class SinoCorInterface:
 class CenterOfRotationFourierAngles:
     """This CoR estimation algo is proposed by V. Valls (BCU). It is based on the Fourier
     transform of the columns on the sinogram.
-    It requires an initial guesss of the CoR wich is retrieved from
+    It requires an initial guess of the CoR which is retrieved from
     dataset_info.dataset_scanner.x_rotation_axis_pixel_position. It is assumed in mm and pixel size in um.
     Options are (for the moment) hard-coded in the SinoCORFinder.cor_finder.extra_options dict.
     """
@@ -225,7 +220,7 @@ class CenterOfRotationFourierAngles:
         """
         Converts a scipy.fft.rfft into the (legacy) scipy.fftpack.rfft format.
         The fftpack.rfft returns a (roughly) twice as long array as fft.rfft as the latter returns an array
-        of complex numbers wheras the former returns an array with real and imag parts in consecutive
+        of complex numbers whereas the former returns an array with real and imag parts in consecutive
         spots in the array.
 
         Parameters
@@ -289,9 +284,8 @@ class CenterOfRotationFourierAngles:
 
     def _px(self, detector_width, abs_pos, near_width, near_std, crop_around_cor, near_step):
         sym_range = None
-        if abs_pos is not None:
-            if crop_around_cor:
-                sym_range = int(abs_pos - near_std * 2), int(abs_pos + near_std * 2)
+        if abs_pos is not None and crop_around_cor:
+            sym_range = int(abs_pos - near_std * 2), int(abs_pos + near_std * 2)
 
         window = near_width
         if sym_range is not None:
