@@ -1,3 +1,4 @@
+import os
 import numpy as np
 
 from scipy.fft import fftn
@@ -6,6 +7,8 @@ from ..processing.azim import azimuthal_integration_skimage_stack, azimuthal_int
 from .alignment import plt
 from .cor import CenterOfRotation
 
+NABU_THREADS = int(os.environ.get("NABU_THREADS", "4"))
+
 
 class CameraFocus(CenterOfRotation):
 
@@ -28,7 +31,7 @@ class CameraFocus(CenterOfRotation):
         return np.stack(d, axis=0)
 
     @staticmethod
-    def _compute_metric_value(data, metric, axes=(-2, -1)):
+    def _compute_metric_value(data, metric, axes=(-2, -1), return_radial_profile=False):
         if metric.lower() == "std":
             return np.std(data, axis=axes) / np.mean(data, axis=axes)
         elif metric.lower() == "grad":
@@ -36,16 +39,17 @@ class CameraFocus(CenterOfRotation):
             grad_mag = np.sqrt(np.sum(grad_data**2, axis=0))
             return np.sum(grad_mag, axis=axes)
         elif metric.lower() == "psd":
-            f_data = fftn(data, axes=axes, workers=4)
-            f_data = np.fft.fftshift(f_data, axes=(-2, -1))
-            # octave-fasttomo3 uses |.|^2, probably with scaled FFT (norm="forward" in python),
-            # but tests show that it's less accurate.
-            f_data = np.abs(f_data)
+            f_data = fftn(data, axes=axes, workers=NABU_THREADS)
+            f_data = np.fft.fftshift(f_data, axes=axes)
+            f_data = (f_data * f_data.conjugate()).real
             ai_func = azimuthal_integration_skimage_stack if __have_skimage__ else azimuthal_integration_imagej_stack
-            az_data = ai_func(f_data, n_threads=4)
+            az_data = ai_func(f_data, n_threads=NABU_THREADS)
             max_vals = np.max(az_data, axis=0)
             az_data /= max_vals[None, :]
-            return np.mean(az_data, axis=-1)
+            ret = np.mean(az_data, axis=-1)
+            if return_radial_profile:
+                ret = ret, az_data
+            return ret
         else:
             raise ValueError("Unknown metric function %s" % metric)
 
@@ -138,7 +142,11 @@ class CameraFocus(CenterOfRotation):
             high_pass=high_pass,
         )
 
-        img_resp = self._compute_metric_value(img_stack, metric=metric, axes=(-2, -1))
+        img_resp = self._compute_metric_value(img_stack, metric=metric, axes=(-2, -1), return_radial_profile=True)
+        if metric == "psd":
+            img_resp, radial_profile = img_resp
+        else:
+            radial_profile = None
 
         # assuming images are equispaced!
         # focus_step = np.mean(np.abs(np.diff(img_pos)))
@@ -159,16 +167,23 @@ class CameraFocus(CenterOfRotation):
 
         if self.verbose:
             self.logger.info(
-                "Fitted focus motor position:",
-                focus_pos,
-                "and corresponding image position:",
-                focus_ind,
+                "Fitted focus motor position: %s and corresponding image position: %s"
+                % (str(focus_pos), str(focus_ind))
             )
-            f, ax = plt.subplots(1, 1)
-            self._add_plot_window(f, ax=ax)
-            ax.stem(img_pos, img_resp)
-            ax.stem(focus_pos, img_resp_max, linefmt="C1-", markerfmt="C1o")
+            n_plots = 2 if metric == "psd" else 1
+
+            fig = plt.figure(figsize=((13, 5) if n_plots == 2 else None))
+            ax = fig.add_subplot(1, n_plots, 1)
+            self._add_plot_window(fig, ax=ax)
+            ax.plot(img_pos, img_resp, ".-")
+            ax.stem(focus_pos, img_resp_max, linefmt="C1-", markerfmt="C1o", bottom=img_resp.min())
             ax.set_title("Images response (metric: %s)" % metric)
+            if n_plots > 1:
+                ax2 = fig.add_subplot(1, n_plots, 2)
+                self._add_plot_window(fig, ax=ax2)
+                ax2.plot(radial_profile.T)
+                ax2.legend(range(radial_profile.shape[0]))
+                ax2.set_title("Normalized radial profile")
             plt.show(block=False)
 
         return focus_pos, focus_ind
@@ -383,10 +398,8 @@ class CameraFocus(CenterOfRotation):
 
         if self.verbose:
             self.logger.info(
-                "Fitted focus motor position:",
-                focus_pos,
-                "and corresponding image position:",
-                focus_ind,
+                "Fitted focus motor position: %s and corresponding image position: %s"
+                % (str(focus_pos), str(focus_ind))
             )
             self.logger.info("Fitted tilts (to be divided by pixel size, and converted to deg): (v, h) %s" % tilts_vh)
             fig = plt.figure()
