@@ -460,6 +460,51 @@ class TestCorOctaveAccurate:
         message = f"Computed CoR {cor_position} and expected CoR {self.cor_pos_abs_blc12781} do not coincide."
         assert np.isclose(self.cor_pos_abs_blc12781, cor_position, atol=self.abs_tol), message
 
+    def test_cor_accurate_pmcc(self):
+        detector_width = self.image_pair_blc12781[0].shape[1]
+        CoR_calc = CenterOfRotationOctaveAccurate()
+        cor_position = CoR_calc.find_shift(
+            self.image_pair_blc12781[0],
+            np.fliplr(self.image_pair_blc12781[1]),
+            "center",
+            return_relative_to_middle=True,
+            pmcc=True,
+        )
+        cor_position = cor_position + detector_width / 2
+        assert np.isscalar(cor_position), f"cor_position expected to be a scalar, {type(cor_position)} returned"
+        message = f"Computed CoR {cor_position} and expected CoR {self.cor_pos_abs_blc12781} do not coincide."
+        assert np.isclose(self.cor_pos_abs_blc12781, cor_position, atol=self.abs_tol), message
+
+    def test_cor_accurate_maxsize(self):
+        detector_width = self.image_pair_blc12781[0].shape[1]
+        CoR_calc = CenterOfRotationOctaveAccurate()
+        cor_position = CoR_calc.find_shift(
+            self.image_pair_blc12781[0],
+            np.fliplr(self.image_pair_blc12781[1]),
+            "center",
+            return_relative_to_middle=True,
+            maxsize=2,
+        )
+        cor_position = cor_position + detector_width / 2
+        assert np.isscalar(cor_position), f"cor_position expected to be a scalar, {type(cor_position)} returned"
+        message = f"Computed CoR {cor_position} and expected CoR {self.cor_pos_abs_blc12781} do not coincide."
+        assert np.isclose(self.cor_pos_abs_blc12781, cor_position, atol=self.abs_tol), message
+
+    def test_cor_accurate_normalize(self):
+        detector_width = self.image_pair_blc12781[0].shape[1]
+        CoR_calc = CenterOfRotationOctaveAccurate()
+        cor_position = CoR_calc.find_shift(
+            self.image_pair_blc12781[0],
+            np.fliplr(self.image_pair_blc12781[1]),
+            "center",
+            return_relative_to_middle=True,
+            normalize=False,
+        )
+        cor_position = cor_position + detector_width / 2
+        assert np.isscalar(cor_position), f"cor_position expected to be a scalar, {type(cor_position)} returned"
+        message = f"Computed CoR {cor_position} and expected CoR {self.cor_pos_abs_blc12781} do not coincide."
+        assert np.isclose(self.cor_pos_abs_blc12781, cor_position, atol=self.abs_tol), message
+
 
 @pytest.mark.usefixtures("bootstrap_cor_fourier", "bootstrap_cor_win")
 class TestCorFourierAngle:
