@@ -1,7 +1,7 @@
 import pytest
 import numpy as np
 from nabu.estimation.tilt import CameraTilt
-from nabu.estimation.tests.test_cor import bootstrap_cor
+from nabu.estimation.tests.test_cor import bootstrap_cor  # noqa: F401
 
 try:
     import skimage.transform as skt  # noqa: F401
