@@ -131,7 +131,7 @@ class CameraTilt(CenterOfRotation):
             img_1, img_2, padding_mode, axes=(-1,), high_pass=high_pass, low_pass=low_pass
         )
 
-        img_shape = img_2.shape
+        img_shape = cc.shape  # cc.shape can differ from img_2.shape, e.g. in case of odd nb of cols
         cc_h_coords = np.fft.fftfreq(img_shape[-1], 1 / img_shape[-1])
 
         (f_vals, fh) = self.extract_peak_regions_1d(cc, peak_radius=peak_fit_radius, cc_coords=cc_h_coords)
@@ -150,10 +150,11 @@ class CameraTilt(CenterOfRotation):
 
         if self.verbose:
             self.logger.info(
-                "Fitted center of rotation (pixels):",
-                cor_offset_pix,
-                "and camera tilt (degrees):",
-                tilt_deg,
+                "Fitted center of rotation (pixels): %s and camera tilt (degrees): %s"
+                % (
+                    str(cor_offset_pix),
+                    str(tilt_deg),
+                )
             )
             f, ax = plt.subplots(1, 1)
             self._add_plot_window(f, ax=ax)
