@@ -1,8 +1,12 @@
 import os
-from nabu.misc.utils import rescale_data
-from nabu.pipeline.params import files_formats
+import logging
+from glob import glob
+from shutil import rmtree
+import numpy
+from silx.io.utils import get_data
+from silx.io.url import DataUrl
 from tomoscan.volumebase import VolumeBase
-from tomoscan.scanbase import TomoScanBase
+from tomoscan.esrf.volume.singleframebase import VolumeSingleFrameBase
 from tomoscan.esrf.volume import (
     EDFVolume,
     HDF5Volume,
@@ -11,17 +15,17 @@ from tomoscan.esrf.volume import (
     TIFFVolume,
 )
 from tomoscan.io import HDF5File
-from silx.io.utils import get_data
-from silx.utils.enum import Enum as _Enum
-import numpy
-from silx.io.url import DataUrl
-from typing import Optional
-import logging
+from tomoscan.esrf.scan.edfscan import EDFTomoScan
+from tomoscan.esrf.scan.nxtomoscan import NXtomoScan
+from ..utils import first_generator_item
+from ..misc.utils import rescale_data
+from ..pipeline.params import files_formats
+from .reader import get_hdf5_file_all_virtual_sources, list_hdf5_entries
 
 _logger = logging.getLogger(__name__)
 
 
-__all__ = ["get_default_output_volume", "cast_volume"]
+__all__ = ["cast_volume", "get_default_output_volume"]
 
 _DEFAULT_OUTPUT_DIR = "vol_cast"
 
@@ -43,7 +47,7 @@ def get_default_output_volume(
     if not isinstance(input_volume, VolumeBase):
         raise TypeError(f"input_volume is expected to be an instance of {VolumeBase}")
     valid_file_formats = set(files_formats.values())
-    if not output_type in valid_file_formats:
+    if output_type not in valid_file_formats:
         raise ValueError(f"output_type is not a valid value ({output_type}). Valid values are {valid_file_formats}")
 
     if isinstance(input_volume, (EDFVolume, TIFFVolume, JP2KVolume)):
@@ -132,14 +136,15 @@ def cast_volume(
     output_data_type: numpy.dtype,
     data_min=None,
     data_max=None,
-    scan: Optional[TomoScanBase] = None,
+    scan=None,
     rescale_min_percentile=RESCALE_MIN_PERCENTILE,
     rescale_max_percentile=RESCALE_MAX_PERCENTILE,
     save=True,
     store=False,
+    remove_input_volume: bool = False,
 ) -> VolumeBase:
     """
-    cast givent volume to output_volume of 'output_data_type' type
+    cast given volume to output_volume of 'output_data_type' type
 
     :param VolumeBase input_volume:
     :param VolumeBase output_volume:
@@ -167,6 +172,7 @@ def cast_volume(
     if not isinstance(output_volume, VolumeBase):
         raise TypeError(f"output_volume is expected to be a {VolumeBase}. {type(output_volume)} provided")
 
+    # ruff: noqa: SIM105, S110
     try:
         output_data_type = numpy.dtype(
             output_data_type
@@ -176,6 +182,17 @@ def cast_volume(
     if not isinstance(output_data_type, numpy.dtype):
         raise TypeError(f"output_data_type is expected to be a {numpy.dtype}. {type(output_data_type)} provided")
 
+    # Make sure the output volume has the same "start_index" as input volume, if relevant
+    if isinstance(input_volume, VolumeSingleFrameBase) and isinstance(output_volume, VolumeSingleFrameBase):
+        try:
+            first_file_name = next(input_volume.browse_data_files())
+            start_idx = int(first_file_name.split(".")[0].split("_")[-1])
+        except (StopIteration, ValueError, TypeError):
+            # StopIteration: Input volume has no file - should not happen
+            # ValueError / TypeError: fail to convert to int, something wrong when extracting slice number (non-default file name scheme)
+            start_idx = 0
+        output_volume.start_index = start_idx
+
     # start processing
     #  check for data_min and data_max
     if data_min is None or data_max is None:
@@ -193,6 +210,9 @@ def cast_volume(
         data_min = data_min if data_min is not None else found_data_min
         data_max = data_max if data_max is not None else found_data_max
 
+    if isinstance(output_volume, JP2KVolume):
+        output_volume.rescale_data = False
+
     data = []
     for input_slice, frame_dumper in zip(
         input_volume.browse_slices(),
@@ -213,6 +233,7 @@ def cast_volume(
                 data_max=data_max,
                 rescale_min_percentile=rescale_min_percentile,
                 rescale_max_percentile=rescale_max_percentile,
+                default_value_for_nan=new_min,
             ).astype(output_data_type)
         else:
             output_slice = input_slice.astype(output_data_type)
@@ -231,6 +252,14 @@ def cast_volume(
     except (OSError, KeyError):
         # if no metadata provided and or saved in disk or if some key are missing
         pass
+
+    if save and output_volume.metadata is not None:
+        output_volume.save_metadata()
+
+    if remove_input_volume:
+        _logger.info(f"Removing {input_volume.data_url.file_path()}")
+        remove_volume(input_volume, check=True)
+
     return output_volume
 
 
@@ -242,32 +271,53 @@ def clamp_and_rescale_data(
     data_max=None,
     rescale_min_percentile=RESCALE_MIN_PERCENTILE,
     rescale_max_percentile=RESCALE_MAX_PERCENTILE,
+    default_value_for_nan=None,
+    do_float64=True,
 ):
     """
     rescale data to 'new_min', 'new_max'
 
-    :param numpy.ndarray data: data to be rescaled
-    :param dtype output_dtype: output dtype
-    :param new_min: rescaled data new min (clamp min value)
-    :param new_max: rescaled data new max (clamp max value)
-    :param data_min: `data` min value to clamp to new_min. Any lower value will also be clamp to new_min.
-    :param data_max: `data` max value to clamp to new_max. Any hight value will also be clamp to new_max.
-    :param rescale_min_percentile: if `data_min` is None will set data_min to 'rescale_min_percentile'
-    :param rescale_max_percentile: if `data_max` is None will set data_min to 'rescale_max_percentile'
+    Parameters
+    ----------
+    data: numpy.ndarray
+        Data to be rescaled (image or volume)
+    new_min: scalar
+        Rescaled data new min (clamp min value)
+    new_max: scalar
+        Rescaled data new min (clamp max value)
+    data_min: scalar, optional
+        Data minimum value. If not provided, will re-compute the min() over data.
+    data_max: scalar, optional
+        Data maximum value. If not provided, will re-compute the min() over data.
+    rescale_min_percentile: scalar, optional
+        if `data_min` is None will set data_min to 'rescale_min_percentile'
+    rescale_max_percentile
+        if `data_max` is None will set data_min to 'rescale_max_percentile'
+    default_value_for_nan: scalar, optional
+        Value that will replace NaNs, if any. Default is None (keep NaNs, will likely raise an error)
+    do_float64
+        Whether to do internal computations in float64. Recommended when casting from float32 to int32 for example.
     """
+    if do_float64 and data.dtype.itemsize < 8:
+        data = numpy.float64(data)
     if data_min is None:
-        data_min = numpy.percentile(data, rescale_min_percentile)
+        data_min = numpy.nanpercentile(data, rescale_min_percentile)
     if data_max is None:
-        data_max = numpy.percentile(data, rescale_max_percentile)
+        data_max = numpy.nanpercentile(data, rescale_max_percentile)
     # rescale data
     rescaled_data = rescale_data(data, new_min=new_min, new_max=new_max, data_min=data_min, data_max=data_max)
+    # Handle NaNs
+    if default_value_for_nan is not None:
+        isnan_mask = numpy.isnan(rescaled_data)
+        if numpy.any(isnan_mask):
+            rescaled_data[isnan_mask] = default_value_for_nan
     # clamp data
     rescaled_data[rescaled_data < new_min] = new_min
     rescaled_data[rescaled_data > new_max] = new_max
     return rescaled_data
 
 
-def find_histogram(volume: VolumeBase, scan: Optional[TomoScanBase] = None) -> Optional[DataUrl]:
+def find_histogram(volume: VolumeBase, scan=None):
     """
     Look for histogram of the provided url. If found one return the DataUrl of the nabu histogram
     """
@@ -285,7 +335,7 @@ def find_histogram(volume: VolumeBase, scan: Optional[TomoScanBase] = None) -> O
                     ]
                 )
             else:
-                data_path = "/".join((volume.url.data_path(), "histogram/results/data"))
+                data_path = f"{volume.url.data_path()}/histogram/results/data"
         else:
             # TODO: FIXME: in some case (if the users provides the full data_url and if the 'DATA_DATASET_NAME' is not used we
             # will endup with an invalid data_path. Hope this case will not happen. Anyway this is a case that we can't handle.)
@@ -311,10 +361,15 @@ def find_histogram(volume: VolumeBase, scan: Optional[TomoScanBase] = None) -> O
             histogram_file = file_path + "_histogram.hdf5"
 
         if scan is not None:
-            data_path = getattr(scan, "entry/histogram/results/data", "entry/histogram/results/data")
+            if isinstance(scan, NXtomoScan):
+                entry = scan.entry
+            elif isinstance(scan, EDFTomoScan):
+                entry = "entry"
+            else:
+                raise NotImplementedError("Scan type not handled")
         else:
 
-            def get_file_entries(file_path: str) -> Optional[tuple]:
+            def get_file_entries(file_path: str):
                 if os.path.exists(file_path):
                     with HDF5File(file_path, mode="r") as h5s:
                         return tuple(h5s.keys())
@@ -327,7 +382,7 @@ def find_histogram(volume: VolumeBase, scan: Optional[TomoScanBase] = None) -> O
             # we can find back the histogram
             entries = get_file_entries(histogram_file)
             if entries is not None and len(entries) == 1:
-                data_path = "/".join((entries[0], "histogram/results/data"))
+                entry = entries[0]
             else:
                 # TODO: FIXME: how to get the entry name in every case ?
                 # what to do if the histogram file has more than one entry.
@@ -335,6 +390,8 @@ def find_histogram(volume: VolumeBase, scan: Optional[TomoScanBase] = None) -> O
                 # or keep as today (in this case it will be recomputed)
                 _logger.info("histogram file found but unable to find relevant histogram")
                 return None
+        data_path = f"{entry}/histogram/results/data"
+
     else:
         raise NotImplementedError(f"volume {type(volume)} not handled")
 
@@ -343,7 +400,7 @@ def find_histogram(volume: VolumeBase, scan: Optional[TomoScanBase] = None) -> O
         return None
 
     with HDF5File(histogram_file, mode="r") as h5f:
-        if not data_path in h5f:
+        if data_path not in h5f:
             _logger.info(f"{data_path} in {histogram_file} not found")
             return None
         else:
@@ -392,3 +449,82 @@ def _min_max_from_histo(url: DataUrl, rescale_min_percentile: int, rescale_max_p
         return _get_hst_saturations(
             hist, bins, numpy.float32(rescale_min_percentile), numpy.float32(rescale_max_percentile)
         )
+
+
+def _remove_volume_singleframe(volume, check=True):
+    volume_directory = volume.data_url.file_path()
+    if check:
+        volume_files = set(volume.browse_data_files())
+        files_names_pattern = os.path.join(volume_directory, "*." + volume.data_extension)
+        files_on_disk = set(glob(files_names_pattern))
+        # Don't check strict equality here, as some files on disk might be already removed.
+        # i.e, there should be no more files on disk than expected files in the volume
+        if not (files_on_disk.issubset(volume_files)):
+            raise RuntimeError(f"Unexpected files present in {volume_directory}: {files_on_disk - volume_files}")
+        # TODO also check for metadata file(s) ?
+    rmtree(volume_directory)
+
+
+def _remove_volume_multiframe(volume, check=True):
+    file_path = volume.data_url.file_path()
+    if check and not os.path.isfile(file_path):
+        raise RuntimeError(f"Expected a file: {file_path}")
+    os.remove(file_path)
+
+
+def _remove_volume_hdf5(volume, check=True):
+    file_path = volume.data_url.file_path()
+    entry = volume.data_url.data_path().lstrip("/").split("/")[0]
+
+    # Nabu HDF5 reconstructions have a folder alongside the HDF5 file, with the same prefix
+    # For example the HDF5 file "/path/to/rec.hdf5" has an associated directory "/path/to/rec"
+    associated_dir, _ = os.path.splitext(os.path.basename(file_path))
+    associated_dir_abs = os.path.join(os.path.dirname(file_path), associated_dir)
+
+    with HDF5File(file_path, "r") as f:
+        fdesc = f[entry]
+        virtual_sources = get_hdf5_file_all_virtual_sources(fdesc, return_only_filenames=True)
+
+    # TODO check if this is legitimate. Nabu reconstruction will only do one VS (for entry/reconstruction/results/data).
+    # Bliss/Lima do have multiple VS (flats/darks/projs), but we generally don't want to remove raw data ?
+    if len(virtual_sources) > 1:
+        raise ValueError("Found more than one virtual source - this looks weird. Interrupting.")
+    #
+    if len(virtual_sources) > 0:
+        h5path, virtual_source_files_paths = first_generator_item(virtual_sources[0].items())
+        if len(virtual_source_files_paths) == 1:
+            target_dir = os.path.dirname(virtual_source_files_paths[0])
+        else:
+            target_dir = os.path.commonpath(virtual_source_files_paths)
+        target_dir_abs = os.path.join(os.path.dirname(file_path), target_dir)
+        if check and (target_dir_abs != associated_dir_abs):
+            raise ValueError(
+                f"The virtual sources in {file_path}:{h5path} reference the directory {target_dir}, but expected was {associated_dir}"
+            )
+        if os.path.isdir(target_dir_abs):
+            rmtree(associated_dir_abs)
+    os.remove(file_path)
+
+
+def remove_volume(volume, check=True):
+    """
+    Remove files belonging to a volume, claim disk space.
+
+    Parameters
+    ----------
+    volume: tomoscan.esrf.volume
+        Volume object
+    check: bool, optional
+        Whether to check if the files that would be removed do not have extra other files ; interrupt the operation if so.
+
+    """
+    if isinstance(volume, (EDFVolume, JP2KVolume, TIFFVolume)):
+        _remove_volume_singleframe(volume, check=check)
+    elif isinstance(volume, MultiTIFFVolume):
+        _remove_volume_multiframe(volume, check=check)
+    elif isinstance(volume, HDF5Volume):
+        if len(list_hdf5_entries(volume.file_path)) > 1:
+            raise NotImplementedError("Removing a HDF5 volume with more than one entry is not supported")
+        _remove_volume_hdf5(volume, check=check)
+    else:
+        raise TypeError("Unknown type of volume")
