@@ -1,25 +1,22 @@
 import os
 from threading import get_ident
-from math import ceil
 from multiprocessing.pool import ThreadPool
 from posixpath import sep as posix_sep, join as posix_join
 import numpy as np
+from h5py import Dataset
 from silx.io import get_data
 from silx.io.dictdump import h5todict
 from tomoscan.io import HDF5File
-from .utils import get_compacted_dataslices, convert_dict_values, get_first_hdf5_entry
+from .utils import convert_dict_values, get_first_hdf5_entry
 from ..misc.binning import binning as image_binning
 from ..utils import (
     check_supported,
     deprecated,
-    deprecated_class,
     deprecation_warning,
     indices_to_slices,
     compacted_views,
     merge_slices,
-    subsample_dict,
     get_3D_subregion,
-    get_num_threads,
     get_shape_from_sliced_dims,
     get_size_from_sliced_dimension,
     safe_format,
@@ -245,299 +242,6 @@ class HDF5Loader:
         return output
 
 
-@deprecated_class("ChunkReader is deprecated since 2024.2.0 and will be removed in a future version", do_print=True)
-class ChunkReader:
-    """
-    A reader of chunk of images.
-    """
-
-    def __init__(
-        self,
-        files,
-        sub_region=None,
-        detector_corrector=None,
-        pre_allocate=True,
-        data_buffer=None,
-        convert_float=False,
-        shape=None,
-        dtype=None,
-        binning=None,
-        dataset_subsampling=None,
-        num_threads=None,
-    ):
-        """
-        Initialize a "ChunkReader". A chunk is a stack of images.
-
-        Parameters
-        ----------
-        files: dict
-            Dictionary where the key is the file/data index, and the value is a
-            silx.io.url.DataUrl pointing to the data.
-            The dict must contain only the files which shall be used !
-            Note: the shape and data type is infered from the first data file.
-        sub_region: tuple, optional
-            If provided, this must be a tuple in the form
-            (start_x, end_x, start_y, end_y). Each image will be cropped to this
-            region. This is used to specify a chunk of files.
-            Each of the parameters can be None, in this case the default start
-            and end are taken in each dimension.
-        pre_allocate: bool
-            Whether to pre-allocate data before reading.
-        data_buffer: array-like, optional
-            If `pre_allocate` is set to False, this parameter has to be provided.
-            It is an array-like object which will hold the data.
-        convert_float: bool
-            Whether to convert data to float32, regardless of the input data type.
-        shape: tuple, optional
-            Shape of each image. If not provided, it is inferred from the first image
-            in the collection.
-        dtype: `numpy.dtype`, optional
-            Data type of each image. If not provided, it is inferred from the first image
-            in the collection.
-        binning: int or tuple of int, optional
-            Whether to bin the data. If multi-dimensional binning is done,
-            the parameter must be in the form (binning_x, binning_y).
-            Each image will be binned by these factors.
-        dataset_subsampling: int or tuple, optional
-            Subsampling factor when reading the images.
-            If an integer `n` is provided, then one image out of `n` will be read.
-            If a tuple of integers (step, begin) is given, the data is read as data[begin::step]
-        num_threads: int, optional
-            Number of threads to use for binning the data.
-            Default is to use all available threads.
-            This parameter has no effect when binning is disabled.
-
-        Notes
-        ------
-        The files are provided as a collection of `silx.io.DataURL`. The file type
-        is inferred from the extension.
-
-        Binning is different from subsampling. Using binning will not speed up
-        the data retrieval (quite the opposite), since the whole (subregion of) data
-        is read and then binning is performed.
-        """
-        self.detector_corrector = detector_corrector
-        self._get_reader_class(files)
-        self.dataset_subsampling = dataset_subsampling
-        self.num_threads = get_num_threads(num_threads)
-        self._set_files(files)
-        self._get_shape_and_dtype(shape, dtype, binning)
-        self._set_subregion(sub_region)
-        self._init_reader()
-        self._loaded = False
-        self.convert_float = convert_float
-        if convert_float:
-            self.out_dtype = np.float32
-        else:
-            self.out_dtype = self.dtype
-        if not ((data_buffer is not None) ^ (pre_allocate is True)):
-            raise ValueError("Please provide either 'data_buffer' or 'pre_allocate'")
-        self.files_data = data_buffer
-        if data_buffer is not None:
-            # overwrite out_dtype
-            self.out_dtype = data_buffer.dtype
-            if data_buffer.shape != self.chunk_shape:
-                raise ValueError("Expected shape %s but got %s" % (self.shape, data_buffer.shape))
-        if pre_allocate:
-            self.files_data = np.zeros(self.chunk_shape, dtype=self.out_dtype)
-        if (self.binning is not None) and (np.dtype(self.out_dtype).kind in ["u", "i"]):
-            raise ValueError(
-                "Output datatype cannot be integer when using binning. Please set the 'convert_float' parameter to True or specify a 'data_buffer'."
-            )
-
-    def _set_files(self, files):
-        if len(files) == 0:
-            raise ValueError("Expected at least one data file")
-        self._files_begin_idx = 0
-        if isinstance(self.dataset_subsampling, (tuple, list)):
-            self._files_begin_idx = self.dataset_subsampling[1]
-            self.dataset_subsampling = self.dataset_subsampling[0]
-        self.n_files = len(files)
-        self.files = files
-        self._sorted_files_indices = sorted(files.keys())
-        self._fileindex_to_idx = dict.fromkeys(self._sorted_files_indices)
-        self._configure_subsampling()
-
-    def _infer_file_type(self, files):
-        fname = files[sorted(files.keys())[0]].file_path()
-        ext = os.path.splitext(fname)[-1].replace(".", "")
-        if ext not in Readers:
-            raise ValueError("Unknown file format %s. Supported formats are: %s" % (ext, str(Readers.keys())))
-        return ext
-
-    def _get_reader_class(self, files):
-        ext = self._infer_file_type(files)
-        reader_class = Readers[ext]
-        self._reader_class = reader_class
-
-    def _get_shape_and_dtype(self, shape, dtype, binning):
-        if shape is None or dtype is None:
-            shape, dtype = self._infer_shape_and_dtype()
-        assert len(shape) == 2, "Expected the shape of an image (2-tuple)"
-        self.shape_total = shape
-        self.dtype = dtype
-        self._set_binning(binning)
-
-    def _configure_subsampling(self):
-        dataset_subsampling = self.dataset_subsampling
-        self.files_subsampled = self.files
-        if dataset_subsampling is not None and dataset_subsampling > 1:
-            self.files_subsampled = subsample_dict(self.files, dataset_subsampling)
-            self.n_files = len(self.files_subsampled)
-            if not (self._reader_class.multi_load):
-                # 3D loading not supported for this reader.
-                # Data is loaded frames by frame, so subsample directly self.files
-                self.files = self.files_subsampled
-                self._sorted_files_indices = sorted(self.files.keys())
-                self._fileindex_to_idx = dict.fromkeys(self._sorted_files_indices)
-
-    def _infer_shape_and_dtype(self):
-        self._reader_entire_image = self._reader_class()
-        first_file_dataurl = self.files[self._sorted_files_indices[0]]
-        first_file_data = self._reader_entire_image.get_data(first_file_dataurl)
-        return first_file_data.shape, first_file_data.dtype
-
-    def _set_subregion(self, sub_region):
-        sub_region = sub_region or (None, None, None, None)
-        start_x, end_x, start_y, end_y = sub_region
-        if start_x is None:
-            start_x = 0
-        if start_y is None:
-            start_y = 0
-        if end_x is None:
-            end_x = self.shape_total[1]
-        if end_y is None:
-            end_y = self.shape_total[0]
-        self.sub_region = (start_x, end_x, start_y, end_y)
-        self.shape = (end_y - start_y, end_x - start_x)
-        if self.binning is not None:
-            self.shape = (self.shape[0] // self.binning[1], self.shape[1] // self.binning[0])
-        self.chunk_shape = (self.n_files,) + self.shape
-        if self.detector_corrector is not None:
-            self.detector_corrector.set_sub_region_transformation(target_sub_region=self.sub_region)
-
-    def _init_reader(self):
-        # instantiate reader with user params
-        if self.detector_corrector is not None:
-            adapted_subregion = self.detector_corrector.get_adapted_subregion(self.sub_region)
-        else:
-            adapted_subregion = self.sub_region
-
-        self.file_reader = self._reader_class(sub_region=adapted_subregion)
-
-    def _set_binning(self, binning):
-        if binning is None:
-            self.binning = None
-            return
-        if np.isscalar(binning):
-            binning = (binning, binning)
-        else:
-            assert len(binning) == 2, "Expected binning in the form (binning_x, binning_y)"
-        if binning[0] == 1 and binning[1] == 1:
-            self.binning = None
-            return
-        for b in binning:
-            if int(b) != b:
-                raise ValueError("Expected an integer number for binning values, but got %s" % binning)
-        self.binning = binning
-
-    def get_data(self, file_url):
-        """
-        Get the data associated to a file url.
-        """
-        arr = self.file_reader.get_data(file_url)
-
-        if arr.ndim == 2:
-            if self.detector_corrector is not None:
-                arr = self.detector_corrector.transform(arr)
-            if self.binning is not None:
-                arr = image_binning(arr, self.binning[::-1])
-
-        else:
-            if self.detector_corrector is not None:
-                if self.detector_corrector is not None:
-                    _, (
-                        src_x_start,
-                        src_x_end,
-                        src_z_start,
-                        src_z_end,
-                    ) = self.detector_corrector.get_actual_shapes_source_target()
-
-                    arr_target = np.empty([len(arr), src_z_end - src_z_start, src_x_end - src_x_start], "f")
-
-                def apply_corrector(i_img_tuple):
-                    i, img = i_img_tuple
-                    arr_target[i] = self.detector_corrector.transform(img)
-
-                with ThreadPool(self.num_threads) as tp:
-                    tp.map(apply_corrector, enumerate(arr))
-                arr = arr_target
-            if self.binning is not None:
-                nz = arr.shape[0]
-                res = np.zeros((nz,) + image_binning(arr[0], self.binning[::-1]).shape, dtype="f")
-
-                def apply_binning(img_res_tuple):
-                    img, res = img_res_tuple
-                    res[:] = image_binning(img, self.binning[::-1])
-
-                with ThreadPool(self.num_threads) as tp:
-                    tp.map(apply_binning, zip(arr, res))
-                arr = res
-
-        return arr
-
-    def _load_single(self):
-        for i, fileidx in enumerate(self._sorted_files_indices):
-            file_url = self.files[fileidx]
-            self.files_data[i] = self.get_data(file_url)
-            self._fileindex_to_idx[fileidx] = i
-
-    def _load_multi(self):
-        urls_compacted = get_compacted_dataslices(
-            self.files, subsampling=self.dataset_subsampling, begin=self._files_begin_idx
-        )
-        loaded = {}
-        start_idx = 0
-        sorted_files_indices = sorted(urls_compacted.keys())
-        for idx in sorted_files_indices:
-            url = urls_compacted[idx]
-            url_str = str(url)
-            is_loaded = loaded.get(url_str, False)
-            if is_loaded:
-                continue
-            ds = url.data_slice()
-            delta_z = ds.stop - ds.start
-            if ds.step is not None and ds.step > 1:
-                delta_z = ceil(delta_z / ds.step)
-            end_idx = start_idx + delta_z
-            self.files_data[start_idx:end_idx] = self.get_data(url)
-            start_idx += delta_z
-            loaded[url_str] = True
-
-    def load_files(self, overwrite: bool = False):
-        """
-        Load the files whose links was provided at class instantiation.
-
-        Parameters
-        -----------
-        overwrite: bool, optional
-            Whether to force reloading the files if already loaded.
-        """
-        if self._loaded and not (overwrite):
-            raise ValueError("Radios were already loaded. Call load_files(overwrite=True) to force reloading")
-        if self.file_reader.multi_load:
-            self._load_multi()
-        else:
-            self._load_single()
-        self._loaded = True
-
-    load_data = load_files
-
-    @property
-    def data(self):
-        return self.files_data
-
-
 class VolReaderBase:
     """
     Base class with common code for data readers (EDFStackReader, NXTomoReader, etc)
@@ -555,7 +259,11 @@ class VolReaderBase:
             slice_x = None
         if isinstance(sub_region, (tuple, list)):
             slice_angle, slice_z, slice_x = sub_region
-        self.sub_region = (slice_angle or slice(None, None), slice_z or slice(None, None), slice_x or slice(None, None))
+        self.sub_region = (
+            slice_angle if slice_angle is not None else slice(None, None),
+            slice_z if slice_z is not None else slice(None, None),
+            slice_x if slice_x is not None else slice(None, None),
+        )
 
     def _set_processing_function(self, processing_func, processing_func_args, processing_func_kwargs):
         self.processing_func = processing_func
@@ -619,7 +327,7 @@ class NXTomoReader(VolReaderBase):
             If provided, this function first argument must be the source buffer (3D array: stack of raw images),
             and the second argument must be the destination buffer (3D array, stack of output images). It can be None.
 
-        Other parameters
+        Other Parameters
         ----------------
         The other parameters are passed to "processing_func" if this parameter is not None.
 
@@ -681,15 +389,20 @@ class NXTomoReader(VolReaderBase):
             # In this case, we can use h5py read_direct() to avoid extraneous memory consumption
             image_key_slice = self._image_key_slices[0]
             # merge image key selection and user selection (if any)
-            self._source_selection = (
-                merge_slices(image_key_slice, self.sub_region[0] or slice(None, None)),
-            ) + self.sub_region[1:]
+            angles_slice = self.sub_region[0]
+            if isinstance(angles_slice, slice) or angles_slice is None:
+                angles_slice = merge_slices(image_key_slice, self.sub_region[0] or slice(None, None))
+            else:  # assuming numpy array
+                # TODO more elegant
+                angles_slice = np.arange(self.data_shape_total[0], dtype=np.uint64)[image_key_slice][angles_slice]
+            self._source_selection = (angles_slice,) + self.sub_region[1:]
         else:
             user_selection_dim0 = self.sub_region[0]
             indices = np.arange(self.data_shape_total[0])
             data_selection_indices_axis0 = np.hstack(
-                [indices[image_key_slice][user_selection_dim0] for image_key_slice in self._image_key_slices]
-            )
+                [indices[image_key_slice] for image_key_slice in self._image_key_slices]
+            )[user_selection_dim0]
+
             self._source_selection = (data_selection_indices_axis0,) + self.sub_region[1:]
 
     def _get_temporary_buffer(self, convert_after_reading):
@@ -792,7 +505,7 @@ class NXDarksFlats:
 
     def get_reduced_current(self, h5_path="{entry}/control/data", method="median"):
         current = self.get_raw_current(h5_path=h5_path)
-        return {k: self._reduce_func[method](current[k]) for k in current.keys()}
+        return {k: self._reduce_func[method](current[k]) for k in current}
 
 
 class EDFStackReader(VolReaderBase):
@@ -986,6 +699,12 @@ def get_entry_from_h5_path(h5_path):
     return v[0] or v[1]
 
 
+def list_hdf5_entries(fname):
+    with HDF5File(fname, "r") as f:
+        entries = list(f.keys())
+    return entries
+
+
 def check_virtual_sources_exist(fname, data_path):
     with HDF5File(fname, "r") as f:
         if data_path not in f:
@@ -1005,6 +724,30 @@ def check_virtual_sources_exist(fname, data_path):
     return True
 
 
+def get_hdf5_file_all_virtual_sources(file_path_or_obj, return_only_filenames=False):
+    result = []
+
+    def collect_vsources(name, obj):
+        if isinstance(obj, Dataset) and obj.is_virtual:
+            vs = obj.virtual_sources()
+            if return_only_filenames:
+                vs = [vs_.file_name for vs_ in vs]
+            result.append({name: vs})
+
+    _self_opened_file = False
+    if isinstance(file_path_or_obj, str):
+        fdesc = HDF5File(file_path_or_obj, "r")
+        _self_opened_file = True
+    else:
+        fdesc = file_path_or_obj
+
+    fdesc.visititems(collect_vsources)
+
+    if _self_opened_file:
+        fdesc.close()
+    return result
+
+
 def import_h5_to_dict(h5file, h5path, asarray=False):
     """
     Wrapper on top of silx.io.dictdump.dicttoh5 replacing "None" with None
