@@ -12,7 +12,7 @@ from tomoscan.esrf.volume import (
     MultiTIFFVolume,
     TIFFVolume,
 )
-from nabu.io.writer import __have_jp2k__
+from tomoscan.esrf.volume.jp2kvolume import has_glymur as __have_jp2k__
 from tomoscan.esrf.scan.edfscan import EDFTomoScan
 from tomoscan.esrf.scan.nxtomoscan import NXtomoScan
 import pytest
@@ -105,7 +105,7 @@ def test_find_histogram_hdf5_volume(tmp_path):
         scheme="silx",
     )
 
-    assert find_histogram(volume=HDF5Volume(file_path=h5_file, data_path="entry")) == None
+    assert find_histogram(volume=HDF5Volume(file_path=h5_file, data_path="entry")) is None
 
 
 def test_find_histogram_single_frame_volume(tmp_path):
