@@ -1,41 +1,22 @@
 from math import ceil
 from tempfile import TemporaryDirectory
-from dataclasses import dataclass
 from tomoscan.io import HDF5File
 import pytest
 import numpy as np
 from nxtomo.application.nxtomo import ImageKey
 from tomoscan.esrf import EDFVolume
 from nabu.pipeline.reader import NXTomoReaderBinning
-from nabu.testutils import utilstest, __do_long_tests__, get_file
+from nabu.testutils import utilstest, __do_long_tests__, get_file, get_dummy_nxtomo_info
 from nabu.utils import indices_to_slices, merge_slices
 from nabu.io.reader import EDFStackReader, NXTomoReader, NXDarksFlats
 
 
-@dataclass
-class SimpleNXTomoDescription:
-    n_darks: int = 0
-    n_flats1: int = 0
-    n_projs: int = 0
-    n_flats2: int = 0
-    n_align: int = 0
-    frame_shape: tuple = None
-    dtype: np.dtype = np.uint16
-
-
 @pytest.fixture(scope="class")
 def bootstrap_nx_reader(request):
     cls = request.cls
-
-    cls.nx_fname = utilstest.getfile("dummy_nxtomo.nx")
-    cls.nx_data_path = "entry/instrument/detector/data"
-    cls.data_desc = SimpleNXTomoDescription(
-        n_darks=10, n_flats1=11, n_projs=100, n_flats2=11, n_align=12, frame_shape=(11, 10), dtype=np.uint16
+    cls.nx_fname, cls.data_desc, cls.image_key, cls.projs_vals, cls.darks_vals, cls.flats1_vals, cls.flats2_vals = (
+        get_dummy_nxtomo_info()
     )
-    cls.projs_vals = np.arange(cls.data_desc.n_projs) + cls.data_desc.n_flats1 + cls.data_desc.n_darks
-    cls.darks_vals = np.arange(cls.data_desc.n_darks)
-    cls.flats1_vals = np.arange(cls.data_desc.n_darks, cls.data_desc.n_darks + cls.data_desc.n_flats1)
-    cls.flats2_vals = np.arange(cls.data_desc.n_darks, cls.data_desc.n_darks + cls.data_desc.n_flats2)
 
     yield
     # teardown
@@ -45,15 +26,15 @@ def bootstrap_nx_reader(request):
 class TestNXReader:
     def test_incorrect_path(self):
         with pytest.raises(FileNotFoundError):
-            reader = NXTomoReader("/invalid/path", self.nx_data_path)
+            reader = NXTomoReader("/invalid/path")
         with pytest.raises(KeyError):
-            reader = NXTomoReader(self.nx_fname, "/bad/data/path")
+            reader = NXTomoReader(self.nx_fname, "/bad/data/path")  # noqa: F841
 
     def test_simple_reads(self):
         """
         Test NXTomoReader with simplest settings
         """
-        reader1 = NXTomoReader(self.nx_fname, self.nx_data_path)
+        reader1 = NXTomoReader(self.nx_fname)
         data1 = reader1.load_data()
         assert data1.shape == (self.data_desc.n_projs,) + self.data_desc.frame_shape
         assert np.allclose(data1[:, 0, 0], self.projs_vals)
@@ -62,15 +43,15 @@ class TestNXReader:
         """
         Test the data selection using "image_key".
         """
-        reader_projs = NXTomoReader(self.nx_fname, self.nx_data_path, image_key=ImageKey.PROJECTION.value)
+        reader_projs = NXTomoReader(self.nx_fname, image_key=ImageKey.PROJECTION.value)
         data = reader_projs.load_data()
         assert np.allclose(data[:, 0, 0], self.projs_vals)
 
-        reader_darks = NXTomoReader(self.nx_fname, self.nx_data_path, image_key=ImageKey.DARK_FIELD.value)
+        reader_darks = NXTomoReader(self.nx_fname, image_key=ImageKey.DARK_FIELD.value)
         data_darks = reader_darks.load_data()
         assert np.allclose(data_darks[:, 0, 0], self.darks_vals)
 
-        reader_flats = NXTomoReader(self.nx_fname, self.nx_data_path, image_key=ImageKey.FLAT_FIELD.value)
+        reader_flats = NXTomoReader(self.nx_fname, image_key=ImageKey.FLAT_FIELD.value)
         data_flats = reader_flats.load_data()
         assert np.allclose(data_flats[:, 0, 0], np.concatenate([self.flats1_vals, self.flats2_vals]))
 
@@ -83,10 +64,10 @@ class TestNXReader:
         def _check_correct_shape_succeeds(shape, sub_region, test_description=""):
             err_msg = "Something wrong with the following test:" + test_description
             data_buffer = np.zeros(shape, dtype="f")
-            reader1 = NXTomoReader(self.nx_fname, self.nx_data_path, sub_region=sub_region)
+            reader1 = NXTomoReader(self.nx_fname, sub_region=sub_region)
             data1 = reader1.load_data(output=data_buffer)
             assert id(data1) == id(data_buffer), err_msg
-            reader2 = NXTomoReader(self.nx_fname, self.nx_data_path, sub_region=sub_region)
+            reader2 = NXTomoReader(self.nx_fname, sub_region=sub_region)
             data2 = reader2.load_data()
             assert np.allclose(data1, data2), err_msg
 
@@ -120,11 +101,10 @@ class TestNXReader:
 
         for test_case in test_cases:
             for wrong_shape in test_case["wrong_shapes"]:
-                with pytest.raises(ValueError):
+                with pytest.raises(ValueError):  # noqa: PT012
                     data_buffer_wrong_shape = np.zeros(wrong_shape, dtype="f")
                     reader = NXTomoReader(
                         self.nx_fname,
-                        self.nx_data_path,
                         sub_region=test_case["sub_region"],
                     )
                     reader.load_data(output=data_buffer_wrong_shape)
@@ -148,7 +128,7 @@ class TestNXReader:
         ]
 
         for test_case in test_cases:
-            reader = NXTomoReader(self.nx_fname, self.nx_data_path, sub_region=test_case["sub_region"])
+            reader = NXTomoReader(self.nx_fname, sub_region=test_case["sub_region"])
             data = reader.load_data()
             assert data.shape == test_case["expected_shape"]
             assert np.allclose(data[:, 0, 0], test_case["expected_values"])
@@ -156,7 +136,7 @@ class TestNXReader:
     def test_reading_with_binning_(self):
         from nabu.pipeline.reader import NXTomoReaderBinning
 
-        reader_with_binning = NXTomoReaderBinning((2, 2), self.nx_fname, self.nx_data_path)
+        reader_with_binning = NXTomoReaderBinning((2, 2), self.nx_fname)
         data = reader_with_binning.load_data()
         assert data.shape == (self.data_desc.n_projs,) + tuple(n // 2 for n in self.data_desc.frame_shape)
 
@@ -171,13 +151,12 @@ class TestNXReader:
 
         distortion_corrector = DetectorDistortionBase(detector_full_shape_vh=data_desc.frame_shape)
         distortion_corrector.set_sub_region_transformation(target_sub_region=sub_region_xy)
-        adapted_subregion = distortion_corrector.get_adapted_subregion(sub_region_xy)
+        # adapted_subregion = distortion_corrector.get_adapted_subregion(sub_region_xy)
         sub_region = (slice(None, None), slice(*sub_region_xy[2:]), slice(*sub_region_xy[:2]))
 
         reader_distortion_corr = NXTomoReaderDistortionCorrection(
             distortion_corrector,
             self.nx_fname,
-            self.nx_data_path,
             sub_region=sub_region,
         )
 
@@ -220,7 +199,7 @@ class TestNXReader:
         for test_case in test_cases:
             binning = test_case.get("binning", None)
             reader_cls = NXTomoReader
-            init_args = [self.nx_fname, self.nx_data_path]
+            init_args = [self.nx_fname]
             init_kwargs = {"sub_region": test_case["sub_region"]}
             if binning is not None:
                 reader_cls = NXTomoReaderBinning
@@ -231,6 +210,29 @@ class TestNXReader:
             assert data.shape == test_case["expected_shape"], err_msg
             assert np.allclose(data[:, 0, 0], test_case["expected_values"]), err_msg
 
+    def test_load_exclude_projections(self):
+        n_z, n_x = self.data_desc.frame_shape
+        # projs_idx = np.where(self.image_key == 0)[0]
+        projs_idx = np.arange(self.data_desc.n_projs, dtype=np.int64)
+        excluded_projs_idx_1 = projs_idx[10:20]
+        excluded_projs_idx_2 = np.concatenate([projs_idx[10:14], projs_idx[50:57]])
+
+        set_to_nparray = lambda x: np.array(sorted(x))
+
+        projs_idx1 = set_to_nparray(set(projs_idx) - set(excluded_projs_idx_1))
+        projs_idx2 = set_to_nparray(set(projs_idx) - set(excluded_projs_idx_2))
+
+        sub_regions_to_test = (
+            (projs_idx1, None, None),
+            (projs_idx1, slice(0, n_z // 2), None),
+            (projs_idx2, None, None),
+            (projs_idx2, slice(3, n_z // 2), None),
+        )
+        for sub_region in sub_regions_to_test:
+            reader = NXTomoReader(self.nx_fname, sub_region=sub_region)
+            data = reader.load_data()
+            assert np.allclose(data[:, 0, 0], self.projs_vals[sub_region[0]])
+
 
 @pytest.fixture(scope="class")
 def bootstrap_edf_reader(request):
@@ -328,7 +330,7 @@ class TestEDFReader:
 
         distortion_corrector = DetectorDistortionBase(detector_full_shape_vh=self.frame_shape)
         distortion_corrector.set_sub_region_transformation(target_sub_region=sub_region_xy)
-        adapted_subregion = distortion_corrector.get_adapted_subregion(sub_region_xy)
+        # adapted_subregion = distortion_corrector.get_adapted_subregion(sub_region_xy)
         sub_region = (slice(None, None), slice(*sub_region_xy[2:]), slice(*sub_region_xy[:2]))
 
         reader_distortion_corr = EDFStackReaderDistortionCorrection(
