@@ -80,7 +80,7 @@ class TestNXWriter:
         writer.write(self.data, "test_no_overwrite")
 
         writer2 = NXProcessWriter(fname, entry="entry0000", overwrite=False)
-        with pytest.raises((RuntimeError, OSError)) as ex:
+        with pytest.raises((RuntimeError, OSError)):
             writer2.write(self.data, "test_no_overwrite")
 
-        message = "Error should have been raised for trying to overwrite, but got the following: %s" % str(ex.value)
+        # message = "Error should have been raised for trying to overwrite, but got the following: %s" % str(ex.value)
