@@ -1,5 +1,4 @@
 import os
-from typing import Optional
 import contextlib
 import h5py
 import numpy as np
@@ -8,6 +7,8 @@ from tomoscan.volumebase import VolumeBase
 from tomoscan.esrf import EDFVolume, HDF5Volume, TIFFVolume, JP2KVolume, MultiTIFFVolume
 from tomoscan.io import HDF5File
 
+from nabu.utils import first_generator_item
+
 
 # This function might be moved elsewhere
 def get_compacted_dataslices(urls, subsampling=None, begin=0):
@@ -100,7 +101,7 @@ def get_compacted_dataslices(urls, subsampling=None, begin=0):
 
 def get_first_hdf5_entry(fname):
     with HDF5File(fname, "r") as fid:
-        entry = list(fid.keys())[0]
+        entry = first_generator_item(fid.keys())
     return entry
 
 
@@ -189,7 +190,7 @@ class _BaseReader(contextlib.AbstractContextManager):
         if url.scheme() not in ("silx", "h5py"):
             raise ValueError("Valid scheme are silx and h5py")
         if url.data_slice() is not None:
-            raise ValueError("Data slices are not managed. Data path should " "point to a bliss node (h5py.Group)")
+            raise ValueError("Data slices are not managed. Data path should point to a bliss node (h5py.Group)")
         self._url = url
         self._file_handler = None
 
@@ -207,7 +208,7 @@ class EntryReader(_BaseReader):
         else:
             entry = self._file_handler[self._url.data_path()]
         if not isinstance(entry, h5py.Group):
-            raise ValueError("Data path should point to a bliss node (h5py.Group)")
+            raise TypeError("Data path should point to a bliss node (h5py.Group)")
         return entry
 
 
@@ -218,7 +219,7 @@ class DatasetReader(_BaseReader):
         self._file_handler = HDF5File(self._url.file_path(), mode="r")
         entry = self._file_handler[self._url.data_path()]
         if not isinstance(entry, h5py.Dataset):
-            raise ValueError("Data path ({}) should point to a dataset (h5py.Dataset)".format(self._url.path()))
+            raise TypeError(f"Data path ({self._url.path()}) should point to a dataset (h5py.Dataset)")
         return entry
 
 
@@ -239,7 +240,7 @@ def file_format_is_hdf5(file_format: str):
     return file_format.lower().lstrip(".") in ("hdf5", "hdf", "nx", "nexus")
 
 
-def get_output_volume(location: str, file_prefix: Optional[str], file_format: str, multitiff=False) -> VolumeBase:
+def get_output_volume(location: str, file_prefix: str | None, file_format: str, multitiff=False) -> VolumeBase:
     # TODO: see strategy. what if user provide a .nx ... ?
     # this function should be more generic
     location, extension = os.path.splitext(location)
@@ -261,3 +262,5 @@ def get_output_volume(location: str, file_prefix: Optional[str], file_format: st
             return MultiTIFFVolume(file_path=location)
         else:
             return TIFFVolume(folder=location, volume_basename=file_prefix)
+    else:
+        raise ValueError
