@@ -3,11 +3,11 @@ import pytest
 from scipy.interpolate import interp1d
 from nabu.testutils import generate_tests_scenarios, get_data
 from nabu.utils import get_cuda_srcfile, updiv
-from nabu.cuda.utils import __has_pycuda__, get_cuda_context
+from nabu.cuda.utils import __has_cupy__
 
-if __has_pycuda__:
-    import pycuda.gpuarray as garray
+if __has_cupy__:
     from nabu.cuda.kernel import CudaKernel
+    import cupy
 
 
 img0 = get_data("brain_phantom.npz")["data"]
@@ -25,11 +25,11 @@ scenarios = generate_tests_scenarios(
 def bootstrap(request):
     cls = request.cls
     cls.tol = 1e-4
-    if __has_pycuda__:
-        cls.ctx = get_cuda_context(cleanup_at_exit=False)
+    if __has_cupy__:
+        ...
     yield
-    if __has_pycuda__:
-        cls.ctx.pop()
+    if __has_cupy__:
+        ...
 
 
 @pytest.mark.usefixtures("bootstrap")
@@ -46,7 +46,7 @@ class TestInterpolation:
 
     # parametrize on a class method will use the same class, and launch this
     # method with different scenarios.
-    @pytest.mark.skipif(not (__has_pycuda__), reason="need pycuda for this test")
+    @pytest.mark.skipif(not (__has_cupy__), reason="need cupy for this test")
     @pytest.mark.parametrize("config", scenarios)
     def test_cuda_interpolation(self, config):
         img = config["image"]
@@ -56,14 +56,12 @@ class TestInterpolation:
         x = np.linspace(xmin, xmax, num=img.shape[0], endpoint=False, dtype="f")
         x_new = x + config["x_to_x_new"]
 
-        d_img = garray.to_gpu(img)
-        d_out = garray.zeros_like(d_img)
-        d_x = garray.to_gpu(x)
-        d_x_new = garray.to_gpu(x_new)
+        d_img = cupy.array(img)
+        d_out = cupy.zeros_like(d_img)
+        d_x = cupy.array(x)
+        d_x_new = cupy.array(x_new)
 
-        cuda_interpolator = CudaKernel(
-            "linear_interp_vertical", get_cuda_srcfile("interpolation.cu"), signature="PPiiPP"
-        )
+        cuda_interpolator = CudaKernel("linear_interp_vertical", get_cuda_srcfile("interpolation.cu"))
         cuda_interpolator(d_img, d_out, Nx, Ny, d_x, d_x_new, grid=(updiv(Nx, 16), updiv(Ny, 16), 1), block=(16, 16, 1))
         err = self._compare(d_out.get(), img, x, x_new)
         err_msg = str("Max error is too high for this configuration: %s" % str(config))
