@@ -3,9 +3,9 @@ import numpy as np
 
 def rescale_data(data, new_min, new_max, data_min=None, data_max=None):
     if data_min is None:
-        data_min = np.min(data)
+        data_min = np.nanmin(data)
     if data_max is None:
-        data_max = np.max(data)
+        data_max = np.nanmax(data)
     return (new_max - new_min) / (data_max - data_min) * (data - data_min) + new_min
 
 
@@ -41,7 +41,7 @@ def psnr(img1, img2):
 #
 
 
-class ConvolutionInfos(object):
+class ConvolutionInfos:
     allowed_axes = {
         "1D": [None],
         "separable_2D_1D_2D": [None, (0, 1), (1, 0)],
