@@ -1,5 +1,5 @@
 import pyopencl.array as parray
-from pyopencl import Program, CommandQueue, kernel_work_group_info
+from pyopencl import Program, kernel_work_group_info
 from ..utils import (
     deprecation_warning,
     catch_warnings,
@@ -15,10 +15,8 @@ class OpenCLKernel(KernelBase):
     -----------
     kernel_name: str
         Name of the OpenCL kernel.
-    ctx: pyopencl.Context
-        OpenCL context to use.
     queue: pyopencl.CommandQueue
-        OpenCL queue to use. If provided, will use this queue's context instead of 'ctx'
+        OpenCL queue to use.
     filename: str, optional
         Path to the file name containing kernels definitions
     src: str, optional
@@ -32,8 +30,7 @@ class OpenCLKernel(KernelBase):
     def __init__(
         self,
         kernel_name,
-        ctx,
-        queue=None,
+        queue,
         filename=None,
         src=None,
         automation_params=None,
@@ -47,12 +44,9 @@ class OpenCLKernel(KernelBase):
             automation_params=automation_params,
             silent_compilation_warnings=silent_compilation_warnings,
         )
-        if queue is not None:
-            self.ctx = queue.context
-            self.queue = queue
-        else:
-            self.ctx = ctx
-            self.queue = None
+
+        self.ctx = queue.context
+        self.queue = queue
         self.compile_kernel_source(kernel_name, build_kwargs)
         self.get_kernel()
 
@@ -125,15 +119,6 @@ class OpenCLKernel(KernelBase):
         return tuple(args)
 
     def call(self, *args, **kwargs):
-        if not isinstance(args[0], CommandQueue):
-            queue = self.queue
-            if queue is None:
-                raise ValueError(
-                    "First argument must be a pyopencl queue - otherwise provide OpenCLKernel(..., queue=queue)"
-                )
-        else:
-            queue = args[0]
-            args = args[1:]
         global_size, local_size, args, kwargs = self._prepare_call(*args, **kwargs)
 
         kwargs.pop("global_size", None)
@@ -141,6 +126,6 @@ class OpenCLKernel(KernelBase):
         kwargs.pop("grid", None)
         kwargs.pop("block", None)
 
-        return self.kernel(queue, global_size, local_size, *args, **kwargs)
+        return self.kernel(self.queue, global_size, local_size, *args, **kwargs)
 
     __call__ = call
