@@ -13,7 +13,7 @@ class OpenCLMemcpy2D(OpenCLProcessing):
 
     def __init__(self, ctx=None, device_type="GPU", queue=None, **kwargs):
         super().__init__(ctx=ctx, device_type=device_type, queue=queue, **kwargs)
-        self.memcpy2D = OpenCLKernel("cpy2d", self.ctx, filename=get_opencl_srcfile("ElementOp.cl"))
+        self.memcpy2D = OpenCLKernel("cpy2d", self.queue, filename=get_opencl_srcfile("ElementOp.cl"))
 
     def __call__(self, dst, src, transfer_shape_xy, dst_offset_xy=None, src_offset_xy=None, wait=True):
         if dst_offset_xy is None:
@@ -21,7 +21,6 @@ class OpenCLMemcpy2D(OpenCLProcessing):
         if src_offset_xy is None:
             src_offset_xy = (0, 0)
         evt = self.memcpy2D(
-            self.queue,
             dst,
             src,
             np.int32(dst.shape[-1]),
