@@ -25,7 +25,7 @@ class OpenCLProcessing(ProcessingBase):
 
         Parameters
         ----------
-        ctx: pycuda.driver.Context, optional
+        ctx: pyopencl context, optional
             Existing context to use. If provided, do not create a new context.
         cleanup_at_exit: bool, optional
             Whether to clean-up the context at exit.
@@ -61,8 +61,7 @@ class OpenCLProcessing(ProcessingBase):
     def kernel(self, kernel_name, filename=None, src=None, automation_params=None, **build_kwargs):
         return OpenCLKernel(
             kernel_name,
-            None,
-            queue=self.queue,
+            self.queue,
             filename=filename,
             src=src,
             automation_params=automation_params,
