@@ -16,43 +16,6 @@ _options_levels = {
 }
 
 
-class NabuConfigParser:
-    @deprecated(
-        "The class 'NabuConfigParser' is deprecated and will be removed in a future version. Please use parse_nabu_config_file instead",
-        do_print=True,
-    )
-    def __init__(self, fname):
-        """
-        Nabu configuration file parser.
-
-        Parameters
-        ----------
-        fname: str
-            File name of the configuration file
-        """
-        parser = ConfigParser(inline_comment_prefixes=("#",))  # allow in-line comments
-        with open(fname) as fid:
-            file_content = fid.read()
-        parser.read_string(file_content)
-        self.parser = parser
-        self.get_dict()
-        self.file_content = file_content.split(linesep)
-
-    def get_dict(self):
-        # Is there an officially supported way to do this ?
-        self.conf_dict = self.parser._sections
-        return self.conf_dict
-
-    def __str__(self):
-        return self.conf_dict.__str__()
-
-    def __repr__(self):
-        return self.conf_dict.__repr__()
-
-    def __getitem__(self, key):
-        return self.conf_dict[key]
-
-
 def parse_nabu_config_file(fname, allow_no_value=False):
     """
     Parse a configuration file and returns a dictionary.
@@ -156,7 +119,7 @@ def _extract_nabuconfig_section(section, default_config):
 
 def _extract_nabuconfig_keyvals(default_config):
     res = {}
-    for section in default_config.keys():
+    for section in default_config:
         res[section] = _extract_nabuconfig_section(section, default_config)
     return res
 
@@ -196,7 +159,7 @@ def validate_config(config, default_config, renamed_keys, errors="warn"):
         Reference configuration. Missing keys/sections from 'config' will be updated with
         keys from this dictionary.
     errors: str, optional
-        What to do when an unknonw key/section is encountered. Possible actions are:
+        What to do when an unknown key/section is encountered. Possible actions are:
           - "warn": throw a warning, continue the validation
           - "raise": raise an error and exit
     """
@@ -257,9 +220,9 @@ def overwrite_config(conf, overwritten_params):
         if section not in conf:
             raise ValueError("Unknown section %s" % section)
         current_section = conf[section]
-        for key in params.keys():
+        for key in params:
             if key not in current_section:
                 raise ValueError("Unknown parameter '%s' in section '%s'" % (key, section))
-            conf[section][key] = overwritten_params[section][key]
+            conf[section][key] = overwritten_params[section][key]  # noqa: PLR1733
     # ---
     return conf
