@@ -1,8 +1,9 @@
+# ruff: noqa: F405
 import os
 
 path = os.path
-from ..utils import check_supported, is_writeable
-from .params import *
+from ..utils import check_supported, deprecation_warning, get_num_threads, is_writeable
+from .params import *  # noqa: F403
 
 """
 A validator is a function with
@@ -96,12 +97,21 @@ def convert_to_bool_noerr(val):
     return res
 
 
-def name_range_checker(name, valid_names, descr, replacements=None):
+def name_range_checker(name, available_names, descr):
+    """
+    Check whether a parameter name is valid, against a list or dictionary of names.
+    """
     name = name.strip().lower()
-    if replacements is not None and name in replacements:
-        name = replacements[name]
-    valid = name in valid_names
-    assert valid, "Invalid %s '%s'. Available are %s" % (descr, name, str(valid_names))
+    valid = name in available_names
+    if isinstance(available_names, dict):
+        # handle replacements, eg. {"edge": "edges"}
+        name = available_names[name]
+        # we could use .keys() instead to be more permissive to the user
+        available_names_str = str(set(available_names.values()))
+    else:
+        # assuming list
+        available_names_str = str(available_names)
+    assert valid, "Invalid %s '%s'. Available are %s" % (descr, name, available_names_str)
     return name
 
 
@@ -310,7 +320,7 @@ def optional_nonzero_float_validator(val):
         assert error is None, "Invalid number"
     else:
         val_float = None
-    if val_float is not None:
+    if val_float is not None:  # noqa: SIM102
         if abs(val_float) < 1e-6:
             val_float = None
     return val_float
@@ -323,7 +333,7 @@ def optional_tuple_of_floats_validator(val):
     err_msg = "Expected a tuple of two numbers, but got %s" % val
     try:
         res = tuple(float(x) for x in val.strip("()").split(","))
-    except Exception as exc:
+    except Exception:
         raise ValueError(err_msg)
     if len(res) != 2:
         raise ValueError(err_msg)
@@ -337,9 +347,7 @@ def cor_validator(val):
         return val_float
     if len(val.strip()) == 0:
         return None
-    val = name_range_checker(
-        val.lower(), set(cor_methods.values()), "center of rotation estimation method", replacements=cor_methods
-    )
+    val = name_range_checker(val, cor_methods, "center of rotation estimation method")
     return val
 
 
@@ -350,9 +358,7 @@ def tilt_validator(val):
         return val_float
     if len(val.strip()) == 0:
         return None
-    val = name_range_checker(
-        val.lower(), set(tilt_methods.values()), "automatic detector tilt estimation method", replacements=tilt_methods
-    )
+    val = name_range_checker(val, tilt_methods, "automatic detector tilt estimation method")
     return val
 
 
@@ -394,53 +400,55 @@ def cor_slice_validator(val):
 
 
 @validator
-def flatfield_enabled_validator(val):
-    return name_range_checker(val, set(flatfield_modes.values()), "flatfield mode", replacements=flatfield_modes)
+def flatfield_validator(val):
+    ret = name_range_checker(val, flatfield_modes, "flatfield mode")
+    if ret in ["force-load", "force-compute"]:
+        deprecation_warning(
+            f"Using 'flatfield = {ret}' is deprecated since version 2025.1.0. Please use the parameter 'flatfield_loading_mode'",
+        )
+    return ret
+
+
+@validator
+def flatfield_loading_mode_validator(val):
+    return name_range_checker(val, flatfield_loading_mode, "flatfield mode")
 
 
 @validator
 def phase_method_validator(val):
-    return name_range_checker(
-        val, set(phase_retrieval_methods.values()), "phase retrieval method", replacements=phase_retrieval_methods
-    )
+    return name_range_checker(val, phase_retrieval_methods, "phase retrieval method")
 
 
 @validator
 def detector_distortion_correction_validator(val):
     return name_range_checker(
         val,
-        set(detector_distortion_correction_methods.values()),
+        detector_distortion_correction_methods,
         "detector_distortion_correction_methods",
-        replacements=detector_distortion_correction_methods,
     )
 
 
 @validator
 def unsharp_method_validator(val):
-    return name_range_checker(
-        val, set(unsharp_methods.values()), "unsharp mask method", replacements=phase_retrieval_methods
-    )
+    return name_range_checker(val, unsharp_methods, "unsharp mask method")
 
 
 @validator
 def padding_mode_validator(val):
-    return name_range_checker(val, set(padding_modes.values()), "padding mode", replacements=padding_modes)
+    return name_range_checker(val, padding_modes, "padding mode")
 
 
 @validator
 def reconstruction_method_validator(val):
-    return name_range_checker(
-        val, set(reconstruction_methods.values()), "reconstruction method", replacements=reconstruction_methods
-    )
+    return name_range_checker(val, reconstruction_methods, "reconstruction method")
 
 
 @validator
 def fbp_filter_name_validator(val):
     return name_range_checker(
         val,
-        set(fbp_filters.values()),
+        fbp_filters,
         "FBP filter",
-        replacements=fbp_filters,
     )
 
 
@@ -448,29 +456,24 @@ def fbp_filter_name_validator(val):
 def reconstruction_implementation_validator(val):
     return name_range_checker(
         val,
-        set(reco_implementations.values()),
+        reco_implementations,
         "Reconstruction method implementation",
-        replacements=reco_implementations,
     )
 
 
 @validator
 def optimization_algorithm_name_validator(val):
-    return name_range_checker(
-        val, set(optim_algorithms.values()), "optimization algorithm name", replacements=iterative_methods
-    )
+    return name_range_checker(val, optim_algorithms, "optimization algorithm name")
 
 
 @validator
 def output_file_format_validator(val):
-    return name_range_checker(val, set(files_formats.values()), "output file format", replacements=files_formats)
+    return name_range_checker(val, files_formats, "output file format")
 
 
 @validator
 def distribution_method_validator(val):
-    val = name_range_checker(
-        val, set(distribution_methods.values()), "workload distribution method", replacements=distribution_methods
-    )
+    val = name_range_checker(val, distribution_methods, "workload distribution method")
     # TEMP.
     if val != "local":
         raise NotImplementedError("Computation method '%s' is not implemented yet" % val)
@@ -480,9 +483,7 @@ def distribution_method_validator(val):
 
 @validator
 def sino_normalization_validator(val):
-    val = name_range_checker(
-        val, set(sino_normalizations.values()), "sinogram normalization method", replacements=sino_normalizations
-    )
+    val = name_range_checker(val, sino_normalizations, "sinogram normalization method")
     return val
 
 
@@ -490,9 +491,8 @@ def sino_normalization_validator(val):
 def sino_deringer_methods(val):
     val = name_range_checker(
         val,
-        set(rings_methods.values()),
+        rings_methods,
         "sinogram rings artefacts correction method",
-        replacements=rings_methods,
     )
     return val
 
@@ -507,6 +507,31 @@ def list_of_int_validator(val):
     return list(set(map(lambda x: x[0], res)))
 
 
+@validator
+def num_threads_validator(val):
+    err_msg = f"Expected either an integer or a fraction of integers (eg. 1/2) for 'num_threads', but got {val}"
+    val_int, err = convert_to_int(val)
+    if err is None:
+        return val_int
+    # COMPAT. with percentage (older config files)
+    if "%" in val:
+        n_cpus = get_num_threads()
+        try:
+            n_cpus = int((float(val.strip("%")) / 100) * get_num_threads())
+        except:
+            raise ValueError(f"Could not extract a percentage from: {val}")
+        return n_cpus
+    #
+    if "/" not in val:
+        raise ValueError(err_msg)
+    num, denom = val.split("/")
+    num, err_n = convert_to_int(num)
+    denom, err_d = convert_to_int(denom)
+    if err_n is not None or err_d is not None:
+        raise ValueError(err_msg)
+    return int((num / denom) * get_num_threads())
+
+
 @validator
 def list_of_shift_validator(values):
     ids = values.replace(" ", "").split(",")
@@ -555,7 +580,7 @@ def nonempty_string_validator(val):
 
 @validator
 def logging_validator(val):
-    return name_range_checker(val, set(log_levels.values()), "logging level", replacements=log_levels)
+    return name_range_checker(val, log_levels, "logging level")
 
 
 @validator
