@@ -1,16 +1,10 @@
 from os import path
+import numpy as np
 from ..resources.logger import LoggerOrPrint
 from .utils import get_subregion
 from .writer import WriterManager
 from ..io.reader import get_hdf5_dataset_shape
 
-try:
-    import pycuda.gpuarray as garray
-
-    __has_pycuda__ = True
-except:
-    __has_pycuda__ = False
-
 
 class DataDumpManager:
     """
@@ -126,7 +120,7 @@ class DataDumpManager:
         read_opts = self.processing_options["read_chunk"]
         if read_opts.get("process_file", None) is None:
             return None
-        dump_start_z, dump_end_z = read_opts["dump_start_z"], read_opts["dump_end_z"]
+        dump_start_z, dump_end_z = read_opts["dump_start_z"], read_opts["dump_end_z"]  # noqa: F841
         relative_start_z = self.z_min - dump_start_z
         relative_end_z = relative_start_z + self.delta_z
         # When using binning, every step after "read" results in smaller-sized data.
@@ -139,7 +133,7 @@ class DataDumpManager:
 
     def _check_resume_from_step(self):
         read_opts = self.processing_options["read_chunk"]
-        expected_radios_shape = get_hdf5_dataset_shape(
+        expected_radios_shape = get_hdf5_dataset_shape(  # noqa: F841
             read_opts["process_file"],
             read_opts["process_h5_path"],
             sub_region=self.get_read_dump_subregion(),
@@ -151,9 +145,11 @@ class DataDumpManager:
             return
         writer = self.data_dump[step_name]
         self.logger.info("Dumping data to %s" % writer.fname)
-        if __has_pycuda__:
-            if isinstance(data, garray.GPUArray):
-                data = data.get()
+        if not (isinstance(data, np.ndarray)):
+            # assuming "device array" (cupy/pycuda/pyopencl)
+            # note that we don't explicitly test the array type, as it would entail to import cupy (for example)
+            # and we want to avoid importing cupy because cuda should not be initialized until really wanted (mess with multiprocessing fork)
+            data = data.get()
 
         margin_up = self.margin[0][0] or None
         margin_down = self.margin[0][1] or None
