@@ -1,16 +1,15 @@
 from os import path
 from time import time
-from math import ceil
 import numpy as np
 from silx.io.url import DataUrl
 
-from ...utils import get_num_threads, remove_items_from_list
+from ...utils import get_num_threads, first_generator_item, remove_items_from_list, get_subregion as get_subregion_xy
 from ...resources.logger import LoggerOrPrint
 from ...resources.utils import extract_parameters
 from ...misc.binning import binning as image_binning
 from ...io.reader import EDFStackReader, HDF5Loader, NXTomoReader
 from ...preproc.ccd import Log, CCDFilter
-from ...preproc.flatfield import FlatField
+from ...preproc.flatfield import FlatField, PCAFlatsNormalizer
 from ...preproc.distortion import DistortionCorrection
 from ...preproc.shift import VerticalShift
 from ...preproc.double_flatfield import DoubleFlatField
@@ -18,7 +17,7 @@ from ...preproc.phase import PaganinPhaseRetrieval
 from ...preproc.ctf import CTFPhaseRetrieval, GeoPars
 from ...reconstruction.sinogram import SinoNormalization
 from ...reconstruction.filtering import SinoFilter
-from ...reconstruction.mlem import __have_corrct__, MLEMReconstructor
+from ...reconstruction.mlem import CCTMLEMReconstructor, NabuMLEMReconstructor
 from ...processing.rotation import Rotation
 from ...reconstruction.rings import MunchDeringer, SinoMeanDeringer, VoDeringer
 from ...processing.unsharp import UnsharpMask
@@ -45,6 +44,7 @@ class ChunkedPipeline:
 
     backend = "numpy"
     FlatFieldClass = FlatField
+    PCAFlatFieldClass = PCAFlatsNormalizer
     DoubleFlatFieldClass = DoubleFlatField
     CCDCorrectionClass = CCDFilter
     PaganinPhaseRetrievalClass = PaganinPhaseRetrieval
@@ -59,12 +59,10 @@ class ChunkedPipeline:
     SinoNormalizationClass = SinoNormalization
     SinoFilterClass = SinoFilter
     FBPClass = Backprojector
-    ConebeamClass = None  # unsupported on CPU
-    MLEMClass = MLEMReconstructor
     HBPClass = None  # unsupported on CPU
     HistogramClass = PartialHistogram
 
-    _default_extra_options = {}
+    _default_extra_options = {"write_in_reverse_order": False}
 
     # These steps are skipped if the reconstruction is done in two stages.
     # The first stage will skip these steps, and the second stage will do these stages after merging sinograms.
@@ -99,7 +97,6 @@ class ChunkedPipeline:
 
         Notes
         ------
-
         Using `margin` results in a lesser number of reconstructed slices.
         More specifically, if `margin = (V, H)`, then there will be `delta_z - 2*V`
         reconstructed slices (if the sub-region is in the middle of the volume)
@@ -111,6 +108,7 @@ class ChunkedPipeline:
         self._init_pipeline()
 
     def _set_params(self, process_config, chunk_shape, extra_options, margin, use_grouped_mode):
+        self._set_extra_options(extra_options)
         self.process_config = process_config
         self.dataset_info = self.process_config.dataset_info
         self.processing_steps = self.process_config.processing_steps.copy()
@@ -118,7 +116,6 @@ class ChunkedPipeline:
         self._set_chunk_shape(chunk_shape, use_grouped_mode)
         self.set_subregion(None)
         self._set_margin(margin)
-        self._set_extra_options(extra_options)
         self._callbacks = {}
         self._steps_name2component = {}
         self._steps_component2name = {}
@@ -127,10 +124,10 @@ class ChunkedPipeline:
         if len(chunk_shape) != 3:
             raise ValueError("Expected chunk_shape to be a tuple of length 3 in the form (n_z, n_y, n_x)")
         self.chunk_shape = tuple(int(c) for c in chunk_shape)  # cast to int, as numpy.int64 can make pycuda crash
-        # TODO: sanity check (eg. compare to size of radios in dataset_info) ?
+        ss_start = getattr(self.process_config, "subsampling_start", 0)
         # (n_a, n_z, n_x)
         self.radios_shape = (
-            ceil(self.chunk_shape[0] / self.process_config.subsampling_factor),
+            np.arange(self.chunk_shape[0])[ss_start :: self.process_config.subsampling_factor].size,
             self.chunk_shape[1] // self.process_config.binning[1],
             self.chunk_shape[2] // self.process_config.binning[0],
         )
@@ -164,7 +161,7 @@ class ChunkedPipeline:
         if self.use_margin:
             self.n_slices -= sum(self.margin[0])
 
-    def set_subregion(self, sub_region):
+    def set_subregion(self, sub_region, output_start_z=None):
         """
         Set the data volume sub-region to process.
         Note that processing margin, if any, is contained within the sub-region.
@@ -175,7 +172,7 @@ class ChunkedPipeline:
             Data volume sub-region, in the form ((start_a, end_a), (start_z, end_z), (start_x, end_x))
             where the data volume has a layout (angles, Z, X)
         """
-        n_angles = self.dataset_info.n_angles
+        # n_angles = self.dataset_info.n_angles
         n_x, n_z = self.dataset_info.radio_dims
         c_a, c_z, c_x = self.chunk_shape
         if sub_region is None:
@@ -190,14 +187,15 @@ class ChunkedPipeline:
             # check sub-region
             for i, start_end in enumerate(sub_region):
                 start, end = start_end
-                if start is not None and end is not None:
+                if start is not None and end is not None:  # noqa: SIM102
                     if end - start != self.chunk_shape[i]:
                         raise ValueError(
                             "Invalid (start, end)=(%d, %d) for sub-region (dimension %d): chunk shape is %s, but %d-%d=%d != %d"
                             % (start, end, i, str(self.chunk_shape), end, start, end - start, self.chunk_shape[i])
                         )
             #
-        self.logger.debug("Set sub-region to %s" % (str(sub_region)))
+        self._output_start_z = output_start_z
+        self.logger.debug(f"Set sub-region to {str(sub_region)} ; output_start_z={self._output_start_z}")
         self.sub_region = sub_region
         self._sub_region_xz = sub_region[2] + sub_region[1]
         self._radios_were_cropped = False
@@ -205,6 +203,7 @@ class ChunkedPipeline:
     def _set_extra_options(self, extra_options):
         self.extra_options = self._default_extra_options.copy()
         self.extra_options.update(extra_options or {})
+        self._write_in_reverse_order = self.extra_options["write_in_reverse_order"]
 
     #
     # Callbacks
@@ -265,7 +264,10 @@ class ChunkedPipeline:
         return (self.n_slices,) + self.sino_shape
 
     def get_slice_start_index(self):
-        return self.z_min + self._margin_up
+        if self._output_start_z is not None:
+            return self._output_start_z
+        z_min = self.sub_region[1][0]
+        return z_min + self._margin_up
 
     #
     # Pipeline initialization
@@ -339,10 +341,28 @@ class ChunkedPipeline:
             subs_angles = None
             subs_z = None
             subs_x = None
+            angular_sub_region = slice(*(self.sub_region[0]))
+
+            # exclude(subsample(.)) != subsample(exclude(.))
+            # Here we want the latter: first exclude the user-defined angular range, and then subsample the remaining indices
+            if len(self.dataset_info.get_excluded_projections_indices()) > 0:
+                angular_sub_region = np.array(
+                    [self.dataset_info.index_to_proj_number(i) for i in sorted(self.dataset_info.projections.keys())]
+                )
             if self.process_config.subsampling_factor:
                 subs_angles = self.process_config.subsampling_factor
+                start = getattr(self.process_config, "subsampling_start", 0) + self.sub_region[0][0]
+                if isinstance(angular_sub_region, slice):
+                    angular_sub_region = slice(
+                        start,
+                        self.sub_region[0][1],
+                        subs_angles,
+                    )
+                else:
+                    angular_sub_region = angular_sub_region[start::subs_angles]
+
             reader_sub_region = (
-                slice(*(self.sub_region[0]) + ((subs_angles,) if subs_angles else ())),
+                angular_sub_region,
                 slice(*(self.sub_region[1]) + ((subs_z,) if subs_z else ())),
                 slice(*(self.sub_region[2]) + ((subs_x,) if subs_x else ())),
             )
@@ -357,7 +377,7 @@ class ChunkedPipeline:
             if self.dataset_info.kind == "nx":
                 self.chunk_reader = NXTomoReader(
                     self.dataset_info.dataset_hdf5_url.file_path(),
-                    self.dataset_info.dataset_hdf5_url.data_path(),
+                    data_path=self.dataset_info.dataset_hdf5_url.data_path(),
                     sub_region=reader_sub_region,
                     image_key=0,
                     **other_reader_kwargs,
@@ -388,50 +408,68 @@ class ChunkedPipeline:
 
     @use_options("flatfield", "flatfield")
     def _init_flatfield(self):
-        self._ff_options = self.processing_options["flatfield"].copy()
+        if self.processing_options["flatfield"]:
+            self._ff_options = self.processing_options["flatfield"].copy()
 
-        # This won't work when resuming from a step (i.e before FF), because we rely on H5Loader()
-        # which re-compacts the data. When data is re-compacted, we have to know the original radios positions.
-        # These positions can be saved in the "file_dump" metadata, but it is not loaded for now
-        # (the process_config object is re-built from scratch every time)
-        self._ff_options["projs_indices"] = self.chunk_reader.get_frames_indices()
+            # This won't work when resuming from a step (i.e before FF), because we rely on H5Loader()
+            # which re-compacts the data. When data is re-compacted, we have to know the original radios positions.
+            # These positions can be saved in the "file_dump" metadata, but it is not loaded for now
+            # (the process_config object is re-built from scratch every time)
+            self._ff_options["projs_indices"] = self.chunk_reader.get_frames_indices()
 
-        if self._ff_options.get("normalize_srcurrent", False):
-            a_start_idx, a_end_idx = self.sub_region[0]
-            subs = self.process_config.subsampling_factor
-            self._ff_options["radios_srcurrent"] = self._ff_options["radios_srcurrent"][a_start_idx:a_end_idx:subs]
+            if self._ff_options.get("normalize_srcurrent", False):
+                a_start_idx, a_end_idx = self.sub_region[0]
+                subs = self.process_config.subsampling_factor
+                self._ff_options["radios_srcurrent"] = self._ff_options["radios_srcurrent"][a_start_idx:a_end_idx:subs]
 
-        distortion_correction = None
-        if self._ff_options["do_flat_distortion"]:
-            self.logger.info("Flats distortion correction will be applied")
-            self.FlatFieldClass = FlatField  # no GPU implementation available, force this backend
-            estimation_kwargs = {}
-            estimation_kwargs.update(self._ff_options["flat_distortion_params"])
-            estimation_kwargs["logger"] = self.logger
-            distortion_correction = DistortionCorrection(
-                estimation_method="fft-correlation", estimation_kwargs=estimation_kwargs, correction_method="interpn"
-            )
+            distortion_correction = None
+            if self._ff_options["do_flat_distortion"]:
+                self.logger.info("Flats distortion correction will be applied")
+                self.FlatFieldClass = FlatField  # no GPU implementation available, force this backend
+                estimation_kwargs = {}
+                estimation_kwargs.update(self._ff_options["flat_distortion_params"])
+                estimation_kwargs["logger"] = self.logger
+                distortion_correction = DistortionCorrection(
+                    estimation_method="fft-correlation",
+                    estimation_kwargs=estimation_kwargs,
+                    correction_method="interpn",
+                )
 
-        # Reduced darks/flats are loaded, but we have to crop them on the current sub-region
-        # and possibly do apply some pre-processing (binning, distortion correction, ...)
-        darks_flats = load_darks_flats(
-            self.dataset_info,
-            self.sub_region[1:],
-            processing_func=self._ff_processing_function,
-            processing_func_args=self._ff_processing_function_args,
-        )
+            if self.processing_options["flatfield"]["method"].lower() != "pca":
+                # Reduced darks/flats are loaded, but we have to crop them on the current sub-region
+                # and possibly do apply some pre-processing (binning, distortion correction, ...)
+                darks_flats = load_darks_flats(
+                    self.dataset_info,
+                    self.sub_region[1:],
+                    processing_func=self._ff_processing_function,
+                    processing_func_args=self._ff_processing_function_args,
+                )
 
-        # FlatField parameter "radios_indices" must account for subsampling
-        self.flatfield = self.FlatFieldClass(
-            self.radios_shape,
-            flats=darks_flats["flats"],
-            darks=darks_flats["darks"],
-            radios_indices=self._ff_options["projs_indices"],
-            interpolation="linear",
-            distortion_correction=distortion_correction,
-            radios_srcurrent=self._ff_options["radios_srcurrent"],
-            flats_srcurrent=self._ff_options["flats_srcurrent"],
-        )
+                # FlatField parameter "radios_indices" must account for subsampling
+                self.flatfield = self.FlatFieldClass(
+                    self.radios_shape,
+                    flats=darks_flats["flats"],
+                    darks=darks_flats["darks"],
+                    radios_indices=self._ff_options["projs_indices"],
+                    interpolation="linear",
+                    distortion_correction=distortion_correction,
+                    radios_srcurrent=self._ff_options["radios_srcurrent"],
+                    flats_srcurrent=self._ff_options["flats_srcurrent"],
+                )
+            else:
+                flats = self.process_config.dataset_info.flats
+                darks = self.process_config.dataset_info.darks
+                if len(darks) != 1:
+                    raise ValueError(f"There should be only one reduced dark. Found {len(darks)}.")
+                else:
+                    dark_key = first_generator_item(darks)
+                nb_pca_components = len(flats) - 1
+                img_subregion = tuple(slice(*sr) for sr in self.sub_region[1:])
+                self.flatfield = self.PCAFlatFieldClass(
+                    np.array([flats[k][img_subregion] for k in range(1, nb_pca_components)]),
+                    darks[dark_key][img_subregion],
+                    flats[0][img_subregion],  # Mean
+                )
 
     @use_options("double_flatfield", "double_flatfield")
     def _init_double_flatfield(self):
@@ -451,7 +489,8 @@ class ChunkedPipeline:
         self.double_flatfield = self.DoubleFlatFieldClass(
             self.radios_shape,
             result_url=result_url,
-            sub_region=self.sub_region[1:],
+            # DoubleFlatField expects sub_region as (start_x, end_x, start_y, end_y)
+            sub_region=get_subregion_xy(self.sub_region[1:][::-1]),
             input_is_mlog=False,
             output_is_mlog=False,
             average_is_on_log=avg_is_on_log,
@@ -574,10 +613,12 @@ class ChunkedPipeline:
         options = self.processing_options["reconstruction"]
         if options["method"] == "FBP" and self.FBPClass is None:
             raise ValueError("No usable FBP module was found")
-        if options["method"] == "cone" and self.ConebeamClass is None:
-            raise ValueError("No usable cone-beam module was found")
-        if options["method"] == "mlem" and self.MLEMClass is None:
-            raise ValueError("No usable MLEM module was found.")
+        if options["method"] is None:
+            self.reconstruction = None
+            return
+
+        if self.dataset_info.flip_frame_lr:
+            self.logger.warning("Slices will be flipped left<->right !")
 
         n_slices = self.n_slices
         if options["method"] in ["FBP", "HBP"]:  # both have the same API
@@ -608,7 +649,17 @@ class ChunkedPipeline:
             pixel_size_m = self.dataset_info.pixel_size * 1e-6
             source_sample_dist = options["source_sample_dist"] / pixel_size_m
             sample_detector_dist = options["sample_detector_dist"] / pixel_size_m
-            self.reconstruction = self.ConebeamClass(  # pylint: disable=E1102
+
+            # Do it here to avoid creating cuda contexts at import time
+            from ...reconstruction.cone import ConebeamReconstructor, NumpyConebeamReconstructor
+
+            cone_implem = options.get("implementation", "nabu")
+            if cone_implem == "astra":
+                cone_cls = NumpyConebeamReconstructor
+            else:
+                cone_cls = ConebeamReconstructor
+
+            self.reconstruction = cone_cls(  # pylint: disable=E1102
                 (self.radios_shape[1],) + self.sino_shape,
                 source_sample_dist,
                 sample_detector_dist,
@@ -616,6 +667,7 @@ class ChunkedPipeline:
                 rot_center=options["rotation_axis_position"],
                 pixel_size=1,
                 padding_mode=options["padding_type"],
+                filter_name=options["fbp_filter_type"] or "none",
                 slice_roi=self.process_config.rec_roi,
                 extra_options={
                     "scale_factor": 1.0 / options["voxel_size_cm"][0],
@@ -623,24 +675,48 @@ class ChunkedPipeline:
                     "clip_outer_circle": options["clip_outer_circle"],
                     "outer_circle_value": options["outer_circle_value"],
                     "filter_cutoff": options["fbp_filter_cutoff"],
+                    "crop_filtered_data": options["crop_filtered_data"],
                 },
             )
 
-        if options["method"] == "mlem" and options["implementation"] in (None, "corrct"):
-            self.reconstruction = self.MLEMClass(  # pylint: disable=E1102
-                (self.radios_shape[1],) + self.sino_shape,
-                angles_rad=-options["angles"],  # WARNING: mind the sign...
-                shifts_uv=self.dataset_info.translations,  # In config file, one line per proj, each line is (tu,tv). Corrct expects one col per proj and (tv,tu).
-                cor=options["rotation_axis_position"],
-                n_iterations=options["iterations"],
-                extra_options={
-                    "compute_shifts": False,
-                    "tomo_consistency": False,
-                    "v_min_for_v_shifts": 0,
-                    "v_max_for_v_shifts": None,
-                    "v_min_for_u_shifts": 0,
-                    "v_max_for_u_shifts": None,
-                },
+        if options["method"] == "mlem":
+            mlem_implem = options.get("implementation", "nabu")
+            # It would be good to have the same interface between CCTMLEMReconstructor and NabuMLEMReconstructor
+            if mlem_implem == "corrct":
+                self.reconstruction = CCTMLEMReconstructor(
+                    (self.radios_shape[1],) + self.sino_shape,
+                    angles_rad=options["angles"],
+                    shifts_uv=self.dataset_info.translations,  # In config file, one line per proj, each line is (tu,tv). Corrct expects one col per proj and (tv,tu).
+                    cor=options["rotation_axis_position"],
+                    n_iterations=options["iterations"],
+                    extra_options={
+                        "compute_shifts": False,
+                        "tomo_consistency": False,
+                        "v_min_for_v_shifts": 0,
+                        "v_max_for_v_shifts": None,
+                        "v_min_for_u_shifts": 0,
+                        "v_max_for_u_shifts": None,
+                        "scale_factor": 1.0 / options["voxel_size_cm"][0],
+                    },
+                )
+            else:
+                self.reconstruction = NabuMLEMReconstructor(
+                    self.sinos_shape[1:],
+                    angles=options["angles"],
+                    rot_center=options["rotation_axis_position"],
+                    halftomo=options["enable_halftomo"],
+                    extra_options={
+                        "scale_factor": 1.0 / options["voxel_size_cm"][0],
+                        "axis_correction": options["axis_correction"],
+                        "clip_outer_circle": options["clip_outer_circle"],
+                        "outer_circle_value": options["outer_circle_value"],
+                    },
+                    cuda_options=None,  # TODO (?)
+                )
+
+        if options.get("crop_filtered_data", True) is False:
+            self.logger.warning(
+                "Using [reconstruction] crop_filtered_data = False. This will use a large amount of memory."
             )
 
         self._allocate_recs(*self.process_config.rec_shape, n_slices=n_slices)
@@ -654,6 +730,9 @@ class ChunkedPipeline:
 
     @use_options("save", "writer")
     def _init_writer(self, **extra_options):
+        if self.reconstruction is None:
+            self.writer = None
+            return
         options = self.processing_options["save"]
         metadata = {
             "process_name": "reconstruction",
@@ -681,6 +760,7 @@ class ChunkedPipeline:
             file_format=options["file_format"],
             overwrite=options["overwrite"],
             start_index=self.get_slice_start_index(),
+            write_in_reverse_order=self._write_in_reverse_order,
             logger=self.logger,
             metadata=metadata,
             histogram=("histogram" in self.processing_steps),
@@ -769,12 +849,16 @@ class ChunkedPipeline:
 
     def _dump_sinogram(self):
         if self.datadump_manager is not None:
-            self.datadump_manager.dump_data_to_file("sinogram", self.radios)
+            if self._radios_were_cropped:
+                # do crop on host
+                self.datadump_manager.dump_data_to_file("sinogram", self._orig_radios, crop_margin=True)
+            else:
+                self.datadump_manager.dump_data_to_file("sinogram", self.radios)
 
     @pipeline_step("sino_deringer", "Removing rings on sinograms")
     def _destripe_sinos(self):
         sinos = np.rollaxis(self.radios, 1, 0)  # view
-        self.sino_deringer.remove_rings(sinos)  # TODO check it works with non-contiguous view
+        self.sino_deringer.remove_rings(sinos)
 
     @pipeline_step("reconstruction", "Reconstruction")
     def _reconstruct(self):
@@ -782,50 +866,57 @@ class ChunkedPipeline:
         Reconstruction for parallel geometry.
         For each target slice: get the corresponding sinogram, apply some processing, then reconstruct
         """
+        x_axis_step = 1 if not (self.dataset_info.flip_frame_lr) else -1
+        x_axis_selection = slice(None, None, x_axis_step)
+
         options = self.processing_options["reconstruction"]
         if options["method"] == "cone":
             self._reconstruct_cone()
-            return
 
-        if options["method"] == "mlem":
-            self.recs = self._reconstruct_mlem()
-            return
-
-        for i in range(self.n_slices):
-            self._tmp_sino[:] = self.radios[:, i, :]  # copy into contiguous array
-            self.reconstruction.fbp(self._tmp_sino, output=self.recs[i])
+        elif options["method"] == "mlem":
+            if options["implementation"] == "corrct":
+                self.recs = self._reconstruct_mlem_corrct()
+            else:
+                n_iterations = self.processing_options["reconstruction"]["iterations"]
+                for i in range(self.n_slices):
+                    self._tmp_sino[:] = self.radios[:, i, x_axis_selection]  # copy into contiguous array
+                    self.reconstruction.reconstruct(self._tmp_sino, output=self.recs[i], n_iterations=n_iterations)
+        else:  # FBP
+            for i in range(self.n_slices):
+                self._tmp_sino[:] = self.radios[:, i, x_axis_selection]  # copy into contiguous array
+                self.reconstruction.fbp(self._tmp_sino, output=self.recs[i])
 
     def _reconstruct_cone(self):
         """
         This reconstructs the entire sinograms stack at once
         """
-
-        n_angles, n_z, n_x = self.radios.shape
-
-        # FIXME
-        # can't do a discontiguous single copy...
-        sinos_contig = self._allocate_array((n_z, n_angles, n_x), np.float32, "sinos_cone")
-        for i in range(n_z):
-            sinos_contig[i] = self.radios[:, i, :]
-        # ---
+        sinos_discontig = self.radios.transpose((1, 0, 2))  # view
+        if self.dataset_info.flip_frame_lr:
+            sinos_discontig = sinos_discontig[..., ::-1]  # still a view (no copy)
 
         # In principle radios are not cropped at this stage,
-        # so self.sub_region[2][0] can be used instead of self.get_slice_start_index() instead of self.sub_region[2][0]
+        # so self.sub_region[2][0] can be used instead
         z_min, z_max = self.sub_region_xz[2:]
         n_z_tot = self.process_config.radio_shape(binning=True)[0]
 
         self.reconstruction.reconstruct(  # pylint: disable=E1101
-            sinos_contig,
+            sinos_discontig,
             output=self.recs,
             relative_z_position=((z_min + z_max) / self.process_config.binning_z / 2) - n_z_tot / 2,
         )
+        if self.reconstruction.implementation == "astra":
+            # See also in chunked_cuda
+            ((U, D), (L, R)) = self.margin
+            U, D = U or None, -D or None
+            self.recs = self.recs[U:D, ...]
 
-    def _reconstruct_mlem(self):
+    def _reconstruct_mlem_corrct(self):
         """
         This reconstructs the entire sinograms stack at once
         """
 
         n_angles, n_z, n_x = self.radios.shape
+        x_axis_selection = slice(None, None, -1 if self.dataset_info.flip_frame_lr else 1)
 
         # FIXME
         # can't do a discontiguous single copy...
@@ -833,12 +924,13 @@ class ChunkedPipeline:
         # data with this order (nb_rows, nb_angles, nb_cols)
         data_vwu = self._allocate_array((n_z, n_angles, n_x), np.float32, "sinos_mlem")
         for i in range(n_z):
-            data_vwu[i] = self.radios[:, i, :]
+            data_vwu[i] = self.radios[:, i, x_axis_selection]
         # ---
 
-        return self.reconstruction.reconstruct(  # pylint: disable=E1101
+        rec = self.reconstruction.reconstruct(  # pylint: disable=E1101
             data_vwu,
         )
+        return rec.astype("f")  # corrct uses float64 data
 
     @pipeline_step("histogram", "Computing histogram")
     def _compute_histogram(self, data=None):
@@ -921,15 +1013,15 @@ class ChunkedPipeline:
             self._init_data_dump()
         self._init_reader()
 
-    def _reset_sub_region(self, sub_region):
-        self.set_subregion(sub_region)
+    def _reset_sub_region(self, sub_region, output_start_z=None):
+        self.set_subregion(sub_region, output_start_z=output_start_z)
         self._reset_reader_subregion()
         self._init_flatfield()  # reset flatfield
         self._init_writer()
         self._init_double_flatfield()
         self._init_data_dump()
 
-    def process_chunk(self, sub_region):
-        self._reset_sub_region(sub_region)
+    def process_chunk(self, sub_region, output_start_z=None):
+        self._reset_sub_region(sub_region, output_start_z=output_start_z)
         self._read_data()
         self._process_chunk()
