@@ -12,15 +12,12 @@ from ...processing.rotation_cuda import CudaRotation
 from ...processing.histogram_cuda import CudaPartialHistogram
 from ...reconstruction.fbp import Backprojector
 from ...reconstruction.hbp import HierarchicalBackprojector
-from ...reconstruction.cone import __have_astra__, ConebeamReconstructor
-from ...cuda.utils import get_cuda_context, __has_pycuda__, __pycuda_error_msg__
+from ...cuda.utils import __has_cupy__
 from ..utils import pipeline_step
 from .chunked import ChunkedPipeline
 
-if __has_pycuda__:
-    import pycuda.gpuarray as garray
-if not (__have_astra__):
-    ConebeamReconstructor = None
+if __has_cupy__:
+    import cupy
 
 
 class CudaChunkedPipeline(ChunkedPipeline):
@@ -45,7 +42,6 @@ class CudaChunkedPipeline(ChunkedPipeline):
     SinoNormalizationClass = CudaSinoNormalization
     SinoFilterClass = CudaSinoFilter
     FBPClass = Backprojector
-    ConebeamClass = ConebeamReconstructor
     HBPClass = HierarchicalBackprojector
     HistogramClass = CudaPartialHistogram
 
@@ -75,14 +71,18 @@ class CudaChunkedPipeline(ChunkedPipeline):
         # Decide when to transfer data to GPU. Normally it's right after reading the data,
         # But sometimes a part of the processing is done on CPU.
         self._when_to_transfer_radios_on_gpu = "read_data"
-        if self.flatfield is not None and self.flatfield.distortion_correction is not None:
-            self._when_to_transfer_radios_on_gpu = "flatfield"
+        if self.flatfield is not None:
+            use_flats_distortion = getattr(self.flatfield, "distortion_correction", None) is not None
+            use_pca_flats = self.processing_options["flatfield"]["method"].lower() == "pca"
+            if use_flats_distortion or use_pca_flats:
+                self._when_to_transfer_radios_on_gpu = "flatfield"
 
     def _init_cuda(self, cuda_options):
-        if not (__has_pycuda__):
-            raise ImportError(__pycuda_error_msg__)
+        if not (__has_cupy__):
+            raise ImportError("Could not import cupy. Cannot use CudaChunkedPipeline")
         cuda_options = cuda_options or {}
-        self.ctx = get_cuda_context(**cuda_options)
+        self._device_id = cuda_options.get("device_id", 0)
+        cupy.cuda.Device(self._device_id).use()
         self._d_radios = None
         self._d_sinos = None
         self._d_recs = None
@@ -93,7 +93,7 @@ class CudaChunkedPipeline(ChunkedPipeline):
         d_arr = getattr(self, d_name, None)
         if d_arr is None:
             self.logger.debug("Allocating %s: %s" % (name, str(shape)))
-            d_arr = garray.zeros(shape, dtype)  # pylint: disable=E0606
+            d_arr = cupy.zeros(shape, dtype)  # pylint: disable=E0606
             setattr(self, d_name, d_arr)
         return d_arr
 
@@ -124,13 +124,14 @@ class CudaChunkedPipeline(ChunkedPipeline):
         super()._reconstruct()
         if "reconstruction" not in self.processing_steps:
             return
-        if self.processing_options["reconstruction"]["method"] == "cone":
+        rec_method = self.processing_options["reconstruction"]["method"]
+        if rec_method == "cone":
             ((U, D), (L, R)) = self.margin
             U, D = U or None, -D or None
             # not sure why slicing can't be done before get()
             self.recs = self.recs.get()[U:D, ...]
-        elif self.processing_options["reconstruction"]["method"] == "mlem":
-            pass
+        elif rec_method == "mlem" and self.processing_options["reconstruction"]["implementation"] == "corrct":
+            pass  # already a numpy array
         else:
             self.recs = self.recs.get()
 
@@ -151,6 +152,6 @@ class CudaChunkedPipeline(ChunkedPipeline):
             data = self.radios
         if step_name not in self.datadump_manager.data_dump:
             return
-        if isinstance(data, garray.GPUArray):
+        if isinstance(data, cupy.ndarray):
             data = data.get()
         self.datadump_manager.dump_data_to_file(step_name, data=data)
