@@ -1,9 +1,10 @@
 import os
 import posixpath
 import numpy as np
+from .get_double_flatfield import get_double_flatfield
 from silx.io import get_data
 from silx.io.url import DataUrl
-from ...utils import copy_dict_items, compare_dicts
+from ...utils import copy_dict_items, compare_dicts, deprecation_warning
 from ...io.utils import hdf5_entry_exists, get_h5_value
 from ...io.reader import import_h5_to_dict
 from ...resources.utils import extract_parameters, get_values_from_file
@@ -11,6 +12,7 @@ from ...resources.nxflatfield import update_dataset_info_flats_darks
 from ...resources.utils import get_quantities_and_units
 from ..estimators import estimate_cor
 from ..processconfig import ProcessConfigBase
+from ..config_validators import convert_to_bool
 from .nabu_config import nabu_config, renamed_keys
 from .dataset_validator import FullFieldDatasetValidator
 from nxtomo.nxobject.nxdetector import ImageKey
@@ -32,6 +34,7 @@ class ProcessConfig(ProcessConfigBase):
 
     (2) update_dataset_info_with_user_config
       - Update flats/darks
+      - Double-flat-field
       - CoR  (value or estimation method) # no estimation yet
       - rotation angles
       - translations files
@@ -73,10 +76,11 @@ class ProcessConfig(ProcessConfigBase):
         Update the 'dataset_info' (DatasetAnalyzer class instance) data structure with options from user configuration.
         """
         self.logger.debug("Updating dataset information with user configuration")
-        if self.dataset_info.kind == "nx":
+        if self.dataset_info.kind == "nx" and self.nabu_config["preproc"]["flatfield"]:
             update_dataset_info_flats_darks(
                 self.dataset_info,
                 self.nabu_config["preproc"]["flatfield"],
+                loading_mode=self.nabu_config["preproc"]["flatfield_loading_mode"],
                 output_dir=self.nabu_config["output"]["location"],
                 darks_flats_dir=self.nabu_config["dataset"]["darks_flats_dir"],
             )
@@ -89,13 +93,41 @@ class ProcessConfig(ProcessConfigBase):
         self.subsampling_factor = subsampling_factor or 1
         self.subsampling_start = subsampling_start or 0
 
+        self._get_double_flatfield()
         self._update_dataset_with_user_overwrites()
         self._get_rotation_axis_position()
         self._update_rotation_angles()
         self._get_translation_file("reconstruction", "translation_movements_file", "translations")
         self._get_user_sino_normalization()
 
+    def _get_double_flatfield(self):
+        self._dff_file = None
+        dff_mode = self.nabu_config["preproc"]["double_flatfield"]
+        if not (dff_mode):
+            return
+        self._dff_file = get_double_flatfield(
+            self.dataset_info,
+            dff_mode,
+            output_dir=self.nabu_config["output"]["location"],
+            darks_flats_dir=self.nabu_config["dataset"]["darks_flats_dir"],
+            dff_options={
+                "dff_sigma": self.nabu_config["preproc"]["dff_sigma"],
+                "do_flatfield": (self.nabu_config["preproc"]["flatfield"] is not False),
+            },
+        )
+
     def _update_dataset_with_user_overwrites(self):
+        # Update info on frames flip
+        user_flip_lr = self.nabu_config["dataset"].get("flip_lr", "auto")
+        if user_flip_lr != "auto":
+            self.logger.warning(f"Resetting flip_lr to {user_flip_lr}")
+            self.dataset_info.flip_frame_lr = user_flip_lr
+        user_flip_ud = self.nabu_config["dataset"].get("flip_ud", "auto")
+        if user_flip_ud != "auto":
+            self.logger.warning(f"Resetting flip_ud to {user_flip_ud}")
+            self.dataset_info.flip_frame_ud = user_flip_ud
+
+        # Update some metadata: pixel_size, distance, energy
         user_overwrites = self.nabu_config["dataset"]["overwrite_metadata"].strip()
         if user_overwrites in ("", None):
             return
@@ -406,8 +438,10 @@ class ProcessConfig(ProcessConfigBase):
         # Flat-field
         #
         if nabu_config["preproc"]["flatfield"]:
+            ff_method = "pca" if nabu_config["preproc"]["flatfield"] == "pca" else "default"
             tasks.append("flatfield")
             options["flatfield"] = {
+                "method": ff_method,
                 #  Data reader handles binning/subsampling by itself,
                 # but FlatField needs "real" indices (after binning/subsampling)
                 "projs_indices": self.projs_indices(subsampling=False),
@@ -415,7 +449,7 @@ class ProcessConfig(ProcessConfigBase):
                 "do_flat_distortion": nabu_config["preproc"]["flat_distortion_correction_enabled"],
                 "flat_distortion_params": extract_parameters(nabu_config["preproc"]["flat_distortion_params"]),
             }
-            normalize_srcurrent = nabu_config["preproc"]["normalize_srcurrent"]
+            normalize_srcurrent = nabu_config["preproc"]["normalize_srcurrent"] and ff_method == "default"
             radios_srcurrent = None
             flats_srcurrent = None
             if normalize_srcurrent:
@@ -439,6 +473,7 @@ class ProcessConfig(ProcessConfigBase):
             if len(dataset_info.darks) > 1:
                 self.logger.warning("Cannot do flat-field with more than one reduced dark. Taking the first one.")
                 dataset_info.darks = dataset_info.darks[sorted(dataset_info.darks.keys())[0]]
+
         #
         # Spikes filter
         #
@@ -451,11 +486,19 @@ class ProcessConfig(ProcessConfigBase):
         #
         # Double flat field
         #
-        if nabu_config["preproc"]["double_flatfield_enabled"]:
+        # ---- COMPAT  ----
+        if convert_to_bool(nabu_config["preproc"].get("double_flatfield_enabled", False))[0]:
+            deprecation_warning(
+                "'double_flatfield_enabled' has been renamed to 'double_flatfield'. Please update your configuration file"
+            )
+            nabu_config["preproc"]["double_flatfield"] = True
+
+        # -------------
+        if nabu_config["preproc"]["double_flatfield"]:
             tasks.append("double_flatfield")
             options["double_flatfield"] = {
                 "sigma": nabu_config["preproc"]["dff_sigma"],
-                "processes_file": nabu_config["preproc"]["processes_file"],
+                "processes_file": self._dff_file or nabu_config["preproc"]["processes_file"],
                 "log_min_clip": nabu_config["preproc"]["log_min_clip"],
                 "log_max_clip": nabu_config["preproc"]["log_max_clip"],
             }
@@ -558,6 +601,7 @@ class ProcessConfig(ProcessConfigBase):
                 self.rec_params,
                 [
                     "method",
+                    "iterations",
                     "implementation",
                     "fbp_filter_type",
                     "fbp_filter_cutoff",
@@ -575,10 +619,16 @@ class ProcessConfig(ProcessConfigBase):
                     "sample_detector_dist",
                     "hbp_legs",
                     "hbp_reduction_steps",
+                    "crop_filtered_data",
                 ],
             )
             rec_options = options["reconstruction"]
             rec_options["rotation_axis_position"] = self.rotation_axis_position(binning=True)
+            if self.dataset_info.flip_frame_lr:
+                # flip 'rotation_axis_position' according to detector lr flip
+                rec_options["rotation_axis_position"] = (
+                    self.sino_shape(binning=True)[-1] - 1 - rec_options["rotation_axis_position"]
+                )
             rec_options["enable_halftomo"] = self.do_halftomo
             rec_options["axis_correction"] = dataset_info.axis_correction
             if dataset_info.axis_correction is not None:
@@ -593,8 +643,6 @@ class ProcessConfig(ProcessConfigBase):
                 voxel_size,
             )  # pix size is in microns in dataset_info
 
-            rec_options["iterations"] = nabu_config["reconstruction"]["iterations"]
-
             # x/y/z position information
             def get_mean_pos(position_array):
                 if position_array is None:
@@ -616,6 +664,8 @@ class ProcessConfig(ProcessConfigBase):
                 rec_options["position"] = mean_positions_xyz
             if rec_options["method"] == "cone" and rec_options["sample_detector_dist"] is None:
                 rec_options["sample_detector_dist"] = self.dataset_info.distance  # was checked to be not None earlier
+            if rec_options["method"].lower() == "mlem" and rec_options["implementation"] in [None, ""]:
+                rec_options["implementation"] = "nabu"
 
             # New key
             rec_options["cor_estimated_auto"] = isinstance(nabu_config["reconstruction"]["rotation_axis_position"], str)
