@@ -1,30 +1,163 @@
 from os import environ
 from os.path import join, isfile, basename, dirname
+from time import sleep
+from multiprocessing.pool import Pool, get_context
 from math import ceil
 import gc
-from psutil import virtual_memory
 from silx.io import get_data
 from silx.io.url import DataUrl
 from tomoscan.esrf.volume.singleframebase import VolumeSingleFrameBase
 from ... import version as nabu_version
-from ...utils import check_supported, subdivide_into_overlapping_segment
-from ...resources.logger import LoggerOrPrint
-from ...resources.utils import is_hdf5_extension
+from ...utils import check_supported, first_generator_item, subdivide_into_overlapping_segment, split_list
+from ...resources.logger import LoggerOrPrint, PrinterLogger, Logger
+from ...resources.utils import is_hdf5_extension, get_available_ram_GB
 from ...io.writer import merge_hdf5_files, NXProcessWriter
-from ...cuda.utils import collect_cuda_gpus, __has_pycuda__
 from ...preproc.phase import compute_paganin_margin
 from ...processing.histogram import PartialHistogram, add_last_bin, hist_as_2Darray
 from .chunked import ChunkedPipeline
 from .computations import estimate_max_chunk_size
-
-if __has_pycuda__:
-    from .chunked_cuda import CudaChunkedPipeline
+from ..tasks import TaskProcessor
 
 
 def variable_idxlen_sort(fname):
     return int(fname.split("_")[-1].split(".")[0])
 
 
+def _proc_collect_cuda_gpus(x):
+    from nabu.cuda.utils import collect_cuda_gpus
+
+    return collect_cuda_gpus()
+
+
+def _collect_cuda_gpus(safe=True):
+    if not safe:
+        return _proc_collect_cuda_gpus(None)
+    try:
+        ctx = get_context("spawn")
+        with Pool(1, context=ctx) as p:
+            v = p.map(_proc_collect_cuda_gpus, [1])[0]
+    except AssertionError:
+        # Can get AssertionError: daemonic processes are not allowed to have children
+        # if the calling code is already a subprocess
+        return _proc_collect_cuda_gpus(None)
+    return v
+
+
+class NannyPipeline:
+    """
+    A "nanny" pipeline that runs in a separate process
+    """
+
+    def __init__(
+        self, process_config, backend="numpy", pipeline_mode="chunked", logging_options=None, **pipeline_kwargs
+    ):
+        self.process_config = process_config
+        self.previous_task = None
+        self.pipeline = None
+        self.worker_id = 0  # placeholder
+        self._get_pipeline_class(backend, pipeline_mode)
+        self.pipeline_kwargs = pipeline_kwargs
+        self._create_logger(logging_options)
+
+    def _create_logger(self, logging_options):
+        if logging_options is None:
+            self.logger = PrinterLogger()
+            return
+        my_logging_options = logging_options.copy()
+        logname = logging_options.pop("logname", "nabu")
+        self.logger = Logger(logname, **my_logging_options)
+
+    def _get_pipeline_class(self, backend, pipeline_mode):
+        self.backend = backend
+        # Check were done before
+        if backend == "cuda":
+            # Don't initialize cuda before creating a new process (with fork) !
+            from .chunked_cuda import CudaChunkedPipeline
+
+            self._pipeline_cls = CudaChunkedPipeline
+        else:
+            self._pipeline_cls = ChunkedPipeline
+        self._pipeline_mode = pipeline_mode
+
+    def _get_chunk_length(self, task):
+        # normally not supported
+        if self._pipeline_mode == "helical":
+            (start_z, end_z) = task["sub_region"]
+            return end_z - start_z
+        #
+        (start_angle, end_angle), (start_z, end_z), _ = task["sub_region"]
+        if self._pipeline_mode == "chunked":
+            return end_z - start_z
+        else:
+            return end_angle - start_angle
+
+    def _instantiate_pipeline(self, task):
+        self.logger.set_prefix("  " * (self.worker_id) + f"[{self.worker_id}] ")
+        self.logger.debug("Creating a new pipeline object")
+        chunk_shape = tuple(s[1] - s[0] for s in task["sub_region"])
+        args = [self.process_config, chunk_shape]
+        pipeline_init_kwargs = self.pipeline_kwargs.copy()
+        if self.backend == "cuda":
+            pipeline_init_kwargs["cuda_options"] = {"device_id": self.worker_id}
+        pipeline_init_kwargs["use_grouped_mode"] = self._pipeline_mode == "grouped"
+        pipeline = self._pipeline_cls(*args, logger=self.logger, margin=task["margin"], **pipeline_init_kwargs)
+        self.pipeline = pipeline
+
+    def _instantiate_pipeline_if_necessary(self, current_task, other_task):
+        """
+        Instantiate a pipeline only if current_task has a different "delta z" than other_task
+        """
+        if self.pipeline is None:
+            self._instantiate_pipeline(current_task)
+            return
+        length_cur = self._get_chunk_length(current_task)
+        length_other = self._get_chunk_length(other_task)
+        if length_cur != length_other or current_task["margin"] != other_task["margin"]:
+            self.logger.debug("Destroying pipeline instance and releasing memory")
+            self._destroy_pipeline()
+            self._instantiate_pipeline(current_task)
+
+    def _destroy_pipeline(self):
+        self.pipeline = None
+
+        gc.collect()
+        # Not elegant, but for now the only way to release Cuda memory
+        import cupy
+
+        cupy.get_default_memory_pool().free_all_blocks()
+        cupy.get_default_pinned_memory_pool().free_all_blocks()
+        gc.collect()
+
+    def process_task(self, task):
+        """
+        'task' is in the form {"sub_region": ((a0, a1), (z0, z1), (x0, x1)), "margin": (m_v, m_h)}
+        """
+        prev_task = self.previous_task
+        self.logger.info("Processing sub-volume %s" % (str(task["sub_region"])))
+        self._instantiate_pipeline_if_necessary(task, prev_task)
+        self.pipeline.process_chunk(task["sub_region"], output_start_z=task.get("output_start_z", None))
+
+        if self._pipeline_mode == "grouped":
+            task_result = self.pipeline.datadump_manager.data_dump["sinogram"].fname
+        else:
+            task_result = self.pipeline.writer.fname if self.pipeline.writer is not None else None
+
+        task_result = {
+            "sub_region": task["sub_region"],
+            "result": task_result,
+            "histogram": None,
+            "data_dumps": {},
+        }
+        if self.pipeline.writer is not None and self.pipeline.writer.histogram:
+            task_result["histogram"] = self.pipeline.writer.histogram_writer.fname
+        if len(self.pipeline.datadump_manager.data_dump) > 0:
+            for step_name, writer in self.pipeline.datadump_manager.data_dump.items():
+                task_result["data_dumps"][step_name] = writer.fname
+
+        self.previous_task = task
+        return task_result
+
+
 class FullFieldReconstructor:
     """
     A reconstructor spawns and manages Pipeline objects, depending on the current chunk/group size.
@@ -61,7 +194,7 @@ class FullFieldReconstructor:
             Dictionary with cuda options passed to `nabu.cuda.processing.CudaProcessing`
 
 
-        Other parameters
+        Other Parameters
         -----------------
         Advanced options can be passed in the 'extra_options' dictionary. These can be:
 
@@ -87,7 +220,6 @@ class FullFieldReconstructor:
             self.process_config.nabu_config["output"]["file_format"]
         )
         self._histogram_merged = False
-        self.pipeline = None
         self._histogram_merged = False
 
     #
@@ -117,25 +249,40 @@ class FullFieldReconstructor:
         self._get_memory()
 
     def _get_memory(self):
-        vm = virtual_memory()
-        self.resources["mem_avail_GB"] = vm.available / 1e9
-        # Account for other memory constraints. There might be a better way
-        slurm_mem_constraint_MB = int(environ.get("SLURM_MEM_PER_NODE", 0))
-        if slurm_mem_constraint_MB > 0:
-            self.resources["mem_avail_GB"] = slurm_mem_constraint_MB / 1e3
-        #
+        self.resources["mem_avail_GB"] = get_available_ram_GB()
         self.cpu_mem = self.resources["mem_avail_GB"] * self.cpu_mem_fraction
 
     def _get_gpu(self):
-        avail_gpus = collect_cuda_gpus() or {}
+        resources_config = self.process_config.nabu_config.get("resources", {})
+
+        # Number of workers
+        self.n_workers = resources_config.get("workers", 1)
+
+        # GPUS / gpu id
+        avail_gpus = _collect_cuda_gpus() or {}
         self.resources["gpus"] = avail_gpus
         if len(avail_gpus) == 0:
+            self._has_cupy = False
             return
-        # pick first GPU by default. TODO: handle user's nabu_config["resources"]["gpu_id"]
-        self.resources["gpu_id"] = self._gpu_id = list(avail_gpus.keys())[0]
+        self._has_cupy = True
+
+        n_gpus = resources_config.get("gpus", 1)
+        gpu_ids = resources_config.get("gpu_id", [])
+        if gpu_ids != [] and len(gpu_ids) != n_gpus:
+            self.logger.warning(
+                "Found a discrepancy between 'gpus' and 'gpu_id' in config file. The parameter 'gpu_id' takes precedence."
+            )
+            n_gpus = len(gpu_ids)
+        if gpu_ids == []:
+            gpu_ids = list(range(n_gpus))  # list(avail_gpus.keys())[:n_gpus] ?
+
+        if len(gpu_ids) > len(avail_gpus):
+            self.logger.error(f"Requested {len(gpu_ids)} GPUs which is more than the {len(avail_gpus)} available GPUs")
+            gpu_ids = gpu_ids[: len(avail_gpus)]
+        self.resources["gpu_id"] = gpu_ids
 
     def _get_backend(self, backend, cuda_options):
-        self._pipeline_cls = ChunkedPipeline
+        # self._pipeline_cls = ChunkedPipeline
         check_supported(backend, self._available_backends, "backend")
         if backend == "cuda":
             self.cuda_options = cuda_options
@@ -144,11 +291,30 @@ class FullFieldReconstructor:
                 self.logger.error("Could not find any cuda device. Falling back on numpy/CPU backend.")
                 backend = "numpy"
             else:
-                self.gpu_mem = self.resources["gpus"][self._gpu_id]["memory_GB"] * self.gpu_mem_fraction
-        if backend == "cuda":
-            if not (__has_pycuda__):
-                raise RuntimeError("pycuda not avilable")
-            self._pipeline_cls = CudaChunkedPipeline  # pylint: disable=E0606
+                for gpu_id in self.resources["gpu_id"]:
+                    gpu_desc = self.resources["gpus"][gpu_id]
+                    gpu_mem_tot = gpu_desc["memory_total_GB"]
+                    gpu_mem_avail = gpu_desc["memory_avail_GB"]
+                    if gpu_mem_avail / gpu_mem_tot < 0.5:
+                        self.logger.warning(
+                            f"It looks like the target GPU number {gpu_id} ({gpu_desc['name']}) is in use (less than half memory available)"
+                        )
+                # Assume all GPU have the same amount of memory # FIXME
+                self.gpu_mem = gpu_mem_avail * self.gpu_mem_fraction
+        if backend == "cuda" and not (self._has_cupy):
+            raise RuntimeError("cupy is not avilable")
+
+        # In the special case where we use astra for cone-beam (FDK) reconstruction,
+        # astra can do the data split automatically better than nabu
+        # In this case, do all the preprocessing CPU, and only FDK on GPU
+        rec_config = self.process_config.nabu_config["reconstruction"]
+        if rec_config["method"] == "cone" and rec_config.get("implementation") == "astra":
+            self.logger.info(
+                "Using reconstruction method='cone' and implementation='astra'. All pre-processing will be done on CPU, and final reconstruction with astra"
+            )
+            backend = "numpy"
+        # ---
+
         self.backend = backend
 
     def _compute_max_chunk_size(self):
@@ -219,6 +385,7 @@ class FullFieldReconstructor:
             pipeline_part="sinos",
             step=10,
         )
+        self.chunk_size_sinos = self.cpu_max_chunk_size_sinos
         if self.backend == "cuda":
             self.gpu_max_chunk_size_sinos = estimate_max_chunk_size(
                 self.gpu_mem,
@@ -256,6 +423,9 @@ class FullFieldReconstructor:
         if (self.process_config.dataset_info.detector_tilt or 0) > 15:
             force_grouped_mode = True
             msg = "Radios rotation with a large angle needs to process full radios"
+        if self.process_config.processing_options.get("flatfield", {}).get("method", "default") == "pca":
+            force_grouped_mode = True
+            msg = "PCA-Flatfield normalization needs to process full radios"
         if self.process_config.resume_from_step == "sinogram" and force_grouped_mode:
             self.logger.warning("Cannot use grouped-radios processing when resuming from sinogram")
             force_grouped_mode = False
@@ -276,7 +446,9 @@ class FullFieldReconstructor:
     #
 
     def _compute_margin(self):
-        user_margin = self.extra_options.get("margin", None)
+        user_margin = self.extra_options.get("margin", None) or self.process_config.nabu_config["pipeline"].get(
+            "processing_margin", None
+        )
         if self.process_config.resume_from_step == "sinogram":
             self.logger.debug("Margin not needed when resuming from sinogram")
             margin_v, margin_h = 0, 0
@@ -289,7 +461,6 @@ class FullFieldReconstructor:
             translations_margin = self._compute_translations_margin()
             cone_margin = self._compute_cone_overlap()
             rot_margin = self._compute_rotation_margin()
-            # TODO radios rotation/movements
             margin_v = max(unsharp_margin[0], phase_margin[0], translations_margin[0], cone_margin[0], rot_margin[0])
             margin_h = max(unsharp_margin[1], phase_margin[1], translations_margin[1], cone_margin[1], rot_margin[1])
             if margin_v > 0:
@@ -307,7 +478,7 @@ class FullFieldReconstructor:
         sigma = opts["unsharp_sigma"]
         # nabu uses cutoff = 4
         cutoff = 4
-        gaussian_kernel_size = int(ceil(2 * cutoff * sigma + 1))
+        gaussian_kernel_size = ceil(2 * cutoff * sigma + 1)
         self.logger.debug("Unsharp mask margin: %d pixels" % gaussian_kernel_size)
         return (gaussian_kernel_size, gaussian_kernel_size)
 
@@ -355,10 +526,11 @@ class FullFieldReconstructor:
         d2 = rec_cfg["sample_detector_dist"]
         n_z, _ = self.process_config.radio_shape(binning=True)
 
-        delta_z = self.process_config.rec_delta_z  # accounts_for_binning
-        overlap = ceil(delta_z * d2 / (d1 + d2))  # sqrt(2) missing ?
+        # delta_z = self.process_config.rec_delta_z  # accounts_for_binning
+        # overlap = ceil(delta_z * d2 / (d1 + d2))  # sqrt(2) missing ?
 
         max_overlap = ceil(n_z * d2 / (d1 + d2))  # sqrt(2) missing ?
+        max_overlap = max(max_overlap, 10)  # use at least 10 pixels
 
         return (max_overlap, 0)
 
@@ -434,7 +606,12 @@ class FullFieldReconstructor:
             z_min = self.z_min + start_z
             z_max = min(self.z_min + end_z, self.n_z)
             sub_region = ((0, self.n_angles), (z_min, z_max), (0, self.chunk_shape[-1]))
-            self._sino_tasks.append({"sub_region": sub_region, "margin": None})
+            task = {"sub_region": sub_region, "margin": None}
+            if self.process_config.dataset_info.flip_frame_ud:
+                task["output_start_z"] = (
+                    self.z_max - 1 - task["sub_region"][1][0] - self._get_task_vertical_margin(task)[0]
+                )
+            self._sino_tasks.append(task)
 
     def _build_tasks_chunked(self):
         margin_v = self._margin_v
@@ -478,7 +655,7 @@ class FullFieldReconstructor:
                 {
                     "sub_region": (
                         (0, self.n_angles),
-                        (curr_z_min - margin_up, curr_z_max + margin_down),
+                        (int(curr_z_min - margin_up), int(curr_z_max + margin_down)),
                         (0, self.chunk_shape[-1]),
                     ),
                     "margin": ((margin_up, margin_down), (0, 0)),
@@ -496,6 +673,24 @@ class FullFieldReconstructor:
         else:
             self._ensure_good_chunk_size_and_margin()
             self._build_tasks_chunked()
+        if self.process_config.dataset_info.flip_frame_ud:
+            for task in self.tasks:
+                # when applying up-down flip we will have a decreasing z index. Indices will be in [start_z, ..., end_z[
+                task["output_start_z"] = (
+                    self.z_max - 1 - task["sub_region"][1][0] - self._get_task_vertical_margin(task)[0]
+                )
+
+    def _get_task_vertical_margin(self, task):
+        margin = task.get("margin", None)
+        if margin is None:
+            return (0, 0)
+        if self._pipeline_mode == "grouped":
+            # margin is in the form (up, down)
+            # TODO this is not consistent
+            return margin
+        else:
+            # margin is in the form ((up, down), (left, right))
+            return margin[0]
 
     def _print_tasks_chunked(self):
         for task in self.tasks:
@@ -535,84 +730,95 @@ class FullFieldReconstructor:
     # Reconstruction
     #
 
-    def _instantiate_pipeline(self, task):
-        self.logger.debug("Creating a new pipeline object")
-        chunk_shape = tuple(s[1] - s[0] for s in task["sub_region"])
-        args = [self.process_config, chunk_shape]
-        kwargs = {}
-        if self.backend == "cuda":
-            kwargs["cuda_options"] = self.cuda_options
-        kwargs["use_grouped_mode"] = self._pipeline_mode == "grouped"
-        pipeline = self._pipeline_cls(*args, logger=self.logger, margin=task["margin"], **kwargs)
-        self.pipeline = pipeline
-
-    def _instantiate_pipeline_if_necessary(self, current_task, other_task):
-        """
-        Instantiate a pipeline only if current_task has a different "delta z" than other_task
-        """
-        if self.pipeline is None:
-            self._instantiate_pipeline(current_task)
-            return
-        length_cur = self._get_chunk_length(current_task)
-        length_other = self._get_chunk_length(other_task)
-        if length_cur != length_other:
-            self.logger.debug("Destroying pipeline instance and releasing memory")
-            self._destroy_pipeline()
-            self._instantiate_pipeline(current_task)
-
-    def _destroy_pipeline(self):
-        self.pipeline = None
-        # Not elegant, but for now the only way to release Cuda memory
-        gc.collect()
-
     def _reconstruct_chunked(self, tasks=None):
         self.results = {}
         self._histograms = {}
         self._data_dumps = {}
         tasks = tasks or self.tasks
-        prev_task = tasks[0]
-        for task in tasks:
-            self.logger.info("Processing sub-volume %s" % (str(task["sub_region"])))
-            self._instantiate_pipeline_if_necessary(task, prev_task)
-            self.pipeline.process_chunk(task["sub_region"])
-            task_key = self.pipeline.sub_region
-            task_result = self.pipeline.writer.fname
-            self.results[task_key] = task_result
-            if self.pipeline.writer.histogram:
-                self._histograms[task_key] = self.pipeline.writer.histogram_writer.fname
-            if len(self.pipeline.datadump_manager.data_dump) > 0:
-                self._data_dumps[task_key] = {}
-                for step_name, writer in self.pipeline.datadump_manager.data_dump.items():
-                    self._data_dumps[task_key][step_name] = writer.fname
-            prev_task = task
+
+        pipeline_kwargs = {}
+        if self.backend == "cuda":
+            pipeline_kwargs["cuda_options"] = self.cuda_options
+
+        # Handle frames vertical flip
+        if self.process_config.dataset_info.flip_frame_ud:
+            self.logger.warning("Using 'flip_ud' setting. Reconstructed volume will be flipped upside down!")
+            pipeline_kwargs["extra_options"] = {"write_in_reverse_order": True}
+
+        # subdivide tasks into workers
+        workers_tasks = split_list(tasks, self.n_workers)
+
+        _log_options = None
+        if isinstance(self.logger, Logger):
+            _log_options = {"level": self.logger.level, "logfile": self.logger.logfile}
+
+        self.tasks_processor = TaskProcessor(
+            NannyPipeline,
+            n_processes=self.n_workers,
+            worker_init_args=(self.process_config,),
+            worker_init_kwargs={
+                "backend": self.backend,
+                "pipeline_mode": self._pipeline_mode,
+                "logging_options": _log_options,
+            }
+            | pipeline_kwargs,
+        )
+
+        def _result_callback(future):
+            result = future.result()
+            if "error" in result:
+                return
+            task_key = result["sub_region"]
+            self.results[task_key] = result["result"]
+            if result["histogram"] is not None:
+                self._histograms[task_key] = result["histogram"]
+            if len(result["data_dumps"]) > 0:
+                self._data_dumps[task_key] = result["data_dumps"]
+
+        self.tasks_processor.start_workers()
+        try:
+            # Submit tasks
+            future_results = {}
+            for worker_id in range(self.n_workers):
+                current_worker_tasks = workers_tasks[worker_id]
+                for task_id, task in enumerate(current_worker_tasks):
+                    fut = self.tasks_processor.submit(worker_id, task)
+                    fut.add_done_callback(_result_callback)
+                    future_results[task_id] = fut
+            # Wait for all results. See also the above callback '_result_callback'
+            while not (all(f.done() for f in future_results.values())):
+                sleep(1)
+
+            # Handle errors that could happen in workers sub-processes.
+            # Note that future.result() is no longer blocking because of the above wait.
+            # We raise a RuntimeError here, because it cannot be raised within the callback.
+            for task_id, future in future_results.items():
+                if "error" in future.result():
+                    msg = f"Something went wrong with task {task_id} ! Aborting reconstruction"
+                    self.logger.fatal(msg)
+                    raise RuntimeError(msg)
+
+        finally:
+            # Terminate workers
+            self.tasks_processor.stop_workers()
 
     def _reconstruct_grouped(self):
         self.results = {}
-        # self._histograms = {}
         self._data_dumps = {}
-        prev_task = self.tasks[0]
 
         # Stage 1: radios processing
         self._modify_processconfig_stage_1()
-        for task in self.tasks:
-            self.logger.info("Processing sub-volume %s" % (str(task["sub_region"])))
-            self._instantiate_pipeline_if_necessary(task, prev_task)
-            self.pipeline.process_chunk(task["sub_region"])
-            task_key = self.pipeline.sub_region
-            task_result = self.pipeline.datadump_manager.data_dump["sinogram"].fname
-            self.results[task_key] = task_result
-            if len(self.pipeline.datadump_manager.data_dump) > 0:
-                self._data_dumps[task_key] = {}
-                for step_name, writer in self.pipeline.datadump_manager.data_dump.items():
-                    self._data_dumps[task_key][step_name] = writer.fname
-            prev_task = task
 
+        self._reconstruct_chunked()
         self.merge_data_dumps(axis=0)
-        self._destroy_pipeline()
+
+        # self._destroy_pipeline()
         self.logger.info("End of first stage of processing. Will now process sinograms saved on disk")
         self._undo_modify_processconfig_stage_1()
 
         # Stage 2: sinograms processing and reconstruction
+        if "reconstruction" not in self.process_config.processing_steps:
+            return
         self._modify_processconfig_stage_2()
         self._pipeline_mode = "chunked"
         self._reconstruct_chunked(tasks=self._sino_tasks)
@@ -633,6 +839,8 @@ class FullFieldReconstructor:
         out_cfg = self.process_config.nabu_config["output"]
         if files is None:
             files = list(self.results.values())
+        if all(f is None for f in files):
+            return []
         try:
             files.sort(key=variable_idxlen_sort)
         except:
@@ -685,7 +893,7 @@ class FullFieldReconstructor:
         # Prevent issue when out_dir is empty, which happens only if dataset/location is a relative path.
         # TODO this should be prevented earlier
         if out_dir is None or len(out_dir.strip()) == 0:
-            out_dir = dirname(dirname(self.results[list(self.results.keys())[0]]))
+            out_dir = dirname(dirname(self.results[first_generator_item(self.results.keys())]))
         #
         if output_file is None:
             output_file = join(out_dir, prefix + out_cfg["file_prefix"]) + ".hdf5"
@@ -766,7 +974,8 @@ class FullFieldReconstructor:
         out_cfg = self.process_config.nabu_config["output"]
         if output_file is None:
             output_file = (
-                join(dirname(list(self._histograms.values())[0]), out_cfg["file_prefix"] + "_histogram") + ".hdf5"
+                join(dirname(first_generator_item(self._histograms.values())), out_cfg["file_prefix"] + "_histogram")
+                + ".hdf5"
             )
         local_files = self.get_relative_files(files=list(self._histograms.values()))
         #
@@ -818,7 +1027,7 @@ class FullFieldReconstructor:
     def merge_data_dumps(self, axis=1):
         # Collect in a dict where keys are step names (instead of task keys)
         dumps = {}
-        for task_key, data_dumps in self._data_dumps.items():
+        for task_key, data_dumps in self._data_dumps.items():  # noqa: PERF102
             for step_name, fname in data_dumps.items():
                 fname = join(basename(dirname(fname)), basename(fname))
                 if step_name not in dumps:
@@ -858,6 +1067,8 @@ class FullFieldReconstructor:
           - Merge histograms (if output format is not HDF5)
         """
         self.merge_data_dumps()
+        if "reconstruction" not in self.process_config.processing_steps:
+            return
         if self._reconstruction_output_format_is_hdf5:
             self.merge_hdf5_reconstructions()
         else:
