@@ -1,4 +1,4 @@
-from ..fullfield.dataset_validator import *
+from ..fullfield.dataset_validator import FullFieldDatasetValidator
 from ...utils import copy_dict_items
 
 
@@ -10,10 +10,9 @@ class HelicalDatasetValidator(FullFieldDatasetValidator):
 
     def _check_slice_indices(self):
         """Slice indices can be far beyond what fullfield pipeline accepts, no check here, but
-        Nabu expects that rec_region is initialised here"""
+        Nabu expects that rec_region is initialised here
+        """
 
         what = ["start_x", "end_x", "start_y", "end_y", "start_z", "end_z"]
 
         self.rec_region = copy_dict_items(self.rec_params, what)
-
-        return
