@@ -1,6 +1,6 @@
-from ...reconstruction.fbp import *
+import numpy as np
+from ...reconstruction.fbp import Backprojector
 from .filtering import HelicalSinoFilter
-from ...utils import convert_index
 
 
 class BackprojectorHelical(Backprojector):
@@ -9,12 +9,12 @@ class BackprojectorHelical(Backprojector):
 
          * the backprojection is decoupled from the filtering. This allows, in the pipeline,
            for doing first a filtering using backprojector_object.sino_filter subobject,
-           then calling backprojector_object.backprojection only after reweigthing the result of
+           then calling backprojector_object.backprojection only after reweighting the result of
            the filter_sino method of sino_filter subobject.
-         * the angles can be resetted on the fly, and the class can work with a variable number of projection.
+         * the angles can be reset on the fly, and the class can work with a variable number of projection.
            As a matter of fact, with the helical_chunked_regridded.py pipeline, the reconstruction is done
            each time with the same set of angles, this because of the regridding mechanism. The feature
-           might return useful in the future if alternative approachs are developed again.
+           might return useful in the future if alternative approaches are developed again.
          *
     """
 
@@ -25,9 +25,9 @@ class BackprojectorHelical(Backprojector):
 
     def set_custom_angles_and_axis_corrections(self, angles_rad, x_per_proj):
         """To arbitrarily change angles
-        Parameters
-        ==========
 
+        Parameters
+        ----------
         angles_rad: array of floats
            one angle per each projection in radians
 
