@@ -68,26 +68,26 @@ class HelicalSinoFilter(CudaSinoFilter):
             "padding",
             filename=get_cuda_srcfile("helical_padding.cu"),
             signature="PPfiiiii",
-            options=[str("-DMIRROR_EDGES")],
+            options=["-DMIRROR_EDGES"],
         )
         self._pad_mirror_constant_kernel = self.cuda.kernel(
             "padding",
             filename=get_cuda_srcfile("helical_padding.cu"),
             signature="PPfiiiiiff",
-            options=[str("-DMIRROR_CONSTANT")],
+            options=["-DMIRROR_CONSTANT"],
         )
 
         self._pad_mirror_edges_variable_rot_pos_kernel = self.cuda.kernel(
             "padding",
             filename=get_cuda_srcfile("helical_padding.cu"),
             signature="PPPiiiii",
-            options=[str("-DMIRROR_EDGES_VARIABLE_ROT_POS")],
+            options=["-DMIRROR_EDGES_VARIABLE_ROT_POS"],
         )
         self._pad_mirror_constant_variable_rot_pos_kernel = self.cuda.kernel(
             "padding",
             filename=get_cuda_srcfile("helical_padding.cu"),
             signature="PPPiiiiiff",
-            options=[str("-DMIRROR_CONSTANT_VARIABLE_ROT_POS")],
+            options=["-DMIRROR_CONSTANT_VARIABLE_ROT_POS"],
         )
 
         self.d_mirror_indexes = self.cuda.allocate_array(
@@ -103,7 +103,7 @@ class HelicalSinoFilter(CudaSinoFilter):
         self._pad_grid = tuple([updiv(n, p) for n, p in zip(self.sino_padded_shape[::-1], self._pad_block)])
 
     def _pad_sino(self, sino, mirror_indexes=None, rot_center=None):
-        """redefined here to adapt the memory copy to the lenght of the sino argument
+        """redefined here to adapt the memory copy to the length of the sino argument
         which, in the general helical case may be varying
         """
         if mirror_indexes is None:
@@ -179,6 +179,7 @@ class HelicalSinoFilter(CudaSinoFilter):
         """
         Perform the sinogram siltering.
         redefined here to use also mirror data
+
         Parameters
         ----------
         sino: numpy.ndarray or pycuda.gpuarray.GPUArray
