@@ -26,13 +26,12 @@ class GriddedAccumulator:
         flats_srcurrent=None,
     ):
         """
-        This class creates, for a selected volume slab, a standard set of radios from an helical dataset.
+        This class creates, for a selected volume slab, a standard set of radios from a helical dataset.
 
         Parameters
-        ==========
-
+        ----------
         gridded_radios : 3D np.array
-           this is the stack of new radios which will be resynthetised, by this class,
+           this is the stack of new radios which will be resynthesized, by this class,
            for a selected slab.
            The object is initialised with this array, and this array will accumulate, during subsequent
            calls to  method extract_preprocess_with_flats, the sum of the transformed contributions
@@ -52,7 +51,7 @@ class GriddedAccumulator:
            During the calls to extract_preprocess_with_flats methods,
            the  radios will collect the transformed data for the angles given by diagnostic_searched_angles_rad_clipped and redundancy
         diagnostic_weights: 3d np.array  a stack composed of two radios
-           Same shape as diagnostic_radios. The weigths for diagnostic radios ( will be zero on pixel
+           Same shape as diagnostic_radios. The weights for diagnostic radios ( will be zero on pixel
            where no data is available, or where the weight is null)
         diagnostic_angles : 1D np.array
            Must have shape==(2*len(diagnostic_searched_angles_rad_clipped),). The entries will be filled with the angles at which the contributions
@@ -97,7 +96,7 @@ class GriddedAccumulator:
         self.flats_srcurrent = flats_srcurrent
 
         self.flat_indexes = flat_indexes
-        self.flat_indexes_reverse_map = dict(
+        self.flat_indexes_reverse_map = dict(  # noqa: C404
             [(global_index, local_index) for (local_index, global_index) in enumerate(flat_indexes)]
         )
         self.flats = flats
@@ -121,16 +120,16 @@ class GriddedAccumulator:
         the accumulators are ready.
 
         Parameters
-        ==========
+        ----------
         subchunk_slice: an object of the python class "slice"
           this slice slices the angular domain which corresponds to the useful
-          projections  which are useful for the chunk, and whose informations
+          projections  which are useful for the chunk, and whose information
           are contained in the companion argument "chunk_info"
           Such slicing correspond to the angular subset, for which we are providing
           data_raw
         subchunk_file_indexes: a sequence of integers.
            they correspond to the projection numbers from which the data in data_raw are coming.
-           They are used to interpolate between falt fields
+           They are used to interpolate between flat fields
         chunk_info: an object returned by the get_chunk_info of the SpanStrategy class
                   this object must have the following members, which relates to the wanted chunk
 
@@ -152,15 +151,15 @@ class GriddedAccumulator:
             Without the phase margin, when the phase margin is zero, hey would correspond exactly to
             the start and end, vertically, of the reconstructed slices.
         dtasrc_start_end: a pair of integers
-            This number are relative to the detector ( they are detector line indexes).
-            They indicated, vertically, the detector portion the data_raw data correspond to
+            These numbers are relative to the detector ( they are detector line indexes).
+            They indicate, vertically, the detector portion the data_raw data correspond to
         data_raw: np.array 3D
             the data which correspond to a limited detector stripe and a limited angular subset
         radios_angular_range_slicing:
             my_subsampled_indexes  is important in order to compare the
             radios positions with respect to the flat position, and these position
             are given as the sequential acquisition number which counts everything ( flats, darks, radios )
-            Insteqd, in order to access array which spans only the radios, we need to have an idea of where we are.
+            Instead, in order to access array which spans only the radios, we need to have an idea of where we are.
             this is provided by radios_angular_range_slicing which addresses the radios domain
 
         """
@@ -278,7 +277,7 @@ class GriddedAccumulator:
                 i_diag_list = [(i0 - 1) // 2, (i0 - 1) // 2 + len(self.diagnostic_searched_angles_rad_clipped)]
                 for i_redundancy, i_diag in enumerate(i_diag_list):
                     # print("IRED ", i_redundancy)
-                    if i_redundancy:
+                    if i_redundancy:  # noqa: SIM102
                         # to avoid, in z_stages with >360 range for one single stage, to fill the second items which should instead be filled by another stage.
                         if abs(original_zpix_transl - self.diagnostic_zpix_transl[i_diag_list[0]]) < 2.0:
                             # print( " >>>>>> stesso z" , i_redundancy )
@@ -306,7 +305,7 @@ class GriddedAccumulator:
                         self.diagnostic_radios[i_diag] += data_token * factor
                         self.diagnostic_weights[i_diag] += weight * factor
                         break
-                    else:
+                    else:  # noqa: RET508
                         pass
 
     class _ReframingInfos:
@@ -483,8 +482,8 @@ def overlap_logic(subr_start_z, subr_end_z, dtasrc_start_z, dtasrc_end_z):
 
 def padding_logic(subr_start_z, subr_end_z, dtasrc_start_z, dtasrc_end_z):
     """.......... and the missing ranges which possibly could be obtained by extension padding"""
-    t_h = subr_end_z - subr_start_z
-    s_h = dtasrc_end_z - dtasrc_start_z
+    # t_h = subr_end_z - subr_start_z
+    # s_h = dtasrc_end_z - dtasrc_start_z
 
     if dtasrc_start_z <= subr_start_z:
         target_lower_padding = None
@@ -501,20 +500,20 @@ def padding_logic(subr_start_z, subr_end_z, dtasrc_start_z, dtasrc_end_z):
 
 def get_reconstruction_space(span_info, min_scanwise_z, end_scanwise_z, phase_margin_pix):
     """Utility function, so far used only by the unit test, which, given the span_info object, creates the auxiliary collection arrays
-    and initialises the  my_z_min, my_z_end variable keeping into account the scan direction
+    and initialises the my_z_min, my_z_end variable taking into account the scan direction
     and the min_scanwise_z, end_scanwise_z input arguments
-    Parameters
-    ==========
 
+    Parameters
+    ----------
     span_info: SpanStrategy
 
     min_scanwise_z: int
-          non negative number, where zero indicates the first feaseable slice doable scanwise.
+          non negative number, where zero indicates the first feasible slice doable scanwise.
           Indicates the first (scanwise) requested slice to be reconstructed
 
     end_scanwise_z: int
-          non negative number, where zero indicates the first feaseable slice doable scanwise.
-          Indicates the end (scanwise) slice which delimity the to be reconstructed requested slab.
+          non negative number, where zero indicates the first feasible slice doable scanwise.
+          Indicates the end (scanwise) slice which delimits the to be reconstructed requested slab.
 
     """
 
@@ -533,7 +532,7 @@ def get_reconstruction_space(span_info, min_scanwise_z, end_scanwise_z, phase_ma
     # regridded dataset, estimating a meaningul angular step representative
     # of the raw data
     my_angle_step = abs(np.diff(span_info.projection_angles_deg).mean())
-    n_gridded_angles = int(round(360.0 / my_angle_step))
+    n_gridded_angles = round(360.0 / my_angle_step)
 
     radios_h = phase_margin_pix + (my_z_end - my_z_min) + phase_margin_pix
 
