@@ -3,6 +3,8 @@ from math import ceil
 from time import time
 import numpy as np
 import copy
+
+from nabu.utils import first_generator_item
 from ...resources.logger import LoggerOrPrint
 from ...io.writer import merge_hdf5_files
 from ...cuda.utils import collect_cuda_gpus
@@ -18,7 +20,7 @@ except:
 
 from .helical_chunked_regridded_cuda import CudaHelicalChunkedRegriddedPipeline
 
-from ..fullfield.reconstruction import collect_cuda_gpus, FullFieldReconstructor
+from ..fullfield.reconstruction import FullFieldReconstructor
 
 avail_gpus = collect_cuda_gpus() or {}
 
@@ -53,7 +55,8 @@ class HelicalReconstructorRegridded:
             Dictionary with advanced options. Please see 'Other parameters' below
         cuda_options: dict, optional
             Dictionary with cuda options passed to `nabu.cuda.processing.CudaProcessing`
-        Other parameters
+
+        Other Parameters
         -----------------
         Advanced options can be passed in the 'extra_options' dictionary. These can be:
 
@@ -165,8 +168,8 @@ class HelicalReconstructorRegridded:
 
             # the meaming of z_min and z_max is: position in slices units from the
             # first available slice and in the direction of the scan
-            self.z_min = int(round(z_start * (0 - z_fract_min) + z_max * z_fract_min))
-            self.z_max = int(round(z_start * (0 - z_fract_max) + z_max * z_fract_max)) + 1
+            self.z_min = round(z_start * (0 - z_fract_min) + z_max * z_fract_min)
+            self.z_max = round(z_start * (0 - z_fract_max) + z_max * z_fract_max) + 1
 
     def _compute_translations_margin(self):
         return 0, 0
@@ -227,7 +230,7 @@ class HelicalReconstructorRegridded:
     # cannot use the estimate_chunk_size from computations.py beacause it has the  estimate_required_memory hard-coded
     def estimate_chunk_size(self, available_memory_GB, process_config, chunk_step=1, user_max_chunk_size=None):
         """
-        Estimate the maximum chunk size given an avaiable amount of memory.
+        Estimate the maximum chunk size given an available amount of memory.
 
         Parameters
         -----------
@@ -242,9 +245,9 @@ class HelicalReconstructorRegridded:
             "reconstruction" in process_config.processing_steps
             and process_config.processing_options["reconstruction"]["enable_halftomo"]
         ):
-            radios_and_sinos = True
+            radios_and_sinos = True  # noqa: F841
 
-        max_dz = process_config.dataset_info.radio_dims[1]
+        # max_dz = process_config.dataset_info.radio_dims[1]
         chunk_size = chunk_step
         last_good_chunk_size = chunk_size
         while True:
@@ -431,7 +434,7 @@ class HelicalReconstructorRegridded:
         angles_deg = np.rad2deg(angles_rad)
 
         redundancy_angle_deg = self.process_config.nabu_config["reconstruction"]["redundancy_angle_deg"]
-        do_helical_half_tomo = self.process_config.nabu_config["reconstruction"]["helical_halftomo"]
+        # do_helical_half_tomo = self.process_config.nabu_config["reconstruction"]["helical_halftomo"]
 
         self.logger.info("Creating SpanStrategy object for helical ")
         t0 = time()
@@ -460,7 +463,7 @@ class HelicalReconstructorRegridded:
         self.logger.debug("Creating a new pipeline object")
         args = [self.process_config, task["sub_region"]]
 
-        dz = self._get_delta_z(task)
+        # dz = self._get_delta_z(task)
 
         pipeline = self._pipeline_cls(
             *args,
@@ -542,7 +545,7 @@ class HelicalReconstructorRegridded:
         # Prevent issue when out_dir is empty, which happens only if dataset/location is a relative path.
         # TODO this should be prevented earlier
         if out_dir is None or len(out_dir.strip()) == 0:
-            out_dir = dirname(dirname(self.results[list(self.results.keys())[0]]))
+            out_dir = dirname(dirname(self.results[first_generator_item(self.results.keys())]))
         #
         if output_file is None:
             output_file = join(out_dir, prefix + out_cfg["file_prefix"]) + ".hdf5"
