@@ -5,17 +5,16 @@ logger = LoggerOrPrint(None)
 
 
 def find_mirror_indexes(angles_deg, tolerance_factor=1.0):
-    """return a list of indexes where the ith elememnt
+    """return a list of indexes where the ith element
     contains the index of the angles_deg array element which has the value the closest
     to angles_deg[i] + 180. It is used for padding in halftomo.
 
-    Parameters:
+    Parameters
     -----------
-
     angles_deg: a nd.array of floats
 
     tolerance: float
-       if the mirror positions are not within a distance less than tolerance fro;
+       if the mirror positions are not within a distance less than tolerance from
        the ideal position a warning is raised
 
     """
