@@ -1,3 +1,4 @@
+# ruff: noqa
 from ..fullfield.nabu_config import *
 import copy
 
@@ -42,7 +43,7 @@ nabu_config["preproc"]["processes_file"] = {
     "validator": optional_file_location_validator,
     "type": "required",
 }
-nabu_config["preproc"]["double_flatfield_enabled"]["default"] = 1
+nabu_config["preproc"]["double_flatfield"]["default"] = 1
 
 
 nabu_config["reconstruction"].update(
