@@ -1,3 +1,4 @@
+# ruff: noqa
 import math
 import numpy as np
 from ...resources.logger import LoggerOrPrint
@@ -57,8 +58,8 @@ class SpanStrategy:
         pixel_size_mm:  float, the pixel size in millimiters
            this value is used to give results in units of " millimeters above the sample stage"
            Althougth internally all is calculated in pixel units, it is useful to incorporate
-           such information in the spanstrategy object which will then be able to  setup
-           reconstruction informations according to several request formats: be they in
+           such information in the SpanStrategy object which will then be able to set up
+           reconstruction information according to several request formats: be they in
            sequential number of reconstructible slices, or millimeters above the stage.
         angular_tolerance_steps: float, defaults to zero
              the angular tolerance, an angular width expressed in units of an angular step, which is tolerated in
@@ -158,7 +159,7 @@ class SpanStrategy:
 
     def get_doable_span(self):
         """return an object with two properties:
-        view_heights_minmax: containining minimum and maximum doable height ( detector reference at iproj=0)
+        view_heights_minmax: containing minimum and maximum doable height ( detector reference at iproj=0)
         z_pix_minmax       : containing minimum and maximum heights above the roto-translational sample stage
         """
         vertical_profile = self.sunshine_image.sum(axis=1)
@@ -216,7 +217,7 @@ class SpanStrategy:
     def get_chunk_info(self, span_v_absolute):
         """
 
-        This method returns an object containing all the informations that are needed to reconstruct
+        This method returns an object containing all the information that are needed to reconstruct
         the corresponding chunk
 
             angle_index_span: a pair of integers indicating the start and the end of useful angles
