@@ -168,18 +168,7 @@ class ProcessConfigBase:
 
     def _get_tilt(self):
         tilt = self.nabu_config["preproc"]["tilt_correction"]
-        user_rot_projs = self.nabu_config["preproc"]["rotate_projections"]
-        if user_rot_projs is not None and tilt is not None:
-            msg = "=" * 80 + "\n"
-            msg += (
-                "Both 'detector_tilt' and 'rotate_projections' options were provided. The option 'rotate_projections' will take precedence. This means that the projections will be rotated by %f degrees and the option 'detector_tilt' will be ignored."
-                % user_rot_projs
-            )
-            msg += "\n" + "=" * 80
-            self.logger.warning(msg)
-            tilt = user_rot_projs
-        #
-        if isinstance(tilt, str):  # auto-tilt
+        if isinstance(tilt, str):  # auto-tilt...
             self.tilt_estimator = DetectorTiltEstimator(
                 self.dataset_info,
                 do_flatfield=self.nabu_config["preproc"]["flatfield"],
