@@ -67,7 +67,7 @@ class GeoPars:
         else:
             self.M_vh = np.array([1, 1])
 
-        self.logger.debug("Magnification : h ({}) ;  v ({}) ".format(self.M_vh[1], self.M_vh[0]))
+        self.logger.debug(f"Magnification : h ({self.M_vh[1]}) ;  v ({self.M_vh[0]}) ")
 
         self.length_scale = length_scale
         self.wavelength = wavelength
@@ -80,17 +80,17 @@ class GeoPars:
 
         which_unit = int(np.sum(np.array([self.pix_size_rec > small for small in [1.0e-6, 1.0e-7]]).astype(np.int32)))
         self.pixelsize_string = [
-            "{:.1f} nm".format(self.pix_size_rec * 1e9),
-            "{:.3f} um".format(self.pix_size_rec * 1e6),
-            "{:.1f} um".format(self.pix_size_rec * 1e6),
+            f"{self.pix_size_rec * 1e9:.1f} nm",
+            f"{self.pix_size_rec * 1e6:.3f} um",
+            f"{self.pix_size_rec * 1e6:.1f} um",
         ][which_unit]
 
         if self.magnification:
             self.logger.debug(
-                "All images are resampled to smallest pixelsize: {}".format(self.pixelsize_string),
+                f"All images are resampled to smallest pixelsize: {self.pixelsize_string}",
             )
         else:
-            self.logger.debug("Pixelsize images:  {}".format(self.pixelsize_string))
+            self.logger.debug(f"Pixelsize images:  {self.pixelsize_string}")
 
 
 class CTFPhaseRetrieval:
@@ -141,9 +141,9 @@ class CTFPhaseRetrieval:
         normalize_by_mean: bool
             Whether to divide the (padded) image with its mean before applying the CTF filter.
         lim1: float >0
-            the regulariser strenght at low  frequencies
+            the regulariser strength at low  frequencies
         lim2: float >0
-            the regulariser strenght at high frequencies
+            the regulariser strength at high frequencies
         use_rfft: bool, optional
             Whether to use real-to-complex (R2C) FFT instead of usual complex-to-complex (C2C).
         fftw_num_threads: bool or None or int, optional
@@ -157,7 +157,7 @@ class CTFPhaseRetrieval:
         """
         self.logger = LoggerOrPrint(logger)
         if not isinstance(geo_pars, GeoPars):
-            raise ValueError("Expected GeoPars instance for 'geo_pars' parameter")
+            raise TypeError("Expected GeoPars instance for 'geo_pars' parameter")
         self.geo_pars = geo_pars
         self._calc_shape(shape, padded_shape, padding_mode)
         self.delta_beta = delta_beta
@@ -202,7 +202,7 @@ class CTFPhaseRetrieval:
         The parameter "length_scale" was mentioned, in the octave code,
         as a rescaling length scale, which is meant to avoid
         having too big or too small numbers.
-        From the mathematical point of view, it is infact completely trasparent:
+        From the mathematical point of view, it is in fact completely transparent:
         its action is on fsamplex, fsampley and  betash, betasv.
         But these latters ( beta's)  are multiplied by  the formers (fsample's) so that
         "length_scale" mathematically disappears, however in case of simple precision float
@@ -286,13 +286,13 @@ class CTFPhaseRetrieval:
         Question:( why the factor 2 appear at the denominator in the square root below?)
         Answer : the below defined cut corresponds to the first maximum of the denominator,
         before arriving to the first zero. In this way the regularisation
-        is already at ( almost) full strenght on the first pole.ation
-        ## is already at ( almost) full strenght on the first pole.
+        is already at ( almost) full strength on the first pole.
+        ## is already at ( almost) full strength on the first pole.
         """
         self.cut_v = math.sqrt(1.0 / 2 / lambda_dist_vh[0]) / fsample_vh[0]
         self.cut_v = min(self.cut_v, 0.5)
 
-        self.logger.debug("Normalized cut-off = {:5.3f}".format(self.cut_v))
+        self.logger.debug(f"Normalized cut-off = {self.cut_v:5.3f}")
 
         self.r = fourier_filters.get_lowpass_filter(
             padded_img_shape,
