@@ -1,15 +1,10 @@
 import numpy as np
 from ..utils import calc_padding_lengths, updiv, get_cuda_srcfile, docstring
 from ..cuda.processing import CudaProcessing
-from ..cuda.utils import __has_pycuda__
 from ..processing.padding_cuda import CudaPadding
 from ..processing.fft_cuda import get_fft_class
-from .phase_cuda import CudaPaganinPhaseRetrieval
 from .ctf import CTFPhaseRetrieval
 
-if __has_pycuda__:
-    from pycuda import gpuarray as garray
-
 
 # TODO:
 #  - better padding scheme (for now 2*shape)
@@ -48,7 +43,7 @@ class CudaCTFPhaseRetrieval(CTFPhaseRetrieval):
         padding_mode: str
             Padding mode. Default is "reflect".
 
-        Other parameters
+        Other Parameters
         -----------------
         Please refer to CTFPhaseRetrieval documentation.
         """
@@ -82,14 +77,10 @@ class CudaCTFPhaseRetrieval(CTFPhaseRetrieval):
         )
 
     def _init_cuda_padding(self):
-        pad_width = calc_padding_lengths(self.shape, self.shape_padded)
-        # Custom coordinate transform to get directly FFT layout
-        R, C = np.indices(self.shape, dtype=np.int32, sparse=True)
-        coords_R = np.roll(np.pad(R.ravel(), pad_width[0], mode=self.padding_mode), -pad_width[0][0])
-        coords_C = np.roll(np.pad(C.ravel(), pad_width[1], mode=self.padding_mode), -pad_width[1][0])
+        self._pad_width = calc_padding_lengths(self.shape, self.shape_padded)
         self.cuda_padding = CudaPadding(
             self.shape,
-            (coords_R, coords_C),
+            pad_width=self._pad_width,
             mode=self.padding_mode,
             # propagate cuda options ?
         )
@@ -104,7 +95,6 @@ class CudaCTFPhaseRetrieval(CTFPhaseRetrieval):
         self.cpxmult_kernel = self.cuda_processing.kernel(
             "CTF_kernel",
             filename=get_cuda_srcfile("ElementOp.cu"),
-            signature="PPPfii",
         )
         Nx = np.int32(self.shape_padded[1] // 2 + 1)
         Ny = np.int32(self.shape_padded[0])
@@ -120,22 +110,25 @@ class CudaCTFPhaseRetrieval(CTFPhaseRetrieval):
         grd = (updiv(Nx, blk[0]), updiv(Ny, blk[1]))
         self._cpxmult_kernel_kwargs = {"grid": grd, "block": blk}
 
-    set_input = CudaPaganinPhaseRetrieval.set_input
+    def _get_output(self, output):
+        pw = self._pad_width
+        sub_region = (slice(pw[0][0], -pw[0][1]), slice(pw[1][0], -pw[1][1]))
+        if output is None:
+            output = self.cuda_processing.allocate_array("d_output", self.shape)
+        output[:, :] = self.d_radio_padded[sub_region]
+        return output
 
     def retrieve_phase(self, image, output=None):
         """
         Perform padding on an image. Please see the documentation of CTFPhaseRetrieval.retrieve_phase().
         """
-        self.set_input(image)
-        self.cuda_padding.pad(image, output=self.d_radio_padded)
+        d_image = self.cuda_processing.set_array("d_image", image, dtype=np.float32)
+        self.cuda_padding.pad(d_image, output=self.d_radio_padded)
         if self.normalize_by_mean:
-            m = garray.sum(self.d_radio_padded).get() / np.prod(self.shape_padded)  # pylint: disable=E0606
+            m = self.d_radio_padded.get().sum() / np.prod(self.shape_padded)  # pylint: disable=E0606
             self.d_radio_padded /= m
         self.cufft.fft(self.d_radio_padded, output=self.d_radio_f)
         self.cpxmult_kernel(*self._cpxmult_kernel_args, **self._cpxmult_kernel_kwargs)
         self.cufft.ifft(self.d_radio_f, output=self.d_radio_padded)
 
-        if output is None:
-            output = self.cuda_processing.allocate_array("d_output", self.shape)
-        output[:, :] = self.d_radio_padded[: self.shape[0], : self.shape[1]]
-        return output
+        return self._get_output(output)
