@@ -1,13 +1,12 @@
 from .double_flatfield import DoubleFlatField
 from ..utils import check_shape
-from ..cuda.utils import __has_pycuda__
+from ..cuda.utils import __has_cupy__
 from ..cuda.processing import CudaProcessing
 from ..processing.unsharp_cuda import CudaUnsharpMask
 from .ccd_cuda import CudaLog
 
-if __has_pycuda__:
-    import pycuda.gpuarray as garray
-    import pycuda.cumath as cumath
+if __has_cupy__:
+    import cupy
 
 
 class CudaDoubleFlatField(DoubleFlatField):
@@ -59,7 +58,7 @@ class CudaDoubleFlatField(DoubleFlatField):
     def _proc_expm(x, o):
         o[:] = x[:]
         o[:] *= -1
-        cumath.exp(o, out=o)  # pylint: disable=E0606
+        cupy.exp(o, out=o)  # pylint: disable=E0606
         return o
 
     def _init_processing(self, input_is_mlog, output_is_mlog, average_is_on_log, sigma_filter, filter_mode):
@@ -108,12 +107,12 @@ class CudaDoubleFlatField(DoubleFlatField):
         recompute: bool, optional
             Whether to recompute the double flatfield if already computed.
         """
-        if not (isinstance(radios, garray.GPUArray)):  # pylint: disable=E0606
-            raise ValueError("Expected pycuda.gpuarray.GPUArray for radios")
+        if not (isinstance(radios, self.cuda_processing.array_class)):  # pylint: disable=E0606
+            raise TypeError("Expected cupy array for radios")
         if self._computed and not (recompute):
             return self.doubleflatfield
-        acc = garray.zeros(radios[0].shape, "f")
-        tmpdat = garray.zeros(radios[0].shape, "f")
+        acc = self.cuda_processing.allocate_array("acc", radios[0].shape, dtype="f")
+        tmpdat = self.cuda_processing.allocate_array("tmpdat", radios[0].shape, dtype="f")
         for i in range(radios.shape[0]):
             self.proc(radios[i], tmpdat)
             acc += tmpdat
@@ -128,9 +127,13 @@ class CudaDoubleFlatField(DoubleFlatField):
         self.doubleflatfield = tmpdat
         # Handle small values to avoid issues when dividing
         # self.doubleflatfield[np.abs(self.doubleflatfield) < self._small] = 1.
-        cumath.fabs(self.doubleflatfield, out=acc)
+        cupy.fabs(self.doubleflatfield, out=acc)
         acc -= self._small  # acc = abs(doubleflatfield) - _small
-        garray.if_positive(acc, self.doubleflatfield, garray.zeros_like(acc) + self._small, out=self.doubleflatfield)
+        # garray.if_positive(acc, self.doubleflatfield, garray.zeros_like(acc) + self._small, out=self.doubleflatfield)
+        # TODO improve the following, too many copies
+        d_eps = self.cuda_processing.allocate_array("d_eps", acc.shape, dtype="f")
+        d_eps += self._small
+        self.doubleflatfield[:] = cupy.where(acc, self.doubleflatfield, d_eps)[:]
 
         if self.writer is not None:
             self.writer.write(self.doubleflatfield.get(), "double_flatfield")
@@ -158,7 +161,7 @@ class CudaDoubleFlatField(DoubleFlatField):
             res = self.compute_double_flatfield(radios)
         else:
             res = self._load_dff_dump()
-            res = garray.to_gpu(res)
+            res = self.cuda_processing.to_device("res", res)
             self._computed = True
         return res
 
