@@ -1,8 +1,11 @@
+import os
 from multiprocessing.pool import ThreadPool
 from bisect import bisect_left
 import numpy as np
+from tomoscan.io import HDF5File
 from ..io.reader import load_images_from_dataurl_dict
 from ..utils import check_supported, deprecated_class, get_num_threads
+from .ccd import CCDFilter
 
 
 class FlatFieldArrays:
@@ -91,7 +94,7 @@ class FlatFieldArrays:
           flatfield_normalization(X') = (X' - D)/(F' - D) = (X - D) / (F - D) * sF/sX
         So current normalization boils down to a scalar multiplication after flat-field.
         """
-        if self._full_shape:
+        if self._full_shape:  # noqa: SIM102
             # this is never going to happen in this base class. But in the derived class for helical
             # which needs to keep the full shape
             if radios_indices is not None:
@@ -102,7 +105,7 @@ class FlatFieldArrays:
         self._precompute_flats_indices_weights()
         self._configure_srcurrent_normalization(radios_srcurrent, flats_srcurrent)
         self.distortion_correction = distortion_correction
-        self.n_threads = min(1, get_num_threads(n_threads) // 2)
+        self.n_threads = max(1, get_num_threads(n_threads) // 2)
 
     def _set_parameters(self, radios_shape, radios_indices, interpolation, nan_value):
         self._set_radios_shape(radios_shape)
@@ -421,7 +424,7 @@ class FlatFieldDataUrls(FlatField):
 
         Other Parameters
         ----------------
-        The other named parameters are passed to ChunkReader(). Please read its
+        The other named parameters are passed to the reading function (sub_region, binning, etc). Please read its
         documentation for more information.
 
         Notes
@@ -450,3 +453,306 @@ class FlatFieldDataUrls(FlatField):
             radios_srcurrent=radios_srcurrent,
             flats_srcurrent=flats_srcurrent,
         )
+
+
+class PCAFlatsNormalizer:
+    """This class implement a flatfield normalization based on a PCA of a series of acquired flatfields.
+    The PCA decomposition is handled by a PCAFlatsDecomposer object.
+
+    This implementation was proposed by Jailin C. et al in https://journals.iucr.org/s/issues/2017/01/00/fv5055/
+
+    Code initially written by ID11 @ ESRF staff.
+    Jonathan Wright - Implementation based on research paper
+    Pedro D. Resende - Added saving and loading from file capabilities
+
+    Jerome Lesaint - Integrated the solution in Nabu.
+
+    """
+
+    def __init__(self, components, dark, mean):
+        """Initializes all variables needed to perform the flatfield normalization.
+
+        Parameters
+        -----------
+        components: ndarray
+            The components of the PCA decomposition.
+        dark: ndarray
+            The dark image. Should be one single 2D image.
+        mean: ndarray
+            The mean image of the series of flats.
+        """
+        ones = np.ones_like(components[0], dtype=np.float32)[np.newaxis]  # This comp will account for I0
+        self.components = np.concatenate([ones, components], axis=0)
+        self.dark = dark
+        self.mean = mean
+        self.n_threads = max(1, get_num_threads() // 2)
+        self._setmask()
+        self.ccdfilter = CCDFilter(mean.shape)
+
+    def _form_lsq_matrix(self):
+        """This function form the Least Square matrix, based on the flats components and the mask."""
+        self.Amat = np.stack([gg for gg in self.g], axis=1)  # JL: this is the matrix for the fit
+
+    def _setmask(self, prop=0.125):
+        """Sets the mask to select where the model is going to be fitted.
+
+        Parameters
+        ----------
+        prop: float, default: 0.125
+            The proportion of the image width to take on each side of the image as a mask.
+
+            By default it sets the strips on each side of the frame in the form:
+                mask[:, lim:] = True
+                mask[:, -lim:] = True
+            Where lim = prop * flat.shape[1]
+
+        If you need a custom mask, see update_mask() method.
+        """
+
+        lim = int(prop * self.mean.shape[1])
+        self.mask = np.zeros(self.mean.shape, dtype=bool)
+        self.mask[:, :lim] = True
+        self.mask[:, -lim:] = True
+        self.g = []
+        for component in self.components:
+            self.g.append(component[self.mask])
+
+        self._form_lsq_matrix()
+
+    def update_mask(self, mask: np.ndarray):
+        """Method to update the mask with a custom mask in the form of a boolean 2+D array.
+
+        Paramters
+        =========
+        mask: np.ndarray of Boolean;
+            The array of boolean allows the selection of the region of the image that will be used to fit against the components.
+
+        It will set the mask, replacing the standard mask created with setmask().
+        """
+        if mask.dtype == bool:
+            self.mask = mask
+            self.g = []
+            for component in self.components:
+                self.g.append(component[self.mask])
+
+            self._form_lsq_matrix()
+        else:
+            raise TypeError("Not a boolean array. Will keep the default mask")
+
+    def normalize_radios(self, projections, mask=None, prop=0.125):
+        """This is to keep the flatfield API in the pipeline."""
+        self.correct_stack(projections, mask=mask, prop=prop)
+
+    def correct_stack(
+        self,
+        projections: np.ndarray,
+        mask: np.ndarray = None,
+        prop: float = 0.125,
+    ):
+        """This functions normalizes the stack of projections.
+
+        Performs correction on a stack of projections based on the calculated decomposition.
+        The normalizations is done in-place. The previous projections before normalization are lost.
+
+        Parameters
+        ----------
+        projections: ndarray
+            Stack of projections to normalize.
+
+        prop: float (default: {0.125})
+            Fraction to mask on the horizontal field of view, assuming vertical rotation axis
+
+        mask: np.ndarray (default: None)
+            Custom mask if your data requires it.
+
+        Returns
+        -------
+        corrected projections: np.ndarray
+            Flat field corrected images. Note that the returned projections are exp-transformed to fit the pipeline (e.g. to allow for phase retrieval).
+        """
+        self.projections = self.ccdfilter.dezinger_correction(projections, self.dark)
+
+        if mask is not None:
+            self.update_mask(mask=mask)
+        else:
+            self._setmask(prop=prop)
+
+        with ThreadPool(self.n_threads) as tp:
+            for i, cor, sol in tp.map(self._readcorrect1, range(len(projections))):
+                # solution[i] = sol
+                projections[i] = np.exp(-cor)
+
+    def _readcorrect1(self, ii):
+        """Method to allow parallelization of the normalization."""
+        corr, s = self.correctproj(self.projections[ii])
+        return ii, corr, s
+
+    def correctproj(self, projection):
+        """Performs the correction on one projection of the stack.
+
+        Parameters
+        ----------
+        projection: np.ndarray, float
+            Radiograph from the acquisition stack.
+
+        Returns
+        -------
+        The fitted projection.
+        """
+        logp = np.log(projection.astype(np.float32) - self.dark)
+        corr = self.mean - logp
+        # model to be fitted !!
+        return self.fit(corr)
+
+    def fit(self, corr):
+        """Fit the (masked) projection to the (masked) components of the PCA decomposition.
+
+        This is for each projection, so worth optimising ...
+        """
+        y = corr[self.mask]
+        solution = np.linalg.lstsq(self.Amat, y, rcond=None)[0]
+        correction = np.einsum("ijk,i->jk", self.components, solution)
+        return corr - correction, solution
+
+
+class PCAFlatsDecomposer:
+    """This class implements a PCA decomposition of a serie of acquired flatfields.
+    The PCA decomposition is used to normalize the projections through a PCAFLatNormalizer object.
+
+    This implementation was proposed by Jailin C. et al in https://journals.iucr.org/s/issues/2017/01/00/fv5055/
+
+    Code initially written by ID11 @ ESRF staff.
+    Jonathan Wright - Implementation based on research paper
+    Pedro D. Resende - Added saving and loading from file capabilities
+
+    Jerome Lesaint - Integrated the solution in Nabu.
+    """
+
+    def __init__(self, flats: np.ndarray, darks: np.ndarray, nsigma=3):
+        """
+
+        Parameters
+        -----------
+        flats: np.ndarray
+            A stack of darks corrected flat field images
+        darks: np.ndarray
+            An image or stack of images of the dark current images of the camera.
+
+        Does the log scaling.
+        Subtracts mean and does eigenvector decomposition.
+        """
+        self.n_threads = max(1, get_num_threads() // 2)
+        self.flats = np.empty(flats.shape, dtype=np.float32)
+
+        darks = darks.astype(np.float32)
+        if darks.ndim == 3:
+            self.dark = np.median(darks, axis=0)
+        else:
+            self.dark = darks.copy()
+        del darks
+
+        self.nsigma = nsigma
+        self.ccdfilter = CCDFilter(self.dark.shape)
+        self._ccdfilter_and_log(flats)  # Log is taken here (after dezinger)
+
+        self.mean = np.mean(self.flats, axis=0)  # average
+
+        self.flats = self.flats - self.mean  # makes a copy
+        self.cov = self.compute_correlation_matrix()
+        self.compute_pca()
+        self.generate_pca_flats(nsigma=nsigma)  # Default nsigma=3
+
+    def __str__(self):
+
+        return f"PCA decomposition from flat images. \nThere are {self.components} components created at {self.sigma} level."
+
+    def _ccdfilter_and_log(self, flats: np.ndarray):
+        """Dezinger (subtract dark, apply median filter) and takes log of the flat stack"""
+
+        self.ccdfilter.dezinger_correction(flats, self.dark)
+
+        with ThreadPool(self.n_threads) as tp:
+            for i, frame in enumerate(tp.map(np.log, flats)):
+                self.flats[i] = frame
+
+    @staticmethod
+    def ccij(args):
+        """Compute the covariance (img[i]*img[j]).sum() / npixels
+        It is a wrapper for threading.
+        args == i, j, npixels, imgs
+        """
+
+        i, j, NY, imgs = args
+        return i, j, np.einsum("ij,ij", imgs[i], imgs[j]) / NY
+
+    def compute_correlation_matrix(self):
+        """Computes an (nflats x nflats) correlation matrix"""
+
+        N = len(self.flats)
+        CC = np.zeros((N, N), float)
+        args = [(i, j, N, self.flats) for i in range(N) for j in range(i + 1)]
+        with ThreadPool(self.n_threads) as tp:
+            for i, j, result in tp.map(self.ccij, args):
+                CC[i, j] = CC[j, i] = result
+        return CC
+
+    def compute_pca(self):
+        """Gets eigenvectors and eigenvalues and sorts them into order"""
+
+        self.eigenvalues, self.eigenvectors = np.linalg.eigh(
+            self.cov
+        )  # Not sure why the eigh is needed. eig should be enough.
+        order = np.argsort(abs(self.eigenvalues))[::-1]  # high to low
+        self.eigenvalues = self.eigenvalues[order]
+        self.eigenvectors = self.eigenvectors[:, order]
+
+    def generate_pca_flats(self, nsigma=3):
+        """Projects the eigenvectors back into image space.
+
+        Parameters
+        ----------
+        nsigma: int (default: 3)
+        """
+
+        self.sigma = nsigma
+        av = abs(self.eigenvalues)
+        N = max(1, (av > (av[-2] * nsigma)).sum())  # Go for 3 sigma. And always take at least one comp.
+
+        self.components = [
+            None,
+        ] * N
+
+        def calculate(ii):
+            calc = np.einsum("i,ijk->jk", self.eigenvectors[:, ii], self.flats)
+            norm = (calc**2).sum()
+            return ii, calc / np.sqrt(norm)
+
+        with ThreadPool(self.n_threads) as tp:
+            for ii, result in tp.map(calculate, range(N)):
+                self.components[ii] = result
+
+        # simple gradients
+        r, c = self.components[0].shape
+        self.components.append(np.outer(np.ones(r), np.linspace(-1 / c, 1 / c, c)))
+        self.components.append(np.outer(np.linspace(-1 / r, 1 / r, r), np.ones(c)))
+
+    def save_decomposition(self, path="PCA_flats.h5", overwrite=True, entry="entry0000"):
+        """Saves the basic information of a PCA decomposition in view of the normalization of projections.
+
+        Parameters
+        ----------
+        path: str (default: "PCA_flats.h5")
+            Full path to the h5 file you want to save your results. It will overwrite!! Be careful.
+        """
+
+        file_exists = os.path.exists(path)
+        if overwrite or not file_exists:
+            with HDF5File(path, "w") as hout:
+                group = hout.create_group(entry)
+                group["eigenvalues"] = self.eigenvalues
+                group["dark"] = self.dark.astype(np.float32)
+                group["p_mean"] = self.mean.astype(np.float32)
+                group["p_components"] = np.array(self.components)
+                hout.flush()
+        else:
+            raise OSError(f"The file {path} already exists and you chose to NOT overwrite.")
