@@ -4,7 +4,6 @@ from nabu.cuda.processing import CudaProcessing
 from ..preproc.flatfield import FlatFieldArrays
 from ..utils import deprecated_class, get_cuda_srcfile
 from ..io.reader import load_images_from_dataurl_dict
-from ..cuda.utils import __has_pycuda__
 
 
 class CudaFlatFieldArrays(FlatFieldArrays):
@@ -58,7 +57,7 @@ class CudaFlatFieldArrays(FlatFieldArrays):
         if self.nan_value is not None:
             options.append("-DNAN_VALUE=%f" % self.nan_value)
         self.cuda_kernel = self.cuda_processing.kernel(
-            "flatfield_normalization", self._cuda_fname, signature="PPPiiiPP", options=options
+            "flatfield_normalization", self._cuda_fname, options=tuple(options)
         )
         self._nx = np.int32(self.shape[1])
         self._ny = np.int32(self.shape[0])
@@ -86,11 +85,11 @@ class CudaFlatFieldArrays(FlatFieldArrays):
 
         Parameters
         -----------
-        radios_shape: `pycuda.gpuarray.GPUArray`
+        radios_shape: cupy array
             Radios chunk.
         """
         if not (isinstance(radios, self.cuda_processing.array_class)):
-            raise ValueError("Expected a pycuda.gpuarray (got %s)" % str(type(radios)))
+            raise TypeError("Expected a cupy array (got %s)" % str(type(radios)))
         if radios.dtype != np.float32:
             raise ValueError("radios must be in float32 dtype (got %s)" % str(radios.dtype))
         if radios.shape != self.radios_shape:
