@@ -5,13 +5,12 @@ from scipy.fft import rfft2, irfft2, fft2, ifft2
 from ..utils import generate_powers, get_decay, check_supported, get_num_threads, deprecation_warning
 
 # COMPAT.
-from .ctf import CTFPhaseRetrieval
 
 #
 
 
 def lmicron_to_db(Lmicron, energy, distance):
-    """
+    r"""
     Utility to convert the "Lmicron" parameter of PyHST
     to a value of delta/beta.
 
@@ -55,7 +54,7 @@ class PaganinPhaseRetrieval:
         fftw_num_threads=None,
         fft_num_threads=None,
     ):
-        """
+        r"""
         Paganin Phase Retrieval for an infinitely distant point source.
         Formula (10) in [1].
 
@@ -98,7 +97,6 @@ class PaganinPhaseRetrieval:
 
         Notes
         ------
-
         **Padding methods**
 
         The phase retrieval is a convolution done in Fourier domain using FFT,
