@@ -17,7 +17,7 @@ class VerticalShift:
         The interpolation is done taking for a pixel (y,x) the pixel found at (y+shft,x)
         in the recorded images.
 
-        The method apply_vertical_shifts performs the correctionson the radios.
+        The method apply_vertical_shifts performs the corrections on the radios.
 
         Parameters
         ----------
@@ -42,7 +42,7 @@ class VerticalShift:
     def _init_interp_coefficients(self):
         self.interp_infos = []
         for s in self.shifts:
-            s0 = int(floor(s))
+            s0 = floor(s)
             f = s - s0
             self.interp_infos.append([s0, f])
 
@@ -51,20 +51,22 @@ class VerticalShift:
         assert np.max(iangles) < len(self.interp_infos)
         assert len(iangles) == radios.shape[0]
 
-    def apply_vertical_shifts(self, radios, iangles, output=None):
+    def apply_vertical_shifts(self, radios, iangles=None, output=None):
         """
         Parameters
         ----------
         radios: a sequence of np.array
             The input radios. If the optional parameter is not given, they are modified in-place
         iangles:  a sequence of integers
-            Must have the same lenght as radios.
+            Must have the same length as radios.
             It contains the index at which the shift is found in `self.shifts`
             given by `shifts` argument in the initialisation of the object.
         output: a sequence of np.array, optional
             If given, it will be modified to contain the shifted radios.
             Must be of the same shape of `radios`.
         """
+        if iangles is None:
+            iangles = np.arange(radios.shape[0])
         self._check(radios, iangles)
 
         newradio = np.zeros_like(radios[0])
