@@ -1,5 +1,4 @@
 import numpy as np
-from ..cuda.utils import __has_pycuda__
 from ..cuda.processing import CudaProcessing
 from ..processing.muladd_cuda import CudaMulAdd
 from .shift import VerticalShift
@@ -19,19 +18,19 @@ class CudaVerticalShift(VerticalShift):
         self._d_interp_infos = self.cuda_processing.to_device("_d_interp_infos", interp_infos_arr)
         self._d_radio_new = self.cuda_processing.allocate_array("_d_radio_new", self.radios_shape[1:], "f")
         self._d_radio = self.cuda_processing.allocate_array("_d_radio", self.radios_shape[1:], "f")
-        self.muladd_kernel = CudaMulAdd(ctx=self.cuda_processing.ctx)
+        self.muladd_kernel = CudaMulAdd()  # ctx=self.cuda_processing.ctx)
 
     def apply_vertical_shifts(self, radios, iangles, output=None):
         """
         Parameters
         ----------
-        radios: 3D pycuda.gpuarray.GPUArray
+        radios: 3D cupy array
             The input radios. If the optional parameter is not given, they are modified in-place
         iangles:  a sequence of integers
-            Must have the same lenght as radios.
+            Must have the same length as radios.
             It contains the index at which the shift is found in `self.shifts`
             given by `shifts` argument in the initialisation of the object.
-        output: 3D pycuda.gpuarray.GPUArray, optional
+        output: 3D cupy array, optional
             If given, it will be modified to contain the shifted radios.
             Must be of the same shape of `radios`.
         """
@@ -51,7 +50,7 @@ class CudaVerticalShift(VerticalShift):
 
         for ia in iangles:
             d_radio_new.fill(0)
-            d_radio[:] = radios[ia, :, :]  # mul-add kernel won't work with pycuda view
+            d_radio[:] = radios[ia, :, :]  # mul-add kernel won't work with pycuda view # TODO try with cupy ?
             S0, f = self.interp_infos[ia]
             f = np.float32(f)
 
