@@ -2,11 +2,10 @@ import pytest
 import numpy as np
 from nabu.utils import median2 as nabu_median_filter
 from nabu.testutils import get_data
-from nabu.cuda.utils import get_cuda_context, __has_pycuda__
+from nabu.cuda.utils import __has_cupy__
 from nabu.preproc.ccd import CCDFilter
 
-if __has_pycuda__:
-    import pycuda.gpuarray as garray
+if __has_cupy__:
     from nabu.preproc.ccd_cuda import CudaCCDFilter
 
 
@@ -17,8 +16,8 @@ def bootstrap(request):
     cls.data = get_data("mri_proj_astra.npz")["data"]
     cls.data /= cls.data.max()
     cls.put_hotspots_in_data()
-    if __has_pycuda__:
-        cls.ctx = get_cuda_context()
+    if __has_cupy__:
+        ...
 
 
 @pytest.mark.usefixtures("bootstrap")
@@ -49,11 +48,13 @@ class TestCCDFilter:
         res = ccd_filter.median_clip_correction(self.data, output=res)
         self.check_detected_hotspots_locations(res)
 
-    @pytest.mark.skipif(not (__has_pycuda__), reason="Need cuda/pycuda for this test")
+    @pytest.mark.skipif(not (__has_cupy__), reason="Need cupy for this test")
     def test_cuda_median_clip(self):
-        d_radios = garray.to_gpu(self.data)
-        cuda_ccd_correction = CudaCCDFilter(d_radios.shape, median_clip_thresh=self.threshold)
-        d_out = garray.zeros_like(d_radios)
+        cuda_ccd_correction = CudaCCDFilter(self.data.shape, median_clip_thresh=self.threshold)
+
+        cuda_proc = cuda_ccd_correction.cuda_median_filter.cuda_processing
+        d_radios = cuda_proc.to_device("d_radios", self.data)
+        d_out = cuda_proc.allocate_array("d_out", d_radios.shape, dtype=d_radios.dtype)
         cuda_ccd_correction.median_clip_correction(d_radios, output=d_out)
         res = d_out.get()
         self.check_detected_hotspots_locations(res)
