@@ -10,10 +10,10 @@ from nabu.preproc import ctf
 from nabu.estimation.distortion import estimate_flat_distortion
 from nabu.misc.filters import correct_spikes
 from nabu.preproc.distortion import DistortionCorrection
-from nabu.cuda.utils import __has_pycuda__, get_cuda_context
+from nabu.cuda.utils import __has_cupy__
 
 __has_cufft__ = False
-if __has_pycuda__:
+if __has_cupy__:
     from nabu.preproc.ctf_cuda import CudaCTFPhaseRetrieval
 
     avail_fft = get_available_fft_implems()
@@ -58,19 +58,17 @@ class TestCtf:
         assert diff.max() < self.abs_tol * (np.abs(ref).mean()), error_message
 
     def test_ctf_id16_way(self):
-        """test the ctf phase retrieval.
-        The cft filter, of the CtfFilter class is iniitalised with the geomety informations contained in geo_pars object
-        of the GeoPars class.
-        The geometry encompass the case of astigmatic wavefront with a vertical and horisontal sources which are
-        at distance z1_vh[0] and z1_vh[1] from the object.
-        In the case of parllel geometry put  z1_vh[0]= z1_vh[1] = R
-        where R is a large value ( meters).
-        SI unit system is used. But the same results shudl be obtained with any homogenuous choice of the distance units.
-        The img_shape is the shape of the images which will be processed.
-        padded_img_shape is an intermediate shape which needs to be larger that the img_shape to avoid border effect due
+        """Test the CTF phase retrieval.
+        The CTF filter, from the CtfFilter class, is initialised with the geometry information contained in the GeoPars object.
+        The geometry encompasses the case of an astigmatic wavefront with vertical and horizontal sources located at
+        distances z1_vh[0] and z1_vh[1] from the object.
+        In the case of parallel geometry, set z1_vh[0] = z1_vh[1] = R where R is a large value (meters).
+        The SI unit system is used, but the same results should be obtained with any homogeneous choice of distance units.
+        The `img_shape` is the shape of the images to be processed.
+        `padded_img_shape` is an intermediate shape that must be larger than `img_shape` to avoid border effects due
         to convolutions.
-        Length scale is an internal parameters which should not affect in anyway the result unless there are serious numerical
-        problems involving very small lenghts. You can safely let the default value.
+        `length_scale` is an internal parameter that should not affect the result unless there are serious numerical
+        problems involving very small lengths. You can safely keep the default value.
         """
         geo_pars = ctf.GeoPars(
             z1_vh=self.z1_vh,
@@ -220,10 +218,10 @@ class TestCtf:
         # Test multi-core FFT
         ctf_fft = ctf.CtfFilter(*ctf_args, **ctf_kwargs, use_rfft=True, fft_num_threads=0)
         if ctf_fft.use_rfft:
-            phase_fft = ctf_fft.retrieve_phase(img)
+            # phase_fft = ctf_fft.retrieve_phase(img)
             self.check_result(phase_r2c, self.ref_plain, "Something wrong with CtfFilter-FFT")
 
-    @pytest.mark.skipif(not (__has_pycuda__ and __has_cufft__), reason="pycuda and (scikit-cuda or vkfft)")
+    @pytest.mark.skipif(not (__has_cupy__ and __has_cufft__), reason="need cupy and vkfft")
     def test_cuda_ctf(self):
         data = nabu_get_data("brain_phantom.npz")["data"]
         delta_beta = 50.0
@@ -232,7 +230,6 @@ class TestCtf:
         pix_size_m = 0.1e-6
 
         geo_pars = ctf.GeoPars(z2=distance_m, pix_size_det=pix_size_m, wavelength=1.23984199e-9 / energy_kev)
-        ctx = get_cuda_context()
 
         for normalize in [True, False]:
             ctf_filter = ctf.CTFPhaseRetrieval(
