@@ -1,10 +1,10 @@
 import os
 import numpy as np
 import pytest
-from nabu.cuda.utils import get_cuda_context, __has_pycuda__
+from nabu.cuda.utils import __has_cupy__
 from nabu.preproc.flatfield import FlatField
 
-if __has_pycuda__:
+if __has_cupy__:
     from nabu.preproc.flatfield_cuda import CudaFlatField
 
 
@@ -139,8 +139,8 @@ def bootstrap(request):
     cls.n_radios = 10
     cls.n_z = 100
     cls.n_x = 512
-    if __has_pycuda__:
-        cls.ctx = get_cuda_context()
+    if __has_cupy__:
+        ...
     yield
     # Tear-down
     for fname in cls.tmp_files:
@@ -238,7 +238,7 @@ class TestFlatField:
             radios_corr, [0.0, 0.35714286, 0.43478261, 0.46875, 0.48780488, 0.5, 0.6, 0.7, 0.8, 0.9]
         )
 
-    @pytest.mark.skipif(not (__has_pycuda__), reason="Need cuda/pycuda for this test")
+    @pytest.mark.skipif(not (__has_cupy__), reason="Need cupy for this test")
     def test_cuda_flatfield(self):
         """
         Test the flat-field with cuda back-end.
@@ -314,7 +314,7 @@ class TestFlatField:
             FF_with_nan_handling.normalize_radios(radios)
         self._check_numerical_issue(radios, config["expected_result"], nan_value)
 
-    @pytest.mark.skipif(not (__has_pycuda__), reason="Need cuda/pycuda for this test")
+    @pytest.mark.skipif(not (__has_cupy__), reason="Need cupy for this test")
     def test_cuda_twoflats_numerical_issue(self):
         """
         Same as above, with the Cuda backend
@@ -354,7 +354,7 @@ class TestFlatField:
         radios_corr = FF.normalize_radios(np.copy(radios))
         self.check_normalized_radios(radios_corr, config["expected_result"])
 
-    @pytest.mark.skipif(not (__has_pycuda__), reason="Need cuda/pycuda for this test")
+    @pytest.mark.skipif(not (__has_cupy__), reason="Need cupy for this test")
     def test_srcurrent_cuda(self):
         radios, flats, darks, config = self.get_test_elements("three_flats_srcurrent")
 
@@ -411,7 +411,7 @@ class FlatFieldTestDataset:
         self._generate_projections()
 
     def get_flat_idx(self, proj_idx):
-        flats_idx = sorted(list(self.flats.keys()))
+        flats_idx = sorted(self.flats.keys())
         if proj_idx <= flats_idx[0]:
             return (flats_idx[0],)
         elif proj_idx > flats_idx[0] and proj_idx < flats_idx[1]:
@@ -478,7 +478,7 @@ class TestFlatFieldH5:
         flatfield.normalize_radios(projs)
         self.check_normalization(projs)
 
-    @pytest.mark.skipif(not (__has_pycuda__), reason="Need cuda/pycuda for this test")
+    @pytest.mark.skipif(not (__has_cupy__), reason="Need cupy for this test")
     def test_cuda_flatfield(self):
         cuda_flatfield = CudaFlatField(
             self.dataset.projs_data.shape,
@@ -558,7 +558,7 @@ class TestFlatFieldMultiFlat:
         print(projs[:, 0, 0])
         self.check_normalization(projs)
 
-    @pytest.mark.skipif(not (__has_pycuda__), reason="Need cuda/pycuda for this test")
+    @pytest.mark.skipif(not (__has_cupy__), reason="Need cupy for this test")
     def test_cuda_flatfield(self):
         cuda_flatfield = CudaFlatField(
             self.radios.shape,
