@@ -1,13 +1,10 @@
 from os.path import dirname
+from warnings import warn
 import numpy as np
-from ..utils import updiv, get_cuda_srcfile
-from ..cuda.utils import __has_pycuda__
+from ..utils import first_dict_item, updiv, get_cuda_srcfile
 from ..misc.utils import ConvolutionInfos
 from ..cuda.processing import CudaProcessing
 
-if __has_pycuda__:
-    from pycuda.compiler import SourceModule
-
 
 class Convolution:
     """
@@ -133,9 +130,10 @@ class Convolution:
         # except for mode="nearest"
         for i, dim_size in enumerate(self.shape):
             if min(self.kernel.shape) > dim_size and i in self.axes:
-                print(
+                warn(
                     "Warning: kernel support is too large for data dimension %d (%d). Forcing convolution mode to 'nearest'"
-                    % (i, dim_size)
+                    % (i, dim_size),
+                    Warning,
                 )
                 self.mode = "nearest"
         #
@@ -159,7 +157,7 @@ class Convolution:
             self.d_kernel = self.cuda.to_device("d_kernel", self.kernel)
         else:
             if not (isinstance(self.kernel, self.cuda.array_class)):
-                raise ValueError("kernel must be either numpy array or pycuda array")
+                raise TypeError("kernel must be either numpy array or cupy array")
             self.d_kernel = self.kernel
         self._old_input_ref = None
         self._old_output_ref = None
@@ -185,51 +183,40 @@ class Convolution:
         self._c_conv_mode = mp[self.mode]
 
     def _init_kernels(self):
-        if self.kernel_ndim > 1:
+        if self.kernel_ndim > 1:  # noqa: SIM102
             if np.abs(np.diff(self.kernel.shape)).max() > 0:
                 raise NotImplementedError("Non-separable convolution with non-square kernels is not implemented yet")
         # Compile source module
-        compile_options = [str("-DUSED_CONV_MODE=%d" % self._c_conv_mode)]
         fname = get_cuda_srcfile("convolution.cu")
         nabu_cuda_dir = dirname(fname)
-        include_dirs = [nabu_cuda_dir]
+        compile_options = (f"-DUSED_CONV_MODE={self._c_conv_mode}", f"-I{nabu_cuda_dir}")
         self.sourcemodule_kwargs["options"] = compile_options
-        self.sourcemodule_kwargs["include_dirs"] = include_dirs
-        with open(fname) as fid:
-            cuda_src = fid.read()
-        self._module = SourceModule(cuda_src, **self.sourcemodule_kwargs)  # pylint: disable=E0606
         # Blocks, grid
         self._block_size = {1: (32, 1, 1), 2: (32, 32, 1), 3: (16, 8, 8)}[self.data_ndim]  # TODO tune
         self._n_blocks = tuple([int(updiv(a, b)) for a, b in zip(self.shape[::-1], self._block_size)])
         # Prepare cuda kernel calls
-        self._cudakernel_signature = {
-            1: "PPPiiii",
-            2: "PPPiiiii",
-            3: "PPPiiiiii",
-        }[self.kernel_ndim]
+
         self.cuda_kernels = {}
         for axis, kern_name in enumerate(self.use_case_kernels):
-            self.cuda_kernels[axis] = self._module.get_function(kern_name)
-            self.cuda_kernels[axis].prepare(self._cudakernel_signature)
+            self.cuda_kernels[axis] = self.cuda.kernel(kern_name, filename=fname, **self.sourcemodule_kwargs)
 
         # Cuda kernel arguments
         kernel_args = [
-            self._n_blocks,
-            self._block_size,
-            None,
-            None,
-            self.d_kernel.gpudata,
+            None,  # input placeholder
+            None,  # output placeholder
+            self.d_kernel,
             np.int32(self.kernel.shape[0]),
             self.Nx,
             self.Ny,
             self.Nz,
         ]
         if self.kernel_ndim == 2:
-            kernel_args.insert(5, np.int32(self.kernel.shape[1]))
+            kernel_args.insert(3, np.int32(self.kernel.shape[1]))
         if self.kernel_ndim == 3:
-            kernel_args.insert(5, np.int32(self.kernel.shape[2]))
-            kernel_args.insert(6, np.int32(self.kernel.shape[1]))
+            kernel_args.insert(3, np.int32(self.kernel.shape[2]))
+            kernel_args.insert(4, np.int32(self.kernel.shape[1]))
         self.kernel_args = tuple(kernel_args)
+        self.kernel_kwargs = {"grid": self._n_blocks, "block": self._block_size}
         # If self.data_tmp is allocated, separable transforms can be performed
         # by a series of batched transforms, without any copy, by swapping refs.
         self.swap_pattern = None
@@ -264,16 +251,12 @@ class Convolution:
 
     def _configure_kernel_args(self, cuda_kernel_args, input_ref, output_ref):
         # TODO more elegant
-        if isinstance(input_ref, self.cuda.array_class):
-            input_ref = input_ref.gpudata
-        if isinstance(output_ref, self.cuda.array_class):
-            output_ref = output_ref.gpudata
         if input_ref is not None or output_ref is not None:
             cuda_kernel_args = list(cuda_kernel_args)
             if input_ref is not None:
-                cuda_kernel_args[2] = input_ref
+                cuda_kernel_args[0] = input_ref
             if output_ref is not None:
-                cuda_kernel_args[3] = output_ref
+                cuda_kernel_args[1] = output_ref
             cuda_kernel_args = tuple(cuda_kernel_args)
         return cuda_kernel_args
 
@@ -290,8 +273,8 @@ class Convolution:
         return ndim
 
     def _check_array(self, arr):
-        if not (isinstance(arr, self.cuda.array_class) or isinstance(arr, np.ndarray)):
-            raise TypeError("Expected either pycuda.gpuarray or numpy.ndarray")
+        if not (isinstance(arr, self.cuda.array_class) or isinstance(arr, np.ndarray)):  # noqa: SIM101
+            raise TypeError("Expected either cupy or numpy.ndarray")
         if arr.dtype != np.float32:
             raise TypeError("Data must be float32")
         if arr.shape != self.shape:
@@ -305,7 +288,7 @@ class Convolution:
             self._old_input_ref = self.data_in
             self.data_in = array
         data_in_ref = self.data_in
-        if output is not None:
+        if output is not None:  # noqa: SIM102
             if not (isinstance(output, np.ndarray)):
                 self._old_output_ref = self.data_out
                 self.data_out = output
@@ -323,12 +306,14 @@ class Convolution:
         # Batched: one kernel call in total
         cuda_kernel = self.cuda_kernels[axis]
         cuda_kernel_args = self._configure_kernel_args(self.kernel_args, input_ref, output_ref)
-        ev = cuda_kernel.prepared_call(*cuda_kernel_args)
+        ev = cuda_kernel(*cuda_kernel_args, **self.kernel_kwargs)
+        return ev
 
     def _nd_convolution(self):
         assert len(self.use_case_kernels) == 1
-        cuda_kernel = self._module.get_function(self.use_case_kernels[0])
-        ev = cuda_kernel.prepared_call(*self.kernel_args)
+        cuda_kernel = first_dict_item(self.use_case_kernels)
+        ev = cuda_kernel(*self.kernel_args, **self.kernel_kwargs)
+        return ev
 
     def _recover_arrays_references(self):
         if self._old_input_ref is not None:
@@ -353,8 +338,8 @@ class Convolution:
         """
         Convolve an array with the class kernel.
 
-        :param array: Input array. Can be numpy.ndarray or pycuda.gpuarray.GPUArray.
-        :param output: Output array. Can be numpy.ndarray or pycuda.gpuarray.GPUArray.
+        :param array: Input array. Can be numpy.ndarray or cupy array
+        :param output: Output array. Can be numpy.ndarray or cupy array
         """
         self._check_array(array)
         self._set_arrays(array, output=output)
