@@ -146,7 +146,7 @@ class PartialHistogram:
         elif self.bin_width == "uint16":
             return self._bin_width_u16(dmin, dmax)
         else:
-            raise ValueError()
+            raise ValueError
 
     def _compute_histogram_fixed_bw(self, data, data_range=None):
         dmin, dmax = data.min(), data.max() if data_range is None else data_range
