@@ -1,10 +1,10 @@
 import numpy as np
 from ..utils import get_cuda_srcfile, updiv
-from ..cuda.utils import __has_pycuda__
+from ..cuda.utils import __has_cupy__
 from .histogram import PartialHistogram, VolumeHistogram
 
-if __has_pycuda__:
-    import pycuda.gpuarray as garray
+if __has_cupy__:
+    import cupy
     from ..cuda.processing import CudaProcessing
 
 
@@ -32,7 +32,6 @@ class CudaPartialHistogram(PartialHistogram):
         self.cuda_hist = self.cuda_processing.kernel(
             "histogram",
             filename=get_cuda_srcfile("histogram.cu"),
-            signature="PiiiffPi",
         )
         self.d_hist = self.cuda_processing.allocate_array("d_hist", self.num_bins, dtype=np.uint32)
 
@@ -40,12 +39,12 @@ class CudaPartialHistogram(PartialHistogram):
         if isinstance(data, np.ndarray):
             data = self.cuda_processing.to_device("data", data)
         if data_range is None:
-            # Should be possible to do both in one single pass with ReductionKernel
-            # and garray.vec.float2, but the last step in volatile shared memory
-            # still gives errors. To be investigated...
+            # Should be possible to do both in one single pass with ReductionKernel,
+            # But not sure how to use native vector types "float2" with these kernels.
+            # Probably the simplest would be a RawKernel with explicit reduction ?
             # pylint: disable=E0606
-            data_min = garray.min(data).get()[()]
-            data_max = garray.max(data).get()[()]
+            data_min = cupy.min(data).get()[()]
+            data_max = cupy.max(data).get()[()]
         else:
             data_min, data_max = data_range
         Nz, Ny, Nx = data.shape
