@@ -1,12 +1,9 @@
 import numpy as np
 from nabu.utils import get_cuda_srcfile, updiv
 from .muladd import MulAdd
-from ..cuda.utils import __has_pycuda__
+from ..cuda.utils import to_int2
 from ..cuda.processing import CudaProcessing
 
-if __has_pycuda__:
-    import pycuda.gpuarray as garray
-
 
 class CudaMulAdd(MulAdd):
     processing_cls = CudaProcessing
@@ -17,11 +14,17 @@ class CudaMulAdd(MulAdd):
     def _init_kernel(self):
         self.muladd_kernel = self.processing.kernel(
             "mul_add",
-            filename=get_cuda_srcfile("ElementOp.cu"),  # signature="PPiiffiiii"
+            filename=get_cuda_srcfile("ElementOp.cu"),
         )
 
     def mul_add(self, dst, other, fac_dst, fac_other, dst_region=None, other_region=None):
         """
+        Performs
+            dst[DST_IDX] = fac_dst*dst[DST_IDX] + fac_other*other[OTHER_IDX]
+        where
+            DST_IDX = dst_start_row:dst_end_row, dst_start_col:dst_end_col
+            OTHER_IDX = other_start_row:other_end_row, other_start_col:other_end_col
+
         'region' should be a tuple (slice(y_start, y_end), slice(x_start, x_end))
         """
 
@@ -41,12 +44,10 @@ class CudaMulAdd(MulAdd):
         if delta_x == 0 or delta_y == 0:
             raise ValueError("delta_x or delta_y is 0")
 
-        # can't use "int4" in pycuda ? int2 seems fine. Go figure
-        # pylint: disable=E0606
-        dst_x_range = np.array(dst_coords[:2], dtype=garray.vec.int2)
-        dst_y_range = np.array(dst_coords[2:], dtype=garray.vec.int2)
-        other_x_range = np.array(other_coords[:2], dtype=garray.vec.int2)
-        other_y_range = np.array(other_coords[2:], dtype=garray.vec.int2)
+        dst_x_range = to_int2(dst_coords[:2])
+        dst_y_range = to_int2(dst_coords[2:])
+        other_x_range = to_int2(other_coords[:2])
+        other_y_range = to_int2(other_coords[2:])
 
         block = (32, 32, 1)
         grid = [updiv(length, b) for (length, b) in zip((delta_x[0], delta_y[0]), block)]
