@@ -21,8 +21,7 @@ class OpenCLRoll:
         self.offset = offset or 0
         self.roll_kernel = OpenCLKernel(
             "roll_forward_x",
-            None,
-            queue=self.processing.queue,
+            self.processing.queue,
             filename=get_opencl_srcfile("roll.cl"),
             options=compile_options,
         )
@@ -31,8 +30,7 @@ class OpenCLRoll:
         if self.direction < 0:
             self.revert_kernel = OpenCLKernel(
                 "revert_array_x",
-                None,
-                queue=self.processing.queue,
+                self.processing.queue,
                 filename=get_opencl_srcfile("roll.cl"),
                 options=compile_options,
             )
