@@ -1,13 +1,9 @@
 import numpy as np
 from .rotation import Rotation
 from ..utils import get_cuda_srcfile, updiv
-from ..cuda.utils import __has_pycuda__, copy_array
+from ..cuda.utils import create_texture
 from ..cuda.processing import CudaProcessing
 
-if __has_pycuda__:
-    from ..cuda.kernel import CudaKernel
-    import pycuda.driver as cuda
-
 
 class CudaRotation(Rotation):
     def __init__(self, shape, angle, center=None, mode="edge", reshape=False, cuda_options=None, **sk_kwargs):
@@ -23,19 +19,18 @@ class CudaRotation(Rotation):
         self._init_rotation_kernel()
 
     def _allocate_arrays(self):
-        self._d_image_cua = cuda.np_to_array(np.zeros(self.shape, "f"), "C")  # pylint: disable=E0606
         self.cuda_processing.init_arrays_to_none(["d_output"])
 
+    def _get_textures(self):
+        self.tex_image, self.tex_image_cua = create_texture(self.shape, "f", address_mode="clamp", filter_mode="linear")
+
     def _init_rotation_kernel(self):
-        self.cuda_rotation_kernel = CudaKernel("rotate", get_cuda_srcfile("rotation.cu"))  # pylint: disable=E0606
-        self.texref_image = self.cuda_rotation_kernel.module.get_texref("tex_image")
-        self.texref_image.set_filter_mode(cuda.filter_mode.LINEAR)  # bilinear
-        self.texref_image.set_address_mode(0, cuda.address_mode.CLAMP)  # TODO tune
-        self.texref_image.set_address_mode(1, cuda.address_mode.CLAMP)  # TODO tune
-        self.cuda_rotation_kernel.prepare("Piiffff", [self.texref_image])
-        self.texref_image.set_array(self._d_image_cua)
-        self._cos_theta = np.cos(np.deg2rad(self.angle))
-        self._sin_theta = np.sin(np.deg2rad(self.angle))
+        self._get_textures()
+        self.cuda_rotation_kernel = self.cuda_processing.kernel(
+            "rotate", filename=get_cuda_srcfile("rotation.cu")
+        )  # pylint: disable=E0606
+        self._cos_theta = np.float32(np.cos(np.deg2rad(self.angle)))
+        self._sin_theta = np.float32(np.sin(np.deg2rad(self.angle)))
         self._Nx = np.int32(self.shape[1])
         self._Ny = np.int32(self.shape[0])
         self._center_x = np.float32(self.center[0])
@@ -44,13 +39,14 @@ class CudaRotation(Rotation):
         self._grid = (updiv(self.shape[1], self._block[1]), updiv(self.shape[0], self._block[0]), 1)
 
     def rotate(self, img, output=None, do_checks=True):
-        copy_array(self._d_image_cua, img, check=do_checks)
+        self.tex_image_cua.copy_from(img)
         if output is not None:
             d_out = output
         else:
             self.cuda_processing.allocate_array("d_output", self.shape, np.float32)
             d_out = self.cuda_processing.d_output
         self.cuda_rotation_kernel(
+            self.tex_image.ptr,
             d_out,
             self._Nx,
             self._Ny,
