@@ -3,22 +3,23 @@ import pytest
 import numpy as np
 from scipy.fft import fftn, ifftn, rfftn, irfftn
 from nabu.testutils import generate_tests_scenarios, get_data, get_array_of_given_shape, __do_long_tests__
-from nabu.cuda.utils import get_cuda_context, __has_pycuda__
-from nabu.processing.fft_cuda import SKCUFFT, VKCUFFT, get_available_fft_implems
-from nabu.opencl.utils import __has_pyopencl__, get_opencl_context
-from nabu.processing.fft_opencl import VKCLFFT, has_vkfft as has_cl_vkfft
+from nabu.cuda.utils import __has_cupy__, get_cuda_stream
+from nabu.cuda.processing import CudaProcessing
+from nabu.opencl.processing import OpenCLProcessing
+from nabu.opencl.utils import __has_pyopencl__
 from nabu.processing.fft_base import is_fast_axes
+from nabu.processing.fft_cuda import VKCUFFT, CupyCUFFT, get_available_fft_implems
+from nabu.processing.fft_opencl import VKCLFFT, has_vkfft as has_cl_vkfft
 
 available_cuda_fft = get_available_fft_implems()
 __has_vkfft__ = "vkfft" in available_cuda_fft
-__has_skcuda__ = "skcuda" in available_cuda_fft
-
+__has_cupyfft__ = "cupy" in available_cuda_fft
 
 scenarios = {
     "shape": [(256,), (300,), (300, 301), (300, 302)],
     "r2c": [True, False],
     "precision": ["simple"],
-    "backend": ["cuda", "opencl"],
+    "backend": ["cuda-vkfft", "cuda-cupy", "opencl-vkfft"],
 }
 
 if __do_long_tests__:
@@ -45,22 +46,27 @@ def bootstrap(request):
             3: 1e-7,
         },
     }
-    if __has_pycuda__:
-        cls.cu_ctx = get_cuda_context(cleanup_at_exit=False)
+    if __has_cupy__:
+        stream = get_cuda_stream(force_create=True)
+        cls.cuda_processing = CudaProcessing(stream=stream)
     if __has_pyopencl__:
-        cls.cl_ctx = get_opencl_context("all")
+        cls.opencl_processing = OpenCLProcessing(device_type="all")
     yield
-    if __has_pycuda__:
-        cls.cu_ctx.pop()
+    if __has_cupy__:
+        ...
 
 
 def _get_fft_cls(backend):
     fft_cls = None
-    if backend == "cuda":
-        if not (__has_vkfft__ and __has_pycuda__):
-            pytest.skip("Need vkfft and pycuda to use VKCUFFT")
+    if backend == "cuda-vkfft":
+        if not (__has_vkfft__ and __has_cupy__):
+            pytest.skip("Need vkfft and cupy to use VKCUFFT")
         fft_cls = VKCUFFT
-    if backend == "opencl":
+    if backend == "cuda-cupy":
+        if not (__has_cupyfft__ and __has_cupy__):
+            pytest.skip("Need cupy to use CupyCUFFT")
+        fft_cls = CupyCUFFT
+    if backend == "opencl-vkfft":
         if not (has_cl_vkfft() and __has_pyopencl__):
             pytest.skip("Need vkfft and pyopencl to use VKCLFFT")
         fft_cls = VKCLFFT
@@ -92,14 +98,18 @@ class TestFFT:
         )
         assert np.allclose(res, ref, atol=tol), err_msg
 
-    def _do_fft(self, data, r2c, axes=None, return_fft_obj=False, backend_cls=None):
-        ctx = self.cu_ctx if backend_cls.backend == "cuda" else self.cl_ctx
-        fft = backend_cls(data.shape, data.dtype, r2c=r2c, axes=axes, ctx=ctx)
+    def _do_fft(self, data, r2c, axes=None, return_fft_obj=False, fft_cls=None):
+        backend_kwargs = {}
+        if fft_cls.backend == "cuda":
+            backend_kwargs["stream"] = self.cuda_processing.stream
+        elif fft_cls.backend == "opencl":
+            backend_kwargs["queue"] = self.opencl_processing.queue
+        fft = fft_cls(data.shape, data.dtype, r2c=r2c, axes=axes, **backend_kwargs)
+
         d_data = fft.processing.allocate_array("_data", data.shape, dtype=data.dtype)
         d_data.set(data)
         d_out = fft.fft(d_data)
-        res = d_out.get()
-        return (res, fft) if return_fft_obj else res
+        return (d_out, fft) if return_fft_obj else d_out
 
     @staticmethod
     def _do_reference_fft(data, r2c, axes=None):
@@ -113,69 +123,8 @@ class TestFFT:
         ref = ref_ifft_func(data, axes=axes)
         return ref
 
-    @pytest.mark.skipif(
-        not (__has_skcuda__ and __has_pycuda__), reason="Need pycuda and (scikit-cuda or vkfft) for this test"
-    )
     @pytest.mark.parametrize("config", scenarios)
-    def test_sckcuda(self, config):
-        r2c = config["r2c"]
-        shape = config["shape"]
-        precision = config["precision"]
-        ndim = len(shape)
-        if ndim == 3 and not (__do_long_tests__):
-            pytest.skip("3D FFTs are done only for long tests - use NABU_LONG_TESTS=1")
-
-        data = self._get_data_array(config)
-
-        res, cufft = self._do_fft(data, r2c, return_fft_obj=True, backend_cls=SKCUFFT)
-        ref = self._do_reference_fft(data, r2c)
-
-        tol = self.abs_tol[precision][ndim]
-        self.check_result(res, ref, config, tol, name="skcuda")
-
-        # Complex-to-complex can also be performed on real data (as in numpy.fft.fft(real_data))
-        if not (r2c):
-            res = self._do_fft(data, False, backend_cls=SKCUFFT)
-            ref = self._do_reference_fft(data, False)
-            self.check_result(res, ref, config, tol, name="skcuda")
-
-        # IFFT
-        res = cufft.ifft(cufft.output_fft).get()
-        self.check_result(res, data, config, tol, name="skcuda")
-        # Perhaps we should also check against numpy/scipy ifft,
-        # but it does not yield the good shape for R2C on odd-sized data
-
-    @pytest.mark.skipif(
-        not (__has_skcuda__ and __has_pycuda__), reason="Need pycuda and (scikit-cuda or vkfft) for this test"
-    )
-    @pytest.mark.parametrize("config", scenarios)
-    def test_skcuda_batched(self, config):
-        shape = config["shape"]
-        if len(shape) == 1:
-            return
-        elif len(shape) == 3 and not (__do_long_tests__):
-            pytest.skip("3D FFTs are done only for long tests - use NABU_LONG_TESTS=1")
-        r2c = config["r2c"]
-        tol = self.abs_tol[config["precision"]][len(shape)]
-
-        data = self._get_data_array(config)
-
-        if data.ndim == 2:
-            axes_to_test = [(0,), (1,)]
-        elif data.ndim == 3:
-            # axes_to_test = [(1, 2), (2, 1), (2,)] # See fft.py: works for C2C but not R2C ?
-            axes_to_test = [(2,)]
-
-        for axes in axes_to_test:
-            res, cufft = self._do_fft(data, r2c, axes=axes, return_fft_obj=True, backend_cls=SKCUFFT)
-            ref = self._do_reference_fft(data, r2c, axes=axes)
-            self.check_result(res, ref, config, tol, name="skcuda batched axes=%s" % (str(axes)))
-            # IFFT
-            res = cufft.ifft(cufft.output_fft).get()
-            self.check_result(res, data, config, tol, name="skcuda")
-
-    @pytest.mark.parametrize("config", scenarios)
-    def test_vkfft(self, config):
+    def test_fft(self, config):
         backend = config["backend"]
         fft_cls = _get_fft_cls(backend)
 
@@ -186,35 +135,37 @@ class TestFFT:
         if ndim == 3 and not (__do_long_tests__):
             pytest.skip("3D FFTs are done only for long tests - use NABU_LONG_TESTS=1")
 
-        if ndim >= 2 and r2c and shape[-1] & 1:
+        if "vkfft" in backend and ndim >= 2 and r2c and shape[-1] & 1:
             pytest.skip("R2C with odd-sized fast dimension is not supported in VKFFT")
 
         # FIXME - vkfft + POCL fail for R2C in one dimension
-        if config["backend"] == "opencl" and r2c and ndim == 1:
-            if self.cl_ctx.devices[0].platform.name.strip().lower() == "portable computing language":
+        if config["backend"] == "opencl-vkfft" and r2c and ndim == 1:  # noqa: SIM102
+            if self.opencl_processing.ctx.devices[0].platform.name.strip().lower() == "portable computing language":
                 pytest.skip("Something wrong with vkfft + pocl for R2C 1D")
         # ---
 
         data = self._get_data_array(config)
 
-        res, fft_obj = self._do_fft(data, r2c, return_fft_obj=True, backend_cls=fft_cls)
+        d_res, fft_obj = self._do_fft(data, r2c, return_fft_obj=True, fft_cls=fft_cls)
+        res = d_res.get()
         ref = self._do_reference_fft(data, r2c)
 
         tol = self.abs_tol[precision][ndim]
-        self.check_result(res, ref, config, tol, name="vkfft_%s" % backend)
+        self.check_result(res, ref, config, tol, name="fft_%s" % backend)
 
         # Complex-to-complex can also be performed on real data (as in numpy.fft.fft(real_data))
         if not (r2c):
-            res = self._do_fft(data, False, backend_cls=fft_cls)
+            d_res = self._do_fft(data, False, fft_cls=fft_cls)
+            res = d_res.get()
             ref = self._do_reference_fft(data, False)
-            self.check_result(res, ref, config, tol, name="vkfft_%s" % backend)
+            self.check_result(res, ref, config, tol, name="fft_%s" % backend)
 
         # IFFT
-        res = fft_obj.ifft(fft_obj.output_fft).get()
-        self.check_result(res, data, config, tol, name="vkfft_%s" % backend)
+        res = fft_obj.ifft(d_res).get()
+        self.check_result(res, data, config, tol, name="fft_%s" % backend)
 
     @pytest.mark.parametrize("config", scenarios)
-    def test_vkfft_batched(self, config):
+    def test_fft_batched(self, config):
         backend = config["backend"]
         fft_cls = _get_fft_cls(backend)
         shape = config["shape"]
@@ -232,19 +183,24 @@ class TestFFT:
 
         # For R2C, only fastest axes are supported by vkfft
         if data.ndim == 2:
-            axes_to_test = [(1,)]
+            if "vkfft" in backend:
+                axes_to_test = [(1,)]
+            else:
+                axes_to_test = [(1,), (0,)]
         elif data.ndim == 3:
-            axes_to_test = [
-                (1, 2),
-                (2,),
-            ]
+            if "vkfft" in backend:
+                axes_to_test = [(1, 2), (2,)]
+            else:
+                axes_to_test = [(1, 2), (0, 1), (0, 2), (2,), (1,), (0,)]
+
         for axes in axes_to_test:
-            res, cufft = self._do_fft(data, r2c, axes=axes, return_fft_obj=True, backend_cls=fft_cls)
+            d_res, cufft = self._do_fft(data, r2c, axes=axes, return_fft_obj=True, fft_cls=fft_cls)
+            res = d_res.get()
             ref = self._do_reference_fft(data, r2c, axes=axes)
-            self.check_result(res, ref, config, tol, name="vkfft_%s batched axes=%s" % (backend, str(axes)))
+            self.check_result(res, ref, config, tol, name="fft_%s batched axes=%s" % (backend, str(axes)))
             # IFFT
-            res = cufft.ifft(cufft.output_fft).get()
-            self.check_result(res, data, config, tol, name="vkfft_%s" % backend)
+            res = cufft.ifft(d_res).get()
+            self.check_result(res, data, config, tol, name="fft_%s" % backend)
 
     @pytest.mark.skipif(not (__do_long_tests__), reason="Use NABU_LONG_TESTS=1 for this test")
     def test_fast_axes_utility_function(self):
