@@ -1,8 +1,8 @@
 import numpy as np
 import pytest
-from nabu.cuda.utils import get_cuda_context, __has_pycuda__
+from nabu.cuda.utils import __has_cupy__
 from nabu.opencl.utils import __has_pyopencl__, get_opencl_context
-from nabu.testutils import get_data, generate_tests_scenarios, __do_long_tests__
+from nabu.testutils import get_data, generate_tests_scenarios
 
 if __has_pyopencl__:
     from nabu.processing.fftshift import OpenCLFFTshift
@@ -22,13 +22,13 @@ def bootstrap(request):
     cls = request.cls
     cls.data = get_data("chelsea.npz")["data"]
     cls.tol = 1e-7
-    if __has_pycuda__:
-        cls.cu_ctx = get_cuda_context(cleanup_at_exit=False)
+    if __has_cupy__:
+        ...
     if __has_pyopencl__:
         cls.cl_ctx = get_opencl_context(device_type="all")
     yield
-    if __has_pycuda__:
-        cls.cu_ctx.pop()
+    if __has_cupy__:
+        ...
 
 
 @pytest.mark.skip(reason="OpenCL fftshift is a prototype")
