@@ -1,11 +1,8 @@
-from ..cuda.utils import __has_pycuda__
+from nabu.processing.muladd_cuda import CudaMulAdd
 from ..processing.convolution_cuda import Convolution
 from ..cuda.processing import CudaProcessing
 from .unsharp import UnsharpMask
 
-if __has_pycuda__:
-    from pycuda.elementwise import ElementwiseKernel
-
 
 class CudaUnsharpMask(UnsharpMask):
     def __init__(self, shape, sigma, coeff, mode="reflect", method="gaussian", **cuda_options):
@@ -31,23 +28,19 @@ class CudaUnsharpMask(UnsharpMask):
         )
 
     def _init_mad_kernel(self):
-        # garray.GPUArray.mul_add is out of place...
-        self.mad_kernel = ElementwiseKernel(  # pylint: disable=E0606
-            "float* array, float fac, float* other, float otherfac",
-            "array[i] = fac * array[i] + otherfac * other[i]",
-            name="mul_add",
-        )
+        self.mul_add = CudaMulAdd()
 
     def unsharp(self, image, output=None):
         if output is None:
             output = self.cuda_processing.allocate_array("_d_out", self.shape, "f")
         self.convolution(image, output=output)
         if self.method == "gaussian":
-            self.mad_kernel(output, -self.coeff, image, 1.0 + self.coeff)
+            # res = (1 + self.coeff) * image - self.coeff * image_b
+            self.mul_add(output, image, -self.coeff, 1.0 + self.coeff)
         elif self.method == "log":
             # output = output * coeff + image   where output was image_blurred
-            self.mad_kernel(output, self.coeff, image, 1.0)
+            self.mul_add(output, image, self.coeff, 1.0)
         else:  # "imagej":
             # output = (image - coeff*image_blurred)/(1-coeff)  where output was image_blurred
-            self.mad_kernel(output, -self.coeff / (1 - self.coeff), image, 1.0 / (1 - self.coeff))
+            self.mul_add(output, image, -self.coeff / (1 - self.coeff), 1.0 / (1 - self.coeff))
         return output
