@@ -1,7 +1,7 @@
 import numpy as np
-import pycuda.driver as cuda
+
 from ..utils import updiv, get_cuda_srcfile
-from ..cuda.utils import copy_array, check_textures_availability
+from ..cuda.utils import create_texture
 from ..cuda.processing import CudaProcessing
 from ..cuda.kernel import CudaKernel
 from .filtering_cuda import CudaSinoFilter
@@ -17,23 +17,13 @@ class CudaBackprojector(BackprojectorBase):
     SinoMultClass = CudaSinoMult
 
     def _check_textures_availability(self):
-        self._use_textures = self.extra_options.get("use_textures", True) and check_textures_availability()
-
-    def _get_kernel_signature(self):
-        kern_full_sig = list("PPiifiiffPPPf")
-        if self._axis_correction is None:
-            kern_full_sig[11] = ""
-        if self._use_textures:
-            # texture references - no object is passed (deprecated, removed in Cuda 12)
-            kern_full_sig[1] = ""
-        return "".join(kern_full_sig)
+        self._use_textures = self.extra_options.get("use_textures", True)
 
     def _get_kernel_options(self):
         super()._get_kernel_options()
         self._kernel_options.update(
             {
                 "file_name": get_cuda_srcfile(self.kernel_filename),
-                "kernel_signature": self._get_kernel_signature(),
                 "texture_name": "tex_projections",
             }
         )
@@ -45,45 +35,54 @@ class CudaBackprojector(BackprojectorBase):
                 "shared_size": self._kernel_options["shared_size"],
             }
         )
-        # texture references - no object is passed (deprecated, removed in Cuda 12)
         if self._use_textures:
-            self.kern_proj_args.pop(1)
+            self.kern_proj_args[1] = self._tex_sino
         else:
             self._d_sino = self._processing.allocate_array("_d_sino", self.sino_shape)
-            self.kern_proj_args[1] = self._d_sino.gpudata
+            self.kern_proj_args[1] = self._d_sino
 
     def _prepare_textures(self):
         if self._use_textures:
-            self.texref_proj = self.gpu_projector.module.get_texref(self._kernel_options["texture_name"])
-            self.texref_proj.set_filter_mode(cuda.filter_mode.LINEAR)
-            self.gpu_projector.prepare(self._kernel_options["kernel_signature"], [self.texref_proj])
-            # Bind texture
-            self._d_sino_cua = cuda.np_to_array(np.zeros(self.sino_shape, "f"), "C")
-            self.texref_proj.set_array(self._d_sino_cua)
-        else:
-            # d_sino_ref = self._d_sino.gpudata
-            # self.kern_proj_args.insert(2, d_sino_ref)
-            self.gpu_projector.prepare(self._kernel_options["kernel_signature"], [])
+            self._tex_sino, self._d_sino_cua = create_texture(
+                self.sino_shape, np.float32, address_mode="border", filter_mode="linear", normalized_coords=False
+            )
 
     def _compile_kernels(self):
+        self._prepare_textures()
         self._prepare_kernel_args()
         if self._use_textures:
             self._kernel_options["sourcemodule_options"].append("-DUSE_TEXTURES")
         self.gpu_projector = CudaKernel(
             self._kernel_options["kernel_name"],
             filename=self._kernel_options["file_name"],
-            options=self._kernel_options["sourcemodule_options"],
+            options=tuple(self._kernel_options["sourcemodule_options"]),
             silent_compilation_warnings=True,  # textures and Cuda 11
         )
         if self.halftomo and self.rot_center < self.dwidth:
-            self.sino_mult = CudaSinoMult(self.sino_shape, self.rot_center, ctx=self._processing.ctx)
-        self._prepare_textures()  # has to be done after compilation for Cuda (to bind texture to built kernel)
+            self.sino_mult = CudaSinoMult(self.sino_shape, self.rot_center)  # , ctx=self._processing.ctx)
+        if self.extra_options["clip_outer_circle"]:
+            self._clip_circle_kernel = CudaKernel(
+                "clip_circle",
+                filename=get_cuda_srcfile("clip_circle.cu"),
+            )
+            self._clip_circle_block = (32, 32)
+            self._clip_circle_kwargs = {
+                "block": self._clip_circle_block,
+                "grid": tuple([updiv(sz, blk) for sz, blk in zip(self.slice_shape[::-1], self._clip_circle_block)]),
+            }
+
+    def _get_filter_init_extra_options(self):
+        return {
+            # "cuda_options": {
+            #     "ctx": self._processing.ctx,
+            # },
+        }
 
     def _transfer_to_texture(self, sino, do_checks=True):
         if do_checks and not (sino.flags.c_contiguous):
             raise ValueError("Expected C-Contiguous array")
         if self._use_textures:
-            copy_array(self._d_sino_cua, sino, check=do_checks)
+            self._d_sino_cua.copy_from(sino)  # TODO stream ?
         else:
             if id(self._d_sino) == id(sino):
                 return
