@@ -1,3 +1,4 @@
+import warnings
 import numpy as np
 from ..utils import updiv, nextpow2, convert_index, deprecation_warning
 from ..processing.processing_base import ProcessingBase
@@ -6,6 +7,19 @@ from .sinogram import SinoMult
 from .sinogram import get_extended_sinogram_width
 
 
+def rot_center_is_in_middle_of_roi(rot_center, roi, tol=2.0):
+    # NB. tolerance should be at least 2,
+    # because in halftomo the extended sinogram width is 2*sino_width - int(2 * XXXX)
+    #   (where  XXX depends on whether the CoR is on the left or on the right)
+    #  because of the int(2 * stuff), we can have a jump of at most two pixels.
+    #
+    start_x, end_x, start_y, end_y = roi
+    return (
+        abs((start_x + end_x - 1) / 2 - rot_center) - 0.5 < tol
+        and abs((start_y + end_y - 1) / 2 - rot_center) - 0.5 < tol
+    )
+
+
 class BackprojectorBase:
     """
     Base class for backprojectors.
@@ -15,13 +29,14 @@ class BackprojectorBase:
     default_padding_mode = "zeros"
     kernel_name = "backproj"
     default_extra_options = {
-        "padding_mode": None,
+        "padding_mode": None,  # deprecated
         "axis_correction": None,
         "centered_axis": False,
         "clip_outer_circle": False,
         "scale_factor": None,
         "filter_cutoff": 1.0,
         "outer_circle_value": 0.0,
+        "crop_filtered_data": True,
     }
 
     kernel_filename = None
@@ -56,7 +71,7 @@ class BackprojectorBase:
             Shape of the slice. By default, the slice shape is (n_x, n_x) where
             `n_x = detector_width`
         angles: array-like, optional
-            Rotation anles in radians.
+            Rotation angles in radians.
             By default, angles are equispaced between [0, pi[.
         rot_center: float, optional
             Rotation axis position. Default is `(detector_width - 1)/2.0`
@@ -82,7 +97,7 @@ class BackprojectorBase:
         backend_options: dict, optional
             OpenCL/Cuda options passed to the OpenCLProcessing or CudaProcessing class.
 
-        Other parameters
+        Other Parameters
         -----------------
         extra_options: dict, optional
             Dictionary with a set of advanced options. The default are the following:
@@ -160,9 +175,6 @@ class BackprojectorBase:
         self.axis_pos = self.rot_center
         self._set_angles(angles, n_angles)
         self._set_slice_roi(slice_roi)
-        #
-        # offset = start - move
-        # move = 0 if not(centered_axis) else start + (n-1)/2. - c
         if self.extra_options["centered_axis"]:
             self.offsets = {
                 "x": self.rot_center - (self.n_x - 1) / 2.0,
@@ -208,6 +220,14 @@ class BackprojectorBase:
         end_x = convert_index(end_x, self.n_x, self.n_x)
         end_y = convert_index(end_y, self.n_y, self.n_y)
         self.slice_shape = (end_y - start_y, end_x - start_x)
+        if self.extra_options["centered_axis"] and not (
+            rot_center_is_in_middle_of_roi(self.rot_center, (start_x, end_x, start_y, end_y))
+        ):
+            warnings.warn(
+                "Using 'centered_axis' when doing a non-centered ROI reconstruction might have side effects: 'start_xy' and 'end_xy' have a different meaning",
+                RuntimeWarning,
+            )
+            # self.extra_options["centered_axis"] = False
         self.n_x = self.slice_shape[-1]
         self.n_y = self.slice_shape[-2]
         self.offsets = {"x": start_x, "y": start_y}
@@ -237,14 +257,20 @@ class BackprojectorBase:
         self._axis_correction = np.zeros((1, self.n_angles), dtype=np.float32)
         self._axis_correction[0, :] = axcorr[:]  # pylint: disable=E1136
 
+    def _get_filter_init_extra_options(self):
+        return {}
+
     def _init_filter(self, filter_name):
         self.filter_name = filter_name
         if filter_name in ["None", "none"]:
             self.sino_filter = None
             return
-        sinofilter_other_kwargs = {}
-        if self.backend != "numpy":
-            sinofilter_other_kwargs["%s_options" % self.backend] = {"ctx": self._processing.ctx}
+
+        # TODO
+        if not (self.extra_options.get("crop_filtered_data", True)):
+            warnings.warn("crop_filtered_data = False is not supported for FBP yet", RuntimeWarning)
+        #
+        sinofilter_other_kwargs = self._get_filter_init_extra_options()
         self.sino_filter = self.SinoFilterClass(
             self.sino_shape,
             filter_name=self.filter_name,
@@ -264,19 +290,15 @@ class BackprojectorBase:
         """
         self.rot_center = rot_center
         self.axis_pos = rot_center
-        # See kernels signature of backproj.cu and backproj.cl.
-        # The ifdef makes things a bit more complicated
         proj_arg_idx = 4
-        if self.backend == "cuda" and self._use_textures:
-            proj_arg_idx = 3
-        self.kern_proj_args[proj_arg_idx] = rot_center
+        self.kern_proj_args[proj_arg_idx] = np.float32(rot_center)
         if self.extra_options["centered_axis"]:
             self.offsets = {
                 "x": self.rot_center - (self.n_x - 1) / 2.0,
                 "y": self.rot_center - (self.n_y - 1) / 2.0,
             }
-            self.kern_proj_args[proj_arg_idx + 3] = self.offsets["x"]
-            self.kern_proj_args[proj_arg_idx + 4] = self.offsets["y"]
+            self.kern_proj_args[proj_arg_idx + 3] = np.float32(self.offsets["x"])
+            self.kern_proj_args[proj_arg_idx + 4] = np.float32(self.offsets["y"])
 
     # Try to factorize some code between Cuda and OpenCL
     # Not ideal, as cuda uses "grid" = n_blocks_launched,
@@ -294,8 +316,6 @@ class BackprojectorBase:
             block = (tpb_x, tpb_y, 1)
             sourcemodule_options.append("-DSHARED_SIZE=%d" % (tpb_x * tpb_y))
         grid = (updiv(updiv(self.n_x, block[0]), 2), updiv(updiv(self.n_y, block[1]), 2))
-        if self.extra_options["clip_outer_circle"]:
-            sourcemodule_options.append("-DCLIP_OUTER_CIRCLE")
         shared_size = int(np.prod(block)) * 2
         if self._axis_correction is not None:
             sourcemodule_options.append("-DDO_AXIS_CORRECTION")
@@ -331,6 +351,14 @@ class BackprojectorBase:
             "grid": self._kernel_options["grid"],
             "block": self._kernel_options["block"],
         }
+        if self.extra_options.get("clip_outer_circle", False):
+            self._clip_circle_args = [
+                np.int32(self.slice_shape[1]),
+                np.int32(self.slice_shape[0]),
+                np.float32((self.slice_shape[1] - 1) / 2),
+                np.float32((self.slice_shape[0] - 1) / 2),
+                np.float32(self.extra_options.get("outer_circle_value", 0)),
+            ]
 
     def _set_output(self, output, check=False):
         self._output_is_ndarray = isinstance(output, np.ndarray)
@@ -343,10 +371,6 @@ class BackprojectorBase:
                 self.slice_shape,
                 output.shape,
             )
-        if self.extra_options.get("clip_outer_circle", False):
-            out_circle_val = self.extra_options.get("outer_circle_value", 0)
-            if out_circle_val != 0:
-                output.fill(out_circle_val)
         return output
 
     def _set_kernel_slice_arg(self, d_slice):
@@ -354,15 +378,20 @@ class BackprojectorBase:
 
     def backproj(self, sino, output=None, do_checks=True):
         if self.halftomo and self.rot_center < self.dwidth:
-            self.sino_mult.prepare_sino(sino)
+            sino = self.sino_mult.prepare_sino(sino)
         self._transfer_to_texture(sino)
         d_slice = self._set_output(output, check=do_checks)
         self._set_kernel_slice_arg(d_slice)
         self.gpu_projector(*self.kern_proj_args, **self.kern_proj_kwargs)
+        if self.extra_options["clip_outer_circle"]:
+            self._clip_circle_kernel(d_slice, *self._clip_circle_args, **self._clip_circle_kwargs)
         if output is not None and not (self._output_is_ndarray):
             return output
         else:
-            return self._processing._d_slice.get(ary=output)
+            return self._get_slice_into(output)
+
+    def _get_slice_into(self, output):
+        return self._processing._d_slice.get(output)
 
     def filtered_backprojection(self, sino, output=None):
         #
@@ -376,6 +405,7 @@ class BackprojectorBase:
             # if a new device array was allocated for sinogram, then the filtering can overwrite it,
             # since it won't affect user argument
             if id(d_sino) != id(sino):
+                # if id(d_sino) != id(sino) and self.extra_options.get("crop_filtered_data", True):
                 filt_kwargs = {"output": d_sino}
             #
             sino_to_backproject = self.sino_filter(d_sino, **filt_kwargs)
