@@ -5,15 +5,6 @@ from silx.image.tomography import compute_fourier_filter, get_next_power
 from ..processing.padding_base import PaddingBase
 from ..utils import check_supported, get_num_threads
 
-# # COMPAT.
-# from .filtering_cuda import CudaSinoFilter
-
-# SinoFilter = deprecated_class(
-#     "From version 2023.1, 'filtering_cuda.CudaSinoFilter' should be used instead of 'filtering.SinoFilter'. In the future, 'filtering.SinoFilter' will be a numpy-only class.",
-#     do_print=True,
-# )(CudaSinoFilter)
-# #
-
 
 class SinoFilter:
     available_filters = [
@@ -44,11 +35,44 @@ class SinoFilter:
         sino_shape,
         filter_name=None,
         padding_mode="zeros",
+        crop_filtered_data=True,
         extra_options=None,
     ):
+        """
+        Initialize a SinoFilter instance.
+
+        Parameters
+        ----------
+        sino_shape: tuple
+            Shape of sinogram, in the form (n_angles, detector_width) or (n_sinos, n_angles, detector_width)
+        filter_name: str, optional
+            Name of the filter. Default is ram-lak.
+        padding_mode: str, optional
+            How to pad the data prior to filtering. Default is zero-padding, corresponding to linear convolution with the filter kernel.
+            In practice this value is often set to "edges" for interior tomography.
+        crop_filtered_data: bool, optional
+            Whether to crop the final, filtered sinogram. Default is True. See notes below.
+        extra_options: dict, optional
+            Dictionary of advanced extra options.
+
+        Notes
+        -----
+        Sinogram filtering done in the Filtered Back-Projection (FBP) method consists, in theory, in applying a high-pass filter
+        to the sinogram prior to backprojection. This high-pass filter is normally the Ramachandran-Lakshminarayanan (Ram-Lak) filter
+        yielding a close-to-ideal reconstruction (see Natterer's "Mathematical methods in image reconstruction").
+        As the filter kernel has a large extent in spatial domain, it's best performed in Fourier domain via the Fourier-convolution theorem.
+        Filtering in Fourier domain should be done with a data padded to at least twice its size.
+        Zero-padding should be used for mathematical correctness (so that multiplication in Fourier domain corresponds to an actual linear convolution).
+        However if the sinogram does not decay to "zero" near the edges (i.e in interior tomography), padding with zeros usually gives artefacts after filtering.
+        In this case, padding with edges is preferred (corresponding to a convolution with the "edges" extension mode).
+
+        After inverse Fourier transform, the (padded and filtered) data is cropped back to its original size.
+        In some cases, it's preferable to keep the data un-cropped for further processing.
+        """
+
         self._init_extra_options(extra_options)
         self._set_padding_mode(padding_mode)
-        self._calculate_shapes(sino_shape)
+        self._calculate_shapes(sino_shape, crop_filtered_data)
         self._init_fft()
         self._allocate_memory()
         self._compute_filter(filter_name)
@@ -67,7 +91,7 @@ class SinoFilter:
         check_supported(padding_mode, self.available_padding_modes, "padding mode")
         self.padding_mode = padding_mode
 
-    def _calculate_shapes(self, sino_shape):
+    def _calculate_shapes(self, sino_shape, crop_filtered_data):
         self.ndim = len(sino_shape)
         if self.ndim == 2:
             n_angles, dwidth = sino_shape
@@ -90,6 +114,11 @@ class SinoFilter:
         self.pad_left = (self.dwidth_padded - self.dwidth) // 2
         self.pad_right = self.dwidth_padded - self.dwidth - self.pad_left
 
+        self.crop_filtered_data = crop_filtered_data
+        self.output_shape = self.sino_shape
+        if not (self.crop_filtered_data):
+            self.output_shape = self.sino_padded_shape
+
     def _init_fft(self):
         pass
 
@@ -147,19 +176,16 @@ class SinoFilter:
         if arr.shape != self.sino_shape:
             raise ValueError("Expected sinogram shape %s, got %s" % (self.sino_shape, arr.shape))
 
-    def filter_sino(self, sino, output=None, no_output=False):
+    def filter_sino(self, sino, output=None):
         """
         Perform the sinogram siltering.
 
         Parameters
         ----------
-        sino: numpy.ndarray or pycuda.gpuarray.GPUArray
+        sino: array
             Input sinogram (2D or 3D)
-        output: numpy.ndarray or pycuda.gpuarray.GPUArray, optional
+        output: array, optional
             Output array.
-        no_output: bool, optional
-            If set to True, no copy is be done. The resulting data lies
-            in self.d_sino_padded.
         """
         self._check_array(sino)
         # sino_padded = np.pad(
@@ -169,16 +195,21 @@ class SinoFilter:
         sino_padded_f = rfft(sino_padded, axis=1, workers=get_num_threads(self.extra_options["fft_threads"]))
         sino_padded_f *= self.filter_f
         sino_filtered = irfft(sino_padded_f, axis=1, workers=get_num_threads(self.extra_options["fft_threads"]))
+
         if output is None:
-            res = np.zeros(self.sino_shape, dtype=np.float32)
+            if not (self.crop_filtered_data):
+                # No need to allocate extra memory here
+                return sino_filtered
+            res = np.zeros(self.output_shape, dtype=np.float32)
         else:
             res = output
-        if self.ndim == 2:
-            # res[:] = sino_filtered[:, : self.dwidth]  # pylint: disable=E1126 # ?!
-            res[:] = sino_filtered[:, self.pad_left : -self.pad_right]  # pylint: disable=E1126 # ?!
+
+        if self.crop_filtered_data:
+            # res[:] = sino_filtered[..., : self.dwidth]  # pylint: disable=E1126 # ?!
+            res[:] = sino_filtered[..., self.pad_left : -self.pad_right]  # pylint: disable=E1126 # ?!
         else:
-            # res[:] = sino_filtered[:, :, : self.dwidth]  # pylint: disable=E1126 # ?!
-            res[:] = sino_filtered[:, :, self.pad_left : -self.pad_right]  # pylint: disable=E1126 # ?!
+            res[:] = sino_filtered[:]
+
         return res
 
     __call__ = filter_sino
@@ -191,6 +222,7 @@ def filter_sinogram(
     padding_mode="constant",
     normalize=True,
     filter_cutoff=1.0,
+    crop_filtered_data=True,
     **padding_kwargs,
 ):
     """
@@ -228,6 +260,8 @@ def filter_sinogram(
     fourier_filter = fourier_filter[: padded_width // 2 + 1]  # R2C
     sino_f = rfft(sinogram_padded, axis=1)
     sino_f *= fourier_filter
-    # sino_filtered = irfft(sino_f, axis=1)[:, :width]  # pylint: disable=E1126 # ?!
-    sino_filtered = irfft(sino_f, axis=1)[:, pad_left:-pad_right]  # pylint: disable=E1126 # ?!
+    sino_filtered = irfft(sino_f, axis=1)
+    if crop_filtered_data:
+        # sino_filtered = sino_filtered[:, :width]  # pylint: disable=E1126 # ?!
+        sino_filtered = sino_filtered[:, pad_left:-pad_right]  # pylint: disable=E1126 # ?!
     return sino_filtered
