@@ -1,22 +1,28 @@
+# Generalized Hierarchical Backprojection (GHBP)
+# for fast tomographic reconstruction from ultra high resolution images at non-negligible fan angles.
+#
+# Authors/Contributions:
+# - Jonas Graetz, Fraunhofer IIS / Universität Würzburg: Algorithm Design and original OpenCL/Python implementation.
+# - Alessandro Mirone, ESRF: CUDA translation, ESRF / BM18 integration, testing <mirone@esrf.fr>
+# - Pierre Paleo, ESRF: ESRF / BM18 integration, testing <pierre.paleo@esrf.fr>
+#
+# JG was funded by the German Federal Ministry of Education and Research (BMBF), grant 05E2019,
+# funding the development of BM18 at ESRF in collaboration with the Fraunhofer Gesellschaft,
+# the Julius-Maximilians-Universität Würzburg, and the University of Passau
+
 import math
 import numpy as np
 
 from ..utils import get_cuda_srcfile
-from ..cuda.processing import __has_pycuda__
+from ..cuda.processing import __has_cupy__
 
-if __has_pycuda__:
+if __has_cupy__:
     from ..cuda.kernel import CudaKernel
     from .sinogram_cuda import CudaSinoMult
-
-from .fbp import CudaBackprojector
-
-
-try:
-    import pycuda.driver as cuda
-    from pycuda import gpuarray as garray
+    from .fbp import CudaBackprojector
 
     __have_hbp__ = True
-except:
+else:
     __have_hbp__ = False
 
 
@@ -60,7 +66,7 @@ class HierarchicalBackprojector(CudaBackprojector):
 
         # to do the reconstruction in reduction_steps steps
         self.reduction_steps = self.extra_options.get("hbp_reduction_steps", 2)
-        reduction_factor = int(math.ceil((sino_shape[-2]) ** (1 / self.reduction_steps)))
+        reduction_factor = math.ceil((sino_shape[-2]) ** (1 / self.reduction_steps))
 
         # TODO customize
         axis_source_meters = 1.0e9
@@ -138,15 +144,15 @@ class HierarchicalBackprojector(CudaBackprojector):
 
         N = self.slice_shape[1] * fac
 
-        angularRange = abs(self.angles.ptp()) / self.sino_shape[0] * reductionFactor
+        angularRange = abs(np.ptp(self.angles)) / self.sino_shape[0] * reductionFactor
 
-        ngrids = int(math.ceil(self.sino_shape[0] / reductionFactor))
+        ngrids = math.ceil(self.sino_shape[0] / reductionFactor)
 
         grid_width = int(
             np.rint(2 * N * self.whf[0])
         )  # double sampling to account/compensate for diamond shaped grid of ray-intersections
-        grid_height = int(
-            math.ceil(angularRange * N * self.whf[1])
+        grid_height = math.ceil(
+            angularRange * N * self.whf[1]
         )  # small-angle approximation, generates as much "lines" as needed to account for all intersection levels
 
         m = (len(self.angles) // reductionFactor) * reductionFactor
@@ -166,7 +172,7 @@ class HierarchicalBackprojector(CudaBackprojector):
             )
         ]
         self.gridInvTransforms += [np.array([np.linalg.inv(t) for t in self.gridTransforms[-1]], dtype=np.float32)]
-        self.grids += [(grid_height, grid_width, int(math.ceil(ngrids / legs)))]
+        self.grids += [(grid_height, grid_width, math.ceil(ngrids / legs))]
         self.reductionFactors += [reductionFactor]
 
         ### intermediate level grids: accumulation grids ###
@@ -177,13 +183,13 @@ class HierarchicalBackprojector(CudaBackprojector):
             # for a reasonable (with regard to memory requirement) grid-aspect ratio in the intermediate levels,
             # the covered angular range per grid should not exceed 28.6°, i.e.,
             # fewer than 7 (6.3) or 13 (12.6)  grids for a 180° / 360° scan is not reasonable
-            if int(math.ceil(ngrids / reductionFactor)) < 20:
+            if math.ceil(ngrids / reductionFactor) < 20:
                 break
             angularRange *= reductionFactor
-            ngrids = int(math.ceil(ngrids / reductionFactor))
+            ngrids = math.ceil(ngrids / reductionFactor)
 
-            grid_height = int(
-                math.ceil(angularRange * N * self.whf[1])
+            grid_height = math.ceil(
+                angularRange * N * self.whf[1]
             )  # implicit small angle approximation, whose validity is
             # asserted by the preceding "break"
             gridAinvT = self._getAinvT(N, grid_height, grid_width)
@@ -205,7 +211,7 @@ class HierarchicalBackprojector(CudaBackprojector):
                 )
             ]
             self.gridInvTransforms += [np.array([np.linalg.inv(t) for t in self.gridTransforms[-1]], dtype=np.float32)]
-            self.grids += [(grid_height, grid_width, int(math.ceil(ngrids / legs)))]
+            self.grids += [(grid_height, grid_width, math.ceil(ngrids / legs))]
             self.reductionFactors += [reductionFactor]
 
         ##### final accumulation grid #################
@@ -281,17 +287,12 @@ class HierarchicalBackprojector(CudaBackprojector):
         self.backprojector = CudaKernel(
             "backprojector",
             filename=get_cuda_srcfile(self.kernel_filename),
-            signature="PPiiiiPiifPi",
-        )
-        self.aggregator = CudaKernel(
-            "aggregator", filename=get_cuda_srcfile(self.kernel_filename), signature="iPPiiiiPiiiP"
-        )
-        self.clip_outer_circle_kernel = CudaKernel(
-            "clip_outer_circle", filename=get_cuda_srcfile(self.kernel_filename), signature="Pii"
         )
+        self.aggregator = CudaKernel("aggregator", filename=get_cuda_srcfile(self.kernel_filename))
+        self.clip_outer_circle_kernel = CudaKernel("clip_outer_circle", filename=get_cuda_srcfile(self.kernel_filename))
         # Duplicate of fbp.py ...
         if self.halftomo and self.rot_center < self.dwidth:
-            self.sino_mult = CudaSinoMult(self.sino_shape, self.rot_center, ctx=self._processing.ctx)
+            self.sino_mult = CudaSinoMult(self.sino_shape, self.rot_center)  # , ctx=self._processing.ctx)
         #
 
     def _set_sino(self, sino, do_checks=True):
@@ -331,7 +332,7 @@ class HierarchicalBackprojector(CudaBackprojector):
             )
 
         else:
-            for leg in list(range(0, self.legs)):
+            for leg in list(range(self.legs)):
                 gridOffset = leg * self.grids[0][2]
                 projOffset = gridOffset * self.reductionFactors[0]
                 gws = getGridSize(self.grids[0], lws)
@@ -403,7 +404,12 @@ class HierarchicalBackprojector(CudaBackprojector):
         # --------
 
         else:
-            return self.gridsD[-1].reshape(self.slice_shape).get(ary=output)
+            res = self.gridsD[-1].reshape(self.slice_shape)
+            if isinstance(output, self.backend_processing_class.array_class):
+                output[:] = res[:]
+                return output
+            else:
+                return res.get(out=output)
 
 
 def get_max_grid_size(grids):
@@ -418,7 +424,7 @@ def get_max_grid_size(grids):
 
 
 def getGridSize(minimum, local):
-    m, l = np.array(minimum), np.array(local)
+    m, l = np.array(minimum), np.array(local)  # noqa: E741
     new = (m // l) * l
     new[new < m] += l[new < m]
     return tuple(map(int, new // l))
