@@ -290,7 +290,7 @@ def match_half_sinos_parts(sino, angles, output=None):
     """
     n_a = angles.size
     n_a_2 = n_a // 2
-    sino_part1 = sino[:n_a_2, :]
+    # sino_part1 = sino[:n_a_2, :]
     sino_part2 = sino[n_a_2:, :]
     angles = np.rad2deg(angles)  # more numerically stable ?
     angles_1 = angles[:n_a_2]
