@@ -1,7 +1,9 @@
+import logging
 import pytest
 import numpy as np
 from scipy.ndimage import gaussian_filter, shift
 from nabu.utils import subdivide_into_overlapping_segment, clip_circle
+from nabu.testutils import __do_long_tests__, generate_tests_scenarios
 
 try:
     import astra
@@ -9,10 +11,10 @@ try:
     __has_astra__ = True
 except ImportError:
     __has_astra__ = False
-from nabu.cuda.utils import __has_pycuda__, get_cuda_context
+from nabu.cuda.utils import __has_cupy__
 
-if __has_pycuda__:
-    from nabu.reconstruction.cone import ConebeamReconstructor
+if __has_cupy__:
+    from nabu.reconstruction.cone import ConebeamReconstructor, NumpyConebeamReconstructor
 if __has_astra__:
     from astra.extrautils import clipCircle
 
@@ -28,11 +30,11 @@ def bootstrap(request):
     cls.volume, cls.cone_data = generate_hollow_cube_cone_sinograms(
         cls.vol_shape, cls.n_angles, cls.src_orig_dist, cls.orig_det_dist, prj_width=cls.prj_width
     )
-    if __has_pycuda__:
-        cls.ctx = get_cuda_context()
+    if __has_cupy__:
+        ...
 
 
-@pytest.mark.skipif(not (__has_pycuda__ and __has_astra__), reason="Need pycuda and astra for this test")
+@pytest.mark.skipif(not (__has_cupy__ and __has_astra__), reason="Need cupy and astra for this test")
 @pytest.mark.usefixtures("bootstrap")
 class TestCone:
     def _create_cone_reconstructor(self, relative_z_position=None):
@@ -42,7 +44,7 @@ class TestCone:
             self.orig_det_dist,
             relative_z_position=relative_z_position,
             volume_shape=self.volume.shape,
-            cuda_options={"ctx": self.ctx},
+            # cuda_options={"ctx": self.ctx},
         )
 
     def test_simple_cone_reconstruction(self):
@@ -123,7 +125,9 @@ class TestCone:
         volume, cone_data = generate_hollow_cube_cone_sinograms(
             vol_shape=(n_z, n_y, n_x), n_angles=n_a, src_orig_dist=src_orig_dist, orig_det_dist=orig_det_dist
         )
-        C_full = ConebeamReconstructor(cone_data.shape, src_orig_dist, orig_det_dist, cuda_options={"ctx": self.ctx})
+        C_full = ConebeamReconstructor(
+            cone_data.shape, src_orig_dist, orig_det_dist
+        )  # , cuda_options={"ctx": self.ctx})
 
         vol_geom = astra.create_vol_geom(n_y, n_x, n_z)
 
@@ -147,7 +151,7 @@ class TestCone:
                 src_orig_dist,
                 orig_det_dist,
                 relative_z_position=rel_z_pos,
-                cuda_options={"ctx": self.ctx},
+                # cuda_options={"ctx": self.ctx},
             )
             proj_id, projs = astra.create_sino3d_gpu(subvolume, C.proj_geom, C.vol_geom)
             astra.data3d.delete(proj_id)
@@ -264,7 +268,7 @@ class TestCone:
         ]
         reconstructor_kwargs = {
             "volume_shape": volume.shape,
-            "cuda_options": {"ctx": self.ctx},
+            # "cuda_options": {"ctx": self.ctx},
         }
         cone_reconstructor = ConebeamReconstructor(*reconstructor_args, **reconstructor_kwargs)
         rec = cone_reconstructor.reconstruct(cone_data)
@@ -273,7 +277,7 @@ class TestCone:
         # Error tolerance has to be higher for these shifts.
         for shift_type, shifts, err_tol in [
             ("integer shifts", shifts_int, 5e-3),
-            ("float shifts", shifts_float, 1.5e-1),
+            ("float shifts", shifts_float, 1.6e-1),
         ]:
             cone_data_shifted = np.zeros_like(cone_data)
             [shift(cone_data[:, i, :], (0, shifts[i]), output=cone_data_shifted[:, i, :]) for i in range(n_a)]
@@ -291,12 +295,6 @@ class TestCone:
             error_profile = np.array([metric(rec[i] - rec_with_correction[i]) for i in range(n_z)])
             assert error_profile.max() < err_tol, "Max error with %s is too high" % shift_type
 
-            # import matplotlib.pyplot as plt
-            # plt.figure()
-            # plt.plot(np.arange(n_z), error_profile)
-            # plt.legend([shift_type])
-            # plt.show()
-
     def test_padding_mode(self):
         n_z = n_y = n_x = 256
         n_a = 500
@@ -317,7 +315,7 @@ class TestCone:
         ]
         reconstructor_kwargs = {
             "volume_shape": volume.shape,
-            "cuda_options": {"ctx": self.ctx},
+            # "cuda_options": {"ctx": self.ctx},
         }
         cone_reconstructor_zero_padding = ConebeamReconstructor(*reconstructor_args, **reconstructor_kwargs)
         rec_z = cone_reconstructor_zero_padding.reconstruct(cone_data)
@@ -366,7 +364,7 @@ class TestCone:
         reconstructor_kwargs = {
             "volume_shape": volume.shape,
             "rot_center": (n_x - 1) / 2 + 10,
-            "cuda_options": {"ctx": self.ctx},
+            # "cuda_options": {"ctx": self.ctx},
         }
         cone_reconstructor_full = ConebeamReconstructor(*reconstructor_args, **reconstructor_kwargs)
         ref = cone_reconstructor_full.reconstruct(cone_data)
@@ -387,6 +385,187 @@ class TestCone:
                 str(roi)
             )
 
+    def test_fdk_preweight(self, caplog):
+        """
+        Check that nabu's FDK pre-weighting give the same results as astra
+        """
+        shapes = [
+            {"n_z": 256, "n_x": 256, "n_a": 500},
+            # {"n_z": 250, "n_x": 340, "n_a": 250}, # Astra reconstruction is incorrect in this case!
+        ]
+        src_orig_dist = 1000
+        orig_det_dist = 50
+
+        rot_centers_from_middle = [0]
+        if __do_long_tests__:
+            rot_centers_from_middle.extend([10, -15])
+
+        params_list = generate_tests_scenarios({"shape": shapes, "rot_center": rot_centers_from_middle})
+
+        for params in params_list:
+            n_z = params["shape"]["n_z"]
+            n_x = n_y = params["shape"]["n_x"]
+            n_a = params["shape"]["n_a"]
+            rc = params["rot_center"]
+            volume, cone_data = generate_hollow_cube_cone_sinograms(
+                vol_shape=(n_z, n_y, n_x),
+                n_angles=n_a,
+                src_orig_dist=src_orig_dist,
+                orig_det_dist=orig_det_dist,
+                apply_filter=False,
+                rot_center_shift=rc,
+            )
+
+            reconstructor_args = [(n_z, n_a, n_x), src_orig_dist, orig_det_dist]
+            reconstructor_kwargs_base = {
+                "volume_shape": volume.shape,
+                "rot_center": (n_x - 1) / 2 + rc,
+                # "cuda_options": {"ctx": self.ctx},
+            }
+            reconstructor_kwargs_astra = {"padding_mode": "zeros", "extra_options": {"use_astra_fdk": True}}
+            reconstructor_kwargs_nabu = {"padding_mode": "zeros", "extra_options": {"use_astra_fdk": False}}
+            reconstructor_astra = ConebeamReconstructor(
+                *reconstructor_args, **{**reconstructor_kwargs_base, **reconstructor_kwargs_astra}
+            )
+            assert reconstructor_astra._use_astra_fdk is True, "reconstructor_astra should use native astra FDK"
+            reconstructor_nabu = ConebeamReconstructor(
+                *reconstructor_args, **{**reconstructor_kwargs_base, **reconstructor_kwargs_nabu}
+            )
+            ref = reconstructor_astra.reconstruct(cone_data)
+            res = reconstructor_nabu.reconstruct(cone_data)
+
+            reconstructor_kwargs_nabu = {"padding_mode": "edges", "extra_options": {"use_astra_fdk": False}}
+            cb_ep = ConebeamReconstructor(
+                *reconstructor_args, **{**reconstructor_kwargs_base, **reconstructor_kwargs_nabu}
+            )
+            res_ep = cb_ep.reconstruct(cone_data)  # noqa: F841
+
+            assert np.max(np.abs(res - ref)) < 2e-3, "Wrong FDK results for parameters: %s" % (str(params))
+
+            # Test with edges padding - only nabu can do that
+            reconstructor_kwargs_nabu["padding_mode"] = "edges"
+            reconstructor_nabu = ConebeamReconstructor(
+                *reconstructor_args, **{**reconstructor_kwargs_base, **reconstructor_kwargs_nabu}
+            )
+            reconstructor_nabu.reconstruct(cone_data)
+            # result is slightly different than "res" in the borders, which is expected
+            # it would be good to test it as well, but it's outside of the scope of this test
+
+            with caplog.at_level(logging.WARNING):
+                reconstructor_kwargs_nabu = {"padding_mode": "edges", "extra_options": {"use_astra_fdk": True}}
+                ConebeamReconstructor(*reconstructor_args, **{**reconstructor_kwargs_base, **reconstructor_kwargs_nabu})
+                assert "cannot use native astra FDK" in caplog.text
+
+    def test_reconstruct_noncontiguous_data(self):
+        n_z = 206
+        n_y = n_x = 256
+        n_a = 500
+        src_orig_dist = 1000
+        orig_det_dist = 50
+
+        volume, cone_data = generate_hollow_cube_cone_sinograms(
+            vol_shape=(n_z, n_y, n_x),
+            n_angles=n_a,
+            src_orig_dist=src_orig_dist,
+            orig_det_dist=orig_det_dist,
+            apply_filter=False,
+            rot_center_shift=10,
+        )
+        cone_reconstructor = ConebeamReconstructor(
+            cone_data.shape,
+            src_orig_dist,
+            orig_det_dist,
+            volume_shape=volume.shape,
+            rot_center=(n_x - 1) / 2 + 10,
+            # cuda_options={"ctx": self.ctx},
+            extra_options={"use_astra_fdk": False},
+        )
+        ref = cone_reconstructor.reconstruct(cone_data)
+
+        radios = cone_reconstructor.cuda.allocate_array("_radios", (n_a, n_z, n_x))
+        for i in range(n_a):
+            radios[i].set(cone_data[:, i, :])
+
+        sinos_discontig = radios.transpose((1, 0, 2))
+        assert cone_reconstructor.cuda.is_contiguous(sinos_discontig) is False
+        res = cone_reconstructor.reconstruct(sinos_discontig)
+        assert np.allclose(res, ref), "Reconstructing non-contiguous data failed"
+
+    def test_cone_clip_circle(self):
+        n_z = 206
+        n_y = n_x = 256
+        n_a = 500
+        src_orig_dist = 1000
+        orig_det_dist = 50
+
+        volume, cone_data = generate_hollow_cube_cone_sinograms(
+            vol_shape=(n_z, n_y, n_x),
+            n_angles=n_a,
+            src_orig_dist=src_orig_dist,
+            orig_det_dist=orig_det_dist,
+            apply_filter=False,
+            rot_center_shift=10,
+        )
+        init_args = [cone_data.shape, src_orig_dist, orig_det_dist]
+        init_kwargs = {
+            "volume_shape": volume.shape,
+            "rot_center": (n_x - 1) / 2 + 10,
+            "extra_options": {"use_astra_fdk": False, "clip_outer_circle": True},
+        }
+
+        # for cone_reconstructor_cls in [ConebeamReconstructor, NumpyConebeamReconstructor]:
+        for cone_reconstructor_cls in [NumpyConebeamReconstructor]:
+            for out_circle_value in [0, np.nan]:
+                init_kwargs["extra_options"]["outer_circle_value"] = out_circle_value
+                cone_reconstructor = cone_reconstructor_cls(*init_args, **init_kwargs)
+                rec = cone_reconstructor.reconstruct(cone_data)
+                for i in range(rec.shape[0]):
+                    err_msg = f"Something wrong with clip_circle for {cone_reconstructor.__class__.__name__} with outer value {out_circle_value} at slice number {i}"
+                    assert np.allclose(clip_circle(rec[i], out_value=out_circle_value), rec[i], equal_nan=True), err_msg
+
+    def _test_numpy_cone(self, cor_shift, padding_mode):
+        d1 = 1000
+        d2 = 10
+
+        rot_center = None
+        neg_cor_shift = None
+        cube, sinos = generate_hollow_cube_cone_sinograms((256, 256, 200), 200, d1, d2, rot_center_shift=cor_shift)
+
+        if cor_shift is not None:
+            rot_center = (sinos.shape[-1] - 1) / 2 + cor_shift
+            neg_cor_shift = -cor_shift
+        rec0 = astra_cone_beam_reconstruction(
+            sinos, np.linspace(0, 2 * np.pi, sinos.shape[1], True), d1, d2, cor_shift=neg_cor_shift
+        )
+
+        cone_reconstructor = NumpyConebeamReconstructor(
+            sinos.shape, d1, d2, rot_center=rot_center, padding_mode=padding_mode
+        )
+        rec = cone_reconstructor.reconstruct(sinos)
+
+        err_profile = np.max(np.abs(rec - rec0), axis=(1, 2))
+        tol = 2e-3
+
+        if padding_mode not in ["zeros", "constant"]:
+            # Astra does not support natively padding mode other than zeros
+            # Here we ignore the region close to the edges, since edges-padding gives a different result
+            err_profile = np.max(np.abs(rec[:, 50:-50, 50:-50] - rec0[:, 50:-50, 50:-50]), axis=(1, 2))[50:-50]
+            tol = 0.15
+
+        assert (
+            np.max(err_profile) < tol
+        ), f"NumpyConebeamReconstructor: FDK reconstruction failed for cor_shift={cor_shift}, padding_mode={padding_mode}"
+
+    def test_numpy_cone(self):
+        test_cases = {
+            "cor_shift": [None, 6.5],
+            "padding_mode": ["zeros", "edges"],
+        }
+        params_list = generate_tests_scenarios(test_cases)
+
+        for params in params_list:
+            self._test_numpy_cone(params["cor_shift"], params["padding_mode"])
+
 
 def generate_hollow_cube_cone_sinograms(
     vol_shape,
@@ -403,13 +582,13 @@ def generate_hollow_cube_cone_sinograms(
     vol_geom = astra.create_vol_geom(n_y, n_x, n_z)
 
     prj_width = prj_width or n_x
-    prj_height = n_z
+    # prj_height = n_z
     angles = np.linspace(0, 2 * np.pi, n_angles, True)
 
-    proj_geom = astra.create_proj_geom("cone", 1.0, 1.0, prj_width, prj_width, angles, src_orig_dist, orig_det_dist)
+    proj_geom = astra.create_proj_geom("cone", 1.0, 1.0, n_z, prj_width, angles, src_orig_dist, orig_det_dist)
     if rot_center_shift is not None:
         proj_geom = astra.geom_postalignment(proj_geom, (-rot_center_shift, 0))
-    magnification = 1 + orig_det_dist / src_orig_dist
+    # magnification = 1 + orig_det_dist / src_orig_dist
 
     # hollow cube
     cube = np.zeros(astra.geom_size(vol_geom), dtype="f")
@@ -434,7 +613,9 @@ def generate_hollow_cube_cone_sinograms(
     return cube, proj_data
 
 
-def astra_cone_beam_reconstruction(cone_data, angles, src_orig_dist, orig_det_dist, demagnify_volume=False):
+def astra_cone_beam_reconstruction(
+    cone_data, angles, src_orig_dist, orig_det_dist, demagnify_volume=False, cor_shift=None
+):
     """
     Handy (but data-inefficient) function to reconstruct data from cone-beam geometry
     """
@@ -442,6 +623,8 @@ def astra_cone_beam_reconstruction(cone_data, angles, src_orig_dist, orig_det_di
     n_z, n_a, n_x = cone_data.shape
 
     proj_geom = astra.create_proj_geom("cone", 1.0, 1.0, n_z, n_x, angles, src_orig_dist, orig_det_dist)
+    if cor_shift is not None:
+        proj_geom = astra.geom_postalignment(proj_geom, (cor_shift, 0))
     sino_id = astra.data3d.create("-sino", proj_geom, data=cone_data)
 
     m = 1 + orig_det_dist / src_orig_dist if demagnify_volume else 1.0
