@@ -1,13 +1,13 @@
 import numpy as np
 import pytest
-from nabu.reconstruction.rings_cuda import CudaSinoMeanDeringer
 from nabu.testutils import compare_arrays, get_data, generate_tests_scenarios, __do_long_tests__
 from nabu.reconstruction.rings import MunchDeringer, SinoMeanDeringer, VoDeringer, __has_algotom__
 from nabu.thirdparty.pore3d_deringer_munch import munchetal_filter
-from nabu.cuda.utils import __has_pycuda__, get_cuda_context
+from nabu.cuda.utils import __has_cupy__
 
-if __has_pycuda__:
-    import pycuda.gpuarray as garray
+if __has_cupy__:
+    import cupy
+    from nabu.reconstruction.rings_cuda import CudaSinoMeanDeringer
     from nabu.processing.fft_cuda import get_available_fft_implems
     from nabu.thirdparty.tomocupy_remove_stripe import __have_tomocupy_deringer__
 
@@ -38,7 +38,7 @@ if __do_long_tests__:
             "sigma": [1.0, 2.0],
             "wname": ["db15", "haar", "rbio4.4"],
             "padding": [None, (100, 100), (50, 71)],
-            "fft_implem": ["skcuda", "vkfft"],
+            "fft_implem": ["vkfft"],
         }
     )
 
@@ -50,12 +50,11 @@ def bootstrap(request):
     cls.sino2 = get_data("sino_bamboo_hercules.npz")["data"]
     cls.tol = 5e-3
     cls.rings = {150: 0.5, -150: 0.5}
-    if __has_pycuda__:
-        cls.ctx = get_cuda_context(cleanup_at_exit=False)
+    if __has_cupy__:
         cls._available_fft_implems = get_available_fft_implems()
     yield
-    if __has_pycuda__:
-        cls.ctx.pop()
+    if __has_cupy__:
+        ...
 
 
 @pytest.mark.usefixtures("bootstrap")
@@ -107,7 +106,7 @@ class TestDeringer:
 
     @pytest.mark.skipif(
         not (__has_cuda_deringer__) or munchetal_filter is None,
-        reason="Need pycuda, pycudwt and (scikit-cuda or pyvkfft) for this test",
+        reason="Need cupy and pyvkfft for this test",
     )
     @pytest.mark.parametrize("config", fw_scenarios)
     def test_cuda_munch_deringer(self, config):
@@ -122,9 +121,9 @@ class TestDeringer:
             wname=config["wname"],
             padding=config["padding"],
             fft_backend=fft_implem,
-            cuda_options={"ctx": self.ctx},
+            # cuda_options={"ctx": self.ctx},
         )
-        d_sino = garray.to_gpu(sino)
+        d_sino = deringer.cuda_processing.to_device("d_sino", sino)
         deringer.remove_rings(d_sino)
         res = d_sino.get()
 
@@ -139,9 +138,9 @@ class TestDeringer:
     )
     def test_vo_deringer(self):
         deringer = VoDeringer(self.sino.shape)
-        sino_deringed = deringer.remove_rings_sinogram(self.sino)
+        sino_deringed = deringer.remove_rings_sinogram(self.sino)  # noqa: F841
         sinos = np.tile(self.sino, (10, 1, 1))
-        sinos_deringed = deringer.remove_rings_sinograms(sinos)
+        sinos_deringed = deringer.remove_rings_sinograms(sinos)  # noqa: F841
         # TODO check result. The generated test sinogram is "too synthetic" for this kind of deringer
 
     @pytest.mark.skipif(
@@ -153,7 +152,7 @@ class TestDeringer:
         # (NaNs on the edges and in some regions). To be investigated
 
         deringer = CudaVoDeringer(self.sino2.shape)
-        d_sino = garray.to_gpu(self.sino2)
+        d_sino = cupy.array(self.sino2)
         deringer.remove_rings_sinogram(d_sino)
         sino = d_sino.get()
 
@@ -176,7 +175,7 @@ class TestDeringer:
         deringer_with_filtering.remove_rings_sinogram(sino)
         # TODO check results
 
-    @pytest.mark.skipif(not (__has_pycuda__), reason="Need pycuda for this test")
+    @pytest.mark.skipif(not (__has_cupy__), reason="Need cupy for this test")
     def test_cuda_mean_deringer(self):
         cuda_deringer = CudaSinoMeanDeringer(
             self.sino.shape,
@@ -185,7 +184,7 @@ class TestDeringer:
                 0,
                 10,
             ),
-            ctx=self.ctx,
+            # ctx=self.ctx,
         )
         deringer = SinoMeanDeringer(
             self.sino.shape,
