@@ -4,16 +4,16 @@ from scipy.ndimage import shift
 from nabu.pipeline.params import fbp_filters
 from nabu.utils import clip_circle
 from nabu.testutils import get_data, generate_tests_scenarios, __do_long_tests__
-from nabu.cuda.utils import get_cuda_context, __has_pycuda__
+from nabu.cuda.utils import __has_cupy__, get_cuda_stream
 from nabu.opencl.utils import get_opencl_context, __has_pyopencl__
 
-from nabu.processing.fft_cuda import has_skcuda, has_vkfft as has_vkfft_cu
+from nabu.processing.fft_cuda import has_vkfft as has_vkfft_cu
 from nabu.processing.fft_opencl import has_vkfft as has_vkfft_cl
 
-__has_pycuda__ = __has_pycuda__ and (has_skcuda() or has_vkfft_cu())
+__has_cupy__ = __has_cupy__ and has_vkfft_cu()
 __has_pyopencl__ = __has_pyopencl__ and has_vkfft_cl()
 
-if __has_pycuda__:
+if __has_cupy__:
     from nabu.reconstruction.fbp import CudaBackprojector
     from nabu.reconstruction.hbp import HierarchicalBackprojector
 if __has_pyopencl__:
@@ -40,15 +40,15 @@ def bootstrap(request):
     # always use contiguous arrays
     cls.sino_511 = np.ascontiguousarray(cls.sino_512[:, :-1])
     # Could be set to 5.0e-2 when using textures. When not using textures, interpolation slightly differs
-    cls.tol = 5.1e-2
+    cls.tol = 2e-2  # 5.1e-2
 
-    if __has_pycuda__:
-        cls.cuda_ctx = get_cuda_context(cleanup_at_exit=False)
+    if __has_cupy__:
+        stream = get_cuda_stream(force_create=True)
     if __has_pyopencl__:
         cls.opencl_ctx = get_opencl_context("all")
     yield
-    if __has_pycuda__:
-        cls.cuda_ctx.pop()
+    if __has_cupy__:
+        del stream  # ?
 
 
 def clip_to_inner_circle(img, radius_factor=0.99, out_value=0):
@@ -61,10 +61,10 @@ class TestFBP:
 
     def _get_backprojector(self, config, *bp_args, **bp_kwargs):
         if config["backend"] == "cuda":
-            if not (__has_pycuda__):
-                pytest.skip("Need pycuda + (scikit-cuda or pyvkfft)")
+            if not (__has_cupy__):
+                pytest.skip("Need cupy and pyvkfft")
             Backprojector = CudaBackprojector
-            ctx = self.cuda_ctx
+            ctx = None  # self.cuda_ctx
         else:
             if not (__has_pyopencl__):
                 pytest.skip("Need pyopencl + pyvkfft")
@@ -98,10 +98,14 @@ class TestFBP:
         B = self._get_backprojector(config, (500, 512))
         res = self.apply_fbp(config, B, self.sino_512)
 
-        delta_clipped = clip_to_inner_circle(res - self.ref_512)
-        err_max = np.max(np.abs(delta_clipped))
+        diff = res - self.ref_512
+        tol = self.tol
+        if not (B._use_textures):
+            diff = clip_to_inner_circle(diff)
+            tol = 5.1e-2
+        err_max = np.max(np.abs(diff))
 
-        assert err_max < self.tol, "Something wrong with config=%s" % (str(config))
+        assert err_max < tol, "Something wrong with config=%s" % (str(config))
 
     @pytest.mark.parametrize("config", scenarios)
     def test_fbp_511(self, config):
@@ -112,10 +116,29 @@ class TestFBP:
         res = self.apply_fbp(config, B, self.sino_511)
         ref = self.ref_512[:-1, :-1]
 
-        delta_clipped = clip_to_inner_circle(res - ref)
-        err_max = np.max(np.abs(delta_clipped))
+        diff = clip_to_inner_circle(res - ref)
+        err_max = np.max(np.abs(diff))
+        tol = self.tol
+        if not (B._use_textures):
+            tol = 5.1e-2
 
-        assert err_max < self.tol, "Something wrong with config=%s" % (str(config))
+        assert err_max < tol, "Something wrong with config=%s" % (str(config))
+
+        # Cropping the singoram to sino[:, :-1]  gives a reconstruction
+        # that is not fully equivalent to rec512[:-1, :-1]  in the upper half of the image, outside FoV.
+        # However, nabu Backprojector gives the same results as astra
+        # Probably we should check this instead:
+
+        # B = self._get_backprojector(config, (500, 511), rot_center=255.5, extra_options={"centered_axis": True})
+        # res = self.apply_fbp(config, B, self.sino_511)
+        # import astra
+        # proj_geom = astra.create_proj_geom('parallel', 1, 511, B.angles)
+        # proj_geom = astra.geom_postalignment(proj_geom, - 0.5)
+        # vol_geom = astra.create_vol_geom(511, 511)
+        # proj_id = astra.create_projector("cuda", proj_geom, vol_geom)
+        # ref = astra.create_reconstruction("FBP_CUDA", proj_id, self.sino_511, proj_id)[1]
+        # err_max = np.max(np.abs(res - ref))
+        # assert err_max < self.tol, "Something wrong with config=%s" % (str(config))
 
     @pytest.mark.parametrize("config", scenarios)
     def test_fbp_roi(self, config):
@@ -194,8 +217,7 @@ class TestFBP:
             )
             res_noclip = B0.fbp(sino)
             ref = clip_to_inner_circle(res_noclip, radius_factor=1)
-            abs_diff = np.abs(res - ref)
-            err_max = np.max(abs_diff)
+            err_max = np.max(np.abs(res - ref))
             assert err_max < tol, "Max error is too high for rot_center=%s ; %s" % (str(rot_center), str(config))
 
             # Test with custom outer circle value
@@ -223,7 +245,6 @@ class TestFBP:
         ref = B0.fbp(self.sino_512)
 
         # Check that "centered_axis" worked
-
         B = self._get_backprojector(config, sino.shape, rot_center=rot_center, extra_options={"centered_axis": True})
         res = self.apply_fbp(config, B, sino)
         # The outside region (outer circle) is different as "res" is a wider slice
@@ -262,11 +283,11 @@ class TestFBP:
         # Need to translate the axis a little bit, because of non-centered differentiation.
         # prepend -> +0.5 ; append -> -0.5
         B = self._get_backprojector(config, sino_diff.shape, filter_name="hilbert", rot_center=255.5 + 0.5)
-        rec = self.apply_fbp(config, B, sino_diff)
+        rec = self.apply_fbp(config, B, sino_diff)  # noqa: F841
         # Looks good, but all frequencies are not recovered. Use a metric like SSIM or FRC ?
 
 
-@pytest.mark.skipif(not (__has_pycuda__), reason="Need pycuda for using HBP")
+@pytest.mark.skipif(not (__has_cupy__), reason="Need cupy for using HBP")
 @pytest.mark.usefixtures("bootstrap")
 class TestHBP:
 
