@@ -1,24 +1,25 @@
 import numpy as np
 import pytest
 from nabu.reconstruction.filtering import SinoFilter, filter_sinogram
-from nabu.cuda.utils import __has_pycuda__
+from nabu.cuda.utils import __has_cupy__
 from nabu.opencl.utils import __has_pyopencl__
 from nabu.testutils import get_data, generate_tests_scenarios, __do_long_tests__
 
-if __has_pycuda__:
-    from nabu.cuda.utils import get_cuda_context
+if __has_cupy__:
     from nabu.reconstruction.filtering_cuda import CudaSinoFilter
-    import pycuda.gpuarray as garray
+    import cupy
 if __has_pyopencl__:
     import pyopencl.array as parray
     from nabu.opencl.processing import OpenCLProcessing
     from nabu.reconstruction.filtering_opencl import OpenCLSinoFilter, __has_vkfft__
 
-filters_to_test = ["ramlak", "shepp-logan", "tukey"]
+filters_to_test = ["ramlak", "shepp-logan"]
 padding_modes_to_test = ["constant", "edge"]
+crop_filtered_data = [True]
 if __do_long_tests__:
-    filters_to_test = ["ramlak", "shepp-logan", "cosine", "hamming", "hann", "tukey", "lanczos"]
+    filters_to_test.extend(["cosine", "hamming", "hann", "lanczos"])
     padding_modes_to_test = SinoFilter.available_padding_modes
+    crop_filtered_data = [True, False]
 
 tests_scenarios = generate_tests_scenarios(
     {
@@ -26,6 +27,7 @@ tests_scenarios = generate_tests_scenarios(
         "padding_mode": padding_modes_to_test,
         "output_provided": [True, False],
         "truncated_sino": [True, False],
+        "crop_filtered_data": crop_filtered_data,
     }
 )
 
@@ -36,10 +38,9 @@ def bootstrap(request):
     cls.sino = get_data("mri_sino500.npz")["data"]
     cls.sino_truncated = np.ascontiguousarray(cls.sino[:, 160:-160])
 
-    if __has_pycuda__:
-        cls.ctx_cuda = get_cuda_context(cleanup_at_exit=False)
-        cls.sino_cuda = garray.to_gpu(cls.sino)
-        cls.sino_truncated_cuda = garray.to_gpu(cls.sino_truncated)
+    if __has_cupy__:
+        cls.sino_cuda = cupy.array(cls.sino)
+        cls.sino_truncated_cuda = cupy.array(cls.sino_truncated)
     if __has_pyopencl__:
         cls.cl = OpenCLProcessing(device_type="all")
         cls.sino_cl = parray.to_device(cls.cl.queue, cls.sino)
@@ -47,8 +48,8 @@ def bootstrap(request):
 
     yield
 
-    if __has_pycuda__:
-        cls.ctx_cuda.pop()
+    if __has_cupy__:
+        ...
 
 
 @pytest.mark.usefixtures("bootstrap")
@@ -61,9 +62,10 @@ class TestSinoFilter:
             sino.shape,
             filter_name=config["filter_name"],
             padding_mode=config["padding_mode"],
+            crop_filtered_data=config["crop_filtered_data"],
         )
         if config["output_provided"]:
-            output = np.zeros_like(sino)
+            output = np.zeros(sino_filter.output_shape, "f")
         else:
             output = None
         res = sino_filter.filter_sino(sino, output=output)
@@ -71,12 +73,16 @@ class TestSinoFilter:
             assert id(res) == id(output), "when providing output, return value must not change"
 
         ref = filter_sinogram(
-            sino, sino_filter.dwidth_padded, filter_name=config["filter_name"], padding_mode=config["padding_mode"]
+            sino,
+            sino_filter.dwidth_padded,
+            filter_name=config["filter_name"],
+            padding_mode=config["padding_mode"],
+            crop_filtered_data=config["crop_filtered_data"],
         )
 
         assert np.allclose(res, ref, atol=4e-6)
 
-    @pytest.mark.skipif(not (__has_pycuda__), reason="Need Cuda + pycuda to use CudaSinoFilter")
+    @pytest.mark.skipif(not (__has_cupy__), reason="Need cupy to use CudaSinoFilter")
     @pytest.mark.parametrize("config", tests_scenarios)
     def test_cuda_filter(self, config):
         sino = self.sino_cuda if not (config["truncated_sino"]) else self.sino_truncated_cuda
@@ -86,10 +92,11 @@ class TestSinoFilter:
             sino.shape,
             filter_name=config["filter_name"],
             padding_mode=config["padding_mode"],
-            cuda_options={"ctx": self.ctx_cuda},
+            crop_filtered_data=config["crop_filtered_data"],
+            # cuda_options={"ctx": self.ctx_cuda},
         )
         if config["output_provided"]:
-            output = garray.zeros(sino.shape, "f")
+            output = cupy.zeros(sino_filter.output_shape, "f")
         else:
             output = None
         res = sino_filter.filter_sino(sino, output=output)
@@ -97,7 +104,11 @@ class TestSinoFilter:
             assert id(res) == id(output), "when providing output, return value must not change"
 
         ref = filter_sinogram(
-            h_sino, sino_filter.dwidth_padded, filter_name=config["filter_name"], padding_mode=config["padding_mode"]
+            h_sino,
+            sino_filter.dwidth_padded,
+            filter_name=config["filter_name"],
+            padding_mode=config["padding_mode"],
+            crop_filtered_data=config["crop_filtered_data"],
         )
 
         assert np.allclose(res.get(), ref, atol=6e-5), "test_cuda_filter: something wrong with config=%s" % (
@@ -109,6 +120,8 @@ class TestSinoFilter:
     )
     @pytest.mark.parametrize("config", tests_scenarios)
     def test_opencl_filter(self, config):
+        if not (config["crop_filtered_data"]):
+            pytest.skip("crop_filtered_data=False is not supported for OpenCL backend yet")
         sino = self.sino_cl if not (config["truncated_sino"]) else self.sino_truncated_cl
         h_sino = self.sino if not (config["truncated_sino"]) else self.sino_truncated
 
@@ -117,6 +130,7 @@ class TestSinoFilter:
             filter_name=config["filter_name"],
             padding_mode=config["padding_mode"],
             opencl_options={"ctx": self.cl.ctx},
+            crop_filtered_data=config["crop_filtered_data"],
         )
         if config["output_provided"]:
             output = parray.zeros(self.cl.queue, sino.shape, "f")
@@ -127,7 +141,11 @@ class TestSinoFilter:
             assert id(res) == id(output), "when providing output, return value must not change"
 
         ref = filter_sinogram(
-            h_sino, sino_filter.dwidth_padded, filter_name=config["filter_name"], padding_mode=config["padding_mode"]
+            h_sino,
+            sino_filter.dwidth_padded,
+            filter_name=config["filter_name"],
+            padding_mode=config["padding_mode"],
+            crop_filtered_data=config["crop_filtered_data"],
         )
 
         assert np.allclose(res.get(), ref, atol=6e-5), "test_opencl_filter: something wrong with config=%s" % (
