@@ -2,18 +2,18 @@ import numpy as np
 import pytest
 from nabu.processing.fft_cuda import get_available_fft_implems
 from nabu.testutils import get_data, generate_tests_scenarios, compare_shifted_images
-from nabu.cuda.utils import get_cuda_context, __has_pycuda__
+from nabu.cuda.utils import __has_cupy__, get_cuda_stream
 from nabu.opencl.utils import get_opencl_context, __has_pyopencl__
 from nabu.thirdparty.algotom_convert_sino import extend_sinogram
 
 __has_cufft__ = False
-if __has_pycuda__:
+if __has_cupy__:
     avail_fft = get_available_fft_implems()
     __has_cufft__ = len(avail_fft) > 0
-__has_pycuda__ = __has_pycuda__ and __has_cufft__  # need both for using Cuda backprojector
+__has_cupy__ = __has_cupy__ and __has_cufft__  # need both for using Cuda backprojector
 
 
-if __has_pycuda__:
+if __has_cupy__:
     from nabu.reconstruction.fbp import CudaBackprojector
     from nabu.reconstruction.hbp import HierarchicalBackprojector
 if __has_pyopencl__:
@@ -30,8 +30,8 @@ def bootstrap(request):
     cls.sino = file_desc["sinogram"] * 1e4
     cls.rot_center = file_desc["rot_center"]
     cls.tol = 5e-3
-    if __has_pycuda__:
-        cls.cuda_ctx = get_cuda_context()
+    if __has_cupy__:
+        cls._cuda_stream = get_cuda_stream(force_create=True)
     if __has_pyopencl__:
         cls.opencl_ctx = get_opencl_context("all")
 
@@ -41,10 +41,10 @@ def bootstrap(request):
 class TestHalftomo:
     def _get_backprojector(self, config, *bp_args, **bp_kwargs):
         if config["backend"] == "cuda":
-            if not (__has_pycuda__):
-                pytest.skip("Need pycuda + scikit-cuda or vkfft")
+            if not (__has_cupy__):
+                pytest.skip("Need cupy and vkfft")
             Backprojector = CudaBackprojector
-            ctx = self.cuda_ctx
+            ctx = None
         else:
             if not (__has_pyopencl__):
                 pytest.skip("Need pyopencl")
@@ -57,6 +57,7 @@ class TestHalftomo:
                 bp_kwargs["extra_options"] = extra_options
         return Backprojector(*bp_args, **bp_kwargs, backend_options={"ctx": ctx})
 
+    # ruff: noqa: PT028
     def test_halftomo_right_side(self, config, sino=None, rot_center=None):
         if sino is None:
             sino = self.sino
@@ -99,13 +100,39 @@ class TestHalftomo:
         rot_center = sino.shape[-1] - 1 - self.rot_center
         return self.test_halftomo_right_side(config, sino=sino, rot_center=rot_center)
 
+    def test_halftomo_plain_backprojection(self, config):
+        backprojector = self._get_backprojector(
+            config,
+            self.sino.shape,
+            rot_center=self.rot_center,
+            halftomo=True,
+            padding_mode="edges",
+            extra_options={"centered_axis": True},
+        )
+        d_sino_filtered = backprojector.sino_filter.filter_sino(self.sino)  # device array
+        h_sino_filtered = d_sino_filtered.get()
+        reference_fbp = backprojector.fbp(self.sino)
+
+        def _check(rec, array_type):
+            assert (
+                np.max(np.abs(rec - reference_fbp)) < 1e-7
+            ), "Something wrong with halftomo backproj using %s array and configuration %s" % (array_type, str(config))
+
+        # Test with device array
+        rec_from_already_filtered_sino = backprojector.backproj(d_sino_filtered)
+        _check(rec_from_already_filtered_sino, "device")
+
+        # Test with numpy array
+        rec_from_already_filtered_sino = backprojector.backproj(h_sino_filtered)
+        _check(rec_from_already_filtered_sino, "numpy")
+
     def test_halftomo_cor_outside_fov(self, config):
         sino = np.ascontiguousarray(self.sino[:, : self.sino.shape[-1] // 2])
         backprojector = self._get_backprojector(config, sino.shape, rot_center=self.rot_center, halftomo=True)
-        res = backprojector.fbp(sino)
+        res = backprojector.fbp(sino)  # noqa: F841
         # Just check that it runs, but no reference results. Who does this anyway ?!
 
-    @pytest.mark.skipif(not (__has_pycuda__), reason="Need pycuda")
+    @pytest.mark.skipif(not (__has_cupy__), reason="Need cupy")
     def test_hbp_halftomo(self, config):
         if config["backend"] == "opencl":
             pytest.skip("No HBP available in OpenCL")
