@@ -5,12 +5,12 @@ gpu.py: general-purpose utilities for GPU
 from ..utils import check_supported
 
 try:
-    from pycuda.driver import Device as CudaDevice
-    from pycuda.driver import device_attribute as dev_attrs
+    from cupy.cuda import Device as CudaDevice
+    from cupy.cuda.runtime import getDeviceProperties
 
-    __has_pycuda__ = True
+    __has_cupy__ = True
 except ImportError:
-    __has_pycuda__ = False
+    __has_cupy__ = False
     CudaDevice = type(None)
 try:
     from pyopencl import Device as CLDevice
@@ -40,7 +40,7 @@ class GPUDescription:
 
         Parameters
         ----------
-        device: `pycuda.driver.Device` or `pyopencl.Device`
+        device: `cupy.cuda.Device` or `pyopencl.Device`
             Class describing a GPU device.
         """
         is_cuda_device = isinstance(device, CudaDevice)
@@ -52,22 +52,27 @@ class GPUDescription:
             self._init_from_cl_device(device)
             self._set_other_attrs(vendor, device_id)
         else:
-            raise ValueError("Expected `pycuda.driver.Device` or `pyopencl.Device`")
+            raise ValueError("Expected `cupy.cuda.device.Device` or `pyopencl.Device`")
 
     def _init_from_cuda_device(self, device):
+        attrs = getDeviceProperties(device.id)
         self._dict = {
             "type": "cuda",
-            "name": device.name(),
-            "memory_GB": device.total_memory() / 1e9,
-            "compute_capability": device.compute_capability(),
-            "device_id": device.get_attribute(dev_attrs.MULTI_GPU_BOARD_GROUP_ID),
+            "name": attrs["name"].decode(),
+            "memory_GB": device.mem_info[1] / 1e9,  # legacy (that was what pyuda reported)
+            "memory_avail_GB": device.mem_info[0] / 1e9,
+            "memory_total_GB": device.mem_info[1] / 1e9,
+            "compute_capability": device.compute_capability,  # /!\ went from tuple (eg. (7, 0)) to str (eg. "70")
+            "device_id": device.id,
         }
 
     def _init_from_cl_device(self, device):
         self._dict = {
             "type": "opencl",
             "name": device.name,
-            "memory_GB": device.global_mem_size / 1e9,
+            "memory_GB": device.global_mem_size / 1e9,  # legacy
+            "memory_avail_GB": device.global_mem_size / 1e9,  # not supported by pyopencl (?)
+            "memory_total_GB": device.global_mem_size / 1e9,  #
             "vendor": device.vendor,
         }
 
@@ -131,7 +136,7 @@ def pick_gpus_auto(cuda_gpus, opencl_platforms, n_gpus):
         return (gpu1["device_id"] == gpu2["device_id"]) and (gpu1["name"] == gpu2["name"])
 
     def is_in_gpus(avail_gpus, query_gpu):
-        for gpu in avail_gpus:
+        for gpu in avail_gpus:  # noqa: SIM110
             if gpu_equal(gpu, query_gpu):
                 return True
         return False
@@ -142,8 +147,8 @@ def pick_gpus_auto(cuda_gpus, opencl_platforms, n_gpus):
     chosen_gpus = list(cuda_gpus.values())
     if len(chosen_gpus) >= n_gpus:
         return chosen_gpus
-    for platform, gpus in opencl_platforms.items():
-        for gpu_id, gpu in gpus.items():
+    for platform, gpus in opencl_platforms.items():  # noqa: PERF102
+        for gpu_id, gpu in gpus.items():  # noqa: PERF102
             if not (is_in_gpus(chosen_gpus, gpu)):
                 # TODO prioritize some OpenCL implementations ?
                 chosen_gpus.append(gpu)
@@ -166,5 +171,5 @@ def pick_gpus_nvidia(cuda_gpus, n_gpus):
     gpus_cc_sorted = sorted(gpus_cc, key=lambda x: x[1], reverse=True)
     res = []
     for i in range(n_gpus):
-        res.append(cuda_gpus[gpus_cc_sorted[i][0]])
+        res.append(cuda_gpus[gpus_cc_sorted[i][0]])  # noqa: PERF401
     return res
