@@ -2,8 +2,8 @@ import logging
 import logging.config
 
 
-class Logger(object):
-    def __init__(self, loggername, level="DEBUG", logfile="logger.log", console=True):
+class Logger:
+    def __init__(self, loggername, level="DEBUG", logfile="logger.log", console=True, prefix=""):
         """
         Configure a Logger object.
 
@@ -26,11 +26,15 @@ class Logger(object):
         self.level = level
         self.logfile = logfile
         self.console = console
+        self.prefix = prefix
         self._configure_logger()
 
+    def set_prefix(self, prefix):
+        self.prefix = prefix
+
     def _configure_logger(self):
         conf = self._get_default_config_dict()
-        for handler in conf["handlers"].keys():
+        for handler in conf["handlers"]:
             conf["handlers"][handler]["level"] = self.level.upper()
         conf["loggers"][self.loggername]["level"] = self.level.upper()
         if not (self.console):
@@ -74,24 +78,24 @@ class Logger(object):
         return conf
 
     def info(self, msg):
-        return self.logger.info(msg)
+        return self.logger.info(self.prefix + msg)
 
     def debug(self, msg):
-        return self.logger.debug(msg)
+        return self.logger.debug(self.prefix + msg)
 
     def warning(self, msg):
-        return self.logger.warning(msg)
+        return self.logger.warning(self.prefix + msg)
 
     warn = warning
 
     def error(self, msg):
-        return self.logger.error(msg)
+        return self.logger.error(self.prefix + msg)
 
     def fatal(self, msg):
-        return self.logger.fatal(msg)
+        return self.logger.fatal(self.prefix + msg)
 
     def critical(self, msg):
-        return self.logger.critical(msg)
+        return self.logger.critical(self.prefix + msg)
 
 
 def LoggerOrPrint(logger):
@@ -103,8 +107,12 @@ def LoggerOrPrint(logger):
     return logger
 
 
-class PrinterLogger(object):
-    def __init__(self):
+class PrinterLogger:
+    def __init__(self, prefix=""):
+        self.prefix = prefix
+        self._configure_logger()
+
+    def _configure_logger(self):
         methods = [
             "debug",
             "warn",
@@ -114,15 +122,23 @@ class PrinterLogger(object):
             "fatal",
             "critical",
         ]
+
+        def _print(msg):
+            print(self.prefix + msg)
+
         for method in methods:
-            self.__setattr__(method, print)
+            self.__setattr__(method, _print)
+
+    def set_prefix(self, prefix):
+        self.prefix = prefix
+        self._configure_logger()
 
 
 LogLevel = {
     "notset": logging.NOTSET,
     "debug": logging.DEBUG,
     "info": logging.INFO,
-    "warn": logging.WARN,
+    "warn": logging.WARNING,
     "warning": logging.WARNING,
     "error": logging.ERROR,
     "critical": logging.CRITICAL,
