@@ -6,6 +6,8 @@ from silx.io import get_data
 from tomoscan.framereducer.reducedframesinfos import ReducedFramesInfos
 from tomoscan.esrf.scan.nxtomoscan import NXtomoScan
 from ..utils import check_supported, is_writeable
+from ..preproc.flatfield import PCAFlatsDecomposer
+from ..io.reader import NXDarksFlats
 
 
 def get_frame_possible_urls(dataset_info, user_dir, output_dir):
@@ -22,7 +24,7 @@ def get_frame_possible_urls(dataset_info, user_dir, output_dir):
         Output processing directory
     """
 
-    frame_types = ["flats", "darks"]
+    frame_types = ["flats", "darks", "pcaflats"]
     h5scan = dataset_info.dataset_scanner  # tomoscan object
 
     def make_dataurl(dirname, frame_type):
@@ -32,8 +34,10 @@ def get_frame_possible_urls(dataset_info, user_dir, output_dir):
 
         if frame_type == "flats":
             dataurl_default_template = h5scan.REDUCED_FLATS_DATAURLS[0]
-        else:
+        elif frame_type == "darks":
             dataurl_default_template = h5scan.REDUCED_DARKS_DATAURLS[0]
+        elif frame_type == "pcaflats":
+            dataurl_default_template = h5scan.PCA_FLATS_DATAURLS[0]
 
         rel_file_path = dataurl_default_template.file_path().format(scan_prefix=h5scan.get_dataset_basename())
         return DataUrl(
@@ -67,20 +71,20 @@ def save_reduced_frames(dataset_info, reduced_frames_arrays, reduced_frames_urls
     darks_flats_dir_url = reduced_frames_urls.get("user", None)
     if darks_flats_dir_url is not None:
         output_url = darks_flats_dir_url
-    elif is_writeable(os.path.dirname(reduced_frames_urls["dataset"]["flats"].file_path())):
+    elif is_writeable(os.path.abspath(os.path.dirname(reduced_frames_urls["dataset"]["flats"].file_path()))):
         output_url = reduced_frames_urls["dataset"]
     else:
         output_url = reduced_frames_urls["output"]
 
     # Get the "ReducedFrameInfos" data structure expected by tomoscan
     def _get_additional_info(frame_type):
-        electric_current = dataset_info.dataset_scanner.electric_current
+        machine_current = dataset_info.dataset_scanner.machine_current
         count_time = dataset_info.dataset_scanner.count_time
-        if electric_current is not None:
-            electric_current = {
-                sl.start: reduce_func[frame_type](electric_current[sl]) for sl in dataset_info.frames_slices(frame_type)
+        if machine_current is not None:
+            machine_current = {
+                sl.start: reduce_func[frame_type](machine_current[sl]) for sl in dataset_info.frames_slices(frame_type)
             }
-            electric_current = [electric_current[k] for k in sorted(electric_current.keys())]
+            machine_current = [machine_current[k] for k in sorted(machine_current.keys())]
         if count_time is not None:
             count_time = {
                 sl.start: reduce_func[frame_type](count_time[sl]) for sl in dataset_info.frames_slices(frame_type)
@@ -88,7 +92,7 @@ def save_reduced_frames(dataset_info, reduced_frames_arrays, reduced_frames_urls
             count_time = [count_time[k] for k in sorted(count_time.keys())]
         info = ReducedFramesInfos()
         info.count_time = count_time
-        info.machine_electric_current = electric_current
+        info.machine_current = machine_current
         return info
 
     flats_info = _get_additional_info("flats")
@@ -149,8 +153,88 @@ def data_url_exists(data_url):
     return group_exists
 
 
+def _compute_and_save_reduced_frames(flatfield_mode, dataset_info, reduced_frames_urls):
+    if flatfield_mode == "pca":
+        dfreader = NXDarksFlats(dataset_info.location)
+        darks = np.concatenate([d for d in dfreader.get_raw_darks()], axis=0)
+        flats = np.concatenate([f for f in dfreader.get_raw_flats()], axis=0)
+        pcaflats_darks = PCAFlatsDecomposer(flats, darks)
+
+        # Get "where to write". tomoscan expects a DataUrl
+        pcaflats_dir_url = reduced_frames_urls.get("user", None)
+        if pcaflats_dir_url is not None:
+            output_url = pcaflats_dir_url
+        elif is_writeable(os.path.dirname(reduced_frames_urls["dataset"]["flats"].file_path())):
+            output_url = reduced_frames_urls["dataset"]
+        else:
+            output_url = reduced_frames_urls["output"]
+        pcaflats_darks.save_decomposition(
+            path=output_url["pcaflats"].file_path(), entry=output_url["pcaflats"].data_path().strip("/").split("/")[0]
+        )
+        dataset_info.logger.info("PCA flats computed and written at %s" % (output_url["pcaflats"].file_path()))
+
+        # Update dataset_info with pca flats and dark
+        dataset_info.darks = {0: pcaflats_darks.dark}
+        flats = {0: pcaflats_darks.mean}
+        for k in range(len(pcaflats_darks.components)):
+            flats.update({k + 1: pcaflats_darks.components[k]})
+        dataset_info.flats = flats
+    else:
+        try:
+            dataset_info.flats = dataset_info.get_reduced_flats()
+            dataset_info.darks = dataset_info.get_reduced_darks()
+        except FileNotFoundError:
+            msg = "Could not find any flats and/or darks"
+            raise FileNotFoundError(msg)
+        _, flats_info, darks_info = save_reduced_frames(
+            dataset_info, {"darks": dataset_info.darks, "flats": dataset_info.flats}, reduced_frames_urls
+        )
+        dataset_info.flats_srcurrent = flats_info.machine_current
+
+
+def _load_existing_flatfields(dataset_info, reduced_frames_urls, frames_types, where_to_load_from):
+    if "pcaflats" not in frames_types:
+        reduced_frames_with_info = {}
+        for frame_type in frames_types:
+            reduced_frames_with_info[frame_type] = tomoscan_load_reduced_frames(
+                dataset_info, frame_type, reduced_frames_urls[where_to_load_from][frame_type]
+            )
+            dataset_info.logger.info(
+                "Loaded %s from %s" % (frame_type, reduced_frames_urls[where_to_load_from][frame_type].file_path())
+            )
+            red_frames_dict, red_frames_info = reduced_frames_with_info[frame_type]
+            setattr(
+                dataset_info,
+                frame_type,
+                {k: get_data(red_frames_dict[k]) for k in red_frames_dict},
+            )
+            if frame_type == "flats":
+                dataset_info.flats_srcurrent = red_frames_info.machine_current
+    else:
+        df_path = reduced_frames_urls[where_to_load_from]["pcaflats"].file_path()
+        entry = reduced_frames_urls[where_to_load_from]["pcaflats"].data_path()
+
+        # Update dark
+        dark_url = DataUrl(f"silx://{df_path}?{entry}/dark")
+        dark = get_data(dark_url)
+        setattr(dataset_info, "dark", {0: dark})  # noqa: B010 - # I know what I'm doing ruff!
+        # Update flats with principal compenents
+        # Take mean as first comp., mask as second, flats thereafter
+        flats_url = DataUrl(f"silx://{df_path}?{entry}/p_components")
+        mean_url = DataUrl(f"silx://{df_path}?{entry}/p_mean")
+        flats = get_data(flats_url)
+        # TODO what do do with this ?
+        mean = get_data(mean_url)  # noqa: F841
+        setattr(dataset_info, "flats", {k: flats[k] for k in range(len(flats))})  # noqa: B010
+
+        dataset_info.flats = {k: flats[k] for k in range(len(flats))}
+        dataset_info.logger.info("Loaded %s from %s" % ("PCA darks/flats", df_path))
+
+
 # pylint: disable=E1136
-def update_dataset_info_flats_darks(dataset_info, flatfield_mode, output_dir=None, darks_flats_dir=None):
+def update_dataset_info_flats_darks(
+    dataset_info, flatfield_mode, loading_mode="load_if_present", output_dir=None, darks_flats_dir=None
+):
     """
     Update a DatasetAnalyzer object with reduced flats/darks (hereafter "reduced frames").
 
@@ -170,23 +254,14 @@ def update_dataset_info_flats_darks(dataset_info, flatfield_mode, output_dir=Non
     if flatfield_mode is False:
         return
 
-    frames_types = ["darks", "flats"]
+    if flatfield_mode == "pca":
+        frames_types = ["pcaflats"]
+    else:
+        frames_types = ["darks", "flats"]
     reduced_frames_urls = get_frame_possible_urls(dataset_info, darks_flats_dir, output_dir)
 
-    def _compute_and_save_reduced_frames():
-        try:
-            dataset_info.flats = dataset_info.get_reduced_flats()
-            dataset_info.darks = dataset_info.get_reduced_darks()
-        except FileNotFoundError:
-            msg = "Could not find any flats and/or darks"
-            raise FileNotFoundError(msg)
-        _, flats_info, darks_info = save_reduced_frames(
-            dataset_info, {"darks": dataset_info.darks, "flats": dataset_info.flats}, reduced_frames_urls
-        )
-        dataset_info.flats_srcurrent = flats_info.machine_electric_current
-
-    if flatfield_mode == "force-compute":
-        _compute_and_save_reduced_frames()
+    if loading_mode == "force-compute":
+        _compute_and_save_reduced_frames(flatfield_mode, dataset_info, reduced_frames_urls)
         return
 
     def _can_load_from(folder_type):
@@ -202,25 +277,10 @@ def update_dataset_info_flats_darks(dataset_info, flatfield_mode, output_dir=Non
     elif _can_load_from("output"):
         where_to_load_from = "output"
 
-    if where_to_load_from == None and flatfield_mode == "force-load":
+    if where_to_load_from is None and flatfield_mode == "force-load":
         raise ValueError("Could not load darks/flats (using 'force-load')")
 
     if where_to_load_from is not None:
-        reduced_frames_with_info = {}
-        for frame_type in frames_types:
-            reduced_frames_with_info[frame_type] = tomoscan_load_reduced_frames(
-                dataset_info, frame_type, reduced_frames_urls[where_to_load_from][frame_type]
-            )
-            dataset_info.logger.info(
-                "Loaded %s from %s" % (frame_type, reduced_frames_urls[where_to_load_from][frame_type].file_path())
-            )
-            red_frames_dict, red_frames_info = reduced_frames_with_info[frame_type]
-            setattr(
-                dataset_info,
-                frame_type,
-                {k: get_data(red_frames_dict[k]) for k in red_frames_dict.keys()},
-            )
-            if frame_type == "flats":
-                dataset_info.flats_srcurrent = red_frames_info.machine_electric_current
+        _load_existing_flatfields(dataset_info, reduced_frames_urls, frames_types, where_to_load_from)
     else:
-        _compute_and_save_reduced_frames()
+        _compute_and_save_reduced_frames(flatfield_mode, dataset_info, reduced_frames_urls)
