@@ -1,8 +1,10 @@
+from os import environ
 from ast import literal_eval
 import numpy as np
+import pint
 from psutil import virtual_memory, cpu_count
-from pyunitsystem.metricsystem import MetricSystem
-from pyunitsystem.energysystem import EnergySI
+
+_ureg = pint.get_application_registry()
 
 
 def get_values_from_file(fname, n_values=None, shape=None, sep=None, any_size=False):
@@ -47,6 +49,18 @@ def get_values_from_file(fname, n_values=None, shape=None, sep=None, any_size=Fa
     return arr
 
 
+def get_available_ram_GB():
+    """
+    Return available memory in GB (not GiB), accounting for possible environment constraints (eg. SLURM)
+    """
+    vm = virtual_memory()
+    avail_mem = vm.available / 1e9
+    slurm_mem_constraint_MB = int(environ.get("SLURM_MEM_PER_NODE", 0))  # noqa: PLW1508
+    if slurm_mem_constraint_MB > 0:
+        avail_mem = slurm_mem_constraint_MB / 1e3
+    return avail_mem
+
+
 def get_memory_per_node(max_mem, is_percentage=True):
     """
     Get the available memory per node in GB.
@@ -163,12 +177,17 @@ def get_quantities_and_units(string, sep=";"):
         value, unit = value_and_unit.split()
         val = float(value)
         # Convert to SI
-        try:
-            # handle metrics
-            conversion_factor = MetricSystem.from_str(unit).value
-        except ValueError:
-            # handle energies
-            conversion_factor = EnergySI.from_str(unit).value / EnergySI.KILOELECTRONVOLT.value
-
-        result[quantity_name] = val * conversion_factor
+        if unit.lower() == "kev":
+            current_unit = _ureg.keV
+        elif unit.lower() == "ev":
+            current_unit = _ureg.eV
+        else:
+            current_unit = _ureg(unit)
+        # handle energies (to move to keV)
+        if _ureg.keV.dimensionality == current_unit.dimensionality:
+            result[quantity_name] = (val * current_unit).to(_ureg.keV).magnitude
+        elif _ureg.meter.dimensionality == current_unit.dimensionality:
+            result[quantity_name] = (val * current_unit).to_base_units().magnitude
+        else:
+            raise ValueError(f"Cannot convert: {unit}")
     return result
