@@ -1,29 +1,25 @@
+from enum import Enum
 import h5py
 import numpy
-from typing import Union
-from silx.utils.enum import Enum as _Enum
-from tomoscan.volumebase import VolumeBase
-from tomoscan.esrf.volume.hdf5volume import HDF5Volume
-from nabu.io.utils import DatasetReader
 
 
-class AlignmentAxis2(_Enum):
-    """Specific alignment named to help users orienting themself with specific name"""
+class AlignmentAxis2(Enum):
+    """Specific alignment names to help users orient themselves with specific labels"""
 
     CENTER = "center"
     LEFT = "left"
     RIGTH = "right"
 
 
-class AlignmentAxis1(_Enum):
-    """Specific alignment named to help users orienting themself with specific name"""
+class AlignmentAxis1(Enum):
+    """Specific alignment names to help users orient themselves with specific labels"""
 
     FRONT = "front"
     CENTER = "center"
     BACK = "back"
 
 
-class _Alignment(_Enum):
+class _Alignment(Enum):
     """Internal alignment to be used for 2D alignment"""
 
     LOWER_BOUNDARY = "lower boundary"
@@ -32,7 +28,7 @@ class _Alignment(_Enum):
 
     @classmethod
     def from_value(cls, value):
-        # cast the AlignmentAxis1 and AlignmentAxis2 values to fit the generic definition
+        # cast the AlignmentAxis1 and AlignmentAxis2 values to fit the generic definition.
         if value in ("front", "left", AlignmentAxis1.FRONT, AlignmentAxis2.LEFT):
             return _Alignment.LOWER_BOUNDARY
         elif value in ("back", "right", AlignmentAxis1.BACK, AlignmentAxis2.RIGTH):
@@ -40,7 +36,7 @@ class _Alignment(_Enum):
         elif value in (AlignmentAxis1.CENTER, AlignmentAxis2.CENTER):
             return _Alignment.CENTER
         else:
-            return super().from_value(value)
+            return super().__new__(cls, value)
 
 
 def align_frame(
@@ -106,7 +102,7 @@ def align_horizontally(data: numpy.ndarray, alignment: AlignmentAxis2, new_width
     :param HAlignment alignment: alignment strategy
     :param int new_width: output data width
     """
-    alignment = AlignmentAxis2.from_value(alignment).value
+    alignment = AlignmentAxis2(alignment).value
     return align_frame(
         data=data, alignment=alignment, new_aligned_axis_size=new_width, pad_mode=pad_mode, alignment_axis=1
     )
@@ -119,7 +115,7 @@ class PaddedRawData:
     The idea behind is to avoid loading all the data in memory
     """
 
-    def __init__(self, data: Union[numpy.ndarray, h5py.Dataset], axis_1_pad_width: tuple) -> None:
+    def __init__(self, data: numpy.ndarray | h5py.Dataset, axis_1_pad_width: tuple) -> None:
         self._axis_1_pad_width = numpy.array(axis_1_pad_width)
         if not (self._axis_1_pad_width.size == 2 and self._axis_1_pad_width[0] >= 0 and self._axis_1_pad_width[1] >= 0):
             raise ValueError(f"'axis_1_pad_width' expects to positive elements. Get {axis_1_pad_width}")
@@ -151,7 +147,7 @@ class PaddedRawData:
     @property
     def shape(self):
         if self._shape is None:
-            self._shape = tuple(
+            self._shape = tuple(  # noqa: C409
                 (
                     self._raw_data_shape[0],
                     numpy.sum(
