@@ -1,8 +1,8 @@
+import pint
 from math import ceil
-from typing import Any, Iterable, Optional, Union, Sized
+from collections.abc import Iterable, Sized
 from dataclasses import dataclass
 import numpy
-from pyunitsystem.metricsystem import MetricSystem
 from nxtomo.paths import nxtomo
 from tomoscan.factory import Factory
 from tomoscan.identifier import VolumeIdentifier, ScanIdentifier
@@ -12,13 +12,14 @@ from ..pipeline.config_validators import (
     convert_to_bool,
 )
 from ..utils import concatenate_dict, convert_str_to_tuple
-from ..io.utils import get_output_volume
 from .overlap import OverlapStitchingStrategy
 from .utils.utils import ShiftAlgorithm
 from .definitions import StitchingType
 from .alignment import AlignmentAxis1, AlignmentAxis2
-from pyunitsystem.metricsystem import MetricSystem
 
+_ureg = pint.get_application_registry()
+
+# ruff: noqa: S105
 
 KEY_IMG_REG_METHOD = "img_reg_method"
 
@@ -128,6 +129,8 @@ SLURM_MODULES_TO_LOADS = "modules"
 
 SLURM_CLEAN_SCRIPTS = "clean_scripts"
 
+SLURM_JOB_NAME = "job_name"
+
 # normalization by sample
 
 NORMALIZATION_BY_SAMPLE_SECTION = "normalization_by_sample"
@@ -193,7 +196,7 @@ HELP_SHIFT_PARAMS = f"""options for shifts algorithms as `key1=value1,key2=value
     - {KEY_HIGH_PASS_FILTER}: high pass filter value for filtering frames before shift research"""
 
 
-def _str_to_dict(my_str: Union[str, dict]):
+def _str_to_dict(my_str: str | dict):
     """convert a string as key_1=value_2;key_2=value_2 to a dict"""
     if isinstance(my_str, dict):
         return my_str
@@ -205,10 +208,10 @@ def _str_to_dict(my_str: Union[str, dict]):
 
 
 def _dict_to_str(ddict: dict):
-    return ";".join([f"{str(key)}={str(value)}" for key, value in ddict.items()])
+    return ";".join([f"{key!s}={value!s}" for key, value in ddict.items()])
 
 
-def str_to_shifts(my_str: Optional[str]) -> Union[str, tuple]:
+def str_to_shifts(my_str: str | None) -> str | tuple:
     if my_str is None:
         return None
     elif isinstance(my_str, str):
@@ -218,7 +221,7 @@ def str_to_shifts(my_str: Optional[str]) -> Union[str, tuple]:
         if my_str == "":
             return None
         try:
-            shift = ShiftAlgorithm.from_value(my_str)
+            shift = ShiftAlgorithm(my_str)
         except ValueError:
             shifts_as_str = filter(None, my_str.replace(";", ",").split(","))
             return [float(shift) for shift in shifts_as_str]
@@ -230,18 +233,18 @@ def str_to_shifts(my_str: Optional[str]) -> Union[str, tuple]:
         raise TypeError("Only str or tuple of str expected expected")
 
 
-def _valid_stitching_kernels_params(my_dict: Union[dict, str]):
+def _valid_stitching_kernels_params(my_dict: dict | str):
     if isinstance(my_dict, str):
         my_dict = _str_to_dict(my_str=my_dict)
 
     valid_keys = (KEY_THRESHOLD_FREQUENCY, KEY_SIDE)
-    for key in my_dict.keys():
-        if not key in valid_keys:
+    for key in my_dict:
+        if key not in valid_keys:
             raise KeyError(f"{key} is a unrecognized key")
     return my_dict
 
 
-def _valid_shifts_params(my_dict: Union[dict, str]):
+def _valid_shifts_params(my_dict: dict | str):
     if isinstance(my_dict, str):
         my_dict = _str_to_dict(my_str=my_dict)
 
@@ -253,13 +256,13 @@ def _valid_shifts_params(my_dict: Union[dict, str]):
         KEY_LOW_PASS_FILTER,
         KEY_SIDE,
     )
-    for key in my_dict.keys():
-        if not key in valid_keys:
+    for key in my_dict:
+        if key not in valid_keys:
             raise KeyError(f"{key} is a unrecognized key")
     return my_dict
 
 
-def _slices_to_list_or_slice(my_str: Optional[str]) -> Union[str, slice]:
+def _slices_to_list_or_slice(my_str: str | None) -> str | slice:
     if my_str is None:
         return None
     if isinstance(my_str, (tuple, list)):
@@ -288,7 +291,7 @@ def _slices_to_list_or_slice(my_str: Optional[str]) -> Union[str, slice]:
         return list(filter(None, my_str.split(";")))
 
 
-def _scalar_or_tuple_to_bool_or_tuple_of_bool(my_str: Union[bool, tuple, str], default=False):
+def _scalar_or_tuple_to_bool_or_tuple_of_bool(my_str: bool | tuple | str, default=False):
     if isinstance(my_str, bool):
         return my_str
     elif isinstance(my_str, str):
@@ -312,6 +315,8 @@ from nabu.stitching.sample_normalization import Method, SampleSide
 
 
 class NormalizationBySample:
+    __hash__ = object.__hash__
+
     def __init__(self) -> None:
         self._active = False
         self._method = Method.MEAN
@@ -333,8 +338,8 @@ class NormalizationBySample:
         return self._method
 
     @method.setter
-    def method(self, method: Union[Method, str]) -> None:
-        self._method = Method.from_value(method)
+    def method(self, method: Method | str) -> None:
+        self._method = Method(method)
 
     @property
     def margin(self) -> int:
@@ -350,8 +355,8 @@ class NormalizationBySample:
         return self._side
 
     @side.setter
-    def side(self, side: Union[SampleSide, str]):
-        self._side = SampleSide.from_value(side)
+    def side(self, side: SampleSide | str):
+        self._side = SampleSide(side)
 
     @property
     def width(self) -> int:
@@ -401,16 +406,17 @@ class NormalizationBySample:
             NORMALIZATION_BY_SAMPLE_WIDTH: self.width,
         }
 
-    def __eq__(self, __value: object) -> bool:
-        if not isinstance(__value, NormalizationBySample):
+    def __eq__(self, value: object, /) -> bool:
+        if not isinstance(value, NormalizationBySample):
             return False
         else:
-            return self.to_dict() == __value.to_dict()
+            return self.to_dict() == value.to_dict()
 
 
 @dataclass
 class SlurmConfig:
-    "configuration for slurm jobs"
+    """configuration for slurm jobs"""
+
     partition: str = ""  # note: must stay empty to make by default we don't use slurm (use by the  configuration file)
     mem: str = "128"
     n_jobs: int = 1
@@ -420,6 +426,7 @@ class SlurmConfig:
     clean_script: bool = ""
     n_tasks: int = 1
     n_cpu_per_task: int = 4
+    job_name: str = ""
 
     def __post_init__(self) -> None:
         # make sure either 'modules' or 'preprocessing_command' is provided
@@ -429,7 +436,7 @@ class SlurmConfig:
             )
 
     def to_dict(self) -> dict:
-        "dump configuration to dict"
+        """dump configuration to dict"""
         return {
             SLURM_PARTITION: self.partition if self.partition is not None else "",
             SLURM_MEM: self.mem,
@@ -440,6 +447,7 @@ class SlurmConfig:
             SLURM_CLEAN_SCRIPTS: self.clean_script,
             SLURM_NUMBER_OF_TASKS: self.n_tasks,
             SLURM_COR_PER_TASKS: self.n_cpu_per_task,
+            SLURM_JOB_NAME: self.job_name,
         }
 
     @staticmethod
@@ -456,18 +464,21 @@ class SlurmConfig:
             preprocessing_command=config.get(SLURM_PREPROCESSING_COMMAND, ""),
             modules_to_load=convert_str_to_tuple(config.get(SLURM_MODULES_TO_LOADS, "")),
             clean_script=convert_to_bool(config.get(SLURM_CLEAN_SCRIPTS, False))[0],
+            job_name=config.get(SLURM_JOB_NAME, ""),
         )
 
 
-def _cast_shift_to_str(shifts: Union[tuple, str, None]) -> str:
+def _cast_shift_to_str(shifts: tuple | numpy.ndarray | str | None) -> str:
     if shifts is None:
         return ""
     elif isinstance(shifts, ShiftAlgorithm):
         return shifts.value
     elif isinstance(shifts, str):
         return shifts
-    elif isinstance(shifts, (tuple, list)):
+    elif isinstance(shifts, (tuple, list, numpy.ndarray)):
         return ";".join([str(value) for value in shifts])
+    else:
+        raise TypeError(f"unexpected type: {type(shifts)}")
 
 
 @dataclass
@@ -476,34 +487,34 @@ class StitchingConfiguration:
     bass class to define stitching configuration
     """
 
-    axis_0_pos_px: Union[tuple, str, None]
+    axis_0_pos_px: tuple | str | None
     "position along axis 0 in absolute. unit: px"
-    axis_1_pos_px: Union[tuple, str, None]
+    axis_1_pos_px: tuple | str | None
     "position along axis 1 in absolute. unit: px"
-    axis_2_pos_px: Union[tuple, str, None]
+    axis_2_pos_px: tuple | str | None
     "position along axis 2 in absolute. unit: px"
 
-    axis_0_pos_mm: Union[tuple, str, None] = None
+    axis_0_pos_mm: tuple | str | None = None
     "position along axis 0 in absolute. unit: mm"
-    axis_1_pos_mm: Union[tuple, str, None] = None
+    axis_1_pos_mm: tuple | str | None = None
     "position along axis 0 in absolute. unit: mm"
-    axis_2_pos_mm: Union[tuple, str, None] = None
+    axis_2_pos_mm: tuple | str | None = None
     "position along axis 0 in absolute. unit: mm"
 
     axis_0_params: dict = None
     axis_1_params: dict = None
     axis_2_params: dict = None
     slurm_config: SlurmConfig = None
-    flip_lr: Union[tuple, bool] = False
+    flip_lr: tuple | bool = False
     "flip frame left-right. For scan this will be append to the NXtransformations of the detector"
-    flip_ud: Union[tuple, bool] = False
+    flip_ud: tuple | bool = False
     "flip frame up-down. For scan this will be append to the NXtransformations of the detector"
 
     overwrite_results: bool = False
     stitching_strategy: OverlapStitchingStrategy = OverlapStitchingStrategy.COSINUS_WEIGHTS
     stitching_kernels_extra_params: dict = None
 
-    slice_for_cross_correlation: Union[str, int] = "middle"
+    slice_for_cross_correlation: str | int = "middle"
 
     # opts for rescaling frame during stitching
     rescale_frames: bool = False
@@ -540,12 +551,12 @@ class StitchingConfiguration:
             STITCHING_SECTION: {
                 STITCHING_TYPE_FIELD: {
                     "default": StitchingType.Z_PREPROC.value,
-                    "help": f"stitching to be applied. Must be in {StitchingType.values()}",
+                    "help": f"stitching to be applied. Must be in {[st.value for st in StitchingType]}",
                     "type": "required",
                 },
                 STITCHING_STRATEGY_FIELD: {
                     "default": "cosinus weights",
-                    "help": f"Policy to apply to compute the overlap area. Must be in {OverlapStitchingStrategy.values()}.",
+                    "help": f"Policy to apply to compute the overlap area. Must be in {[ov.value for ov in OverlapStitchingStrategy]}.",
                     "type": "required",
                 },
                 CROSS_CORRELATION_SLICE_FIELD: {
@@ -625,7 +636,7 @@ class StitchingConfiguration:
                 },
                 ALIGNMENT_AXIS_2_FIELD: {
                     "default": "center",
-                    "help": f"In case frame have different frame widths how to align them (so along volume axis 2). Valid keys are {AlignmentAxis2.values()}",
+                    "help": f"In case frame have different frame widths how to align them (so along volume axis 2). Valid keys are {[aa.value for aa in AlignmentAxis2]}",
                     "type": "advanced",
                 },
                 PAD_MODE_FIELD: {
@@ -747,7 +758,7 @@ class StitchingConfiguration:
                 AXIS_2_POS_PX: _cast_shift_to_str(self.axis_2_pos_px),
                 AXIS_2_POS_MM: _cast_shift_to_str(self.axis_2_pos_mm),
                 AXIS_2_PARAMS: _dict_to_str(self.axis_2_params or {}),
-                STITCHING_STRATEGY_FIELD: OverlapStitchingStrategy.from_value(self.stitching_strategy).value,
+                STITCHING_STRATEGY_FIELD: OverlapStitchingStrategy(self.stitching_strategy).value,
                 FLIP_UD: self.flip_ud,
                 FLIP_LR: self.flip_lr,
                 RESCALE_FRAMES: self.rescale_frames,
@@ -784,7 +795,7 @@ class SingleAxisStitchingConfiguration(StitchingConfiguration, metaclass=SingleA
     base class to define z-stitching parameters
     """
 
-    slices: Union[slice, tuple, None] = (
+    slices: slice | tuple | None = (
         None  # slices to reconstruct. Over axis 0 for pre-processing, over axis 1 for post-processing. If None will reconstruct all
     )
 
@@ -836,8 +847,8 @@ class PreProcessedSingleAxisStitchingConfiguration(SingleAxisStitchingConfigurat
     input_scans: tuple = ()  # tuple of ScanBase
     output_file_path: str = ""
     output_data_path: str = ""
-    output_nexus_version: Optional[float] = None
-    pixel_size: Optional[float] = None
+    output_nexus_version: float | None = None
+    pixel_size: float | None = None
 
     @property
     def stitching_type(self) -> StitchingType:
@@ -926,7 +937,7 @@ class PreProcessedSingleAxisStitchingConfiguration(SingleAxisStitchingConfigurat
         if self.pixel_size is None:
             pixel_size_mm = ""
         else:
-            pixel_size_mm = self.pixel_size * MetricSystem.MILLIMETER.value
+            pixel_size_mm = (self.pixel_size * _ureg.meter).to(_ureg.millimeter).magnitude
         return concatenate_dict(
             super().to_dict(),
             {
@@ -990,10 +1001,10 @@ class PreProcessedSingleAxisStitchingConfiguration(SingleAxisStitchingConfigurat
         if pixel_size == "":
             pixel_size = None
         else:
-            pixel_size = float(pixel_size) / MetricSystem.MM
+            pixel_size = (float(pixel_size) * _ureg.millimeter).to_base_units().magnitude
 
         return cls(
-            stitching_strategy=OverlapStitchingStrategy.from_value(
+            stitching_strategy=OverlapStitchingStrategy(
                 config[STITCHING_SECTION].get(
                     STITCHING_STRATEGY_FIELD,
                     OverlapStitchingStrategy.COSINUS_WEIGHTS,
@@ -1034,12 +1045,13 @@ class PreProcessedSingleAxisStitchingConfiguration(SingleAxisStitchingConfigurat
                     config[STITCHING_SECTION].get(STITCHING_KERNELS_EXTRA_PARAMS, {}),
                 )
             ),
-            alignment_axis_2=AlignmentAxis2.from_value(
+            alignment_axis_2=AlignmentAxis2(
                 config[STITCHING_SECTION].get(ALIGNMENT_AXIS_2_FIELD, AlignmentAxis2.CENTER)
             ),
             pad_mode=config[STITCHING_SECTION].get(PAD_MODE_FIELD, "constant"),
-            duplicate_data=not config[STITCHING_SECTION].get(AVOID_DATA_DUPLICATION_FIELD, False),
-            normalization_by_sample=NormalizationBySample.from_dict(config.get(NORMALIZATION_BY_SAMPLE_SECTION, {})),
+            duplicate_data=not _scalar_or_tuple_to_bool_or_tuple_of_bool(
+                config[STITCHING_SECTION].get(AVOID_DATA_DUPLICATION_FIELD, False)
+            ),
         )
 
 
@@ -1050,8 +1062,8 @@ class PostProcessedSingleAxisStitchingConfiguration(SingleAxisStitchingConfigura
     """
 
     input_volumes: tuple = ()  # tuple of VolumeBase
-    output_volume: Optional[VolumeIdentifier] = None
-    voxel_size: Optional[float] = None
+    output_volume: VolumeIdentifier | None = None
+    voxel_size: float | None = None
     alignment_axis_1: AlignmentAxis1 = AlignmentAxis1.CENTER
 
     @property
@@ -1154,11 +1166,11 @@ class PostProcessedSingleAxisStitchingConfiguration(SingleAxisStitchingConfigura
         if voxel_size == "":
             voxel_size = None
         else:
-            voxel_size = float(voxel_size) * MetricSystem.MM
+            voxel_size = (float(voxel_size) * _ureg.millimeter).to_base_units().magnitude
 
         # on the next section the one with a default value qre the optional one
         return cls(
-            stitching_strategy=OverlapStitchingStrategy.from_value(
+            stitching_strategy=OverlapStitchingStrategy(
                 config[STITCHING_SECTION].get(
                     STITCHING_STRATEGY_FIELD,
                     OverlapStitchingStrategy.COSINUS_WEIGHTS,
@@ -1189,14 +1201,16 @@ class PostProcessedSingleAxisStitchingConfiguration(SingleAxisStitchingConfigura
                     config[STITCHING_SECTION].get(STITCHING_KERNELS_EXTRA_PARAMS, {}),
                 )
             ),
-            alignment_axis_1=AlignmentAxis1.from_value(
+            alignment_axis_1=AlignmentAxis1(
                 config[STITCHING_SECTION].get(ALIGNMENT_AXIS_1_FIELD, AlignmentAxis1.CENTER)
             ),
-            alignment_axis_2=AlignmentAxis2.from_value(
+            alignment_axis_2=AlignmentAxis2(
                 config[STITCHING_SECTION].get(ALIGNMENT_AXIS_2_FIELD, AlignmentAxis2.CENTER)
             ),
             pad_mode=config[STITCHING_SECTION].get(PAD_MODE_FIELD, "constant"),
-            duplicate_data=not config[STITCHING_SECTION].get(AVOID_DATA_DUPLICATION_FIELD, False),
+            duplicate_data=not _scalar_or_tuple_to_bool_or_tuple_of_bool(
+                config[STITCHING_SECTION].get(AVOID_DATA_DUPLICATION_FIELD, False)
+            ),
             normalization_by_sample=NormalizationBySample.from_dict(config.get(NORMALIZATION_BY_SAMPLE_SECTION, {})),
         )
 
@@ -1204,7 +1218,7 @@ class PostProcessedSingleAxisStitchingConfiguration(SingleAxisStitchingConfigura
         if self.voxel_size is None:
             voxel_size_mm = ""
         else:
-            voxel_size_mm = numpy.array(self.voxel_size) / MetricSystem.MM
+            voxel_size_mm = numpy.array((self.voxel_size * _ureg.meter).to(_ureg.millimeter).magnitude)
 
         return concatenate_dict(
             super().to_dict(),
@@ -1239,7 +1253,7 @@ class PostProcessedSingleAxisStitchingConfiguration(SingleAxisStitchingConfigura
                 STITCHING_SECTION: {
                     ALIGNMENT_AXIS_1_FIELD: {
                         "default": "center",
-                        "help": f"alignment to apply over axis 1 if needed. Valid values are {AlignmentAxis1.values()}",
+                        "help": f"alignment to apply over axis 1 if needed. Valid values are {[aa for aa in AlignmentAxis1]}",
                         "type": "advanced",
                     }
                 },
@@ -1270,7 +1284,7 @@ def dict_to_config_obj(config: dict):
     if stitching_type is None:
         raise ValueError("Unable to find stitching type from config dict")
     else:
-        stitching_type = StitchingType.from_value(stitching_type)
+        stitching_type = StitchingType(stitching_type)
         if stitching_type is StitchingType.Z_POSTPROC:
             return PostProcessedZStitchingConfiguration.from_dict(config)
         elif stitching_type is StitchingType.Z_PREPROC:
@@ -1281,7 +1295,7 @@ def dict_to_config_obj(config: dict):
             raise NotImplementedError(f"stitching type {stitching_type.value} not handled yet")
 
 
-def get_default_stitching_config(stitching_type: Optional[Union[StitchingType, str]]) -> tuple:
+def get_default_stitching_config(stitching_type: StitchingType | str | None) -> tuple:
     """
     Return a default configuration for doing stitching.
 
@@ -1293,7 +1307,7 @@ def get_default_stitching_config(stitching_type: Optional[Union[StitchingType, s
     if stitching_type is None:
         return concatenate_dict(z_postproc_stitching_config, z_preproc_stitching_config)
 
-    stitching_type = StitchingType.from_value(stitching_type)
+    stitching_type = StitchingType(stitching_type)
     if stitching_type is StitchingType.Z_POSTPROC:
         return z_postproc_stitching_config
     elif stitching_type is StitchingType.Z_PREPROC:
