@@ -1,7 +1,7 @@
-from silx.utils.enum import Enum as _Enum
+from enum import Enum
 
 
-class StitchingType(_Enum):
+class StitchingType(Enum):
     Y_PREPROC = "y-preproc"
     Z_PREPROC = "z-preproc"
     Z_POSTPROC = "z-postproc"
