@@ -1,7 +1,6 @@
 import numpy
 import logging
-from typing import Optional, Union
-from silx.utils.enum import Enum as _Enum
+from enum import Enum
 from nabu.misc import fourier_filters
 from scipy.fft import rfftn as local_fftn
 from scipy.fft import irfftn as local_ifftn
@@ -10,7 +9,7 @@ from tomoscan.utils.geometry import BoundingBox1D
 _logger = logging.getLogger(__name__)
 
 
-class OverlapStitchingStrategy(_Enum):
+class OverlapStitchingStrategy(Enum):
     MEAN = "mean"
     COSINUS_WEIGHTS = "cosinus weights"
     LINEAR_WEIGHTS = "linear weights"
@@ -39,17 +38,17 @@ class ImageStichOverlapKernel(OverlapKernelBase):
     def __init__(
         self,
         stitching_axis: int,
-        frame_unstitched_axis_size: tuple,
+        frame_unstitched_axis_size: int,
         stitching_strategy: OverlapStitchingStrategy = DEFAULT_OVERLAP_STRATEGY,
         overlap_size: int = DEFAULT_OVERLAP_SIZE,
-        extra_params: Optional[dict] = None,
+        extra_params: dict | None = None,
     ) -> None:
         """
         :param stitching_axis: axis along which stitching is operate. Must be in '0', '1'
         :param frame_unstitched_axis_size: according to the stitching axis the stitched framed will always have a constant size:
             * If stitching_axis == 0 then it will be the frame width
             * If stitching_axis == 1 then it will be the frame height
-        :param stitching_strategy: stategy / algorithm to use in order to generate the stitching
+        :param stitching_strategy: strategy / algorithm to use in order to generate the stitching
         :param overlap_size: size (int) of the overlap (stitching) between the two images
         :param extra_params: possibly extra parameters to operate the stitching
         """
@@ -64,7 +63,7 @@ class ImageStichOverlapKernel(OverlapKernelBase):
                 f"frame_width is expected to be a positive int, {frame_unstitched_axis_size} - not {frame_unstitched_axis_size} ({type(frame_unstitched_axis_size)})"
             )
 
-        if not stitching_axis in (0, 1):
+        if stitching_axis not in (0, 1):
             raise ValueError(
                 "stitching_axis is expected to be the axis along which stitching must be done. It should be '0' or '1'"
             )
@@ -72,7 +71,7 @@ class ImageStichOverlapKernel(OverlapKernelBase):
         self._stitching_axis = stitching_axis
         self._overlap_size = abs(overlap_size)
         self._frame_unstitched_axis_size = frame_unstitched_axis_size
-        self._stitching_strategy = OverlapStitchingStrategy.from_value(stitching_strategy)
+        self._stitching_strategy = OverlapStitchingStrategy(stitching_strategy)
         self._weights_img_1 = None
         self._weights_img_2 = None
         if extra_params is None:
@@ -124,11 +123,11 @@ class ImageStichOverlapKernel(OverlapKernelBase):
         return self._img_2
 
     @property
-    def weights_img_1(self) -> Optional[numpy.ndarray]:
+    def weights_img_1(self) -> numpy.ndarray | None:
         return self._weights_img_1
 
     @property
-    def weights_img_2(self) -> Optional[numpy.ndarray]:
+    def weights_img_2(self) -> numpy.ndarray | None:
         return self._weights_img_2
 
     @property
@@ -313,7 +312,7 @@ def compute_image_higher_signal(img_1: numpy.ndarray, img_2: numpy.ndarray):
     return numpy.where(img_1 >= img_2, img_1, img_2)
 
 
-def check_overlaps(frames: Union[tuple, numpy.ndarray], positions: tuple, axis: int, raise_error: bool):
+def check_overlaps(frames: tuple | numpy.ndarray, positions: tuple, axis: int, raise_error: bool):
     """
     check over frames if there is a single overlap other juxtaposed frames (at most and at least)
 
