@@ -1,6 +1,5 @@
 import os
 import copy
-from typing import Optional, Union
 import numpy
 from silx.io.url import DataUrl
 from tomoscan.tomoobject import TomoObject
@@ -154,7 +153,7 @@ def split_stitching_configuration_to_slurm_job(
             yield job
 
 
-def split_slices(slices: Union[slice, tuple], n_parts: int):
+def split_slices(slices: slice | tuple, n_parts: int):
     if not isinstance(n_parts, int):
         raise TypeError(f"n_parts should be an int. {type(n_parts)} provided")
     if isinstance(slices, slice):
@@ -177,7 +176,7 @@ def split_slices(slices: Union[slice, tuple], n_parts: int):
         raise TypeError(f"slices type ({type(slices)}) is not handled. Must be a slice or an Iterable")
 
 
-def get_working_directory(obj: TomoObject) -> Optional[str]:
+def get_working_directory(obj: TomoObject) -> str | None:  # noqa: PLR0911
     """
     return working directory for a specific TomoObject
     """
@@ -201,4 +200,4 @@ def get_working_directory(obj: TomoObject) -> Optional[str]:
         else:
             return os.path.abspath(os.path.dirname(obj.master_file))
     else:
-        raise RuntimeError(f"obj type not handled ({type(obj)})")
+        raise RuntimeError(f"obj type not handled ({type(obj)})")  # noqa: TRY004
