@@ -2,7 +2,7 @@ import logging
 import numpy
 import os
 import h5py
-from typing import Union
+import pint
 from nabu.stitching.config import PostProcessedSingleAxisStitchingConfiguration
 from nabu.stitching.alignment import AlignmentAxis1
 from nabu.stitching.alignment import PaddedRawData
@@ -13,11 +13,9 @@ from tomoscan.esrf import NXtomoScan
 from tomoscan.series import Series
 from tomoscan.volumebase import VolumeBase
 from tomoscan.esrf.volume import HDF5Volume
-from typing import Iterable
+from collections.abc import Iterable
 from contextlib import AbstractContextManager
-from pyunitsystem.metricsystem import MetricSystem
 from nabu.stitching.config import (
-    PostProcessedSingleAxisStitchingConfiguration,
     KEY_IMG_REG_METHOD,
 )
 from nabu.stitching.utils.utils import find_volumes_relative_shifts
@@ -26,6 +24,8 @@ from .single_axis import SingleAxisStitcher
 
 _logger = logging.getLogger(__name__)
 
+_ureg = pint.get_application_registry()
+
 
 class FlippingValueError(ValueError):
     pass
@@ -267,7 +267,7 @@ class PostProcessingStitching(SingleAxisStitcher):
                 axis_N_pos_px = []
                 for volume, pos_in_mm in zip(self.series, pos_as_mm):
                     voxel_size_m = self.configuration.voxel_size or volume.voxel_size
-                    axis_N_pos_px.append((pos_in_mm / MetricSystem.MILLIMETER.value) / voxel_size_m[0])
+                    axis_N_pos_px.append((pos_in_mm * _ureg.millimeter).to_base_units().magnitude / voxel_size_m[0])
                 return axis_N_pos_px
             else:
                 # deduce from motor position and pixel size
@@ -397,7 +397,7 @@ class PostProcessingStitching(SingleAxisStitcher):
 
         data_type = self.get_output_data_type()
 
-        if self.progress:
+        if self.progress is not None:
             self.progress.total = final_volume_shape[1]
 
         y_index = 0
@@ -412,7 +412,7 @@ class PostProcessingStitching(SingleAxisStitcher):
             "dtype": data_type,
             "dumper": self.dumper,
         }
-        from .dumper.postprocessing import PostProcessingStitchingDumperNoDD
+        from .dumper.postprocessing import PostProcessingStitchingDumperNoDD, PostProcessingStitchingDumperWithCache
 
         # TODO: FIXME: for now not very elegant but in the case of avoiding data duplication
         # we need to provide the the information about the stitched part shape.
@@ -421,8 +421,12 @@ class PostProcessingStitching(SingleAxisStitcher):
             output_dataset_args["stitching_sources_arr_shapes"] = tuple(
                 [(abs(overlap), n_slices, self._stitching_constant_length) for overlap in self._axis_0_rel_final_shifts]
             )
+        elif isinstance(self.dumper, PostProcessingStitchingDumperWithCache):
+            self.dumper.set_final_volume_shape(final_volume_shape)
 
-        with self.dumper.OutputDatasetContext(**output_dataset_args):
+        bunch_size = 50
+        # how many frame to we stitch between two read from disk / save to disk
+        with self.dumper.OutputDatasetContext(**output_dataset_args):  # noqa: SIM117
             # note: output_dataset is a HDF5 dataset if final volume is an HDF5 volume else is a numpy array
             with _RawDatasetsContext(
                 self._input_volumes,
@@ -431,11 +435,14 @@ class PostProcessingStitching(SingleAxisStitcher):
                 # note: raw_datasets can be numpy arrays or HDF5 dataset (in the case of HDF5Volume)
                 # to speed up we read by bunch of dataset. For numpy array this doesn't change anything
                 # but for HDF5 dataset this can speed up a lot the processing (depending on HDF5 dataset chuncks)
-                # note: we read trhough axis 1
+                # note: we read through axis 1
                 if isinstance(self.dumper, PostProcessingStitchingDumperNoDD):
                     self.dumper.raw_regions_hdf5_dataset = raw_datasets
+                if isinstance(self.dumper, PostProcessingStitchingDumperWithCache):
+                    self.dumper.init_cache(dump_axis=1, dtype=data_type, size=bunch_size)
+
                 for bunch_start, bunch_end in PostProcessingStitching._data_bunch_iterator(
-                    slices=self._slices_to_stitch, bunch_size=50
+                    slices=self._slices_to_stitch, bunch_size=bunch_size
                 ):
                     for data_frames in PostProcessingStitching._get_bunch_of_data(
                         bunch_start,
@@ -470,6 +477,9 @@ class PostProcessingStitching(SingleAxisStitcher):
                             self.progress.update()
                         y_index += 1
 
+                    if isinstance(self.dumper, PostProcessingStitchingDumperWithCache):
+                        self.dumper.dump_cache(nb_frames=(bunch_end - bunch_start))
+
     # alias to general API
     def _create_stitching(self, store_composition):
         self._create_stitched_volume(store_composition=store_composition)
@@ -518,7 +528,8 @@ class _RawDatasetsContext(AbstractContextManager):
                 else:
                     data = volume.load_data(store=False)
                     if data is None:
-                        raise ValueError(f"No data found for volume {volume.get_identifier()}")
+                        # TODO
+                        raise ValueError(f"No data found for volume {volume.get_identifier()}")  # noqa: TRY301
                 if axis_1_need_padding:
                     data = self.add_padding(data=data, axis_1_dim=axis_1_dim, alignment=self.alignment_axis_1)
                 datasets.append(data)
@@ -526,7 +537,7 @@ class _RawDatasetsContext(AbstractContextManager):
             # if some errors happen during loading HDF5
             for file_handled in self.__file_handlers:
                 file_handled.close()
-            raise e
+            raise e  # noqa: TRY201
 
         return datasets
 
@@ -536,9 +547,10 @@ class _RawDatasetsContext(AbstractContextManager):
             success = success and file_handler.close()
         if exc_type is None:
             return success
+        return None
 
-    def add_padding(self, data: Union[h5py.Dataset, numpy.ndarray], axis_1_dim, alignment: AlignmentAxis1):
-        alignment = AlignmentAxis1.from_value(alignment)
+    def add_padding(self, data: h5py.Dataset | numpy.ndarray, axis_1_dim: int, alignment: AlignmentAxis1):
+        alignment = AlignmentAxis1(alignment)
         if alignment is AlignmentAxis1.BACK:
             axis_1_pad_width = (axis_1_dim - data.shape[1], 0)
         elif alignment is AlignmentAxis1.CENTER:
