@@ -2,7 +2,8 @@ import numpy
 import logging
 import h5py
 import os
-from typing import Iterable
+import pint
+from collections.abc import Iterable
 from silx.io.url import DataUrl
 from silx.io.utils import get_data
 from datetime import datetime
@@ -26,8 +27,8 @@ from nabu.stitching.config import (
 from nabu.stitching.utils import find_projections_relative_shifts
 from functools import lru_cache as cache
 from .single_axis import SingleAxisStitcher
-from pyunitsystem.metricsystem import MetricSystem
 
+_ureg = pint.get_application_registry()
 
 _logger = logging.getLogger(__name__)
 
@@ -40,7 +41,6 @@ class PreProcessingStitching(SingleAxisStitcher):
     """
 
     def __init__(self, configuration, progress=None) -> None:
-        """ """
         if not isinstance(configuration, PreProcessedSingleAxisStitchingConfiguration):
             raise TypeError(
                 f"configuration is expected to be an instance of {PreProcessedSingleAxisStitchingConfiguration}. Get {type(configuration)} instead"
@@ -239,18 +239,18 @@ class PreProcessingStitching(SingleAxisStitcher):
             if not scan_0.field_of_view == scan_1.field_of_view:
                 raise ValueError(f"{scan_0} and {scan_1} have different field of view")
             # check distance
-            if scan_0.distance is None:
+            if scan_0.sample_detector_distance is None:
                 _logger.warning(f"no distance found for {scan_0}")
-            elif not numpy.isclose(scan_0.distance, scan_1.distance, rtol=10e-3):
+            elif not numpy.isclose(scan_0.sample_detector_distance, scan_1.sample_detector_distance, rtol=10e-3):
                 raise ValueError(f"{scan_0} and {scan_1} have different sample / detector distance")
             # check pixel size
-            if not numpy.isclose(scan_0.x_pixel_size, scan_1.x_pixel_size):
+            if not numpy.isclose(scan_0.sample_x_pixel_size, scan_1.sample_x_pixel_size):
                 raise ValueError(
-                    f"{scan_0} and {scan_1} have different x pixel size. {scan_0.x_pixel_size} vs {scan_1.x_pixel_size}"
+                    f"{scan_0} and {scan_1} have different x pixel size. {scan_0.sample_x_pixel_size} vs {scan_1.sample_x_pixel_size}"
                 )
-            if not numpy.isclose(scan_0.y_pixel_size, scan_1.y_pixel_size):
+            if not numpy.isclose(scan_0.sample_y_pixel_size, scan_1.sample_y_pixel_size):
                 raise ValueError(
-                    f"{scan_0} and {scan_1} have different y pixel size. {scan_0.y_pixel_size} vs {scan_1.y_pixel_size}"
+                    f"{scan_0} and {scan_1} have different y pixel size. {scan_0.sample_y_pixel_size} vs {scan_1.sample_y_pixel_size}"
                 )
 
         for scan in self.series:
@@ -292,7 +292,7 @@ class PreProcessingStitching(SingleAxisStitcher):
                 axis_N_pos_px = []
                 for scan, pos_in_mm in zip(self.series, pos_as_mm):
                     pixel_size_m = self.configuration.pixel_size or scan.pixel_size
-                    axis_N_pos_px.append((pos_in_mm / MetricSystem.MILLIMETER.value) / pixel_size_m)
+                    axis_N_pos_px.append((pos_in_mm * _ureg.millimeter).to_base_units().magnitude / pixel_size_m)
                 return axis_N_pos_px
             else:
                 # deduce from motor position and pixel size
@@ -472,7 +472,7 @@ class PreProcessingStitching(SingleAxisStitcher):
         """
         nx_tomo = NXtomo()
 
-        nx_tomo.energy = self.series[0].energy
+        nx_tomo.energy = self.series[0].energy * _ureg.keV
         start_times = list(filter(None, [scan.start_time for scan in self.series]))
         end_times = list(filter(None, [scan.end_time for scan in self.series]))
 
@@ -496,9 +496,9 @@ class PreProcessingStitching(SingleAxisStitcher):
 
         # handle detector (without frames)
         nx_tomo.instrument.detector.field_of_view = self.series[0].field_of_view
-        nx_tomo.instrument.detector.distance = self.series[0].distance
-        nx_tomo.instrument.detector.x_pixel_size = self.series[0].x_pixel_size
-        nx_tomo.instrument.detector.y_pixel_size = self.series[0].y_pixel_size
+        nx_tomo.instrument.detector.distance = self.series[0].sample_detector_distance * _ureg.meter
+        nx_tomo.instrument.detector.x_pixel_size = self.series[0].x_pixel_size * _ureg.meter
+        nx_tomo.instrument.detector.y_pixel_size = self.series[0].y_pixel_size * _ureg.meter
         nx_tomo.instrument.detector.image_key_control = [ImageKey.PROJECTION] * n_proj
         nx_tomo.instrument.detector.tomo_n = n_proj
         # note: stitching process insure un-flipping of frames. So make sure transformations is defined as an empty set
@@ -506,13 +506,13 @@ class PreProcessingStitching(SingleAxisStitcher):
 
         if isinstance(self.series[0], NXtomoScan):
             # note: first scan is always the reference as order to read data (so no rotation_angle inversion here)
-            rotation_angle = numpy.asarray(self.series[0].rotation_angle)
+            rotation_angle = numpy.asarray(self.series[0].rotation_angle) * _ureg.degree
             nx_tomo.sample.rotation_angle = rotation_angle[
                 numpy.asarray(self.series[0].image_key_control) == ImageKey.PROJECTION.value
             ]
         elif isinstance(self.series[0], EDFTomoScan):
-            nx_tomo.sample.rotation_angle = numpy.linspace(
-                start=0, stop=self.series[0].scan_range, num=self.series[0].tomo_n
+            nx_tomo.sample.rotation_angle = (
+                numpy.linspace(start=0, stop=self.series[0].scan_range, num=self.series[0].tomo_n) * _ureg.degree
             )
         else:
             raise NotImplementedError(
@@ -526,12 +526,15 @@ class PreProcessingStitching(SingleAxisStitcher):
             if isinstance(slices, slice):
                 return array[slices.start : slices.stop : 1]
             elif isinstance(slices, Iterable):
-                return list([array[index] for index in slices])
+                return [array[index] for index in slices]
             else:
-                raise RuntimeError("slices must be instance of a slice or of an iterable")
+                raise RuntimeError("slices must be instance of a slice or of an iterable")  # noqa: TRY004
 
-        nx_tomo.sample.rotation_angle = apply_slices_selection(
-            array=nx_tomo.sample.rotation_angle, slices=self._slices_to_stitch
+        nx_tomo.sample.rotation_angle = (
+            apply_slices_selection(
+                array=nx_tomo.sample.rotation_angle.to_base_units().magnitude, slices=self._slices_to_stitch
+            )
+            * _ureg.degree
         )
 
         # handle sample
@@ -560,7 +563,7 @@ class PreProcessingStitching(SingleAxisStitcher):
                 # note: if at least one has missing values the numpy.Array(x_translation) with create an error as well
                 x_translation = [0.0] * n_proj
                 _logger.warning("Unable to fin input nxtomo x_translation values. Set it to 0.0")
-            nx_tomo.sample.x_translation = x_translation
+            nx_tomo.sample.x_translation = x_translation * _ureg.meter
 
             y_translation = [
                 get_sample_translation_for_projs(scan, "y_translation")
@@ -575,7 +578,7 @@ class PreProcessingStitching(SingleAxisStitcher):
             else:
                 y_translation = [0.0] * n_proj
                 _logger.warning("Unable to fin input nxtomo y_translation values. Set it to 0.0")
-            nx_tomo.sample.y_translation = y_translation
+            nx_tomo.sample.y_translation = y_translation * _ureg.meter
             z_translation = [
                 get_sample_translation_for_projs(scan, "z_translation")
                 for scan in self.series
@@ -589,7 +592,7 @@ class PreProcessingStitching(SingleAxisStitcher):
             else:
                 z_translation = [0.0] * n_proj
                 _logger.warning("Unable to fin input nxtomo z_translation values. Set it to 0.0")
-            nx_tomo.sample.z_translation = z_translation
+            nx_tomo.sample.z_translation = z_translation * _ureg.meter
 
             nx_tomo.sample.name = self.series[0].sample_name
 
@@ -678,7 +681,7 @@ class PreProcessingStitching(SingleAxisStitcher):
             scans_projections_indexes = []
             for scan, reverse in zip(self.series, self.reading_orders):
                 scans_projections_indexes.append(sorted(scan.projections.keys(), reverse=(reverse == -1)))
-            if self.progress:
+            if self.progress is not None:
                 self.progress.total = self.get_n_slices_to_stitch()
 
             if isinstance(self._slices_to_stitch, slice):
@@ -760,13 +763,13 @@ class PreProcessingStitching(SingleAxisStitcher):
         """
         goal is to load contiguous projections as much as possible...
 
-        :param int bunch_start: begining of the bunch
+        :param int bunch_start: beginning of the bunch
         :param int bunch_end: end of the bunch
         :param int scans: ordered scan for which we want to get data
         :param scans_projections_indexes: tuple with scans and scan projection indexes to be loaded
         :param tuple flip_lr_arr: extra information from the user to left-right flip frames
         :param tuple flip_ud_arr: extra information from the user to up-down flip frames
-        :return: list of list. For each frame we want to stitch contains the (flat fielded) frames to stich together
+        :return: list of lists. Each frame to stitch contains the (flat-fielded) frames to stitch together
         """
         assert len(scans) == len(scans_projections_indexes)
         assert isinstance(flip_lr_arr, tuple)
@@ -794,7 +797,7 @@ class PreProcessingStitching(SingleAxisStitcher):
         ):
             i_frame = 0
             _, set_of_compacted_slices = get_compacted_dataslices(scan_urls, return_url_set=True)
-            for _, url in set_of_compacted_slices.items():
+            for url in set_of_compacted_slices.values():
                 scan = scans[i_scan]
                 url = DataUrl(
                     file_path=url.file_path(),
@@ -826,9 +829,9 @@ class PreProcessingStitching(SingleAxisStitcher):
                 else:
                     has_reduced_metadata = (
                         scan.reduced_flats_infos is not None
-                        and len(scan.reduced_flats_infos.machine_electric_current) > 0
+                        and len(scan.reduced_flats_infos.machine_current) > 0
                         and scan.reduced_darks_infos is not None
-                        and len(scan.reduced_darks_infos.machine_electric_current) > 0
+                        and len(scan.reduced_darks_infos.machine_current) > 0
                     )
                     if not has_reduced_metadata:
                         _logger.warning("no metadata about current found. Won't normalize according to machine current")
@@ -838,10 +841,8 @@ class PreProcessingStitching(SingleAxisStitcher):
                         flats=scan.reduced_flats,
                         darks=scan.reduced_darks,
                         radios_indices=radio_indices,
-                        radios_srcurrent=scan.electric_current[radio_indices] if has_reduced_metadata else None,
-                        flats_srcurrent=(
-                            scan.reduced_flats_infos.machine_electric_current if has_reduced_metadata else None
-                        ),
+                        radios_srcurrent=scan.machine_current[radio_indices] if has_reduced_metadata else None,
+                        flats_srcurrent=(scan.reduced_flats_infos.machine_current if has_reduced_metadata else None),
                     )
                     # note: we need to cast radios to float 32. Darks and flats are cast to anyway
                     data = ff_arrays.normalize_radios(raw_radios.astype(numpy.float32))
@@ -886,6 +887,9 @@ class PreProcessingStitching(SingleAxisStitcher):
         """
         make sure reduced dark and flats are existing otherwise compute them
         """
+        # TODO
+        # ruff: noqa: SIM105, S110
+        # --
         for scan in self.series:
             try:
                 reduced_darks, darks_infos = scan.load_reduced_darks(return_info=True)
@@ -896,7 +900,7 @@ class PreProcessingStitching(SingleAxisStitcher):
                 try:
                     # if we don't have write in the folder containing the .nx for example
                     scan.save_reduced_darks(reduced_darks, darks_infos=darks_infos)
-                except Exception as e:
+                except Exception:
                     pass
             scan.set_reduced_darks(reduced_darks, darks_infos=darks_infos)
 
@@ -909,7 +913,7 @@ class PreProcessingStitching(SingleAxisStitcher):
                 try:
                     # if we don't have write in the folder containing the .nx for example
                     scan.save_reduced_flats(reduced_flats, flats_infos=flats_infos)
-                except Exception as e:
+                except Exception:
                     pass
             scan.set_reduced_flats(reduced_flats, flats_infos=flats_infos)
 
@@ -945,13 +949,13 @@ class PreProcessingStitching(SingleAxisStitcher):
         """
         goal is to load contiguous projections as much as possible...
 
-        :param int bunch_start: begining of the bunch
+        :param int bunch_start: beginning of the bunch
         :param int bunch_end: end of the bunch
         :param int scans: ordered scan for which we want to get data
         :param scans_projections_indexes: tuple with scans and scan projection indexes to be loaded
         :param tuple flip_lr_arr: extra information from the user to left-right flip frames
         :param tuple flip_ud_arr: extra information from the user to up-down flip frames
-        :return: list of list. For each frame we want to stitch contains the (flat fielded) frames to stich together
+        :return: list of lists. Each frame to stitch contains the (flat-fielded) frames to stitch together
         """
         assert len(scans) == len(scans_projections_indexes)
         assert isinstance(flip_lr_arr, tuple)
@@ -979,7 +983,7 @@ class PreProcessingStitching(SingleAxisStitcher):
         ):
             i_frame = 0
             _, set_of_compacted_slices = get_compacted_dataslices(scan_urls, return_url_set=True)
-            for _, url in set_of_compacted_slices.items():
+            for url in set_of_compacted_slices.values():
                 scan = scans[i_scan]
                 url = DataUrl(
                     file_path=url.file_path(),
@@ -1000,20 +1004,20 @@ class PreProcessingStitching(SingleAxisStitcher):
 
                 missing = []
                 if len(scan.reduced_flats) == 0:
-                    missing = "flats"
+                    missing.append("flats")
                 if len(scan.reduced_darks) == 0:
-                    missing = "darks"
+                    missing.append("darks")
 
                 if len(missing) > 0:
-                    _logger.warning(f"missing {'and'.join(missing)}. Unable to do flat field correction")
+                    _logger.warning(f"missing {' and '.join(missing)}. Unable to do flat field correction")
                     ff_arrays = None
                     data = raw_radios
                 else:
                     has_reduced_metadata = (
                         scan.reduced_flats_infos is not None
-                        and len(scan.reduced_flats_infos.machine_electric_current) > 0
+                        and len(scan.reduced_flats_infos.machine_current) > 0
                         and scan.reduced_darks_infos is not None
-                        and len(scan.reduced_darks_infos.machine_electric_current) > 0
+                        and len(scan.reduced_darks_infos.machine_current) > 0
                     )
                     if not has_reduced_metadata:
                         _logger.warning("no metadata about current found. Won't normalize according to machine current")
@@ -1023,10 +1027,8 @@ class PreProcessingStitching(SingleAxisStitcher):
                         flats=scan.reduced_flats,
                         darks=scan.reduced_darks,
                         radios_indices=radio_indices,
-                        radios_srcurrent=scan.electric_current[radio_indices] if has_reduced_metadata else None,
-                        flats_srcurrent=(
-                            scan.reduced_flats_infos.machine_electric_current if has_reduced_metadata else None
-                        ),
+                        radios_srcurrent=scan.machine_current[radio_indices] if has_reduced_metadata else None,
+                        flats_srcurrent=(scan.reduced_flats_infos.machine_current if has_reduced_metadata else None),
                     )
                     # note: we need to cast radios to float 32. Darks and flats are cast to anyway
                     data = ff_arrays.normalize_radios(raw_radios.astype(numpy.float32))
