@@ -1,8 +1,6 @@
-import h5py
 import numpy
 import logging
-from math import ceil
-from typing import Optional, Iterable, Union
+from collections.abc import Iterable
 from tomoscan.series import Series
 from tomoscan.identifier import BaseIdentifier
 from nabu.stitching.stitcher.base import _StitcherBase, get_obj_constant_side_length
@@ -354,7 +352,7 @@ class SingleAxisStitcher(_StitcherBase, metaclass=_SingleAxisMetaClass):
 
     @staticmethod
     def stitch_frames(
-        frames: Union[tuple, numpy.ndarray],
+        frames: tuple | numpy.ndarray,
         axis,
         x_relative_shifts: tuple,
         y_relative_shifts: tuple,
@@ -368,7 +366,7 @@ class SingleAxisStitcher(_StitcherBase, metaclass=_SingleAxisMetaClass):
         return_composition_cls=False,
         alignment="center",
         pad_mode="constant",
-        new_width: Optional[int] = None,
+        new_width: int | None = None,
     ) -> numpy.ndarray:
         """
         shift frames according to provided `shifts` (as y, x tuples) then stitch all the shifted frames together and
@@ -471,12 +469,13 @@ class SingleAxisStitcher(_StitcherBase, metaclass=_SingleAxisMetaClass):
             pad_mode=pad_mode,
             new_unstitched_axis_size=new_width,
         )
-        dumper.save_stitched_frame(
-            stitched_frame=stitched_frame,
-            composition_cls=composition_cls,
-            i_frame=i_frame,
-            axis=1,
-        )
+        if dumper is not None:
+            dumper.save_stitched_frame(
+                stitched_frame=stitched_frame,
+                composition_cls=composition_cls,
+                i_frame=i_frame,
+                axis=1,
+            )
 
         if return_composition_cls:
             return stitched_frame, composition_cls
