@@ -1,6 +1,10 @@
 from nabu.stitching.stitcher.pre_processing import PreProcessingStitching
 from nabu.stitching.stitcher.post_processing import PostProcessingStitching
-from .dumper import PreProcessingStitchingDumper, PostProcessingStitchingDumperNoDD, PostProcessingStitchingDumper
+from .dumper import (
+    PreProcessingStitchingDumper,
+    PostProcessingStitchingDumperNoDD,
+    PostProcessingStitchingDumperWithCache,
+)
 from nabu.stitching.stitcher.single_axis import _SingleAxisMetaClass
 
 
@@ -26,12 +30,12 @@ class PreProcessingZStitcher(
 class PostProcessingZStitcher(
     PostProcessingStitching,
     metaclass=_SingleAxisMetaClass,
-    dumper_cls=PostProcessingStitchingDumper,
+    dumper_cls=PostProcessingStitchingDumperWithCache,
     axis=0,
 ):
     @property
     def serie_label(self) -> str:
-        return "z-serie"
+        return "z-series"
 
 
 class PostProcessingZStitcherNoDD(
@@ -42,4 +46,4 @@ class PostProcessingZStitcherNoDD(
 ):
     @property
     def serie_label(self) -> str:
-        return "z-serie"
+        return "z-series"
