@@ -10,7 +10,7 @@ from nabu.stitching.overlap import OverlapStitchingStrategy
 from nabu.stitching import config as stiching_config
 
 
-_stitching_types = list(stiching_config.StitchingType.values())
+_stitching_types = [st.value for st in list(stiching_config.StitchingType)]
 _stitching_types.append(None)
 
 
@@ -37,18 +37,16 @@ def test_stitching_config(stitching_type, option_level):
 
         assert "stitching" in config
         assert "type" in config["stitching"]
-        stitching_type = stiching_config.StitchingType.from_value(config["stitching"]["type"])
+        stitching_type = stiching_config.StitchingType(config["stitching"]["type"])
         if stitching_type is stiching_config.StitchingType.Z_POSTPROC:
             assert isinstance(
                 stiching_config.dict_to_config_obj(config),
                 stiching_config.PostProcessedSingleAxisStitchingConfiguration,
             )
-        elif stitching_type is stiching_config.StitchingType.Z_PREPROC:
-            assert isinstance(
-                stiching_config.dict_to_config_obj(config),
-                stiching_config.PreProcessedSingleAxisStitchingConfiguration,
-            )
-        elif stitching_type is stiching_config.StitchingType.Y_PREPROC:
+        elif (
+            stitching_type is stiching_config.StitchingType.Z_PREPROC
+            or stitching_type is stiching_config.StitchingType.Y_PREPROC
+        ):
             assert isinstance(
                 stiching_config.dict_to_config_obj(config),
                 stiching_config.PreProcessedSingleAxisStitchingConfiguration,
@@ -84,7 +82,7 @@ def test_stitching_config(stitching_type, option_level):
         assert isinstance(config_class_instance.to_dict(), dict)
 
 
-@pytest.mark.parametrize("stitching_strategy", OverlapStitchingStrategy.values())
+@pytest.mark.parametrize("stitching_strategy", [oss for oss in OverlapStitchingStrategy])
 @pytest.mark.parametrize("overwrite_results", (True, "False", 0, "1"))
 @pytest.mark.parametrize(
     "axis_shifts",
@@ -92,7 +90,7 @@ def test_stitching_config(stitching_type, option_level):
         "",
         None,
         "None",
-        "",
+        "",  # noqa: PT014
         "skimage",
         "nabu-fft",
     ),
@@ -176,7 +174,8 @@ def test_PreProcessedZStitchingConfiguration(
 
     from_dict = stiching_config.PreProcessedZStitchingConfiguration.from_dict(pre_process_config.to_dict())
     # workaround for scans because a new object is created each time
-    pre_process_config.settle_inputs
+    # ???
+    pre_process_config.settle_inputs  # noqa: B018
     assert len(from_dict.input_scans) == len(pre_process_config.input_scans)
     from_dict.input_scans = None
     pre_process_config.input_scans = None
