@@ -1,6 +1,6 @@
 import numpy
 import pytest
-from nabu.stitching.sample_normalization import normalize_frame, SampleSide, Method
+from nabu.stitching.sample_normalization import normalize_frame
 
 
 def test_normalize_frame():
