@@ -1,4 +1,4 @@
-from nabu.stitching.utils import has_itk, find_shift_with_itk
+from nabu.stitching.utils.utils import has_itk, find_shift_with_itk
 from scipy.ndimage import shift as shift_scipy
 import numpy
 import pytest
