@@ -395,7 +395,7 @@ def test_vol_z_stitching_with_alignment_axis_2(tmp_path, alignment_axis_2):
         axis_2_params={"img_reg_method": ShiftAlgorithm.NONE},
         slice_for_cross_correlation="middle",
         voxel_size=None,
-        alignment_axis_2=AlignmentAxis2.from_value(alignment_axis_2),
+        alignment_axis_2=AlignmentAxis2(alignment_axis_2),
     )
 
     stitcher = PostProcessZStitcher(z_stich_config, progress=None)
@@ -512,7 +512,7 @@ def test_vol_z_stitching_with_alignment_axis_1(tmp_path, alignment_axis_1):
         axis_2_params={"img_reg_method": ShiftAlgorithm.NONE},
         slice_for_cross_correlation="middle",
         voxel_size=None,
-        alignment_axis_1=AlignmentAxis1.from_value(alignment_axis_1),
+        alignment_axis_1=AlignmentAxis1(alignment_axis_1),
     )
 
     stitcher = PostProcessZStitcher(z_stich_config, progress=None)
@@ -770,6 +770,6 @@ def test_data_duplication(tmp_path, data_duplication):
     if not data_duplication:
         # make sure an error is raised if we try to ask for no data duplication and if we get some flips
         z_stich_config.flip_ud = (False, True, False)
-        with pytest.raises(ValueError):
+        with pytest.raises(ValueError):  # noqa: PT012
             stitcher = PostProcessZStitcherNoDD(z_stich_config, progress=None)
             stitcher.stitch()
