@@ -1,15 +1,13 @@
-from distutils.version import StrictVersion
-from typing import Optional, Union
+from enum import Enum
+from packaging.version import parse as parse_version
 import logging
 import functools
 import numpy
 from tomoscan.scanbase import TomoScanBase
 from tomoscan.volumebase import VolumeBase
 from nxtomo.utils.transformation import build_matrix, DetYFlipTransformation
-from silx.utils.enum import Enum as _Enum
 from scipy.fft import rfftn as local_fftn
 from scipy.fft import irfftn as local_ifftn
-from ..overlap import OverlapStitchingStrategy, ImageStichOverlapKernel
 from ..alignment import AlignmentAxis1, AlignmentAxis2, PaddedRawData
 from ...misc import fourier_filters
 from ...estimation.alignment import AlignmentBase
@@ -37,7 +35,7 @@ else:
     __has_sk_phase_correlation__ = True
 
 
-class ShiftAlgorithm(_Enum):
+class ShiftAlgorithm(Enum):
     """All generic shift search algorithm"""
 
     NABU_FFT = "nabu-fft"
@@ -59,7 +57,7 @@ class ShiftAlgorithm(_Enum):
         if value in ("", None):
             return ShiftAlgorithm.NONE
         else:
-            return super().from_value(value=value)
+            return super().__new__(cls, value)
 
 
 def find_frame_relative_shifts(
@@ -69,13 +67,13 @@ def find_frame_relative_shifts(
     overlap_axis: int,
     x_cross_correlation_function=None,
     y_cross_correlation_function=None,
-    x_shifts_params: Optional[dict] = None,
-    y_shifts_params: Optional[dict] = None,
+    x_shifts_params: dict | None = None,
+    y_shifts_params: dict | None = None,
 ):
     """
-    :param overlap_axis: axis in [0, 1] on which the overlap exists. In image space. So 0 is aka y and 1 as x
+    :param overlap_axis: axis in [0, 1] on which the overlap exists. In image space. So 0 is aka y and 1 as x.
     """
-    if not overlap_axis in (0, 1):
+    if overlap_axis not in (0, 1):
         raise ValueError(f"overlap_axis should be in (0, 1). Get {overlap_axis}")
     from nabu.stitching.config import (
         KEY_LOW_PASS_FILTER,
@@ -146,7 +144,7 @@ def find_frame_relative_shifts(
     }
 
     res_algo = {}
-    for shift_alg in set((x_cross_correlation_function, y_cross_correlation_function)):
+    for shift_alg in set((x_cross_correlation_function, y_cross_correlation_function)):  # noqa: C405
         if shift_alg not in shift_methods:
             raise ValueError(f"requested image alignment function not handled ({shift_alg})")
         try:
@@ -173,11 +171,11 @@ def find_volumes_relative_shifts(
     dtype,
     flip_ud_upper_frame: bool = False,
     flip_ud_lower_frame: bool = False,
-    slice_for_shift: Union[int, str] = "middle",
+    slice_for_shift: int | str = "middle",
     x_cross_correlation_function=None,
     y_cross_correlation_function=None,
-    x_shifts_params: Optional[dict] = None,
-    y_shifts_params: Optional[dict] = None,
+    x_shifts_params: dict | None = None,
+    y_shifts_params: dict | None = None,
     alignment_axis_2="center",
     alignment_axis_1="center",
 ):
@@ -200,8 +198,8 @@ def find_volumes_relative_shifts(
     else:
         raise ValueError(f"Stitching is done in 3D space. Expect axis to be in [0,2]. Get {overlap_axis}")
 
-    alignment_axis_2 = AlignmentAxis2.from_value(alignment_axis_2)
-    alignment_axis_1 = AlignmentAxis1.from_value(alignment_axis_1)
+    alignment_axis_2 = AlignmentAxis2(alignment_axis_2)
+    alignment_axis_1 = AlignmentAxis1(alignment_axis_1)
     assert dim_axis_1 > 0, "dim_axis_1 <= 0"
 
     if isinstance(slice_for_shift, str):
@@ -249,7 +247,7 @@ def find_volumes_relative_shifts(
 
     w_window_size = int(y_shifts_params.get(KEY_WINDOW_SIZE, 400))
     start_overlap = max(estimated_shifts[0] // 2 - w_window_size // 2, 0)
-    end_overlap = min(estimated_shifts[0] // 2 + w_window_size // 2, min(upper_frame.shape[0], lower_frame.shape[0]))
+    end_overlap = min(estimated_shifts[0] // 2 + w_window_size // 2, upper_frame.shape[0], lower_frame.shape[0])
 
     if start_overlap == 0:
         overlap_upper_frame = upper_frame[-end_overlap:]
@@ -302,12 +300,12 @@ def find_projections_relative_shifts(
     axis: int,
     flip_ud_upper_frame: bool = False,
     flip_ud_lower_frame: bool = False,
-    projection_for_shift: Union[int, str] = "middle",
+    projection_for_shift: int | str = "middle",
     invert_order: bool = False,
     x_cross_correlation_function=None,
     y_cross_correlation_function=None,
-    x_shifts_params: Optional[dict] = None,
-    y_shifts_params: Optional[dict] = None,
+    x_shifts_params: dict | None = None,
+    y_shifts_params: dict | None = None,
 ) -> tuple:
     """
     deduce the relative shift between the two scans.
@@ -321,7 +319,7 @@ def find_projections_relative_shifts(
     :param int axis: axis on which the overlap / stitching is happening. In the 3D space (sample, detector referential)
     :param bool flip_ud_upper_frame: is the upper frame flipped
     :param bool flip_ud_lower_frame: is the lower frame flipped
-    :param Union[int,str] projection_for_shift: index fo the projection to use (in projection space or in scan space ?. For now in projection) or str. If str must be in (`middle`, `first`, `last`)
+    :param projection_for_shift: index for the projection to use (in projection space or scan space?). If str must be in (`middle`, `first`, `last`)
     :param bool invert_order: are projections inverted between the two scans (case if rotation angle are inverted)
     :param str x_cross_correlation_function: optional method to refine x shift from computing cross correlation. For now valid values are: ("skimage", "nabu-fft")
     :param str y_cross_correlation_function: optional method to refine y shift from computing cross correlation. For now valid values are: ("skimage", "nabu-fft")
@@ -385,18 +383,16 @@ def find_projections_relative_shifts(
             cor_options=cor_options,
         )
 
-        estimated_shifts = tuple(
-            [
-                estimated_shifts[0],
-                (lower_scan_pos - upper_scan_pos),
-            ]
-        )
+        estimated_shifts = [
+            estimated_shifts[0],
+            (lower_scan_pos - upper_scan_pos),
+        ]
         x_cross_correlation_function = ShiftAlgorithm.NONE
 
     # } else we will compute shift from the flat projections
 
     def get_flat_fielded_proj(
-        scan: TomoScanBase, proj_index: int, reverse: bool, transformation_matrix: Optional[numpy.ndarray]
+        scan: TomoScanBase, proj_index: int, reverse: bool, transformation_matrix: numpy.ndarray | None
     ):
         first_proj_idx = sorted(lower_scan.projections.keys(), reverse=reverse)[proj_index]
         ff = scan.flat_field_correction(
@@ -464,7 +460,8 @@ def find_projections_relative_shifts(
     start_overlap = max(estimated_shifts[axis_proj_space] // 2 - w_window_size // 2, 0)
     end_overlap = min(
         estimated_shifts[axis_proj_space] // 2 + w_window_size // 2,
-        min(upper_proj.shape[axis_proj_space], lower_proj.shape[axis_proj_space]),
+        upper_proj.shape[axis_proj_space],
+        lower_proj.shape[axis_proj_space],
     )
     o_upper_sel = numpy.array(range(-end_overlap, -start_overlap))
     overlap_upper_frame = numpy.take_along_axis(
@@ -502,7 +499,7 @@ def find_shift_correlate(img1, img2, padding_mode="reflect"):
         padding_mode,
     )
 
-    img_shape = img1.shape[-2:]
+    img_shape = cc.shape  # Because cc.shape can differ from img_2.shape (e.g. in case of odd nb of cols)
     cc_vs = numpy.fft.fftfreq(img_shape[-2], 1 / img_shape[-2])
     cc_hs = numpy.fft.fftfreq(img_shape[-1], 1 / img_shape[-1])
 
@@ -523,7 +520,7 @@ def find_shift_with_itk(img1: numpy.ndarray, img2: numpy.ndarray) -> tuple:
         _logger.warning("itk is not installed. Please install it to find shift with it")
         return (0, 0)
 
-    if StrictVersion(itk.Version.GetITKVersion()) < StrictVersion("4.9.0"):
+    if parse_version(itk.Version.GetITKVersion()) < parse_version("4.9.0"):
         _logger.error("ITK 4.9.0 is required to find shift with it.")
         return (0, 0)
 
@@ -582,7 +579,7 @@ def find_shift_with_itk(img1: numpy.ndarray, img2: numpy.ndarray) -> tuple:
     return numpy.round(translation_along_y), numpy.round(translation_along_x)
 
 
-def from_slice_to_n_elements(slice_: Union[slice, tuple]):
+def from_slice_to_n_elements(slice_: slice | tuple):
     """Return the number of element in a slice or in a tuple"""
     if isinstance(slice_, slice):
         return (slice_.stop - slice_.start) / (slice_.step or 1)
