@@ -1,3 +1,4 @@
+from dataclasses import dataclass
 from itertools import product
 import tarfile
 import os
@@ -5,6 +6,7 @@ import numpy as np
 from scipy.signal.windows import gaussian
 from silx.resources import ExternalResources
 from silx.io.dictdump import nxtodict, dicttonx
+from nxtomo.application.nxtomo import ImageKey
 
 utilstest = ExternalResources(
     project="nabu", url_base="http://www.silx.org/pub/nabu/data/", env_key="NABU_DATA", timeout=60
@@ -14,14 +16,14 @@ __big_testdata_dir__ = os.environ.get("NABU_BIGDATA_DIR")
 if __big_testdata_dir__ is None or not (os.path.isdir(__big_testdata_dir__)):
     __big_testdata_dir__ = None
 
-__do_long_tests__ = os.environ.get("NABU_LONG_TESTS", False)
+__do_long_tests__ = os.environ.get("NABU_LONG_TESTS", False)  # noqa: PLW1508
 if __do_long_tests__:
     try:
         __do_long_tests__ = bool(int(__do_long_tests__))
     except:
         __do_long_tests__ = False
 
-__do_large_mem_tests__ = os.environ.get("NABU_LARGE_MEM_TESTS", False)
+__do_large_mem_tests__ = os.environ.get("NABU_LARGE_MEM_TESTS", False)  # noqa: PLW1508
 if __do_large_mem_tests__:
     try:
         __do_large_mem_tests__ = bool(int(__do_large_mem_tests__))
@@ -56,6 +58,38 @@ def get_data(*dataset_path):
     return np.load(dataset_downloaded_path)
 
 
+@dataclass
+class SimpleNXTomoDescription:
+    n_darks: int = 0
+    n_flats1: int = 0
+    n_projs: int = 0
+    n_flats2: int = 0
+    n_align: int = 0
+    frame_shape: tuple = None
+    dtype: np.dtype = np.uint16
+
+
+def get_dummy_nxtomo_info():
+    nx_fname = utilstest.getfile("dummy_nxtomo.nx")
+    data_desc = SimpleNXTomoDescription(
+        n_darks=10, n_flats1=11, n_projs=100, n_flats2=11, n_align=12, frame_shape=(11, 10), dtype=np.uint16
+    )
+    image_key = np.concatenate(
+        [
+            np.zeros(data_desc.n_darks, dtype=np.int32) + ImageKey.DARK_FIELD.value,
+            np.zeros(data_desc.n_flats1, dtype=np.int32) + ImageKey.FLAT_FIELD.value,
+            np.zeros(data_desc.n_projs, dtype=np.int32) + ImageKey.PROJECTION.value,
+            np.zeros(data_desc.n_flats2, dtype=np.int32) + ImageKey.FLAT_FIELD.value,
+            np.zeros(data_desc.n_align, dtype=np.int32) + ImageKey.ALIGNMENT.value,
+        ]
+    )
+    projs_vals = np.arange(data_desc.n_projs) + data_desc.n_flats1 + data_desc.n_darks
+    darks_vals = np.arange(data_desc.n_darks)
+    flats1_vals = np.arange(data_desc.n_darks, data_desc.n_darks + data_desc.n_flats1)
+    flats2_vals = np.arange(data_desc.n_darks, data_desc.n_darks + data_desc.n_flats2)
+    return nx_fname, data_desc, image_key, projs_vals, darks_vals, flats1_vals, flats2_vals
+
+
 def get_array_of_given_shape(img, shape, dtype):
     """
     From a given image, returns an array of the wanted shape and dtype.
@@ -81,12 +115,34 @@ def get_big_data(filename):
 
 
 def uncompress_file(compressed_file_path, target_directory):
-    with tarfile.open(compressed_file_path) as f:
-        f.extractall(path=target_directory)
+    if not tarfile.is_tarfile(compressed_file_path):
+        raise ValueError(f"Invalid tar file: {compressed_file_path}")
+
+    def is_safe_member(member, target_directory):
+        """Ensure the member does not extract outside the target directory."""
+        if not isinstance(member, tarfile.TarInfo):
+            return False  # Reject any unexpected type
+
+        abs_target = os.path.abspath(target_directory)
+        member_path = os.path.abspath(os.path.join(target_directory, member.name))
+        return member_path.startswith(abs_target)
+
+    def get_valid_members(tar):
+        members = [m for m in tar.getmembers() if is_safe_member(m, target_directory)]
+        if not members:
+            raise ValueError("No valid files to extract or archive contains unsafe paths.")
+        for member in members:
+            if not is_safe_member(member, target_directory):
+                raise ValueError(f"Unsafe path detected: {member.name}")
+
+    with tarfile.open(compressed_file_path, "r") as tar:
+        tar.extractall(  # noqa: S202 - what can be done in addition of the above checks ?
+            path=target_directory, members=get_valid_members(tar)
+        )
 
 
 def get_file(fname):
-    downloaded_file = dataset_downloaded_path = utilstest.getfile(fname)
+    downloaded_file = utilstest.getfile(fname)
     if ".tar" in fname:
         uncompress_file(downloaded_file, os.path.dirname(downloaded_file))
         downloaded_file = downloaded_file.split(".tar")[0]
@@ -104,7 +160,7 @@ def compare_arrays(arr1, arr2, tol, diff=None, absolute_value=True, percent=None
     arr2: numpy.ndarray
         Second array to compare
     tol: float
-        Tolerance indicating whether arrays are close to eachother.
+        Tolerance indicating whether arrays are close to each other.
     diff: numpy.ndarray, optional
         Difference `arr1 - arr2`. If provided, this array is taken instead of `arr1`
         and `arr2`.
@@ -204,10 +260,7 @@ def generate_nx_dataset(out_fname, image_key, data_volume=None, rotation_angle=N
     nx_entry = nx_dict["entry"]
 
     def _get_field(dict_, path):
-        if path.startswith("/"):
-            path = path[1:]
-        if path.endswith("/"):
-            path = path[:-1]
+        path = path.strip("/")
         split_path = path.split("/")
         if len(split_path) == 1:
             return dict_[split_path[0]]
