@@ -49,7 +49,6 @@ License follows.
 # *************************************************************************** #
 
 try:
-    import pycuda.gpuarray as garray
     import cupy as cp
     import pywt
     from cupyx.scipy.ndimage import median_filter
@@ -576,7 +575,6 @@ def remove_all_stripe_sinos(sinos, snr=3, la_size=61, sm_size=21, dim=1):
     return sinos
 
 
-from ..cuda.utils import pycuda_to_cupy
 def remove_all_stripe_pycuda(array, layout="radios", device_id=0, **kwargs):
     """
     Nabu interface to tomocupy "remove_all_stripe".
@@ -594,11 +592,9 @@ def remove_all_stripe_pycuda(array, layout="radios", device_id=0, **kwargs):
     See parameters of 'remove_all_stripe
     """
 
-    # Init cupy. Nabu currently does not use cupy, with exception of this module,
-    # so the initialization has to be done here.
     if getattr(remove_all_stripe, "_cupy_init", False) is False:
-        from cupy import cuda
-        cuda.Device(device_id).use()
+        import cupy # TODO elsewhere ?
+        cupy.cuda.Device(device_id).use()
         setattr(remove_all_stripe, "_cupy_init", True)
 
     # remove_all_stripe() in tomocupy expects a 3D array to build the "matindex" data structure.
@@ -611,10 +607,9 @@ def remove_all_stripe_pycuda(array, layout="radios", device_id=0, **kwargs):
         sinos = array
     # is_contiguous = sinos.flags.c_contigious
     n_sinos, n_a, n_x = sinos.shape
-    sinos_tmp = garray.zeros((1, n_a, n_x), dtype="f")
+    sinos_tmp = cupy.zeros((1, n_a, n_x), dtype="f")
     for i in range(n_sinos):
         sinos_tmp[0] = sinos[i]
-        cupy_sinos = pycuda_to_cupy(sinos_tmp) # no memory copy, the internal pointer is passed to pycuda
-        remove_all_stripe_sinos(cupy_sinos, **kwargs)
+        remove_all_stripe_sinos(sinos_tmp, **kwargs)
         sinos[i] = sinos_tmp[0]
     return array
