@@ -1,7 +1,8 @@
+from bisect import bisect_left
 from fnmatch import fnmatch
 from functools import partial
 import os
-from functools import partial, lru_cache
+from functools import lru_cache
 from itertools import product
 import warnings
 from time import time
@@ -74,6 +75,15 @@ def get_resource_file(filename, subfolder=None):
     return os.path.join(abspath, filename)
 
 
+def check_array(array, expected_shape, expected_dtype="f", array_name="array", check_contiguous=True):
+    if array.shape != expected_shape:
+        raise ValueError(f"Expected array with shape {expected_shape} for {array_name}, but got {array.shape}")
+    if array.dtype != np.dtype(expected_dtype):
+        raise ValueError(f"Expected array with data type {expected_dtype} for {array_name}, but got {array.dtype}")
+    if check_contiguous and (not (isinstance(array, np.ndarray)) or not (array.flags["C_CONTIGUOUS"])):
+        raise ValueError(f"Expected C-Contiguous array for {array_name}")
+
+
 def indices_to_slices(indices):
     """
     From a series of integer indices, return corresponding slice() objects.
@@ -94,7 +104,7 @@ def indices_to_slices(indices):
     jumps = np.hstack([-1, jumps, len(indices) - 1])
     slices = []
     for i in range(len(jumps) - 1):
-        slices.append(slice(indices[jumps[i] + 1], indices[jumps[i + 1]] + 1))
+        slices.append(slice(indices[jumps[i] + 1], indices[jumps[i + 1]] + 1))  # noqa: PERF401
     return slices
 
 
@@ -180,10 +190,12 @@ def list_match_queries(available, queries):
     Given a list of strings, return all items matching any of one elements of "queries"
     """
     matches = []
+    if isinstance(queries, str):
+        queries = [queries]
     for a in available:
         for q in queries:
             if fnmatch(a, q):
-                matches.append(a)
+                matches.append(a)  # noqa: PERF401
     return matches
 
 
@@ -211,7 +223,7 @@ def _sizeof(Type):
     return np.dtype(Type).itemsize
 
 
-class _Default_format(dict):
+class _DefaultFormat(dict):
     """
     https://docs.python.org/3/library/stdtypes.html
     """
@@ -224,7 +236,7 @@ def safe_format(str_, **kwargs):
     """
     Alternative to str.format(), but does not throw a KeyError when fields are missing.
     """
-    return str_.format_map(_Default_format(**kwargs))
+    return str_.format_map(_DefaultFormat(**kwargs))
 
 
 def get_ftype(url):
@@ -296,7 +308,7 @@ def view_as_images_stack(img):
 
 
 def rescale_integers(items, new_tot):
-    """ "
+    """
     From a given sequence of integers, create a new sequence
     where the sum of all items must be equal to "new_tot".
     The relative contribution of each item to the new total is approximately kept.
@@ -329,6 +341,8 @@ def merged_shape(shapes, axis=0):
         return (shapes[0][0], n_img, shapes[0][2])
     elif axis == 2:
         return shapes[0][:2] + (n_img,)
+    else:
+        raise ValueError
 
 
 def is_device_backend(backend):
@@ -350,7 +364,7 @@ def get_decay(curve, cutoff=1e3, vmax=None):
     """
     if vmax is None:
         vmax = curve.max()
-    return np.argmax(np.abs(curve) < vmax / cutoff)
+    return int(np.argmax(np.abs(curve) < vmax / cutoff))
 
 
 @lru_cache(maxsize=1)
@@ -371,7 +385,7 @@ def generate_powers():
     powers = product(*valuations)
     res = []
     for pw in powers:
-        res.append(np.prod(list(map(lambda x: x[0] ** x[1], zip(primes, pw)))))
+        res.append(np.prod(list(map(lambda x: x[0] ** x[1], zip(primes, pw)))))  # noqa: PERF401
     return np.unique(res)
 
 
@@ -422,10 +436,14 @@ def partition_dict(dict_, n_partitions):
 
 
 def first_dict_item(dict_):
-    keys = sorted(list(dict_.keys()))
+    keys = sorted(dict_.keys())
     return dict_[keys[0]]
 
 
+def first_generator_item(gen):
+    return next(iter(gen))  # instead of list(gen)[0]
+
+
 def subsample_dict(dic, subsampling_factor):
     """
     Subsample a dict where keys are integers.
@@ -437,7 +455,7 @@ def subsample_dict(dic, subsampling_factor):
     return res
 
 
-def compare_dicts(dic1, dic2):
+def compare_dicts(dic1, dic2):  # noqa: PLR0911
     """
     Compare two dictionaries. Return None if and only iff the dictionaries are the same.
 
@@ -530,6 +548,37 @@ def restore_items_in_list(list_, removed_items):
         list_.insert(idx, val)
 
 
+def split_list(lst, n):
+    """
+    Split a list into n roughly equal parts, similar to numpy.array_split.
+
+    Parameters
+    -----------
+    lst: list
+        The list to split
+    n: int
+        Number of parts
+
+    Returns
+    --------
+    list of lists: The split parts
+    """
+    k, m = divmod(len(lst), n)
+    return [lst[i * k + min(i, m) : (i + 1) * k + min(i + 1, m)] for i in range(n)]
+
+
+def search_sorted(arr, val):
+    """
+    Binary search that returns the "nearest" index given a query,
+    i.e find "i" that minimizes   abs(arr[i] - val)
+    It does not return the "insersion point" contrarily to numpy.searchsorted() or bisect_left
+    """
+    pos = bisect_left(arr, val)
+    if pos == len(arr):
+        return len(arr) - 1
+    return pos - 1 if abs(val - arr[pos - 1]) < abs(arr[pos] - val) else pos
+
+
 def check_supported(param_value, available, param_desc):
     if param_value not in available:
         raise ValueError("Unsupported %s '%s'. Available are: %s" % (param_desc, param_value, str(available)))
@@ -629,7 +678,7 @@ def get_num_threads(n=None):
         return min(n_avail, n)
 
 
-class DictToObj(object):
+class DictToObj:
     """utility class to transform a dictionary into an object with dictionary items as members.
     Example:
 
@@ -644,16 +693,19 @@ class DictToObj(object):
 
 def remove_parenthesis_or_brackets(input_str):
     """
-    clear string from left and or roght parenthesis / braquets
+    Clear a string from left and/or right parentheses or brackets.
     """
-    if input_str.startswith("(") and input_str.endswith(")") or input_str.startswith("[") and input_str.endswith("]"):
+    if (input_str.startswith("(") and input_str.endswith(")")) or (
+        input_str.startswith("[") and input_str.endswith("]")
+    ):
         input_str = input_str[1:-1]
     return input_str
 
 
 def filter_str_def(elmt):
-    """clean elemt if is a string defined from a text file.
-    Remove some character that could have be put on left or right and some empty spaces"""
+    """Clean element if it is a string defined from a text file.
+    Remove characters that could have been put on the left or right, and strip empty spaces.
+    """
     if elmt is None:
         return None
     assert isinstance(elmt, str)
@@ -676,7 +728,7 @@ def convert_str_to_tuple(input_str: str, none_if_empty: bool = False):
     if isinstance(input_str, tuple):
         return input_str
     if not isinstance(input_str, str):
-        raise TypeError("input_str should be a string not {}, {}".format(type(input_str), input_str))
+        raise TypeError(f"input_str should be a string not {type(input_str)}, {input_str}")
     input_str = input_str.lstrip(" ").lstrip("(").lstrip("[").lstrip(" ").rstrip(" ")
     input_str = remove_parenthesis_or_brackets(input_str)
     input_str = input_str.replace("\n", ",")
@@ -702,9 +754,9 @@ def concatenate_dict(dict_1, dict_2) -> dict:
     return res
 
 
-class BaseClassError:
+class BaseClassError(BaseException):
     def __init__(self, *args, **kwargs):
-        raise ValueError("Base class")
+        raise NotImplementedError("Base class")
 
 
 def MissingComponentError(msg):
@@ -767,7 +819,7 @@ def median2(img):
     Roughly same speed as scipy median filter, but more memory demanding.
     """
     img2 = extend_image_onepixel(img)
-    I = np.array(
+    img3 = np.array(
         [
             img2[0:-2, 0:-2],
             img2[0:-2, 1:-1],
@@ -780,13 +832,18 @@ def median2(img):
             img2[2:, 2:],
         ]
     )
-    return np.median(I, axis=0)
+    return np.median(img3, axis=0)
 
 
 # ------------------------------------------------------------------------------
 # ---------------------------- Decorators --------------------------------------
 # ------------------------------------------------------------------------------
 
+
+def no_decorator(func):
+    return func
+
+
 _warnings = {}
 
 
@@ -820,6 +877,7 @@ def warning(msg):
                 print(msg)
                 res = func(*args, **kwargs)
                 return res
+            return None
 
         return wrapper
 
@@ -840,7 +898,7 @@ def deprecated(msg, do_print=False):
 
 def deprecated_class(msg, do_print=False):
     def decorator(cls):
-        class wrapper:
+        class Wrapper:
             def __init__(self, *args, **kwargs):
                 deprecation_warning(msg, do_print=do_print, func_name=cls.__name__)
                 self.wrapped = cls(*args, **kwargs)
@@ -849,7 +907,7 @@ def deprecated_class(msg, do_print=False):
             def __getattr__(self, name):
                 return getattr(self.wrapped, name)
 
-        return wrapper
+        return Wrapper
 
     return decorator
 
@@ -905,7 +963,7 @@ from warnings import catch_warnings
 # catch_warnings() does not have "action=XX" kwarg for python < 3.11
 from sys import version_info
 
-if version_info.major == 3 and version_info.minor < 11:
+if version_info.major == 3 and version_info.minor < 11:  # noqa: YTT204 - not sure about this
 
     def dummy(*args, **kwargs):
         pass
